@@ -1,41 +1,9 @@
-;*---------------------------------------------------------------------*/
-;*   A pratical implementation for the Scheme programming language     */
-;*                                                                     */
-;*                                    ,--^,                            */
-;*                              _ ___/ /|/                             */
-;*                          ,;'( )__, ) '                              */
-;*                         ;;  //   L__.                               */
-;*                         '   \\   /  '                               */
-;*                              ^   ^                                  */
-;*                                                                     */
-;*   Copyright (c) 1992-1999 Manuel Serrano                            */
-;*                                                                     */
-;*     Bug descriptions, use reports, comments or suggestions are      */
-;*     welcome. Send them to                                           */
-;*       bigloo-request@kaolin.unice.fr                                */
-;*       http://kaolin.unice.fr/bigloo                                 */
-;*                                                                     */
-;*   This program is free software; you can redistribute it            */
-;*   and/or modify it under the terms of the GNU General Public        */
-;*   License as published by the Free Software Foundation; either      */
-;*   version 2 of the License, or (at your option) any later version.  */
-;*                                                                     */
-;*   This program is distributed in the hope that it will be useful,   */
-;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
-;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
-;*   GNU General Public License for more details.                      */
-;*                                                                     */
-;*   You should have received a copy of the GNU General Public         */
-;*   License along with this program; if not, write to the Free        */
-;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
-;*   MA 02111-1307, USA.                                               */
-;*---------------------------------------------------------------------*/
 ;*=====================================================================*/
 ;*    serrano/prgm/project/bigloo/bde/bdepend/bdepend.scm              */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Sun May 31 07:37:29 1998                          */
-;*    Last change :  Fri Oct 30 07:29:13 1998 (serrano)                */
+;*    Last change :  Thu Feb  8 12:05:05 2001 (serrano)                */
 ;*    -------------------------------------------------------------    */
 ;*    The Bigloo depend utility.                                       */
 ;*=====================================================================*/
@@ -44,25 +12,28 @@
 ;*    The module                                                       */
 ;*---------------------------------------------------------------------*/
 (module bdepend
-   (include "include/args.sch")
-   (main    main))
+   (main main))
 
 ;*---------------------------------------------------------------------*/
 ;*    Global parameters                                                */
 ;*---------------------------------------------------------------------*/
-(define *bdepend-version*      "0.0")
-(define *bdepend-path*         '())
-(define *bdepend-exclude-path* '())
-(define *bdepend-source-files* '())
-(define *bdepend-makefile*     #f)
-(define *bdepend-suffixes*     '("scm" "sch" "bgl"))
+(define *bdepend-version*          "0.1")
+(define *bdepend-path*             '())
+(define *bdepend-exclude-path*     '())
+(define *bdepend-source-files*     '())
+(define *bdepend-makefile*         #f)
+(define *bdepend-suffixes*         '("scm" "sch" "bgl"))
+(define *bdepend-gui-suffix*       "bld")
 (define *bdepend-initial-suffixes* *bdepend-suffixes*)
-(define *bdepend-verbose*      #f)
+(define *bdepend-verbose*          #f)
 
-(define *bdepend-start-sentinel* "#bdepend start (don't edit)")
-(define *bdepend-stop-sentinel*  "#bdepend stop")
-(define *bdepend-iport*          #f)
-(define *bdepend-mco?*           #t)
+(define *bdepend-start-sentinel*   "#bdepend start (don't edit)")
+(define *bdepend-stop-sentinel*    "#bdepend stop")
+(define *bdepend-iport*            #f)
+(define *bdepend-mco?*             #t)
+(define *bdepend-jvm?*             #t)
+(define *bdepend-obj/mco-dir*      "")
+(define *bdepend-include-path*     '("."))
 
 ;*---------------------------------------------------------------------*/
 ;*    main ...                                                         */
@@ -97,31 +68,41 @@
    (define (usage args-parse-usage level)
       (print "usage: bdepend [options] file ...")
       (newline)
-      (args-parse-usage)
+      (args-parse-usage #f)
       (newline))
    (args-parse (cdr cmd-args)
       (("?")
        (usage args-parse-usage 1))
-      (("-help" (synopsis "This help message"))
+      (("-help" (help "This help message"))
        (usage args-parse-usage 1))
-      (("-search-path" ?path (synopsis "Add search path"))
+      (("-search-path" ?path (help "Add search path"))
        (if (directory? path)
 	   (set! *bdepend-path* (cons path *bdepend-path*))
 	   (warning "bdepend" "Can't find search directory -- " path)))
-      (("-exclude-path" ?path (synopsis "Exclude search path"))
+      (("-exclude-path" ?path (help "Exclude search path"))
        (if (directory? path)
 	   (set! *bdepend-exclude-path* (cons path *bdepend-exclude-path*))
 	   (warning "bdepend" "Can't find search directory -- " path)))
-      (("-suffix" ?suf (synopsis "Add Bigloo source suffixes"))
+      (("-I" ?path (help "Add <name> to the include path"))
+       (set! *bdepend-include-path* (cons path *bdepend-include-path*)))
+      (("-suffix" ?suf (help "Add Bigloo source suffixes"))
        (set! *bdepend-suffixes* (cons suf *bdepend-suffixes*)))
-      (("-o" ?name (synopsis "Set makefile name"))
+      (("-gui-suffix" ?suf (help "Set GUI suffixe"))
+       (set! *bdepend-gui-suffix* suf))
+      (("-o" ?name (help "Set makefile name"))
        (set! *bdepend-makefile* name))
-      (("-v" (synopsis "Be verbose"))
+      (("-v" (help "Be verbose"))
        (set! *bdepend-verbose* #t))
-      (("-fno-mco" (synopsis "Don't produce mco dependences"))
+      (("-fno-mco" (help "Don't produce mco dependences"))
        (set! *bdepend-mco?* #f))
-      (("-fmco" (synopsis "Do produce mco dependences"))
+      (("-fmco" (help "Do produce mco dependences"))
        (set! *bdepend-mco?* #t))
+      (("-fno-jvm" (help "Don't produce dependences for .class"))
+       (set! *bdepend-jvm?* #f))
+      (("-fjvm" (help "Do produce dependences for .class"))
+       (set! *bdepend-jvm?* #t))
+      (("-obj-dir" ?pref (help "directory prefix for .o/.mco files"))
+       (set! *bdepend-obj/mco-dir* (string-append pref "/")))
       (else
        (set! *bdepend-source-files* (cons else *bdepend-source-files*)))))
 
@@ -280,7 +261,12 @@
    ;; and we start emitting objects
    (let ((objects (find-all-dependences *bdepend-source-files*)))
       (for-each (lambda (module)
-		   (wprin (source->object (car module)) ": ")
+		   (wprin (if (source? (car module))
+			      (source->object (car module))
+			      (car module)))
+		   (if (and (source? (car module)) *bdepend-jvm?*)
+		       (wprin " " (source->class (car module))))
+		   (wprin ": ")
 		   (for-each (lambda (object)
 				(let ((len (+fx 1 (string-length object))))
 				   (if (>fx len 74)
@@ -304,6 +290,18 @@
 (define (source? fname)
    (member (suffix fname) *bdepend-suffixes*))
 
+;*---------------------------------------------------------------------*/
+;*    gui? ...                                                         */
+;*---------------------------------------------------------------------*/
+(define (gui? fname)
+   (string=? (suffix fname) *bdepend-gui-suffix*))
+
+;*---------------------------------------------------------------------*/
+;*    gui->source ...                                                  */
+;*---------------------------------------------------------------------*/
+(define (gui->source fname)
+   (string-append (prefix fname) ".scm"))
+
 ;*---------------------------------------------------------------------*/
 ;*    source->module ...                                               */
 ;*---------------------------------------------------------------------*/
@@ -337,7 +335,7 @@
 			      (if (not (memq fname *bdepend-exclude-path*))
 				  (find-all-bigloo-sources/basename
 				   (string-append fname "/")
-				   (directory->list fname)))) 
+				   (directory->list fname))))
 			     ((source? fname)
 			      (let ((module (source->module fname)))
 				 (if (symbol? module)
@@ -352,12 +350,9 @@
 ;*---------------------------------------------------------------------*/
 ;*    *object-env* ...                                                 */
 ;*---------------------------------------------------------------------*/
-(define *object-env*
-   (make-hash-table 1024
-		    (lambda (o) (string->0..2^x-1 o 10))
-		    (lambda (x) (car x))
-		    string=?
-		    64))
+(define *object-env* (make-hashtable))
+
+(define *include-env* (make-hashtable))
 
 ;*---------------------------------------------------------------------*/
 ;*    find-all-dependences ...                                         */
@@ -367,23 +362,38 @@
        (print "Generating object list..."))
    (for-each add-one-source! source-files)
    (let ((res '()))
-      (for-each-hash (lambda (src)
-			(if (pair? (cdr src))
-			    (set! res (cons src res))))
-		     *object-env*)
+      (hashtable-for-each *object-env*
+			  (lambda (key src)
+			     (if (pair? (cdr src))
+				 (set! res (cons src res)))))
       res))
 
 ;*---------------------------------------------------------------------*/
 ;*    add-one-source! ...                                              */
 ;*---------------------------------------------------------------------*/
 (define (add-one-source! source)
-   (if (not (get-hash source *object-env*))
+   (if (not (hashtable-get *object-env* source))
        (let ((object (list source)))
 	  (if *bdepend-verbose* (print source ":"))
-	  (put-hash! object *object-env*)
-	  (set-cdr! object (find-imported-modules source))
-	  source)))
+	  (hashtable-put! *object-env* source object)
+	  (set-cdr! object (find-dependences source))
+	  (let ((include-files (hashtable-get *include-env* source)))
+	     (when (and include-files *bdepend-mco?*)
+		(let ((key (source->mco source)))
+		   (hashtable-put! *object-env*
+				   key
+				   (cons  key (cdr include-files))))))
+	  source)
+       source))
 
+;*---------------------------------------------------------------------*/
+;*    find-dependences ...                                             */
+;*---------------------------------------------------------------------*/
+(define (find-dependences source)
+   (if (gui? source)
+       (list (gui->source source))
+       (find-imported-modules source)))
+			  
 ;*---------------------------------------------------------------------*/
 ;*    find-imported-modules ...                                        */
 ;*---------------------------------------------------------------------*/
@@ -396,9 +406,9 @@
 		 (try (let ((module (read port)))
 			 (match-case module
 			    ((module (? symbol?) . ?clauses)
-			     (find-imported-modules/clauses clauses))
+			     (find-imported-modules/clauses fname clauses))
 			    ((directives . ?clauses)
-			     (find-imported-modules/clauses clauses))
+			     (find-imported-modules/clauses fname clauses))
 			    (else
 			     '())))
 		      (lambda (escape obj proc msg)
@@ -410,18 +420,35 @@
 ;*    source->object ...                                               */
 ;*---------------------------------------------------------------------*/
 (define (source->object src)
-   (string-append (prefix src) ".o"))
+   (string-append *bdepend-obj/mco-dir* (prefix src) ".o"))
 
 ;*---------------------------------------------------------------------*/
 ;*    source->mco ...                                                  */
 ;*---------------------------------------------------------------------*/
 (define (source->mco src)
-   (string-append (prefix src) ".mco"))
+   (string-append *bdepend-obj/mco-dir* (prefix src) ".mco"))
+
+;*---------------------------------------------------------------------*/
+;*    source->class ...                                                */
+;*---------------------------------------------------------------------*/
+(define (source->class src)
+   (string-append *bdepend-obj/mco-dir* (prefix src) ".class"))
 
 ;*---------------------------------------------------------------------*/
 ;*    find-imported-modules/clauses ...                                */
 ;*---------------------------------------------------------------------*/
-(define (find-imported-modules/clauses clauses)
+(define (find-imported-modules/clauses source clauses)
+   (define (find-imported-file file)
+      (let find ((search-dirs *bdepend-path*))
+	 (if (null? search-dirs)
+	     file
+	     (let ((find-file (string-append
+			       (car search-dirs)
+			       "/"
+			       file)))
+		(if (file-exists? find-file)
+		    find-file
+		    (find (cdr search-dirs)))))))
    (define (find-imported-modules/import import)
       (match-case import
 	 (((and ?module (? symbol?)) (and ?fname (? string?)) . ?rest)
@@ -467,9 +494,50 @@
 			  (if (pair? aux)
 			      (append aux res)
 			      res))))))
+ 	 ((extern . ?eclauses)
+	  ;; we only fetch external include clauses
+ 	  (let loop ((clauses eclauses)
+		     (all     '())
+ 		     (res     '()))
+	     (cond
+		((null? clauses)
+		 (if (pair? all)
+		     (hashtable-put! *include-env* source (cons source all)))
+		 res)
+		(else
+		 (match-case (car clauses)
+		    ((include ?fname)
+		     (let ((f (find-file/path fname *bdepend-include-path*)))
+			;; we don't have to include a file if it does not
+			;; exists otherwise we could have dependencies
+			;; such as:
+			;;   foo.scm: signal.h
+			(if (string? f)
+			    (loop (cdr clauses)
+				  (cons f all)
+				  (cons fname res))
+			    (loop (cdr clauses)
+				  all
+				  res))))
+		    (else
+		     (loop (cdr clauses) all res)))))))
+ 	 ((load . ?tags)
+ 	  (let loop ((tag tags)
+ 		     (fnames '()))
+	     (cond
+		((null? tag)
+		 (let ((actual-includes (map find-imported-file fnames)))
+		    (for-each add-one-source! actual-includes)
+		    actual-includes)
+		 fnames)
+		((pair? (car tag))
+		 (loop (cdr tag) (append fnames (cdar tag))))
+		(else
+		 (loop (cdr tag) fnames)))))
 	 ((include . ?fnames)
-	  (for-each add-one-source! fnames)
-	  fnames)
+ 	  (let ((actual-includes (map find-imported-file fnames)))
+ 	     (for-each add-one-source! actual-includes)
+ 	     actual-includes))
 	 (else
 	  '())))
    (let loop ((clauses clauses)
@@ -480,4 +548,3 @@
 		(append (find-imported-modules/clause (car clauses)) res)))))
 
    
-
