@@ -1,41 +1,9 @@
-;*---------------------------------------------------------------------*/
-;*   A pratical implementation for the Scheme programming language     */
-;*                                                                     */
-;*                                    ,--^,                            */
-;*                              _ ___/ /|/                             */
-;*                          ,;'( )__, ) '                              */
-;*                         ;;  //   L__.                               */
-;*                         '   \\   /  '                               */
-;*                              ^   ^                                  */
-;*                                                                     */
-;*   Copyright (c) 1992-1999 Manuel Serrano                            */
-;*                                                                     */
-;*     Bug descriptions, use reports, comments or suggestions are      */
-;*     welcome. Send them to                                           */
-;*       bigloo-request@kaolin.unice.fr                                */
-;*       http://kaolin.unice.fr/bigloo                                 */
-;*                                                                     */
-;*   This program is free software; you can redistribute it            */
-;*   and/or modify it under the terms of the GNU General Public        */
-;*   License as published by the Free Software Foundation; either      */
-;*   version 2 of the License, or (at your option) any later version.  */
-;*                                                                     */
-;*   This program is distributed in the hope that it will be useful,   */
-;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
-;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
-;*   GNU General Public License for more details.                      */
-;*                                                                     */
-;*   You should have received a copy of the GNU General Public         */
-;*   License along with this program; if not, write to the Free        */
-;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
-;*   MA 02111-1307, USA.                                               */
-;*---------------------------------------------------------------------*/
 ;*=====================================================================*/
 ;*    serrano/prgm/project/bigloo/bde/bpp/bpp.scm                      */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Marc Feeley (c)                                   */
 ;*    Creation    :  Thu Jan  7 13:02:43 1993                          */
-;*    Last change :  Sat Aug 15 08:50:58 1998 (serrano)                */
+;*    Last change :  Thu Nov 29 10:53:24 2001 (serrano)                */
 ;*    -------------------------------------------------------------    */
 ;*    A Scheme pretty-printer                                          */
 ;*=====================================================================*/
@@ -219,7 +187,7 @@
     (define (length1? l) (and (pair? l) (null? (cdr l))))
     (let ((head (car l)) (tail (cdr l)))
       (case head
-        ((QUOTE QUASIQUOTE UNQUOTE UNQUOTE-SPLICING) (length1? tail))
+        ((quote quasiquote unquote unquote-splicing) (length1? tail))
         (else                                        #f))))
 
   (define (read-macro-body l)
@@ -228,10 +196,10 @@
   (define (read-macro-prefix l)
     (let ((head (car l)) (tail (cdr l)))
       (case head
-        ((QUOTE)            "'")
-        ((QUASIQUOTE)       "`")
-        ((UNQUOTE)          ",")
-        ((UNQUOTE-SPLICING) ",@"))))
+        ((quote)            "'")
+        ((quasiquote)       "`")
+        ((unquote)          ",")
+        ((unquote-splicing) ",@"))))
 
   (define (out str col)
     (and col (output str) (+ col (string-length str))))
@@ -252,11 +220,7 @@
                      (else      (out ")" (wr l (out " . " col)))))))
         (out "()" col)))
 
-    (cond ((match-case obj
-	      ((COMMENT (? integer?) (? string?))
-	       #t)
-	      (else
-	       #f))
+    (cond ((comment? obj)
 	   (let ((add (- *width* (string-length (caddr obj)) 3)))
 	      (if (<= add 0)
 		  (out (caddr obj) col)
@@ -268,22 +232,25 @@
 				      (out (vector-prefix obj) col)))
           ((boolean? obj)     (out (if obj "#t" "#f") col))
           ((number? obj)      (out (number->string obj) col))
-          ((symbol? obj)      (case *case*
-				 ((respect)
-				  (out (symbol->string obj) col))
-				 ((upper)
-				  (out (string-upcase (symbol->string obj))
-				       col))
-				 (else
-				  (out (string-downcase (symbol->string obj))
-				       col))))
+          ((symbol? obj)      (let ((str (if (whitespace-symbol? obj)
+ 					     (let ((p (open-output-string)))
+						(write obj p)
+						(close-output-port p))
+					     (symbol->string obj))))
+				 (case *case*
+				    ((respect)
+				     (out str col))
+				    ((upper)
+				     (out (string-upcase str) col))
+				    (else
+				     (out (string-downcase str) col)))))
           ((procedure? obj)   (out "#<procedure>" col))
           ((string? obj)      (let ((obj (string-for-read obj)))
 				 (if display?
 				     (out obj col)
 				     (let loop ((i 0)
 						(j 0) 
-						(col (out "#\"" col)))
+						(col (out "\"" col)))
 					(if (and col (< j (string-length obj)))
 					    (let ((c (string-ref obj j)))
 					       (loop i (+ j 1) col))
@@ -323,7 +290,8 @@
              (spaces (- to col) col))))
 
     (define (pr obj col extra pp-pair)
-      (if (or (pair? obj) (vector? obj)) ; may have to split on multiple lines
+      (if (or (and (not (comment? obj)) (pair? obj))
+	      (vector? obj)) ; may have to split on multiple lines
         (let ((result '())
               (left (min (+ (- (- width col) extra) 1) max-expr-width)))
           (generic-write obj display? #f
@@ -384,10 +352,19 @@
       (let loop ((l l) (col col1))
         (and col
              (cond ((pair? l)
-                    (let ((rest (cdr l)))
-                      (let ((extra (if (null? rest) (+ extra 1) 0)))
-                        (loop rest
-                              (pr (car l) (indent col2 col) extra pp-item)))))
+		    (if (and (comment? (car l)) (null? (cdr l)))
+			(begin
+			   (pr (car l)
+			       (indent col2 col)
+			       extra pp-item)
+			   (newline)
+			   (out ")" col))
+			(let ((rest (cdr l)))
+			   (let ((extra (if (null? rest) (+ extra 1) 0)))
+			      (loop rest
+				    (pr (car l)
+					(indent col2 col)
+					extra pp-item))))))
                    ((null? l)
                     (out ")" col))
                    (else
@@ -398,7 +375,6 @@
                              pp-item)))))))
 
     (define (pp-general expr col extra named? pp-1 pp-2 pp-3)
-
       (define (tail1 rest col1 col2 col3)
         (if (and pp-1 (pair? rest))
           (let* ((val1 (car rest))
@@ -434,7 +410,7 @@
     (define (pp-expr-defun l col extra)
       (pp-defun-form l col extra pp-expr))
     
-    (define (pp-DEFiNE expr col extra)
+    (define (pp-DEFINE expr col extra)
       (pp-general expr col extra #f pp-expr-list #f pp-expr)
       (out #"\n" 0))
 
@@ -500,18 +476,18 @@
       (case (if (eq? *case* 'respect)
 		(string->symbol (string-upcase (symbol->string head)))
 		head)
-        ((LAMBDA LET* LETREC)                                pp-LAMBDA)
-	((DEFINE DEFINE-INLINE DEFINE-METHOD DEFINE-GENERIC) pp-DEFINE)
-	((MODULE)                                            pp-MODULE) 
-	((DEFUN DE)                                          pp-DEFUN)
-        ((IF SET!)                                           pp-IF)
-        ((COND)                                              pp-COND)
-        ((CASE)                                              pp-CASE)
-        ((AND OR)                                            pp-AND)
-        ((LET)                                               pp-LET)
-        ((BEGIN)                                             pp-BEGIN)
-        ((DO)                                                pp-DO)
-	((COMMENT)                                           pp-COMMENT)
+        ((lambda let* letrec)                                pp-LAMBDA)
+	((define define-inline define-method define-generic) pp-DEFINE)
+	((module)                                            pp-MODULE) 
+	((defun de)                                          pp-DEFUN)
+        ((if set!)                                           pp-IF)
+        ((cond)                                              pp-COND)
+        ((case)                                              pp-CASE)
+        ((and or)                                            pp-AND)
+        ((let)                                               pp-LET)
+        ((begin)                                             pp-BEGIN)
+        ((do)                                                pp-DO)
+	((comment)                                           pp-COMMENT)
         (else                                                #f)))
 
     (pr obj col 0 pp-expr))
@@ -583,4 +559,26 @@
     (generic-write obj #f 79 (lambda (str) (set! result (cons str result)) #t))
     (reverse-string-append result)))
 
+; is an `expression' an comment ?
+
+(define (comment? obj)
+   (match-case obj
+      ((COMMENT (? integer?) (? string?))
+       #t)
+      (else
+       #f)))
+
+;; whitespace symbols
+
+(define (whitespace-symbol? sym)
+   (let* ((s (symbol->string sym))
+	  (l (string-length s)))
+      (let loop ((i 0))
+	 (cond
+	    ((=fx i l)
+	     #f)
+	    ((memq (string-ref s i) '(#\space #\Newline #\tab))
+	     #t)
+	    (else
+	     (loop (+fx i 1)))))))
 
