@@ -1,41 +1,10 @@
-;*---------------------------------------------------------------------*/
-;*   A pratical implementation for the Scheme programming language     */
-;*                                                                     */
-;*                                    ,--^,                            */
-;*                              _ ___/ /|/                             */
-;*                          ,;'( )__, ) '                              */
-;*                         ;;  //   L__.                               */
-;*                         '   \\   /  '                               */
-;*                              ^   ^                                  */
-;*                                                                     */
-;*   Copyright (c) 1992-1999 Manuel Serrano                            */
-;*                                                                     */
-;*     Bug descriptions, use reports, comments or suggestions are      */
-;*     welcome. Send them to                                           */
-;*       bigloo-request@kaolin.unice.fr                                */
-;*       http://kaolin.unice.fr/bigloo                                 */
-;*                                                                     */
-;*   This program is free software; you can redistribute it            */
-;*   and/or modify it under the terms of the GNU General Public        */
-;*   License as published by the Free Software Foundation; either      */
-;*   version 2 of the License, or (at your option) any later version.  */
-;*                                                                     */
-;*   This program is distributed in the hope that it will be useful,   */
-;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
-;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
-;*   GNU General Public License for more details.                      */
-;*                                                                     */
-;*   You should have received a copy of the GNU General Public         */
-;*   License along with this program; if not, write to the Free        */
-;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
-;*   MA 02111-1307, USA.                                               */
-;*---------------------------------------------------------------------*/
 ;*=====================================================================*/
-;*    serrano/prgm/project/bigloo/bde/btags/btags.scm                  */
+;*    serrano/prgm/project/bigloo2.2a/bde/btags/btags.scm              */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Thu May 28 07:24:34 1998                          */
-;*    Last change :  Mon Sep 21 08:34:14 1998 (serrano)                */
+;*    Last change :  Sat Jul  8 08:35:00 2000 (serrano)                */
+;*    Copyright   :  2000 Manuel Serrano                               */
 ;*    -------------------------------------------------------------    */
 ;*    The Bigloo tag generator.                                        */
 ;*=====================================================================*/
@@ -44,21 +13,23 @@
 ;*    The module                                                       */
 ;*---------------------------------------------------------------------*/
 (module btags
-   (include "include/args.sch")
-   (main    main))
+   (main main))
 
 ;*---------------------------------------------------------------------*/
 ;*    Global parameters                                                */
 ;*---------------------------------------------------------------------*/
-(define *btags-version*    "0.0")
-(define *btags-append*     #f)
-(define *btags-language*   #f)
-(define *btags-table-name* "TAGS")
-(define *btags-etags*      "etags")
-(define *btags-files*      '())
-(define *btags-modules*    #f)
-
-(define *oport*            #f)
+(define *btags-version*         "0.0")
+(define *btags-append*          #f)
+(define *btags-language*        #f)
+(define *btags-table-name*      "TAGS")
+(define *btags-etags*           "etags")
+(define *btags-files*           '())
+(define *btags-modules*         #f)
+(define *btags-extern?*         #f)
+(define *oport*                 #f)
+(define *btags-define-var-list* '(define define-struct))
+(define *btags-define-fun-list* '(define define-inline define-generic
+				    define-macro define-expander))
 
 ;*---------------------------------------------------------------------*/
 ;*    main ...                                                         */
@@ -66,9 +37,11 @@
 (define (main argv)
    ;; we parse command line arguments
    (parse-args argv)
-   ;; ew start erasing the old table if not in append mode
+   ;; we start erasing the old table if not in append mode
    (if (not *btags-append*)
        (delete-file *btags-table-name*))
+   ;; we emit the argument option for other bee tools (such as kbrowse)
+   (btags-emit-configuration)
    ;; we produce tags entrie regarding the source language
    (for-each btags-file (reverse! *btags-files*))
    ;; we close the output port if currently opened
@@ -84,20 +57,24 @@
    (define (usage args-parse-usage level)
       (print "usage: btags [options] file ...")
       (newline)
-      (args-parse-usage)
+      (args-parse-usage #f)
       (newline))
    (args-parse (cdr cmd-args)
       (("?")
        (usage args-parse-usage 1))
-      (("-help" (synopsis "This help message"))
+      (("-help" (help "This help message"))
        (usage args-parse-usage 1))
-      (("-a" (synopsis "-a, --append" "Append to existing tag file."))
+      (("-a" (help "-a, --append" "Append to existing tag file."))
        (set! *btags-append* #t))
       (("--append")
        (set! *btags-append* #t))
+      (("-e" (help "-e, --extern" "Parse Bigloo extern clauses"))
+       (set! *btags-extern?* #t))
+      (("--extern")
+       (set! *btags-extern?* #t))
       (("-l"
 	?language
-	(synopsis "-l language, --language-language"
+	(help "-l language, --language-language"
 		  "Parse the following files according to the given language."))
        (if (string=? language "none")
 	   (set! *btags-language* #f)
@@ -108,20 +85,28 @@
 	   (set! *btags-language* language)))
       (("-o"
 	?tagfile
-	(synopsis "-o tagfile, --output=tagfile"
+	(help "-o tagfile, --output=tagfile"
 		  "Explict name of file for tag table (default `TAGS')."))
        (set! *btags-table-name* tagfile))
       (("--output=?tagfile")
        (set! *btags-table-name* tagfile))
-      (("-module" (synopsis "Generate tag entry for module identifier"))
+      (("-module" (help "Generate tag entry for module identifier"))
        (set! *btags-modules* #t))
-      (("-v" (synopsis "-v, --version"
+      (("-v" (help "-v, --version"
 		       "Print the current-version of the program"))
        (print *btags-version*)
        (exit 0))
       (("--version")
        (print *btags-version*)
        (exit 0))
+      (("--define-var" ?k (synopsis "Add define keyword `(KWD VAR VAL)'"))
+       (set! *btags-define-var-list*
+	     (cons (string->symbol (string-upcase k))
+		   *btags-define-var-list*)))
+      (("--define-fun" ?k (synopsis "Add define keyword `(KWD (ID ...) VAL)'"))
+       (set! *btags-define-fun-list*
+	     (cons (string->symbol (string-upcase k))
+		   *btags-define-fun-list*)))
       (else
        (if (string? *btags-language*)
 	   (set! *btags-files* (cons (cons else *btags-language*)
@@ -165,6 +150,35 @@
 				 file)))
       (system command)))
       
+;*---------------------------------------------------------------------*/
+;*    btags-emit-configuration ...                                     */
+;*---------------------------------------------------------------------*/
+(define (btags-emit-configuration)
+   (define (btags-list l)
+      (for-each (lambda (x)
+		   (display "(meta-define define " *oport*)
+		   (display (string-downcase (symbol->string x)) *oport*)
+		   (fprint *oport* (string #a127 #\0 #\, #\0)))
+		l))
+   ;; first, we open the output file
+   (if (not (output-port? *oport*))
+       (begin
+	  (set! *oport* (append-output-file *btags-table-name*))
+	  (if (not (output-port? *oport*))
+	      (error "btags"
+		     "Can't open file for output"
+		     *btags-table-name*))))
+   ;; then we do the real job
+   (if (or (pair? *btags-define-var-list*)
+	   (pair? *btags-define-fun-list*))
+       (begin
+	  (fprint *oport* #a012)
+	  (fprint *oport* "-,0")
+	  (if (pair? *btags-define-var-list*)
+	      (btags-list *btags-define-var-list*))
+	  (if (pair? *btags-define-fun-list*)
+	      (btags-list *btags-define-fun-list*)))))
+
 ;*---------------------------------------------------------------------*/
 ;*    bigloo-btags ...                                                 */
 ;*    -------------------------------------------------------------    */
@@ -179,6 +193,7 @@
 	      (error "btags"
 		     "Can't open file for output"
 		     *btags-table-name*))))
+   ;; the real btags jobs
    (bigloo-btags-file *oport* file))
 
 ;*---------------------------------------------------------------------*/
@@ -240,28 +255,57 @@
 ;*    tags-substring ...                                               */
 ;*---------------------------------------------------------------------*/
 (define (tags-substring str1 str2)
-   (let* ((len1 (string-length str1))
-          (len2 (string-length str2))
-          (stop (- len1 len2)))
-      (define (substring-at? start)
-         (let loop ((i1 start)
-                    (i2 0))
-            (cond
-               ((=fx i2 len2)
-                #t)
-               ((char-ci=? (string-ref str1 i1) (string-ref str2 i2))
-                (loop (+ i1 1) (+ i2 1)))
-               (else
-                #f))))
-      (let loop ((i 0))
-         (cond
-            ((> i stop)
-             #f)
-            ((substring-at? i)
-	     (substring str1 0 (+fx i len2)))
-            (else
-             (loop (+ i 1)))))))
-
+   (let ((len1 (string-length str1)))
+      (if (pair? str2)
+	  ;; if the argument is a pair instead of a string, it means that
+	  ;; it is a typed identifier (such as a class or a method) that
+	  ;; must be written with its type information. Nevertheless we
+	  ;; have to take care that we must strip off all the extra
+	  ;; information found a the line. For instead for a class declaration
+	  ;; such as: (class point-3d::point (z (default 0)))
+	  ;; we have to write:
+	  ;; (class point-3d::point^?...
+	  (let loop ((r 0))
+	     (cond
+		((=fx r len1)
+		 str1)
+		((or (char=? (string-ref str1 r) #\))
+		     (char=? (string-ref str1 r) #\;))
+		 (substring str1 0 r))
+		(else
+		 (loop (+fx r 1)))))
+	  ;; this is neither a class nor a method...
+	  (let* ((len2 (string-length str2))
+		 (stop (- len1 len2)))
+	     (define (substring-at? start)
+		(let loop ((i1 start)
+			   (i2 0))
+		   (cond
+		      ((=fx i2 len2)
+		       #t)
+		      ((char-ci=? (string-ref str1 i1) (string-ref str2 i2))
+		       (loop (+ i1 1) (+ i2 1)))
+		      (else
+		       #f))))
+	     (let ((start (let loop ((j 0))
+			     (cond
+				((=fx j len1)
+				 #f)
+				((char=? (string-ref str1 j) #\space)
+				 (+fx j 1))
+				(else
+				 (loop (+fx j 1)))))))
+		(if (integer? start)
+		    (let loop ((i start))
+		       (cond
+			  ((> i stop)
+			   #f)
+			  ((substring-at? i)
+			   (substring str1 0 (+fx i len2)))
+			  (else
+			   (loop (+ i 1)))))
+		    #f))))))
+ 
 ;*---------------------------------------------------------------------*/
 ;*    find-declaration-lines ...                                       */
 ;*    -------------------------------------------------------------    */
@@ -356,7 +400,9 @@
 			      (append (btags-static+export-clause clause)
 				      lines))
 			     ((extern)
-			      (append (btags-extern-clause clause) lines))
+			      (if *btags-extern?*
+				  (append (btags-extern-clause clause) lines)
+				  lines))
 			     (else
 			      lines))
 			  lines)))
@@ -395,7 +441,7 @@
 	 ((import (and (? symbol?) ?var))
 	  (let ((lnum (line-number decl)))
 	     (if (number? lnum)
-		 (cons lnum var))))))
+		 (cons lnum (symbol->string var)))))))
    (btags-clause-walker eval-parse clause))
 
 ;*---------------------------------------------------------------------*/
@@ -407,7 +453,7 @@
 	 (((and ?class (or class final-class wide-class)) ?name . ?-)
 	  (let ((lnum (line-number decl)))
 	     (if (number? lnum)
-		 (cons lnum (id->string name decl)))))))
+		 (cons lnum (list (id->string name decl))))))))
    (btags-clause-walker static+export-parse clause))
 
 ;*---------------------------------------------------------------------*/
@@ -444,17 +490,27 @@
 ;*    btags-expression ...                                             */
 ;*---------------------------------------------------------------------*/
 (define (btags-expression expr)
+   (define (var-keyword? id)
+      (and (symbol? id) (memq id *btags-define-var-list*)))
+   (define (fun-keyword? id)
+      (and (symbol? id) (memq id *btags-define-fun-list*)))
    (match-case expr
       ((begin . ?rest)
        (apply append (map btags-expression rest)))
-      (((or define define-inline define-generic
-	    define-method define-macro define-expander)
-	(?fun . ?-) . ?-)
+      ((define-method (?fun . ?-) . ?-)
+       ;; method are special because we are not seeking an exact match
+       ;; for method because we want to write the type for which the
+       ;; method overrides
+       (let ((lnum (line-number expr)))
+	  (if (number? lnum)
+	      (list (cons lnum (list (id->string fun expr))))
+	      '())))
+      (((? fun-keyword?) (?fun . ?-) . ?-)
        (let ((lnum (line-number expr)))
 	  (if (number? lnum)
 	      (list (cons lnum (id->string fun expr)))
 	      '())))
-      (((or define define-struct) (and (? symbol?) ?var) . ?-)
+      (((? var-keyword?) (and (? symbol?) ?var) . ?-)
        (let ((lnum (line-number expr)))
 	  (if (number? lnum)
 	      (list (cons lnum (id->string var expr)))
