@@ -1,41 +1,9 @@
-;*---------------------------------------------------------------------*/
-;*   A pratical implementation for the Scheme programming language     */
-;*                                                                     */
-;*                                    ,--^,                            */
-;*                              _ ___/ /|/                             */
-;*                          ,;'( )__, ) '                              */
-;*                         ;;  //   L__.                               */
-;*                         '   \\   /  '                               */
-;*                              ^   ^                                  */
-;*                                                                     */
-;*   Copyright (c) 1992-1999 Manuel Serrano                            */
-;*                                                                     */
-;*     Bug descriptions, use reports, comments or suggestions are      */
-;*     welcome. Send them to                                           */
-;*       bigloo-request@kaolin.unice.fr                                */
-;*       http://kaolin.unice.fr/bigloo                                 */
-;*                                                                     */
-;*   This program is free software; you can redistribute it            */
-;*   and/or modify it under the terms of the GNU General Public        */
-;*   License as published by the Free Software Foundation; either      */
-;*   version 2 of the License, or (at your option) any later version.  */
-;*                                                                     */
-;*   This program is distributed in the hope that it will be useful,   */
-;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
-;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
-;*   GNU General Public License for more details.                      */
-;*                                                                     */
-;*   You should have received a copy of the GNU General Public         */
-;*   License along with this program; if not, write to the Free        */
-;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
-;*   MA 02111-1307, USA.                                               */
-;*---------------------------------------------------------------------*/
 ;*=====================================================================*/
 ;*    serrano/prgm/project/bigloo/bmacs/bee/bee-bdb.el                 */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Sun Nov 15 06:58:27 1998                          */
-;*    Last change :  Thu Feb 11 17:39:03 1999 (serrano)                */
+;*    Last change :  Wed Jun  7 06:25:09 2000 (serrano)                */
 ;*    -------------------------------------------------------------    */
 ;*    The connexion to bdb (via dbg).                                  */
 ;*=====================================================================*/
@@ -79,7 +47,7 @@
 	 (set-default symbol value)
 	 (setq dbg-emacs-option value)))
 
-(defcustom bee-dbg-prompt-regexp "(bdb) "
+(defcustom bee-dbg-prompt-regexp "(bdb\\(:[^)]+\\)?) "
   "*The prompt of Bdb."
   :group 'bee
   :type 'string
@@ -99,7 +67,7 @@
 	 (setq dbg-marker-regexp value)))
 
 ;; bdb command
-(defcustom bee-dbg-run-command "brun"
+(defcustom bee-dbg-run-command "run"
   "*The run command."
   :group 'bee
   :type 'string
@@ -109,7 +77,7 @@
 	 (setq dbg-breakpoint-hook-regexp (dbg-make-breakpoint-hook-regexp))
 	 (setq dbg-display-hook-regexp (dbg-make-display-hook-regexp))))
 
-(defcustom bee-dbg-step-command "bstep"
+(defcustom bee-dbg-step-command "step"
   "*The step command."
   :group 'bee
   :type 'string
@@ -118,7 +86,7 @@
 	 (setq dbg-step-command value)
 	 (setq dbg-display-hook-regexp (dbg-make-display-hook-regexp))))
 
-(defcustom bee-dbg-cont-command "bcont"
+(defcustom bee-dbg-cont-command "cont"
   "*The cont command."
   :group 'bee
   :type 'string
@@ -127,7 +95,7 @@
 	 (setq dbg-cont-command value)
 	 (setq dbg-display-hook-regexp (dbg-make-display-hook-regexp))))
 
-(defcustom bee-dbg-call-command "bcall"
+(defcustom bee-dbg-call-command "call"
   "*The call command."
   :group 'bee
   :type 'string
@@ -136,7 +104,7 @@
 	 (setq dbg-call-command value)
 	 (setq dbg-display-hook-regexp (dbg-make-display-hook-regexp))))
 
-(defcustom bee-dbg-return-command "breturn"
+(defcustom bee-dbg-return-command "return"
   "*The return command."
   :group 'bee
   :type 'string
@@ -145,7 +113,7 @@
 	 (setq dbg-return-command value)
 	 (setq dbg-display-hook-regexp (dbg-make-display-hook-regexp))))
 
-(defcustom bee-dbg-next-command "bnext"
+(defcustom bee-dbg-next-command "next"
   "*The next command."
   :group 'bee
   :type 'string
@@ -154,7 +122,7 @@
 	 (setq dbg-next-command value)
 	 (setq dbg-display-hook-regexp (dbg-make-display-hook-regexp))))
 
-(defcustom bee-dbg-until-command "buntil"
+(defcustom bee-dbg-until-command "until"
   "*The until command."
   :group 'bee
   :type 'string
@@ -163,7 +131,7 @@
 	 (setq dbg-until-command value)
 	 (setq dbg-display-hook-regexp (dbg-make-display-hook-regexp))))
 
-(defcustom bee-dbg-finish-command "bfinish"
+(defcustom bee-dbg-finish-command "finish"
   "*The finish command."
   :group 'bee
   :type 'string
@@ -172,7 +140,7 @@
 	 (setq dbg-finish-command value)
 	 (setq dbg-display-hook-regexp (dbg-make-display-hook-regexp))))
 
-(defcustom bee-dbg-break-command "bbreak"
+(defcustom bee-dbg-break-command "break"
   "*The break command."
   :group 'bee
   :type 'string
@@ -226,7 +194,7 @@
 	 (setq dbg-breakpoint-hook-regexp
 	       (dbg-make-breakpoint-hook-regexp))))
 
-(defcustom bee-dbg-tbreak-command "btbreak"
+(defcustom bee-dbg-tbreak-command "tbreak"
   "*The tbreak command."
   :group 'bee
   :type 'string
@@ -236,7 +204,7 @@
 	 (setq dbg-breakpoint-hook-regexp
 	       (dbg-make-breakpoint-hook-regexp))))
 
-(defcustom bee-dbg-cond-command "bcond"
+(defcustom bee-dbg-cond-command "cond"
   "*The cond command."
   :group 'bee
   :type 'string
@@ -246,27 +214,7 @@
 	 (setq dbg-breakpoint-hook-regexp
 	       (dbg-make-breakpoint-hook-regexp))))
 
-(defcustom bee-dbg-assert-command "assert"
-  "*The assert command."
-  :group 'bee
-  :type 'string
-  :set (lambda (symbol value)
-	 (set-default symbol value)
-	 (setq dbg-assert-command value)
-	 (setq dbg-breakpoint-hook-regexp
-	       (dbg-make-breakpoint-hook-regexp))))
-
-(defcustom bee-dbg-unassert-command "unassert"
-  "*The unassert command."
-  :group 'bee
-  :type 'string
-  :set (lambda (symbol value)
-	 (set-default symbol value)
-	 (setq dbg-unassert-command value)
-	 (setq dbg-breakpoint-hook-regexp
-	       (dbg-make-breakpoint-hook-regexp))))
-
-(defcustom bee-dbg-complete-command "bcomplete"
+(defcustom bee-dbg-complete-command "complete"
   "*The complete command."
   :group 'bee
   :type 'string
@@ -274,7 +222,7 @@
 	 (set-default symbol value)
 	 (setq dbg-complete-command value)))
 
-(defcustom bee-dbg-info-args-command "info bargs"
+(defcustom bee-dbg-info-args-command "info args"
   "*The info args command."
   :group 'bee
   :type 'string
@@ -282,7 +230,7 @@
 	 (set-default symbol value)
 	 (setq dbg-info-args-command value)))
 
-(defcustom bee-dbg-info-locals-command "info blocals"
+(defcustom bee-dbg-info-locals-command "info locals"
   "*The info locals command."
   :group 'bee
   :type 'string
@@ -290,7 +238,7 @@
 	 (set-default symbol value)
 	 (setq dbg-info-locals-command value)))
 
-(defcustom bee-dbg-display-command "bdisplay"
+(defcustom bee-dbg-display-command "display"
   "*The info locals command."
   :group 'bee
   :type 'string
@@ -300,7 +248,7 @@
 
 ;; bdb regular expression
 (defcustom bee-dbg-breakpoint-regexp
-"\\([0-9]+\\)\\s-*\\S-*\\s-*\\S-*\\s-*\\(.\\)\\s-*0x[0-9a-fA-F]+\\s-+in\\s-+[a-zA-Z0-9_]+\\s-+at\\s-+\\([^:]+\\)[:]\\([0-9]+\\)"
+"\\([0-9]+\\)\\s-*\\S-*\\s-*\\S-*\\s-*\\(.\\)\\s-*0x[0-9a-fA-F]+\\s-+in\\s-+[a-zA-Z0-9_?><:!@$%^&*+=\\|~-]+\\s-+at\\s-+\\([^:]+\\)[:]\\([0-9]+\\)"
   "The regexp that matches a bdb breakpoint as printed by the
 `info breakpoint' command. This regular expression matches the
 first line of the breakpoint. It does not match the line
@@ -354,8 +302,8 @@ reported by `info breakpoint'."
 ;*---------------------------------------------------------------------*/
 (defun bdb-eval-region (region)
   (if (and (> (length region) 0) (not (eq (aref region 0) ?\()))
-      (bdb-remote-call (format "(begin %s)" region))
-    (bdb-remote-call region)))
+      (dbg-remote-call (format "(begin %s)" region))
+    (dbg-remote-call region)))
 
 ;*---------------------------------------------------------------------*/
 ;*    bdb-bee-source-menu-entries ...                                  */
@@ -368,12 +316,12 @@ reported by `info breakpoint'."
     (list (vector "(bdb) eval"
 		  (list 'bdb-eval-region region)
 		  t)
-	  (vector "(bdb) bprint"
-		  (list 'dbg-command-region "bprint " region)
+	  (vector "(bdb) print"
+		  (list 'dbg-command-region "print " region)
 		  t)
 	  "-"
-	  (vector "(bdb) bwhatis"
-		  (list 'dbg-command-region "bwhatis " region)
+	  (vector "(bdb) whatis"
+		  (list 'dbg-command-region "whatis " region)
 		  t)
 	  "-"
 	  (vector "(bdb) break"
@@ -388,6 +336,13 @@ reported by `info breakpoint'."
 			file
 			line)
 		  t)
+	  "-"
+	  (vector "(bdb) footprint"
+		  (list 'bdb-remote-set-breakpoint
+			(concat bee-dbg-footprint-command " ")
+			file
+			line)
+		  t)
 	  "--:shadowDoubleEtchedOut")))
 
 ;*---------------------------------------------------------------------*/
@@ -459,18 +414,6 @@ reported by `info breakpoint'."
 ;*---------------------------------------------------------------------*/
 (defun bee-dbg-stack-mode-menu ()
   (list "dbg stack"
-	(if (eq bee-dbg-stack 'SCHEME)
-	    (vector "Show C frames"
-		    '(progn (setq bee-dbg-stack 'C)
-			    (setq dbg-info-stack-hook-command "info stack")
-			    (dbg-stack-hook t))
-		    t)
-	  (vector "Hidden C frames"
-		  '(progn (setq bee-dbg-stack 'SCHEME)
-			  (setq dbg-info-stack-hook-command "binfo stack")
-			  (dbg-stack-hook t))
-		  t))
-	"--:shadowEtchedOut"
 	(vector "Close stack frame display"
 		'(dbg-stack-quit)
 		t)))
@@ -500,9 +443,7 @@ reported by `info breakpoint'."
   (setq dbg-info-break-command bee-dbg-info-break-command)
   (setq dbg-tbreak-command bee-dbg-tbreak-command)
   (setq dbg-cond-command bee-dbg-cond-command)
-  (setq dbg-assert-command bee-dbg-assert-command)
   (setq dbg-info-assert-command bee-dbg-info-assert-command)
-  (setq dbg-unassert-command bee-dbg-unassert-command)
   (setq dbg-complete-command bee-dbg-complete-command)
   (setq dbg-info-args-command bee-dbg-info-args-command)
   (setq dbg-info-locals-command bee-dbg-info-locals-command)
@@ -523,7 +464,7 @@ reported by `info breakpoint'."
   (setq dbg-display-hook-regexp (dbg-make-display-hook-regexp))
   ;; stack printing
   (setq dbg-stack-mode-menu 'bee-dbg-stack-mode-menu)
-  (setq dbg-info-stack-hook-command "binfo stack")
+  (setq dbg-info-stack-hook-command "info stack")
   ;; bee bdb connexion
   (dbg-add-connect-hook 'bee-mode 'bee-connect-buffer-hook)
   (dbg-add-disconnect-hook 'bee-mode 'bee-disconnect-buffer-hook)
