@@ -1,41 +1,9 @@
-;*---------------------------------------------------------------------*/
-;*   A pratical implementation for the Scheme programming language     */
-;*                                                                     */
-;*                                    ,--^,                            */
-;*                              _ ___/ /|/                             */
-;*                          ,;'( )__, ) '                              */
-;*                         ;;  //   L__.                               */
-;*                         '   \\   /  '                               */
-;*                              ^   ^                                  */
-;*                                                                     */
-;*   Copyright (c) 1992-1999 Manuel Serrano                            */
-;*                                                                     */
-;*     Bug descriptions, use reports, comments or suggestions are      */
-;*     welcome. Send them to                                           */
-;*       bigloo-request@kaolin.unice.fr                                */
-;*       http://kaolin.unice.fr/bigloo                                 */
-;*                                                                     */
-;*   This program is free software; you can redistribute it            */
-;*   and/or modify it under the terms of the GNU General Public        */
-;*   License as published by the Free Software Foundation; either      */
-;*   version 2 of the License, or (at your option) any later version.  */
-;*                                                                     */
-;*   This program is distributed in the hope that it will be useful,   */
-;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
-;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
-;*   GNU General Public License for more details.                      */
-;*                                                                     */
-;*   You should have received a copy of the GNU General Public         */
-;*   License along with this program; if not, write to the Free        */
-;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
-;*   MA 02111-1307, USA.                                               */
-;*---------------------------------------------------------------------*/
 ;*=====================================================================*/
 ;*    serrano/prgm/project/bigloo/bmacs/bee/bee-indent.el              */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Mon May 25 07:27:11 1998                          */
-;*    Last change :  Tue Nov 24 18:26:06 1998 (serrano)                */
+;*    Last change :  Sat Nov  3 07:02:51 2001 (serrano)                */
 ;*    -------------------------------------------------------------    */
 ;*    The Bee indent (this file is adapted from the Scheme mode by     */
 ;*    Bill Rozas).                                                     */
@@ -84,7 +52,6 @@
 			 (buffer-name)
 			 bee-external-indent
 			 (format "-w%d" (- (frame-width) 1))
-			 "-lower"
 			 (buffer-file-name)))
     (set-process-sentinel bee-external-indent-process
 			  'bee-external-indent-sentinel))))
@@ -132,8 +99,9 @@ rigidly along with this one."
     (setq beg (point))
     (skip-chars-forward " \t")
     (if (or (looking-at ";;;")
-            (looking-at ";\\*"))
-	;; Don't alter indentation of a ;;; comment line.
+            (looking-at ";\\*")
+	    (looking-at ";[*]"))
+	;; Don't alter indentation of a ;;; or a ;* comment line.
 	nil
       (if (listp indent) (setq indent (car indent)))
       (setq shift-amt (- indent (current-column)))
@@ -545,63 +513,77 @@ of the start of the containing expression."
 ;*    Bee indent forms                                                 */
 ;*---------------------------------------------------------------------*/
 ;; basic forms
-(put 'begin                 'bee-indent-hook 0)
-(put 'case                  'bee-indent-hook 1)
-(put 'delay                 'bee-indent-hook 0)
-(put 'do                    'bee-indent-hook 2)
-(put 'lambda                'bee-indent-hook 1)
-(put 'cond                  'bee-indent-hook 0)
-(put 'when                  'bee-indent-hook 1)
-(put 'unless                'bee-indent-hook 1)
+(put 'begin                  'bee-indent-hook 0)
+(put 'case                   'bee-indent-hook 1)
+(put 'delay                  'bee-indent-hook 0)
+(put 'do                     'bee-indent-hook 0)
+(put 'lambda                 'bee-indent-hook 1)
+(put 'cond                   'bee-indent-hook 0)
+(put 'when                   'bee-indent-hook 1)
+(put 'unless                 'bee-indent-hook 1)
 
 ;; binding forms
-(put 'let                   'bee-indent-hook 'bee-let-indent)
-(put 'let*                  'bee-indent-hook 1)
-(put 'letrec                'bee-indent-hook 1)
-(put 'labels                'bee-indent-hook 'bee-let-indent)
-(put 'let-syntax            'bee-indent-hook 'bee-let-indent)
-(put 'letrec-syntax         'bee-indent-hook 'bee-let-indent)
+(put 'let                    'bee-indent-hook 'bee-let-indent)
+(put 'let*                   'bee-indent-hook 1)
+(put 'letrec                 'bee-indent-hook 1)
+(put 'labels                 'bee-indent-hook 'bee-let-indent)
+(put 'let-syntax             'bee-indent-hook 'bee-let-indent)
+(put 'letrec-syntax          'bee-indent-hook 'bee-let-indent)
+(put 'co-instantiate         'bee-indent-hook 'bee-let-indent)
 
 ;; output/input command
-(put 'call-with-input-file  'bee-indent-hook 1)
-(put 'with-input-from-file  'bee-indent-hook 1)
-(put 'with-input-from-port  'bee-indent-hook 1)
-(put 'call-with-output-file 'bee-indent-hook 1)
-(put 'with-output-to-file   'bee-indent-hook 1)
-(put 'with-output-to-port   'bee-indent-hook 1)
+(put 'call-with-input-file   'bee-indent-hook 1)
+(put 'with-input-from-file   'bee-indent-hook 1)
+(put 'with-input-from-port   'bee-indent-hook 1)
+(put 'with-input-from-string 'bee-indent-hook 1)
+(put 'call-with-output-file  'bee-indent-hook 1)
+(put 'with-output-to-file    'bee-indent-hook 1)
+(put 'with-output-to-port    'bee-indent-hook 1)
+(put 'with-output-to-string  'bee-indent-hook 0)
+(put 'with-error-to-port     'bee-indent-hook 1)
+(put 'with-error-to-file     'bee-indent-hook 1)
+(put 'with-error-to-string   'bee-indent-hook 0)
 
 ;; define forms
-(put 'define-macro          'bee-indent-hook 1)
-(put 'define-generic        'bee-indent-hook 1)
-(put 'define-method         'bee-indent-hook 1)
+(put 'define-macro           'bee-indent-hook 1)
+(put 'define-generic         'bee-indent-hook 1)
+(put 'define-method          'bee-indent-hook 1)
 
 ;; exceptions
-(put 'bind-exit             'bee-indent-hook 1)
-(put 'unwind-protect        'bee-indent-hook 0)
-(put 'dynamic-wind          'bee-indent-hook 0)
+(put 'bind-exit              'bee-indent-hook 1)
+(put 'unwind-protect         'bee-indent-hook 0)
+(put 'dynamic-wind           'bee-indent-hook 0)
 
 ;; multiple values
-(put 'multiple-value-bind   'bee-indent-hook 1)
-(put 'call-with-values      'bee-indent-hook 0)
+(put 'multiple-value-bind    'bee-indent-hook 1)
+(put 'receive                'bee-indent-hook 1)
+(put 'call-with-values       'bee-indent-hook 0)
 
 ;; parsing
-(put 'regular-grammar       'bee-indent-hook 'bee-let-indent)
-(put 'lalr-grammar          'bee-indent-hook 0)
+(put 'regular-grammar        'bee-indent-hook 'bee-let-indent)
+(put 'lalr-grammar           'bee-indent-hook 0)
 
 ;; module indentation
-(put 'module                'bee-indent-hook 1)
-(put 'directives            'bee-indent-hook 0)
-(put 'class                 'bee-indent-hook 1)
-(put 'wide-class            'bee-indent-hook 1)
-(put 'final-class           'bee-indent-hook 1)
+(put 'module                 'bee-indent-hook 1)
+(put 'directives             'bee-indent-hook 0)
+(put 'class                  'bee-indent-hook 1)
+(put 'abstract-class         'bee-indent-hook 1)
+(put 'wide-class             'bee-indent-hook 1)
+(put 'final-class            'bee-indent-hook 1)
 
 ;; matching indentation
-(put 'args-parse            'bee-indent-hook 1)
-(put 'match-case            'bee-indent-hook 1)
-(put 'string-case           'bee-indent-hook 1)
-(put 'match-lambda          'bee-indent-hook 0)
-(put 'syntax-rules          'bee-indent-hook 1)
+(put 'args-parse             'bee-indent-hook 1)
+(put 'match-case             'bee-indent-hook 1)
+(put 'cond-expand            'bee-indent-hook 0)
+(put 'string-case            'bee-indent-hook 1)
+(put 'match-lambda           'bee-indent-hook 0)
+(put 'syntax-rules           'bee-indent-hook 1)
+(put 'event-case             'bee-indent-hook 1)
+(put 'on-event               'bee-indent-hook 1)
 
+;; profiling
+(put 'profile                'bee-indent-hook 1)
+(put 'profile/gc             'bee-indent-hook 1)
 
 
 
