@@ -1,41 +1,9 @@
-;*---------------------------------------------------------------------*/
-;*   A pratical implementation for the Scheme programming language     */
-;*                                                                     */
-;*                                    ,--^,                            */
-;*                              _ ___/ /|/                             */
-;*                          ,;'( )__, ) '                              */
-;*                         ;;  //   L__.                               */
-;*                         '   \\   /  '                               */
-;*                              ^   ^                                  */
-;*                                                                     */
-;*   Copyright (c) 1992-1999 Manuel Serrano                            */
-;*                                                                     */
-;*     Bug descriptions, use reports, comments or suggestions are      */
-;*     welcome. Send them to                                           */
-;*       bigloo-request@kaolin.unice.fr                                */
-;*       http://kaolin.unice.fr/bigloo                                 */
-;*                                                                     */
-;*   This program is free software; you can redistribute it            */
-;*   and/or modify it under the terms of the GNU General Public        */
-;*   License as published by the Free Software Foundation; either      */
-;*   version 2 of the License, or (at your option) any later version.  */
-;*                                                                     */
-;*   This program is distributed in the hope that it will be useful,   */
-;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
-;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
-;*   GNU General Public License for more details.                      */
-;*                                                                     */
-;*   You should have received a copy of the GNU General Public         */
-;*   License along with this program; if not, write to the Free        */
-;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
-;*   MA 02111-1307, USA.                                               */
-;*---------------------------------------------------------------------*/
 ;*=====================================================================*/
 ;*    serrano/prgm/project/bigloo/bmacs/bee/bee-keymap.el              */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Tue May 26 07:34:07 1998                          */
-;*    Last change :  Thu Jan 21 07:34:20 1999 (serrano)                */
+;*    Last change :  Fri Nov 16 09:41:49 2001 (serrano)                */
 ;*    -------------------------------------------------------------    */
 ;*    The bee menus                                                    */
 ;*=====================================================================*/
@@ -75,7 +43,7 @@
 	     bee-about-icon))
   
 ;*---------------------------------------------------------------------*/
-;*    bee-find-module-menu ...                                         */
+;*    bee-make-find-module-menu ...                                    */
 ;*    -------------------------------------------------------------    */
 ;*    For that menu, we look if the afile already exists. If it does   */
 ;*    exists, we create a sub menu with the module list. Otherwise,    */
@@ -83,26 +51,39 @@
 ;*    afile file if it does not exists because it is a too long        */
 ;*    computation.                                                     */
 ;*---------------------------------------------------------------------*/
+(defun bee-make-find-module-menu (module-list)
+  (if module-list
+      (let ((menu-list (ude-split-menu-entries
+			(mapcar '(lambda (x)
+				   (vector x
+					   (list 'bee-find-module-nokeymap x)
+					   t))
+				module-list)))
+	    (ident (ude-fetch-identifier (point))))
+	(if (stringp ident)
+	    (append (list "module..."
+			  (vector ident
+				  (list 'bee-find-module-nokeymap ident)
+				  t)
+			  "-")
+		    menu-list)
+	  (cons "module..."
+		menu-list)))
+    ["module" bee-find-module t]))
+
+;*---------------------------------------------------------------------*/
+;*    bee-find-module-menu ...                                         */
+;*---------------------------------------------------------------------*/
 (defun bee-find-module-menu ()
-  (let ((module-list (bee-find-module-list)))
-    (if module-list
-	(let ((menu-list (ude-split-menu-entries
-			  (mapcar '(lambda (x)
-				     (vector x
-					     (list 'bee-find-module-nokeymap x)
-					     t))
-				  module-list)))
-	      (ident (ude-fetch-identifier (point))))
-	  (if (stringp ident)
-	      (append (list "module..."
-			    (vector ident
-				    (list 'bee-find-module-nokeymap ident)
-				    t)
-			    "-")
-		      menu-list)
-	    (cons "module..."
-		  menu-list)))
-      ["module" bee-find-module t])))
+  (bee-make-find-module-menu (bee-find-module-list)))
+
+;*---------------------------------------------------------------------*/
+;*    bee-project-menu ...                                             */
+;*---------------------------------------------------------------------*/
+(defun bee-project-menu ()
+  (list (list "Project..."
+	      ["Root" ude-user-set-root-directory t]
+	      ["Set name" ude-makefile-set-name t])))
   
 ;*---------------------------------------------------------------------*/
 ;*    bee-find-menu ...                                                */
@@ -115,6 +96,31 @@
 	      ["usage" bee-usage-find t]
 	      (bee-find-module-menu))))
 
+;*---------------------------------------------------------------------*/
+;*    bee-find-interface-module-menu ...                               */
+;*---------------------------------------------------------------------*/
+(defun bee-find-interface-module-menu ()
+  (if (not (bee-find-afile-create-p))
+      '()
+    (let ((old (bee-find-module-list))
+	  (res '()))
+      (while (consp old)
+	(let ((fname (bee-find-afile-module (car old))))
+	  (if fname
+	      (if (bee-interface-builder-module-p (car old) fname)
+		  (setq res (cons (car old) res))))
+	  (setq old (cdr old))))
+      (list (bee-make-find-module-menu (reverse res))))))
+
+;*---------------------------------------------------------------------*/
+;*    bee-builder-menu ...                                             */
+;*---------------------------------------------------------------------*/
+(defun bee-builder-menu ()
+  (list (append (list "Builder..."
+		      ["Start Builder" bee-interface-builder-start t]
+		      ["Edit interface" bee-find-interface-builder-module t]))))
+;* 		(bee-find-interface-module-menu))))                    */
+
 ;*---------------------------------------------------------------------*/
 ;*    bee-makefile-menu ...                                            */
 ;*---------------------------------------------------------------------*/
@@ -123,6 +129,12 @@
 	      ["Edit Makefile"
 	      ude-edit-makefile
 	      (file-exists-p (concat ude-root-directory ude-makefile))]
+	      ["Remove Makefile"
+	      ude-remove-makefile
+	      (file-exists-p (concat ude-root-directory ude-makefile))]
+	      ["Add entry"
+	      ude-add-user-makefile-entry
+	      (file-exists-p (concat ude-root-directory ude-makefile))]
 	      (if (file-exists-p (concat ude-root-directory ude-makefile))
 		  (vector "Update" 'ude-update-makefile t)
 		(list "Generate Makefile"
@@ -131,7 +143,7 @@
 				    bee-bmake-application-option)
 			      t)
 		      (vector "library"
-			      (list 'bee-generate-makefile
+			      (list 'ude-generate-makefile
 				    bee-bmake-library-option)
 			      t))))))
 
@@ -208,8 +220,21 @@
 ;*---------------------------------------------------------------------*/
 (defun bee-profile-menu ()
   '(("Profile..."
-     ["Compile for profile" ude-compile-for-profile t]
-     ["Run for profile" ude-run-for-profile t])))
+     ["Start profiler" bee-profiler-start t]
+     ["Inspect function" bee-profiler-inspect t])))
+
+;*---------------------------------------------------------------------*/
+;*    bee-jinsight-menu ...                                            */
+;*---------------------------------------------------------------------*/
+(defun bee-jinsight-menu ()
+  '(("Jinsight..."
+     ["Start jinsight" bee-jinsight-start t]
+     ("Module"
+      ["Select" bee-jinsight-select-module t]
+      ["Cpu time" bee-jinsight-module-cpu t]
+      ["Allocations" bee-jinsight-module-instances t])
+     ["Class Instances" bee-jinsight-class-instances t]
+     ["Select Function" bee-jinsight-select-function t])))
 
 ;*---------------------------------------------------------------------*/
 ;*    bee-html-menu ...                                                */
@@ -219,6 +244,21 @@
      ["Generate Html buffer" ude->html t]
      ["Write Html file" ude->html-file t])))
   
+;*---------------------------------------------------------------------*/
+;*    bee-ref-menu ...                                                 */
+;*---------------------------------------------------------------------*/
+(defun bee-ref-menu ()
+  (list (list "@label..."
+	      ["Make New label" ude-mouse-make-label t]
+	      (vector (if (ude-mouse-ref-armed-p)
+			  (concat "Make Ref (" (ude-mouse-ref-label) ")")
+			"Make Ref")
+		      'ude-mouse-make-ref '(ude-mouse-ref-armed-p))
+	      (vector (if (ude-mouse-ref-armed-p)
+			  (concat "Make Mutual Ref (" (ude-mouse-ref-label) ")")
+			"Make Mutual Ref")
+		      'ude-mouse-make-mutual-ref '(ude-mouse-ref-armed-p)))))
+  
 ;*---------------------------------------------------------------------*/
 ;*    bee-module-menu ...                                              */
 ;*---------------------------------------------------------------------*/
@@ -245,7 +285,11 @@
 (defun bee-make-menu ()
   (let* ((separator (list "--:shadowEtchedOut"))
 	 (user-menu (bee-user-menu))
-	 (menu      (append (bee-find-menu)
+	 (menu      (append (bee-project-menu)
+			    (bee-find-menu)
+			    (if (bee-interface-builder-ready-p)
+				(bee-builder-menu)
+			      '())
 			    (ude-compile-menu)
 			    (bee-makefile-menu)
 			    (bee-repl-menu)
@@ -254,7 +298,9 @@
 			    (bee-expand-menu)
 			    (bee-module-menu)
 			    (bee-profile-menu)
+			    (bee-jinsight-menu)
 			    (bee-html-menu)
+			    (bee-ref-menu)
 			    (if user-menu
 				(append separator user-menu)
 			      '()))))
@@ -367,16 +413,31 @@
   (define-key bee-mode-map "\e."              'bee-tag-find)
   (define-key bee-mode-map "\e,"              'bee-tags-find-next)
 
+  ;; interface builder
+  (define-key bee-prefixed-map "\C-g\C-s"     'bee-interface-builder-start)
+  (define-key bee-prefixed-map "\C-g\C-f"     'bee-find-interface-builder-module)
+
+  ;; project
+  (define-key bee-prefixed-map "\C-p\C-r"     'ude-user-set-root-directory)
+  (define-key bee-prefixed-map "\C-p\C-n"     'ude-makefile-set-name)
+    
   ;; compilation
   (define-key bee-prefixed-map "\C-c\C-c"     'ude-mode-compile-from-menu)
+  (define-key bee-prefixed-map "\C-c\C-x"     'ude-mode-jcompile-from-menu)
+  (define-key bee-prefixed-map "\C-c\C-r"     'ude-execute)
   (define-key bee-prefixed-map "\C-ck"        'kill-compilation)
   (define-key bee-prefixed-map "\C-cs"        'ude-set-compile-command)
   (define-key bee-prefixed-map "\C-cm"        'ude-view-last-compile-messages)
 
   ;; makefile
   (define-key bee-prefixed-map "\C-ce"        'ude-edit-makefile)
+  (define-key bee-prefixed-map "\C-cn"        'ude-add-user-makefile-entry)
+  (define-key bee-prefixed-map "\C-cr"        'ude-remove-makefile)
   (define-key bee-prefixed-map "\C-ca"        'ude-generate-makefile)
   (define-key bee-prefixed-map "\C-cu"        'ude-update-makefile)
+  (define-key bee-prefixed-map "\C-c\C-d"     'ude-makefile-debug-mode)
+  (define-key bee-prefixed-map "\C-c\C-v"     'ude-makefile-devel-mode)
+  (define-key bee-prefixed-map "\C-c\C-f"     'ude-makefile-final-mode)
 
   ;; repl
   (define-key bee-prefixed-map "\C-r\C-r"     'ude-repl-other-frame)
@@ -416,12 +477,25 @@
   (define-key bee-prefixed-map "\C-vd"        'ude-diff-file-version)
 
   ;; profile
-  (define-key bee-prefixed-map "\C-pc"        'ude-compile-for-profile)
-  (define-key bee-prefixed-map "\C-pr"        'ude-run-for-profile)
+  (define-key bee-prefixed-map "\C-pc"        'bee-profiler-start)
+  (define-key bee-prefixed-map "\C-pi"        'bee-profiler-inspect)
+
+  ;; jinsight
+  (define-key bee-prefixed-map "\C-jc"        'bee-jinsight-start)
+  (define-key bee-prefixed-map "\C-js"        'bee-jinsight-select-module)
+  (define-key bee-prefixed-map "\C-jf"        'bee-jinsight-select-function)
+  (define-key bee-prefixed-map "\C-jm"        'bee-jinsight-module-cpu)
+  (define-key bee-prefixed-map "\C-jM"        'bee-jinsight-module-instances)
+  (define-key bee-prefixed-map "\C-jk"        'bee-jinsight-class-instances)
 
   ;; mouse bindings
   (define-key bee-mode-map ude-mouse-binding  'ude-predicate-mouse-event)
 
+  ;; ref bindings
+  (define-key bee-prefixed-map "\C-ml"        'ude-mouse-make-label)
+  (define-key bee-prefixed-map "\C-ms"        'ude-mouse-make-ref)
+  (define-key bee-prefixed-map "\C-mm"        'ude-mouse-make-mutual-ref)
+  
   ;; the popup menus
   (ude-add-menu '(lambda (event) t)
 		'bee-popup-menu)
@@ -432,6 +506,6 @@
   (if (featurep 'menubar)
       (progn
 	(set-buffer-menubar current-menubar)
-	(add-submenu nil (cons "bee" (bee-make-menu)))
-	(add-submenu nil (cons "version" (bee-make-version-menu)))
-	(add-submenu nil (cons "config" (bee-make-customize-menu))))))
+	(add-submenu nil (cons "Bee" (bee-make-menu)))
+	(add-submenu nil (cons "Version" (bee-make-version-menu)))
+	(add-submenu nil (cons "Config" (bee-make-customize-menu))))))
