@@ -1,41 +1,9 @@
-;*---------------------------------------------------------------------*/
-;*   A pratical implementation for the Scheme programming language     */
-;*                                                                     */
-;*                                    ,--^,                            */
-;*                              _ ___/ /|/                             */
-;*                          ,;'( )__, ) '                              */
-;*                         ;;  //   L__.                               */
-;*                         '   \\   /  '                               */
-;*                              ^   ^                                  */
-;*                                                                     */
-;*   Copyright (c) 1992-1999 Manuel Serrano                            */
-;*                                                                     */
-;*     Bug descriptions, use reports, comments or suggestions are      */
-;*     welcome. Send them to                                           */
-;*       bigloo-request@kaolin.unice.fr                                */
-;*       http://kaolin.unice.fr/bigloo                                 */
-;*                                                                     */
-;*   This program is free software; you can redistribute it            */
-;*   and/or modify it under the terms of the GNU General Public        */
-;*   License as published by the Free Software Foundation; either      */
-;*   version 2 of the License, or (at your option) any later version.  */
-;*                                                                     */
-;*   This program is distributed in the hope that it will be useful,   */
-;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
-;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
-;*   GNU General Public License for more details.                      */
-;*                                                                     */
-;*   You should have received a copy of the GNU General Public         */
-;*   License along with this program; if not, write to the Free        */
-;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
-;*   MA 02111-1307, USA.                                               */
-;*---------------------------------------------------------------------*/
 ;*=====================================================================*/
 ;*    serrano/prgm/project/bigloo/bmacs/bee/bee-module.el              */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Thu Jun 11 11:58:51 1998                          */
-;*    Last change :  Sun Jan  3 08:31:12 1999 (serrano)                */
+;*    Last change :  Tue Jul 25 15:37:41 2000 (serrano)                */
 ;*    -------------------------------------------------------------    */
 ;*    Module handling                                                  */
 ;*=====================================================================*/
@@ -195,16 +163,22 @@
 			 (point))
 		     (error nil))))
 	  (if end
-	      ;; we search for an exportation clause
-	      (if (let ((eval t))
-		    (while (and eval
+	      ;; we search for wanted clause (export or import)
+	      (if (let ((eval t)
+			(extern t))
+		    (while (and (or eval extern)
 				(re-search-forward
 				 (format "(%s[ \t\n]" (regexp-quote clause))
 				 end
 				 t))
-		      (setq eval (bee-module-declaration-p (match-beginning 0)
-							   "(eval[ \n]")))
-		    (not eval))
+		      (let ((match-0 (match-beginning 0)))
+			(setq eval (bee-module-declaration-p
+				    match-0
+				    "(eval[ \n]"))
+			(setq extern (bee-module-declaration-p
+				      match-0
+				      "(extern[ \n]"))))
+		    (and (not eval) (not extern)))
 		  (progn
 		    (goto-char (match-beginning 0))
 		    ;; it may be the export clause of an eval form...
@@ -424,8 +398,11 @@
 				 var import-module)))
 	     (t
 	      (set-buffer current)
-	      (save-excursion
-		(bee-add-import (format "(%s %s)" var import-module))))))))))
+	      (let ((import (if (eq bee-import-mode 'all)
+				import-module
+			      (format "(%s %s)" var import-module))))
+		(save-excursion
+		  (bee-add-import import))))))))))
 	     
 
 
