@@ -1,41 +1,9 @@
-;*---------------------------------------------------------------------*/
-;*   A pratical implementation for the Scheme programming language     */
-;*                                                                     */
-;*                                    ,--^,                            */
-;*                              _ ___/ /|/                             */
-;*                          ,;'( )__, ) '                              */
-;*                         ;;  //   L__.                               */
-;*                         '   \\   /  '                               */
-;*                              ^   ^                                  */
-;*                                                                     */
-;*   Copyright (c) 1992-1999 Manuel Serrano                            */
-;*                                                                     */
-;*     Bug descriptions, use reports, comments or suggestions are      */
-;*     welcome. Send them to                                           */
-;*       bigloo-request@kaolin.unice.fr                                */
-;*       http://kaolin.unice.fr/bigloo                                 */
-;*                                                                     */
-;*   This program is free software; you can redistribute it            */
-;*   and/or modify it under the terms of the GNU General Public        */
-;*   License as published by the Free Software Foundation; either      */
-;*   version 2 of the License, or (at your option) any later version.  */
-;*                                                                     */
-;*   This program is distributed in the hope that it will be useful,   */
-;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
-;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
-;*   GNU General Public License for more details.                      */
-;*                                                                     */
-;*   You should have received a copy of the GNU General Public         */
-;*   License along with this program; if not, write to the Free        */
-;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
-;*   MA 02111-1307, USA.                                               */
-;*---------------------------------------------------------------------*/
 ;*=====================================================================*/
 ;*    serrano/prgm/project/bigloo/bmacs/dbg/dbg-breakpoint.el          */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Wed Apr 22 06:12:31 1998                          */
-;*    Last change :  Thu Feb 11 18:03:39 1999 (serrano)                */
+;*    Last change :  Wed Nov 15 21:27:41 2000 (serrano)                */
 ;*    -------------------------------------------------------------    */
 ;*    Bdb breakpoint handling.                                         */
 ;*=====================================================================*/
@@ -56,7 +24,8 @@
 ;*---------------------------------------------------------------------*/
 (defvar dbg-red-bp-glyph
   (make-glyph
-   "/* XPM */
+   (concat
+    "/* XPM */
 static char *mini-redlight[] = {
 /* width height num_colors chars_per_pixel */
 \"    21    14       13            1\",
@@ -68,7 +37,7 @@ static char *mini-redlight[] = {
 \"c c #800000\",
 \"d c #808080\",
 \"e c #9898f8\",
-\"f c None s None\",
+\"f c " dbg-margin-color "\",
 \"g c #f80000\",
 \"h c #f86430\",
 \"i c #f864c8\",
@@ -89,12 +58,13 @@ static char *mini-redlight[] = {
 \"fffbkbaaaa.ekdaaabdff\",
 \"fffbeeeaa.dekbaabbfff\",
 \"fffffaaebbbbababbdfff\",
-};")
+};"))
   "The glyph to be use for red breakpoint display.")
 
 (defvar dbg-orange-bp-glyph
   (make-glyph
-   "/* XPM */
+   (concat
+    "/* XPM */
 static char *mini-redlight[] = {
 /* width height num_colors chars_per_pixel */
 \"    21    14       13            1\",
@@ -106,7 +76,7 @@ static char *mini-redlight[] = {
 \"c c #800000\",
 \"d c #808080\",
 \"e c #9898f8\",
-\"f c None s None\",
+\"f c " dbg-margin-color "\",
 \"g c #f80000\",
 \"h c #f86430\",
 \"i c #f864c8\",
@@ -127,12 +97,13 @@ static char *mini-redlight[] = {
 \"fffbkbaaa#.ekddaadbff\",
 \"fffbeeeaa.dekbaabbfff\",
 \"fffffaaebbbbababbdfff\",
-};")
+};"))
   "The glyph to be use for red breakpoint display.")
 
 (defvar dbg-green-bp-glyph
   (make-glyph
-   "/* XPM */
+   (concat 
+    "/* XPM */
 static char *mini-redlight[] = {
 /* width height num_colors chars_per_pixel */
 \"    21    14       13            1\",
@@ -144,7 +115,7 @@ static char *mini-redlight[] = {
 \"c c #008000\",
 \"d c #808080\",
 \"e c #9898f8\",
-\"f c None s None\",
+\"f c " dbg-margin-color "\",
 \"g c #00f800\",
 \"h c #64f830\",
 \"i c #f864c8\",
@@ -165,17 +136,18 @@ static char *mini-redlight[] = {
 \"fffbkhgghh.ekbfchffff\",
 \"fffbkcfghc#ekbbbcffff\",
 \"fffffaaebbbbababfffff\",
-};")
+};"))
   "The glyph to be use for green breakpoint display.")
 
 (defvar dbg-warning-glyph
   (make-glyph
-   "/* XPM */
+   (concat
+    "/* XPM */
 static char *mini-redlight[] = {
 /* width height num_colors chars_per_pixel */
 \"    21    14       4            1\",
 /* colors */
-\"  c None s None\",
+\"  c " dbg-margin-color "\",
 \"x c #ff2020\",
 \"o c black\",
 \". c white\",
@@ -194,11 +166,74 @@ static char *mini-redlight[] = {
 \"    xx.........xx    \",
 \"   xxxxxxxxxxxxxxx   \",
 \"    xxxxxxxxxxxxx    \",
-};")
+};"))
   "The glyph to be used for assertions.")
 
-(make-face 'dbg-breakpoint-face)
-(set-face-background 'dbg-breakpoint-face dbg-margin-color)
+(defvar dbg-footprint-glyph
+  (make-glyph
+   (concat 
+   "/* XPM */
+static char *footprint[] = {
+/* width height num_colors chars_per_pixel */
+\"    19    12        2            1\",
+/* colors */
+\"_ c " dbg-margin-color "\",
+\"d c red\",
+/* pixels */
+\"___________dd______\",
+\"___________dd_dd___\",
+\"______________dd___\",
+\"__________ddd______\",
+\"__________dddd_dd__\",
+\"__dd______dddd_dd__\",
+\"__dd_dd____ddd_____\",
+\"_____dd____________\",
+\"_ddd_______________\",
+\"_dddd_dd___________\",
+\"_dddd_dd___________\",
+\"__ddd______________\",
+};"))
+  "The glyph to be used for footprints.")
+
+(defvar dbg-footprint-disable-glyph
+  (make-glyph
+   (concat
+    "/* XPM */
+static char *footprint[] = {
+/* width height num_colors chars_per_pixel */
+\"    19    14        2            1\",
+/* colors */
+\"_ c " dbg-margin-color "\",
+\"d c #00fc00\",
+/* pixels */
+\"___________________\",
+\"___________dd______\",
+\"___________dd_dd___\",
+\"______________dd___\",
+\"__________ddd______\",
+\"__________dddd_dd__\",
+\"__dd______dddd_dd__\",
+\"__dd_dd____ddd_____\",
+\"_____dd____________\",
+\"_ddd_______________\",
+\"_dddd_dd___________\",
+\"_dddd_dd___________\",
+\"__ddd______________\",
+\"___________________\",
+};"))
+  "The glyph to be used for disabled footprints.")
+
+(defface dbg-breakpoint-face
+  (list (list '((class color))
+	      (list ':background dbg-margin-color
+		    ':bold nil))
+	'(t (:bold nil)))
+  "Dbg breakpoint face"
+  :group 'dbg)
+;*                                                                     */
+;*                                                                     */
+;* (make-face 'dbg-breakpoint-face)                                    */
+;* (set-face-background 'dbg-breakpoint-face dbg-margin-color)         */
 
 ;*---------------------------------------------------------------------*/
 ;*    breakpoint keymap                                                */
@@ -361,6 +396,7 @@ static char *mini-redlight[] = {
 				(string-match
 				 dbg-breakpoint-assert-regexp string)
 				0)))
+		 (fpp     nil)
 		 (msg     (cond
 			   (assertp
 			    ;; we wait for comint to be ready
@@ -387,6 +423,7 @@ static char *mini-redlight[] = {
 			   (t
 			    nil)))
 		 (kind    (cond
+			   (fpp 'footprint)
 			   (assertp 'assert)
 			   (condp 'cond)
 			   (t nil))))
@@ -414,6 +451,7 @@ static char *mini-redlight[] = {
 							      enablep
 							      file
 							      line
+							      fpp
 							      condp
 							      assertp
 							      msg)))
@@ -425,6 +463,7 @@ static char *mini-redlight[] = {
 					      enablep
 					      file
 					      line
+					      fpp
 					      condp
 					      assertp
 					      msg)))
@@ -452,11 +491,12 @@ static char *mini-redlight[] = {
 ;*      - enablep : a bool        : is the breakpoint enabled          */
 ;*      - file    : a string      : the file name of this breakpoint   */
 ;*      - line    : an integer    : the line number of this breakpoint */
+;*      - fpp     : a bool        : is the breakpoint a footprint      */
 ;*      - condp   : a bool        : is the breakpoint conditional      */
 ;*      - assertp : a bool        : is the breakpoint an assertion     */
 ;*      - msg     : a string      : the expr of the breakpoint         */
 ;*---------------------------------------------------------------------*/
-(defun dbg-make-breakpoint (num enablep file line condp assertp msg)
+(defun dbg-make-breakpoint (num enablep file line fpp condp assertp msg)
   (save-excursion
     (let ((buffer (dbg-find-file file)))
       (if (bufferp buffer)
@@ -466,13 +506,21 @@ static char *mini-redlight[] = {
 			     enablep
 			     file
 			     line
-			     (cond (assertp 'assert) (condp 'cond) (t nil))
+			     (cond
+			      (fpp 'footprint)
+			      (assertp 'assert)
+			      (condp 'cond)
+			      (t nil))
 			     extent
 			     msg)))
 	    ;; we set the left margin for the bdb source buffer
 	    (dbg-connect-buffer buffer)
 	    (set-extent-face extent 'dbg-breakpoint-face)
 	    (set-extent-begin-glyph extent (cond
+					    (fpp
+					     (if enablep
+						 dbg-footprint-glyph
+					       dbg-footprint-disable-glyph))
 					    (assertp dbg-warning-glyph)
 					    ((not enablep) dbg-green-bp-glyph)
 					    (condp dbg-orange-bp-glyph)
@@ -495,9 +543,13 @@ static char *mini-redlight[] = {
   (if (extentp object)
       (let ((bp (extent-property object 'dbg-breakpoint)))
 	(if (dbg-breakpoint-p bp)
-	    (let ((str (concat (if (eq (dbg-breakpoint-kind bp) 'assert)
-				   "assertion: "
-				 "breakpoint: ")
+	    (let ((str (concat (cond
+				((eq (dbg-breakpoint-kind bp) 'assert)
+				 "assertion: ")
+				((eq (dbg-breakpoint-kind bp) 'footprint)
+				 "footprint: ")
+				(t
+				 "breakpoint: "))
 			       (number-to-string (dbg-breakpoint-num bp)))))
 	      (put-text-property 0 10
 				 'face
@@ -581,9 +633,13 @@ static char *mini-redlight[] = {
 (defun dbg-breakpoint-update-popup-menu (breakpoint pos buffer)
   "The popup menu to print when the cursor hit a breakpoint glyph."
   (let* ((msg (dbg-breakpoint-msg breakpoint))
-	 (kind (if (eq (dbg-breakpoint-kind breakpoint) 'assert)
-		   "assertion"
-		 "breakpoint"))
+	 (kind (cond
+		((eq (dbg-breakpoint-kind breakpoint) 'assert)
+		 "assertion")
+		((eq (dbg-breakpoint-kind breakpoint) 'footprint)
+		 "footprint")
+		(t
+		 "breakpoint")))
 	 (rest (if (eq (dbg-breakpoint-kind breakpoint) 'assert)
 		   (list (vector "Unassert"
 				 (list 'dbg-remote-unassert-breakpoint
@@ -697,6 +753,12 @@ static char *mini-redlight[] = {
 			   line)
 		     t)
 	     "-"
+	     (vector "Set footprint"
+		     (list 'dbg-remote-set-breakpoint
+			   dbg-footprint-command
+			   fname line)
+		     t)
+	     "-"
 	     (vector "Continue until here"
 		     (list 'dbg-remote-continue fname line)
 		     t))))))
