@@ -1,41 +1,9 @@
-;*---------------------------------------------------------------------*/
-;*   A pratical implementation for the Scheme programming language     */
-;*                                                                     */
-;*                                    ,--^,                            */
-;*                              _ ___/ /|/                             */
-;*                          ,;'( )__, ) '                              */
-;*                         ;;  //   L__.                               */
-;*                         '   \\   /  '                               */
-;*                              ^   ^                                  */
-;*                                                                     */
-;*   Copyright (c) 1992-1999 Manuel Serrano                            */
-;*                                                                     */
-;*     Bug descriptions, use reports, comments or suggestions are      */
-;*     welcome. Send them to                                           */
-;*       bigloo-request@kaolin.unice.fr                                */
-;*       http://kaolin.unice.fr/bigloo                                 */
-;*                                                                     */
-;*   This program is free software; you can redistribute it            */
-;*   and/or modify it under the terms of the GNU General Public        */
-;*   License as published by the Free Software Foundation; either      */
-;*   version 2 of the License, or (at your option) any later version.  */
-;*                                                                     */
-;*   This program is distributed in the hope that it will be useful,   */
-;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
-;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
-;*   GNU General Public License for more details.                      */
-;*                                                                     */
-;*   You should have received a copy of the GNU General Public         */
-;*   License along with this program; if not, write to the Free        */
-;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
-;*   MA 02111-1307, USA.                                               */
-;*---------------------------------------------------------------------*/
 ;*=====================================================================*/
 ;*    serrano/prgm/project/bigloo/bmacs/dbg/dbg.el                     */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Sun Apr 19 15:16:46 1998                          */
-;*    Last change :  Fri Dec  4 15:03:43 1998 (serrano)                */
+;*    Last change :  Wed Sep 15 16:09:39 1999 (serrano)                */
 ;*    -------------------------------------------------------------    */
 ;*    The main DBG function.                                           */
 ;*=====================================================================*/
@@ -70,6 +38,13 @@
 ;*---------------------------------------------------------------------*/
 (defvar dbg-started-p nil)
 
+;*---------------------------------------------------------------------*/
+;*    dbg-wrapper-caller                                               */
+;*    -------------------------------------------------------------    */
+;*    A possible function that is in charge to call the debugger.      */
+;*---------------------------------------------------------------------*/
+(defvar dbg-wrapper-caller nil)
+
 ;*---------------------------------------------------------------------*/
 ;*    dbg ...                                                          */
 ;*---------------------------------------------------------------------*/
@@ -184,9 +159,11 @@ and source-file directory for your debugger."
 ;*    dbg-remote-call ...                                              */
 ;*---------------------------------------------------------------------*/
 (defun dbg-remote-call (command)
-  (if dbg-verbose-remote
-      (dbg-verbose-remote-call command)
-    (dbg-silent-remote-call command)))
+  (if (functionp dbg-wrapper-caller)
+      (funcall dbg-wrapper-caller command)
+    (if dbg-verbose-remote
+	(dbg-verbose-remote-call command)
+      (dbg-silent-remote-call command))))
 
 ;*---------------------------------------------------------------------*/
 ;*    dbg-verbose-remote-call ...                                      */
