@@ -1,35 +1,3 @@
-;*---------------------------------------------------------------------*/
-;*   A pratical implementation for the Scheme programming language     */
-;*                                                                     */
-;*                                    ,--^,                            */
-;*                              _ ___/ /|/                             */
-;*                          ,;'( )__, ) '                              */
-;*                         ;;  //   L__.                               */
-;*                         '   \\   /  '                               */
-;*                              ^   ^                                  */
-;*                                                                     */
-;*   Copyright (c) 1992-1999 Manuel Serrano                            */
-;*                                                                     */
-;*     Bug descriptions, use reports, comments or suggestions are      */
-;*     welcome. Send them to                                           */
-;*       bigloo-request@kaolin.unice.fr                                */
-;*       http://kaolin.unice.fr/bigloo                                 */
-;*                                                                     */
-;*   This program is free software; you can redistribute it            */
-;*   and/or modify it under the terms of the GNU General Public        */
-;*   License as published by the Free Software Foundation; either      */
-;*   version 2 of the License, or (at your option) any later version.  */
-;*                                                                     */
-;*   This program is distributed in the hope that it will be useful,   */
-;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
-;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
-;*   GNU General Public License for more details.                      */
-;*                                                                     */
-;*   You should have received a copy of the GNU General Public         */
-;*   License along with this program; if not, write to the Free        */
-;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
-;*   MA 02111-1307, USA.                                               */
-;*---------------------------------------------------------------------*/
 ;;; etags.el --- etags facility for Emacs
 
 ;; Copyright (C) 1985, 1986, 1988, 1989, 1992, 1993, 1994, 1995
@@ -114,6 +82,9 @@ and thus, does not uses the correct syntax table (regarding the mode of the
 buffer that is callings tags). To fix this, we have introduced this variable
 that holds the caller buffer syntax table.")
 
+(defvar tags-auto-reload-p t
+  "Reload new TAGS table when updated without user request.")
+
 ;;;###autoload
 (defvar find-tag-hook nil
   "*Hook to be run by \\[find-tag] after finding a tag.  See `run-hooks'.
@@ -353,9 +324,11 @@ Returns non-nil iff it is a valid table."
 	(set-buffer (get-file-buffer file))
 	(setq win (or verify-tags-table-function (initialize-new-tags-table)))
 	(if (or (verify-visited-file-modtime (current-buffer))
-		(not (yes-or-no-p
-		      (format "Tags file %s has changed, read new contents? "
-			      file))))
+		(not (or
+		      tags-auto-reload-p
+		      (yes-or-no-p
+		       (format "Tags file %s has changed, read new contents? "
+			       file)))))
 	    (and win (funcall verify-tags-table-function))
 	  (revert-buffer t t)
 	  (initialize-new-tags-table)))
@@ -854,7 +827,7 @@ See documentation of variable `tags-file-name'."
 ;;;###autoload (define-key ctl-x-4-map "." 'find-tag-other-window)
 
 ;;;###autoload
-(defun find-tag-other-frame (tagname &optional next-p)
+(defun find-tag-other-frame (tagname &optional next-p regexp-p)
   "Find tag (in current tags table) whose name contains TAGNAME.
 Select the buffer containing the tag's definition in another frame, and
 move point there.  The default for TAGNAME is the expression in the buffer
@@ -869,8 +842,52 @@ just \\[negative-argument]), pop back to the previous tag gone to.
 See documentation of variable `tags-file-name'."
   (interactive (find-tag-interactive "Find tag other frame: "))
   (let ((pop-up-frames t))
-    (find-tag-other-window tagname next-p)))
+    (find-tag-other-window tagname next-p regexp-p)))
 
+;;;###autoload
+(defun find-tag-other-frame-if-new (tagname &optional next-p regexp-p)
+  "Find tag (in current tags table) whose name contains TAGNAME.
+Select the buffer containing the tag's definition in another window, and
+move point there.  The default for TAGNAME is the expression in the buffer
+around or before point.
+
+If second arg NEXT-P is t (interactively, with prefix arg), search for
+another tag that matches the last tagname or regexp used.  When there are
+multiple matches for a tag, more exact matches are found first.  If NEXT-P
+is negative (interactively, with prefix arg that is a negative number or
+just \\[negative-argument]), pop back to the previous tag gone to.
+
+See documentation of variable `tags-file-name'."
+  (interactive (find-tag-interactive "Find tag other window: "))
+  ;; This hair is to deal with the case where the tag is found in the
+  ;; selected window's buffer; without the hair, point is moved in both
+  ;; windows.  To prevent this, we save the selected window's point before
+  ;; doing find-tag-noselect, and restore it after.
+  (let* ((window-point (window-point (selected-window)))
+	 (tagbuf (find-tag-noselect tagname next-p regexp-p))
+	 (tagpoint (progn (set-buffer tagbuf) (point))))
+    (set-window-point (prog1
+			  (selected-window)
+			(if (not (let ((win (get-buffer-window tagbuf t)))
+				   (and win
+					(let ((frame (window-frame win)))
+					  (raise-frame frame)
+					  (select-frame frame)
+					  (select-window win)
+					  (set-buffer tagbuf)
+					  (goto-char tagpoint)
+					  (set-window-point win tagpoint)
+					  t))))
+			    (let ((pop-up-frames t))
+			      (switch-to-buffer-other-window tagbuf)))
+			;; We have to set this new window's point; it
+			;; might already have been displaying a
+			;; different portion of tagbuf, in which case
+			;; switch-to-buffer-other-window doesn't set
+			;; the window's point from the buffer.
+			(set-window-point (selected-window) tagpoint))
+		      window-point)))
+  
 ;;;###autoload (define-key ctl-x-5-map "." 'find-tag-other-frame)
 
 ;;;###autoload
@@ -920,10 +937,6 @@ See documentation of variable `tags-file-name'."
 			  next-line-after-failure-p
 			  matching
 			  first-search)
-
-  (if tags-syntax-table
-      (set-syntax-table tags-syntax-table))
-       
   (let (file				;name of file containing tag
 	tag-info			;where to find the tag in FILE
 	(first-table t)
