@@ -1,41 +1,9 @@
-;*---------------------------------------------------------------------*/
-;*   A pratical implementation for the Scheme programming language     */
-;*                                                                     */
-;*                                    ,--^,                            */
-;*                              _ ___/ /|/                             */
-;*                          ,;'( )__, ) '                              */
-;*                         ;;  //   L__.                               */
-;*                         '   \\   /  '                               */
-;*                              ^   ^                                  */
-;*                                                                     */
-;*   Copyright (c) 1992-1999 Manuel Serrano                            */
-;*                                                                     */
-;*     Bug descriptions, use reports, comments or suggestions are      */
-;*     welcome. Send them to                                           */
-;*       bigloo-request@kaolin.unice.fr                                */
-;*       http://kaolin.unice.fr/bigloo                                 */
-;*                                                                     */
-;*   This program is free software; you can redistribute it            */
-;*   and/or modify it under the terms of the GNU General Public        */
-;*   License as published by the Free Software Foundation; either      */
-;*   version 2 of the License, or (at your option) any later version.  */
-;*                                                                     */
-;*   This program is distributed in the hope that it will be useful,   */
-;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
-;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
-;*   GNU General Public License for more details.                      */
-;*                                                                     */
-;*   You should have received a copy of the GNU General Public         */
-;*   License along with this program; if not, write to the Free        */
-;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
-;*   MA 02111-1307, USA.                                               */
-;*---------------------------------------------------------------------*/
 ;*=====================================================================*/
 ;*    serrano/prgm/project/bigloo/bmacs/ude/ude-compile.el             */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Wed May 27 10:41:34 1998                          */
-;*    Last change :  Thu Jan 28 10:16:10 1999 (serrano)                */
+;*    Last change :  Fri Nov 16 09:47:51 2001 (serrano)                */
 ;*    -------------------------------------------------------------    */
 ;*    The ude compilation                                              */
 ;*=====================================================================*/
@@ -75,6 +43,7 @@
 	(fset 'compilation-goto-locus 'ude-compilation-goto-locus)
 	;; we prepare the hook for toolbar setting
 	(add-hook 'compilation-mode-hook 'ude-compilation-init-toolbar)
+	(add-hook 'compilation-mode-hook 'ude-set-compilation-modeline)
 	;; we make variable for error regexp and keywords local variable
 	;; so programming mode are able to use their own value for these
 	;; variables
@@ -132,6 +101,7 @@ Executed hooks are removed from that list before being executed.")
 ;*    buffer that as requested a compilation.                          */
 ;*---------------------------------------------------------------------*/
 (defvar ude-compile-root-directory nil)
+(defvar ude-compile-compile-mode nil)
 
 ;*---------------------------------------------------------------------*/
 ;*    ude-compile ...                                                  */
@@ -149,17 +119,68 @@ Executed hooks are removed from that list before being executed.")
   ;; initialize the compile package
   (ude-compile-init)
   ;; start the real compilation process
-  (let* ((new-height (if (bufferp ude-last-compile-buffer)
-			 (let ((w (get-buffer-window
-				   ude-last-compile-buffer t)))
-			   (if (windowp w)
-			       (window-height w)
-			     ude-compilation-window-height))
+  (setq ude-compile-root-directory ude-root-directory)
+  (setq ude-compile-compile-mode ude-compile-mode)
+  ;; do process the compilation
+  (if (or (and (not (ude-detached-compile-buffer-p))
+	       (consp (cdr (window-list))))
+	  ude-always-detach-compile-buffer)
+      (ude-frame-compile)
+    (ude-window-compile)))
+
+;*---------------------------------------------------------------------*/
+;*    ude-detached-compile-buffer-p ...                                */
+;*    -------------------------------------------------------------    */
+;*    Is the compilation buffer detached.                              */
+;*---------------------------------------------------------------------*/
+(defun ude-detached-compile-buffer-p ()
+  (or ude-always-detach-compile-buffer
+      (and (bufferp ude-last-compile-buffer)
+	   (ude-one-frame-buffer-p ude-last-compile-buffer))))
+
+;*---------------------------------------------------------------------*/
+;*    ude-window-compile ...                                           */
+;*---------------------------------------------------------------------*/
+(defun ude-window-compile ()
+  (let* ((win        (if (bufferp ude-last-compile-buffer)
+			 (get-buffer-window ude-last-compile-buffer t)
+		       nil))
+	 (new-height (if (windowp win)
+			 (window-height win)
 		       ude-compilation-window-height))
 	 (compilation-window-height new-height)
 	 (compilation-read-command nil)
-	 (compile-command ude-compile-command))
-    (setq ude-compile-root-directory ude-root-directory)
+	 (compile-command ude-compile-command)
+	 (frame (if (windowp win) (window-frame win))))
+    ;; we raise the compilation window before compiling
+    (if (framep frame)
+	(raise-frame frame))
+    (if (char= (aref ude-root-directory (- (length ude-root-directory) 1))
+	       ?/)
+	(let ((default-directory ude-root-directory))
+	  (call-interactively 'compile))
+      (let ((default-directory (concat ude-root-directory "/")))
+	(call-interactively 'compile)))))
+
+;*---------------------------------------------------------------------*/
+;*    ude-frame-compile ...                                            */
+;*---------------------------------------------------------------------*/
+(defun ude-frame-compile ()
+  ;; if there is a frame for compilation, we raise it
+  (let* ((win    (if (bufferp ude-last-compile-buffer)
+		     (get-buffer-window ude-last-compile-buffer t)
+		   nil))
+	 (frame  (if (windowp win) (window-frame win))))
+    ;; we raise the compilation window before compiling
+    (if (framep frame)
+	(raise-frame frame)))
+  ;; the we start the regular compilation stuff
+  (let* ((default-frame-alist (cons 'height
+				    (cons ude-compilation-frame-height
+					  default-frame-alist)))
+	 (compilation-read-command nil)
+	 (compile-command ude-compile-command)
+	 (pop-up-frames t))
     (if (char= (aref ude-root-directory (- (length ude-root-directory) 1))
 	       ?/)
 	(let ((default-directory ude-root-directory))
@@ -174,9 +195,32 @@ Executed hooks are removed from that list before being executed.")
 ;*---------------------------------------------------------------------*/
 (defun ude-compile-compile ()
   (interactive)
-  (let ((ude-root-directory ude-compile-root-directory))
+  (let ((ude-root-directory ude-compile-root-directory)
+	(ude-compile-mode ude-compile-compile-mode))
     (ude-compile)))
 
+;*---------------------------------------------------------------------*/
+;*    ude-compile-edit-makefile ...                                    */
+;*    -------------------------------------------------------------    */
+;*    Change to the proper ude-root-directory then compile.            */
+;*---------------------------------------------------------------------*/
+(defun ude-compile-edit-makefile ()
+  (interactive)
+  (let ((ude-root-directory ude-compile-root-directory)
+	(ude-compile-mode ude-compile-compile-mode))
+    (ude-edit-makefile)))
+
+;*---------------------------------------------------------------------*/
+;*    ude-compile-update-makefile ...                                  */
+;*    -------------------------------------------------------------    */
+;*    Change to the proper ude-root-directory then compile.            */
+;*---------------------------------------------------------------------*/
+(defun ude-compile-update-makefile ()
+  (interactive)
+  (let ((ude-root-directory ude-compile-root-directory)
+	(ude-compile-mode ude-compile-compile-mode))
+    (ude-update-makefile)))
+
 ;*---------------------------------------------------------------------*/
 ;*    ude-mode-compile ...                                             */
 ;*---------------------------------------------------------------------*/
@@ -199,6 +243,18 @@ Executed hooks are removed from that list before being executed.")
       (funcall ude-mode-menu-compile)
     (ude-compile)))
 
+;*---------------------------------------------------------------------*/
+;*    ude-mode-jcompile-from-menu ...                                  */
+;*---------------------------------------------------------------------*/
+(defun ude-mode-jcompile-from-menu ()
+  (interactive)
+  ;; use a dynamic variable in order to change the value of the
+  ;; UDE-COMPILE-COMMAND variable
+  (let ((ude-compile-command ude-jcompile-command))
+    (if (functionp ude-mode-menu-compile)
+	(funcall ude-mode-menu-compile)
+      (ude-compile))))
+
 ;*---------------------------------------------------------------------*/
 ;*    ude-set-compile-command ...                                      */
 ;*---------------------------------------------------------------------*/
@@ -263,15 +319,10 @@ Executed hooks are removed from that list before being executed.")
 					    default-frame-alist)))
 	   (window (get-buffer-window buffer)))
       (if (windowp window)
-	  (progn
-	    (if (not (one-window-p window))
-		(progn
-		  (delete-window window)
-		  (pop-to-buffer buffer)))
-	    (beginning-of-buffer)
-	    (if (and ude-error-regexp
-		     (re-search-forward ude-error-regexp (point-max) t))
-		(goto-char (match-beginning 0)))))))
+	  (if (not (one-window-p window))
+	      (progn
+		(delete-window window)
+		(pop-to-buffer buffer))))))
    (ude-enlarge-compilation-window-on-error
     ;; we have to enlarge the compilation height to its emacs default value
     (let ((window (get-buffer-window buffer)))
@@ -284,11 +335,24 @@ Executed hooks are removed from that list before being executed.")
 		(let ((o-window (selected-window)))
 		  (select-window window)
 		  (enlarge-window (- max-size height))
-		  (select-window o-window)))))
-      (beginning-of-buffer)
-      (if (and ude-error-regexp
-	       (re-search-forward ude-error-regexp (point-max) t))
-	  (goto-char (match-beginning 0)))))))
+		  (select-window o-window)))))))))
+
+;*---------------------------------------------------------------------*/
+;*    ude-compilation-all-done-p ...                                   */
+;*    -------------------------------------------------------------    */
+;*    Are all the current compilation done?                            */
+;*---------------------------------------------------------------------*/
+(defun ude-compilation-all-done-p (lst)
+  (let ((res t))
+    (while (consp lst)
+      (let ((proc (car lst)))
+	(if (and (processp proc)
+		 (eq (process-status proc) 'run))
+	    (progn
+	      (setq res nil)
+	      (setq lst nil))
+	  (setq lst (cdr lst)))))
+    res))
 
 ;*---------------------------------------------------------------------*/
 ;*    ude-compilation-succeed ...                                      */
@@ -300,14 +364,16 @@ Executed hooks are removed from that list before being executed.")
     (if (functionp ude-success-hook)
 	(let ((hook ude-success-hook))
 	  (setq ude-success-hook nil)
-	  (funcall hook buffer msg))
-      (if (and ude-delete-compilation-window-on-success
-	       (not (ude-detached-compile-buffer-p)))
-	  (progn
-	    (sit-for 5)
-	    (if (windowp window)
-		(delete-window window))
-	    (message "compilation completed..."))))
+	  (funcall hook buffer msg)))
+    (if (and ude-delete-compilation-window-on-success
+	     (not (ude-detached-compile-buffer-p)))
+	(progn
+	  (sit-for 5)
+	  (if (and (windowp window)
+		   (or (null compilation-in-progress)
+		       (ude-compilation-all-done-p compilation-in-progress)))
+	      (delete-window window))
+	  (message "compilation completed...")))
     t))
 
 ;*---------------------------------------------------------------------*/
@@ -386,13 +452,24 @@ Selects a window with point at SOURCE, with another window displaying ERROR."
 	  (goto-char (cdr next-error))))))
 
 ;*---------------------------------------------------------------------*/
-;*    ude-detached-compile-buffer-p ...                                */
-;*    -------------------------------------------------------------    */
-;*    Is the compilation buffer detached.                              */
+;*    delete-frame-or-quit ...                                         */
 ;*---------------------------------------------------------------------*/
-(defun ude-detached-compile-buffer-p ()
-  (and (bufferp ude-last-compile-buffer)
-       (ude-one-frame-buffer-p ude-last-compile-buffer)))
+(defun delete-frame-or-quit ()
+  (interactive)
+  (let ((lst (device-frame-list)))
+    (if (and (consp lst) (null (cdr lst)))
+	(kill-emacs)
+      (delete-frame))))
+
+;*---------------------------------------------------------------------*/
+;*    delete-frame-or-quit ...                                         */
+;*---------------------------------------------------------------------*/
+(defun delete-window-or-quit ()
+  (interactive)
+  (let ((lst (device-frame-list)))
+    (if (and (consp lst) (null (cdr lst)))
+	(kill-emacs)
+      (delete-window))))
 
 ;*---------------------------------------------------------------------*/
 ;*    Various compile toolbar button                                   */
@@ -403,6 +480,8 @@ Selects a window with point at SOURCE, with another window displaying ERROR."
   (toolbar-make-button-list ude-quit-icon))
 (defvar ude-compile-button
   (toolbar-make-button-list ude-compile-icon))
+(defvar ude-make-make-button
+  (toolbar-make-button-list ude-make-make-icon))
 (defvar ude-compile-detach-button
   (toolbar-make-button-list ude-compile-detach-icon))
 (defvar ude-next-error-button
@@ -411,6 +490,10 @@ Selects a window with point at SOURCE, with another window displaying ERROR."
   (toolbar-make-button-list ude-help-icon))
 (defvar ude-compile-quit-button
   (toolbar-make-button-list ude-quit-icon))
+(defvar ude-edit-makefile-button
+  (toolbar-make-button-list ude-edit-makefile-icon))
+(defvar ude-execute-button
+  (toolbar-make-button-list ude-execute-icon))
 
 ;*---------------------------------------------------------------------*/
 ;*    ude-compilation-attached-opened-toolbar ...                      */
@@ -421,7 +504,7 @@ Selects a window with point at SOURCE, with another window displaying ERROR."
     [:style 2d :size 2]
     
     ;; the quit button
-    [ude-compile-quit-button delete-window t "Close Compile Window"]
+    [ude-compile-quit-button delete-window-or-quit t "Close Compile Window"]
     [:style 2d :size 2]
 
     ;; the detach button
@@ -429,6 +512,8 @@ Selects a window with point at SOURCE, with another window displaying ERROR."
     ude-detach-compile-buffer
     t
     "Pop a new frame for Compile Buffer"]
+    ;; edit makefile
+    [ude-edit-makefile-button ude-compile-edit-makefile t "Edit Makefile"]
     [:style 2d :size 2]
 
     ;; the stop button
@@ -441,8 +526,14 @@ Selects a window with point at SOURCE, with another window displaying ERROR."
     
     ;; the compile button
     [ude-compile-button ude-compile-compile t "Compile"]
+    ;; the make-make button
+    [ude-make-make-button ude-compile-update-makefile t "Update Makefile"]
     [:style 2d :size 2]
 
+    ;; the execution button
+    [ude-execute-button ude-execute t "Execute"]
+    [:style 2d :size 2]
+    
     ;; flushing right
     nil
     [:style 2d :size 2]
@@ -458,9 +549,13 @@ Selects a window with point at SOURCE, with another window displaying ERROR."
     [:style 2d :size 2]
     
     ;; the quit button
-    [ude-compile-quit-button delete-frame t "Close Compilation Frame"]
+    [ude-compile-quit-button delete-frame-or-quit t "Close Compilation Frame"]
     [:style 2d :size 2]
     
+    ;; edit makefile
+    [ude-edit-makefile-button ude-compile-edit-makefile t "Edit Makefile"]
+    [:style 2d :size 2]
+
     ;; the stop button
     [ude-compilation-stop-button kill-compilation t "Abort Compilation"]
     [:style 2d :size 2]
@@ -471,8 +566,14 @@ Selects a window with point at SOURCE, with another window displaying ERROR."
     
     ;; the compile button
     [ude-compile-button ude-compile-compile t "Compile"]
+    ;; the compile button
+    [ude-make-make-button ude-compile-update-makefile t "Update Makefile"]
     [:style 2d :size 2]
 
+    ;; the execution button
+    [ude-execute-button ude-execute t "Execute"]
+    [:style 2d :size 2]
+    
     ;; flushing right
     nil
     [:style 2d :size 2]
@@ -567,7 +668,10 @@ Selects a window with point at SOURCE, with another window displaying ERROR."
 ;*---------------------------------------------------------------------*/
 (defun ude-compile-menu ()
   (let ((entries (ude-fetch-makefile-entries))
-	(mkmf    (concat ude-root-directory ude-makefile)))
+	(mkmf    (concat ude-root-directory ude-makefile))
+	(mode    (ude-fetch-makefile-mode-entry)))
+    (if (stringp mode)
+	(setq ude-compile-mode (intern mode)))
     (if (and (file-exists-p mkmf) (consp entries))
 	(progn
 	  (if (not (stringp ude-compile-last-entry))
@@ -592,14 +696,81 @@ Selects a window with point at SOURCE, with another window displaying ERROR."
 					   '(ude-mode-compile-from-menu)))
 				   t))
 			entries))
+		 ["Execute" ude-execute t]
 		 ["Kill compilation" kill-compilation compilation-in-progress]
 		 ["View last messages" ude-view-last-compile-messages
-		 (ude-compile-messages-available-p)])))
+		 (ude-compile-messages-available-p)]
+		 '("Development mode"
+		   ["Debug" ude-makefile-debug-mode
+		   :style toggle :selected (eq ude-compile-mode 'debug)]
+		   ["Devel" ude-makefile-devel-mode
+		   :style toggle :selected (eq ude-compile-mode 'devel)]
+		   ["Final" ude-makefile-final-mode
+		   :style toggle :selected (eq ude-compile-mode 'final)]))))
       (list
        (list "Compilation..."
 	     (vector (ude-string-excerpt (ude-get-compile-command))
 		     'ude-mode-compile-from-menu
 		     t)
+	     ["Execute" ude-execute t]
 	     ["Kill compilation" kill-compilation compilation-in-progress]
-	     ["Set compile command" ude-set-compile-command t])))))
+	     ["Set compile command" ude-set-compile-command t]
+	     '("Development mode"
+	       ["Debug" ude-makefile-debug-mode
+	       :style toggle :selected (eq ude-compile-mode 'debug)]
+	       ["Devel" ude-makefile-devel-mode
+	       :style toggle :selected (eq ude-compile-mode 'devel)]
+	       ["Final" ude-makefile-final-mode
+	       :style toggle :selected (eq ude-compile-mode 'final)]))))))
+
+;*---------------------------------------------------------------------*/
+;*    ude-execute-default-args ...                                     */
+;*---------------------------------------------------------------------*/
+(defvar ude-execute-default-args "")
+
+;*---------------------------------------------------------------------*/
+;*    ude-execute ...                                                  */
+;*---------------------------------------------------------------------*/
+(defun ude-execute (arg)
+  (interactive
+   (let ((arg (read-string (format "Run argument: [%s] "
+				   ude-execute-default-args))))
+     (list (if (and (stringp arg) (> (length arg) 0))
+	       arg
+	     ude-execute-default-args))))
+  ;; we remember the run argument for later profile
+  (setq ude-execute-default-args arg)
+  ;; the run for profiling
+  (let* ((ude-compile-command (format "%s %s"
+				      (ude-fetch-makefile-binary-entry)
+				      arg)))
+    (ude-compile))) 
+
+;*---------------------------------------------------------------------*/
+;*    ude-set-compilation-modeline ...                                 */
+;*    -------------------------------------------------------------    */
+;*    This function sets the modeline according to the compile mode.   */
+;*---------------------------------------------------------------------*/
+(defun ude-set-compilation-modeline ()
+  (let* ((id (if (symbolp ude-compile-compile-mode)
+		 (let* ((text (symbol-name ude-compile-compile-mode))
+			(glyph (make-glyph text)))
+		   (set-glyph-face glyph 'modeline-mousable)
+		   glyph)
+	       (let ((glyph (make-glyph "plain")))
+		 (set-glyph-face glyph 'ude-modeline-no-root-face)
+		 glyph))))
+    (if (consp ude-modeline-id)
+	(rplacd ude-modeline-id id)
+      (progn
+	(setq ude-modeline-id
+	      (cons (copy-extent modeline-buffer-id-left-extent) id))
+	(setq mode-line-buffer-identification
+	      (cons ude-modeline-id
+		    (mapcar '(lambda (prop)
+			       (let ((name (cdr prop)))
+				 (if (string= "XEmacs%N:" name)
+				     (cons (car prop) "%N:")
+				   prop)))
+			    mode-line-buffer-identification)))))))
 
