@@ -1,41 +1,9 @@
-;*---------------------------------------------------------------------*/
-;*   A pratical implementation for the Scheme programming language     */
-;*                                                                     */
-;*                                    ,--^,                            */
-;*                              _ ___/ /|/                             */
-;*                          ,;'( )__, ) '                              */
-;*                         ;;  //   L__.                               */
-;*                         '   \\   /  '                               */
-;*                              ^   ^                                  */
-;*                                                                     */
-;*   Copyright (c) 1992-1999 Manuel Serrano                            */
-;*                                                                     */
-;*     Bug descriptions, use reports, comments or suggestions are      */
-;*     welcome. Send them to                                           */
-;*       bigloo-request@kaolin.unice.fr                                */
-;*       http://kaolin.unice.fr/bigloo                                 */
-;*                                                                     */
-;*   This program is free software; you can redistribute it            */
-;*   and/or modify it under the terms of the GNU General Public        */
-;*   License as published by the Free Software Foundation; either      */
-;*   version 2 of the License, or (at your option) any later version.  */
-;*                                                                     */
-;*   This program is distributed in the hope that it will be useful,   */
-;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
-;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
-;*   GNU General Public License for more details.                      */
-;*                                                                     */
-;*   You should have received a copy of the GNU General Public         */
-;*   License along with this program; if not, write to the Free        */
-;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
-;*   MA 02111-1307, USA.                                               */
-;*---------------------------------------------------------------------*/
 ;*=====================================================================*/
 ;*    serrano/prgm/project/bigloo/bmacs/ude/ude-docline.el             */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Thu Nov 12 21:29:10 1998                          */
-;*    Last change :  Wed Feb 10 15:00:51 1999 (serrano)                */
+;*    Last change :  Tue Apr 10 09:47:22 2001 (serrano)                */
 ;*    -------------------------------------------------------------    */
 ;*    Ude online documentation handling.                               */
 ;*    -------------------------------------------------------------    */
@@ -199,6 +167,22 @@ This command is designed to be used whether you are already in Info or not."
 			   fn (intern val)))))
   (ude-info-ref-internal var))
 
+;*---------------------------------------------------------------------*/
+;*    ude-info-docline-ident ...                                       */
+;*    -------------------------------------------------------------    */
+;*    This function calls an online documentation with respect         */
+;*    to the active context.                                           */
+;*    -------------------------------------------------------------    */
+;*    If a region is active, this function prints the documentation    */
+;*    for the s-expression.                                            */
+;*---------------------------------------------------------------------*/
+(defun ude-info-docline-ident (flock ident)
+  "Popup an online documentation according to context."
+  (interactive)
+  (ude-docline-init)
+  (ude-info-init flock)
+  (ude-info-ref-internal ident))
+
 ;*---------------------------------------------------------------------*/
 ;*    ude-info-docline ...                                             */
 ;*    -------------------------------------------------------------    */
@@ -265,10 +249,12 @@ This command is designed to be used whether you are already in Info or not."
 	   (anchor-regexp "@ref[ \t]+\\([^@:]+\\):\\([^@]+\\)@")
 	   (node-regexp   "@node[ \t]+\\([^@]+\\)@")
 	   (doc-regexp    "@deffn ?[^@]*@")
+	   (eval-regexp   "@eval[ \t]+\\([^@]+\\)@")
 	   (union-regexp  (concat path-regexp "\\|"
 				  anchor-regexp "\\|"
 				  node-regexp "\\|"
-				  doc-regexp))
+				  doc-regexp "\\|"
+				  eval-regexp))
 	   (path '()))
       (while (re-search-forward union-regexp (point-max) t)
 	(goto-char (match-beginning 0))
@@ -321,6 +307,8 @@ This command is designed to be used whether you are already in Info or not."
 					 'mouse-face 'highlight)
 		    (set-extent-property extent
 					 'keymap ude-node-mouse-map)))))
+	   ((re-search-forward eval-regexp end t)
+	    (eval-region (match-beginning 1) (match-end 1)))
 	   (t
 	    (error (format "Illegal match %S" end))))
 	  (goto-char end))))))
@@ -396,10 +384,10 @@ This command is designed to be used whether you are already in Info or not."
 			 nil))))
     (if id
 	(let* ((eregexp (concat "@deffn[x]?[ \t]*\\{[^}]+\\}[ \t]+"
-				id
+				(regexp-quote id)
 				"[\n \t]"))
 	       (regexp (concat "@deffn[x]?[ \t]*{[^}]+}[ \t]+"
-			       id
+			       (regexp-quote id)
 			       "[\n \t]"))
 	       (file (ude-find-in-files eregexp files)))
 	  (if (stringp file)
@@ -450,3 +438,87 @@ This command is designed to be used whether you are already in Info or not."
 			       node file)))
 	      (ude-error "Can't find buffer for %S" file)))
 	(ude-error "Can't find buffer for %S" file)))))
+
+;*---------------------------------------------------------------------*/
+;*    Reference positioning ...                                        */
+;*---------------------------------------------------------------------*/
+(defvar ude-ref-point nil)
+(defvar ude-ref-buffer nil)
+(defvar ude-ref-label nil)
+
+;*---------------------------------------------------------------------*/
+;*    ude-mouse-ref-armed-p ...                                        */
+;*---------------------------------------------------------------------*/
+(defun ude-mouse-ref-armed-p ()
+  (stringp ude-ref-label))
+
+;*---------------------------------------------------------------------*/
+;*    ude-mouse-ref-label ...                                          */
+;*---------------------------------------------------------------------*/
+(defun ude-mouse-ref-label ()
+  ude-ref-label)
+
+;*---------------------------------------------------------------------*/
+;*    ude-mouse-make-label ...                                         */
+;*---------------------------------------------------------------------*/
+(defun ude-mouse-make-label (str)
+  (interactive "sReference label: ")
+  (setq ude-ref-label str)
+  (setq ude-ref-point (point))
+  (setq ude-ref-buffer (current-buffer))
+  (insert (format "@label %s@" ude-ref-label))
+  (message "Label %S sets at %S:%d"
+	   ude-ref-label
+	   (buffer-name ude-ref-buffer)
+	   ude-ref-point))
+
+;*---------------------------------------------------------------------*/
+;*    ude-mouse-make-ref ...                                           */
+;*---------------------------------------------------------------------*/
+(defun ude-mouse-make-ref ()
+  (interactive)
+  (condition-case nil
+      (progn
+	(message "Setting reference %S from buffer %S to buffer %S"
+		 ude-ref-label
+		 (buffer-name (current-buffer))
+		 (buffer-name ude-ref-buffer))
+	(let* ((fname (buffer-file-name (current-buffer)))
+	       (fname2 (file-relative-name
+			(buffer-file-name ude-ref-buffer)
+			(file-name-directory fname))))
+	  (insert (format "@ref %s:%s@" fname2 ude-ref-label))))
+    (error
+     (ude-error "Can't set %S" ude-ref-label))))
+
+;*---------------------------------------------------------------------*/
+;*    ude-mouse-make-mutual-ref ...                                    */
+;*---------------------------------------------------------------------*/
+(defun ude-mouse-make-mutual-ref ()
+  (interactive)
+  (condition-case nil
+      (progn
+	(message "Setting mutual reference %S from buffer %S to buffer %S"
+		 ude-ref-label
+		 (buffer-name (current-buffer))
+		 (buffer-name ude-ref-buffer))
+	(let ((fname (buffer-file-name (current-buffer))))
+	  ;; set the new local label
+	  (insert (format "@label %s@" ude-ref-label))
+	  (save-excursion
+	    (let ((fname2 (file-relative-name
+			   fname
+			   (file-name-directory
+			    (buffer-file-name ude-ref-buffer))))
+		  (fname3 (file-relative-name
+			   (buffer-file-name ude-ref-buffer)
+			   (file-name-directory fname))))
+	      (insert (format "@ref %s:%s@" fname3 ude-ref-label))
+	      (set-buffer ude-ref-buffer)
+	      (save-excursion
+		(goto-char ude-ref-point)
+		(insert (format "@ref %s:%s@" fname2 ude-ref-label)))))))
+    (error
+     (ude-error "Can't set %S" ude-ref-label))))
+    
+  
