@@ -1,41 +1,9 @@
-;*---------------------------------------------------------------------*/
-;*   A pratical implementation for the Scheme programming language     */
-;*                                                                     */
-;*                                    ,--^,                            */
-;*                              _ ___/ /|/                             */
-;*                          ,;'( )__, ) '                              */
-;*                         ;;  //   L__.                               */
-;*                         '   \\   /  '                               */
-;*                              ^   ^                                  */
-;*                                                                     */
-;*   Copyright (c) 1992-1999 Manuel Serrano                            */
-;*                                                                     */
-;*     Bug descriptions, use reports, comments or suggestions are      */
-;*     welcome. Send them to                                           */
-;*       bigloo-request@kaolin.unice.fr                                */
-;*       http://kaolin.unice.fr/bigloo                                 */
-;*                                                                     */
-;*   This program is free software; you can redistribute it            */
-;*   and/or modify it under the terms of the GNU General Public        */
-;*   License as published by the Free Software Foundation; either      */
-;*   version 2 of the License, or (at your option) any later version.  */
-;*                                                                     */
-;*   This program is distributed in the hope that it will be useful,   */
-;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
-;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
-;*   GNU General Public License for more details.                      */
-;*                                                                     */
-;*   You should have received a copy of the GNU General Public         */
-;*   License along with this program; if not, write to the Free        */
-;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
-;*   MA 02111-1307, USA.                                               */
-;*---------------------------------------------------------------------*/
 ;*=====================================================================*/
 ;*    serrano/prgm/project/bigloo/cigloo/Parser/parser.scm             */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Fri Nov 24 11:17:34 1995                          */
-;*    Last change :  Sat May  9 11:58:26 1998 (serrano)                */
+;*    Last change :  Mon Dec  3 18:26:09 2001 (serrano)                */
 ;*    -------------------------------------------------------------    */
 ;*    The C syntax                                                     */
 ;*=====================================================================*/
@@ -49,7 +17,13 @@
 	    engine_param
 	    parser_lexer
 	    translate_decl)
-   (export  parser))
+   (export  parser
+	    *anonymous-struct-alist*))
+
+;*---------------------------------------------------------------------*/
+;*    *anonymous-struct-alist* ...                                     */
+;*---------------------------------------------------------------------*/
+(define *anonymous-struct-alist* '())
 
 ;*---------------------------------------------------------------------*/
 ;*    parser ...                                                       */
@@ -59,20 +33,22 @@
 
       ;; tokens
       (CONSTANT PAR-OPEN PAR-CLO BRA-OPEN BRA-CLO ANGLE-OPEN ANGLE-CLO
-       SEMI-COMMA COMMA DOT LDOTS -> ! % ^ & * = / + ~ - >> << | != ==
-       <<= >>= += -= *= /= %= ^= &= |= ++ -- ID && || ? : TYPE-ID <= >=
+       SEMI-COMMA COMMA DOT LDOTS -> ! % ^ & * = / + ~ - >> << BOR != ==
+       <<= >>= += -= *= /= %= ^= &= OR= ++ -- ID && OR ? : TYPE-ID <= >=
        < > asm auto break case char const __const continue default do double
-       else enum extern float for fortran goto if int long register
+       else enum extern float for fortran goto if int long register FILE
        return short signed sizeof static struct switch typedef union unsigned
-       void volatile while __attribute__ inline __inline__)
+       void volatile while __attribute__ inline __inline__ __extension__ obj_t)
  
       ;; we build the overall list in a inversed way in order to
       ;; remove a shift/reduce in the grammar.
-      (file
+      (start
        (()
 	'())
-       ((file external-definition)
-	`(,external-definition ,@file)))
+       ((start external-definition)
+	`(,external-definition ,@start))
+       ((start c++-definition)
+	`(,@c++-definition ,@start)))
 
       (external-definition
        ((function-definition)
@@ -80,6 +56,20 @@
        ((declaration)
 	declaration))
 
+      (c++-definition
+       ((extern CONSTANT BRA-OPEN c++-declarations BRA-CLO SEMI-COMMA)
+	(if (string=? (cadr CONSTANT) "\"C\"")
+	    c++-declarations
+	    (error "cigloo"
+		   "illegal syntax"
+		   (list 'extern (car extern) "Illegal construction")))))
+
+      (c++-declarations
+       ((declaration)
+	(list declaration))
+       ((c++-declarations declaration)
+	`(,declaration ,@c++-declarations)))
+      
       (declaration
        ((declaration-specifiers SEMI-COMMA)
 	(ast-declare #f
@@ -100,6 +90,38 @@
 				(t-id    (ident-id t-ident)))
 			    (define-type-id t-id)))
 		      init-declarator-list))
+	
+	;; Record the alias of an anonymous struct.
+	(let ((tspecs (type-spec-of-decl-spec declaration-specifiers)))
+	   (for-each 
+	    (lambda (decl)
+	       (let* ((decl    (cond
+				  ((decl? decl)
+				   decl)
+				  ((and (pair? decl)
+					(decl? (car decl))
+					(or (null? (cdr decl))
+					    (and (pair? (cdr decl))
+						 (eq? (cadr decl) #unspecified)
+						 (null? (cddr decl)))))
+				   (car decl))
+				  (else
+				   (error "cigloo"
+					  "internal error"
+					  "parsing"))))
+		      (t-ident (get-decl-ident decl))
+		      (t-id    (ident-id t-ident)))
+		  ;; Look at the type being aliased.  If it is an 
+		  ;; anonymous struct set its c-name to be this alias.
+		  (for-each
+		   (lambda (tspec)
+		      (let* ((value (type-spec-value tspec))
+			     (entry (and (struct-spec? value)
+					 (assq value *anonymous-struct-alist*))))
+			 (when entry
+			    (set-cdr! entry t-ident))))
+		   tspecs)))
+	    init-declarator-list))
 	(ast-declare #f
 		     declaration-specifiers
 		     init-declarator-list)))
@@ -127,42 +149,48 @@
 	`(,declarator ,initializer)))
       
       (storage-class-specifier
-       ((TYPEDEF)
-	(ast-storage-class-spec (car TYPEDEF) 'TYPEDEF))
-       ((EXTERN)
-	(ast-storage-class-spec (car EXTERN) 'EXTERN))
-       ((STATIC)
-	(ast-storage-class-spec (car STATIC) 'STATIC))
-       ((AUTO)
-	(ast-storage-class-spec (car AUTO) 'AUTO))
-       ((REGISTER)
-	(ast-storage-class-spec (car REGISTER) 'REGISTER)))
+       ((typedef)
+	(ast-storage-class-spec (car typedef) 'typedef))
+       ((__extension__ typedef)
+	(ast-storage-class-spec (car typedef) 'typedef))
+       ((extern)
+	(ast-storage-class-spec (car extern) 'extern))
+       ((static)
+	(ast-storage-class-spec (car static) 'static))
+       ((auto)
+	(ast-storage-class-spec (car auto) 'auto))
+       ((register)
+	(ast-storage-class-spec (car register) 'register)))
       
       (type-specifier
-       ((CHAR)
-	(ast-type-spec (car CHAR) 'CHAR "char" #unspecified))
-       ((SHORT)
-	(ast-type-spec (car SHORT) 'SHORT "short" #unspecified))
-       ((INT)
-	(ast-type-spec (car INT) 'INT "int" #unspecified))
-       ((LONG)
-	(ast-type-spec (car LONG) 'LONG "long" #unspecified))
-       ((SIGNED)
-	(ast-type-spec (car SIGNED) 'SIGNED "signed" #unspecified))
-       ((UNSIGNED)
-	(ast-type-spec (car UNSIGNED) 'UNSIGNED "unsigned" #unspecified))
-       ((FLOAT)
-	(ast-type-spec (car FLOAT) 'FLOAT "float" #unspecified))
-       ((DOUBLE)
-	(ast-type-spec (car DOUBLE) 'DOUBLE "double" #unspecified))
-       ((CONST)
-	(ast-type-spec (car CONST) 'CONST "const" #unspecified))
-       ((__CONST)
-	(ast-type-spec (car __CONST) 'CONST "const" #unspecified))
-       ((VOLATILE)
-	(ast-type-spec (car VOLATILE) 'VOLATILE "volatile" #unspecified))
-       ((VOID)
-	(ast-type-spec (car VOID) 'VOID "void" #unspecified))
+       ((FILE)
+	(ast-type-spec (car FILE) 'FILE "FILE" #unspecified))
+       ((char)
+	(ast-type-spec (car char) 'char "char" #unspecified))
+       ((short)
+	(ast-type-spec (car short) 'short "short" #unspecified))
+       ((int)
+	(ast-type-spec (car int) 'int "int" #unspecified))
+       ((long)
+	(ast-type-spec (car long) 'long "long" #unspecified))
+       ((signed)
+	(ast-type-spec (car signed) 'signed "signed" #unspecified))
+       ((unsigned)
+	(ast-type-spec (car unsigned) 'unsigned "unsigned" #unspecified))
+       ((float)
+	(ast-type-spec (car float) 'float "float" #unspecified))
+       ((double)
+	(ast-type-spec (car double) 'double "double" #unspecified))
+       ((const)
+	(ast-type-spec (car const) 'const "const" #unspecified))
+       ((__const)
+	(ast-type-spec (car __const) 'const "const" #unspecified))
+       ((volatile)
+	(ast-type-spec (car volatile) 'volatile "volatile" #unspecified))
+       ((void)
+	(ast-type-spec (car void) 'void "void" #unspecified))
+       ((obj_t)
+	(ast-type-spec (car obj_t) 'obj_t "obj" #unspecified))
        ((struct-or-union-specifier)
 	(ast-type-spec (ast-coord struct-or-union-specifier)
 		       'struct
@@ -188,12 +216,16 @@
 			 (ast-ident (car TYPE-ID) (cadr TYPE-ID))
 			 struct-declaration-list))
        ((struct-or-union BRA-OPEN struct-declaration-list BRA-CLO)
-	(ast-struct-spec (car struct-or-union)
-			 (cdr struct-or-union)
-			 (ast-ident #f (symbol->string
-					(gensym (string-append *iname*
-							       "__s"))))
-			 struct-declaration-list))
+	(let* ((id (ast-ident #f (symbol->string
+				  (gensym (string-append *iname* "__s")))))
+	       (result (ast-struct-spec (car struct-or-union)
+					(cdr struct-or-union)
+					id
+					struct-declaration-list)))
+	  ;; Create an entry in the alist for this struct.
+	  (set! *anonymous-struct-alist*
+		(cons (list result) *anonymous-struct-alist*))
+	  result))
        ((struct-or-union identifier)
 	(ast-struct-spec (car struct-or-union)
 			 (cdr struct-or-union)
@@ -206,10 +238,10 @@
 			 '())))
       
       (struct-or-union
-       ((STRUCT)
-	(cons (car STRUCT) 'STRUCT))
-       ((UNION)
-	(cons (car UNION) 'UNION)))
+       ((struct)
+	(cons (car struct) 'struct))
+       ((union)
+	(cons (car union) 'union)))
       
       (struct-declaration-list
        ((struct-declaration)
@@ -236,12 +268,12 @@
 	declarator))
       
       (enum-specifier
-       ((ENUM BRA-OPEN enumerator-list BRA-CLO)
-	(ast-enum-spec (car ENUM) #f enumerator-list))
-       ((ENUM identifier BRA-OPEN enumerator-list BRA-CLO)
-	(ast-enum-spec (car ENUM) identifier enumerator-list))
-       ((ENUM identifier)
-	(ast-enum-spec (car ENUM) identifier #f)))
+       ((enum BRA-OPEN enumerator-list BRA-CLO)
+	(ast-enum-spec (car enum) #f enumerator-list))
+       ((enum identifier BRA-OPEN enumerator-list BRA-CLO)
+	(ast-enum-spec (car enum) identifier enumerator-list))
+       ((enum identifier)
+	(ast-enum-spec (car enum) identifier #f)))
       
       (enumerator-list
        ((enumerator)
@@ -428,9 +460,9 @@
 	`("--" ,unary-expr))
        ((unary-operator cast-expr)
 	`(,unary-operator ,cast-expr))
-       ((SIZEOF unary-expr)
+       ((sizeof unary-expr)
 	`("sizeof(" ,unary-expr ")"))
-       ((SIZEOF PAR-OPEN type-name PAR-CLO)
+       ((sizeof PAR-OPEN type-name PAR-CLO)
 	`("sizeof(" ,type-name ")")))
       
       (unary-operator
@@ -514,7 +546,7 @@
       (inclusive-or-expr
        ((exclusive-or-expr)
 	exclusive-or-expr)
-       ((inclusive-or-expr | exclusive-or-expr)
+       ((inclusive-or-expr BOR exclusive-or-expr)
 	`(,inclusive-or-expr "|" ,exclusive-or-expr)))
       
       (logical-and-expr
@@ -526,7 +558,7 @@
       (logical-or-expr
        ((logical-and-expr)
 	logical-and-expr)
-       ((logical-or-expr || logical-and-expr)
+       ((logical-or-expr OR logical-and-expr)
 	`(,logical-or-expr "||" ,logical-and-expr)))
       
       (conditional-expr
@@ -562,8 +594,8 @@
 	&=)
        ((^=)
 	^=)
-       ((|=)
-	|=))
+       ((OR=)
+	OR=))
       
       (expr
        ((assignment-expr)
@@ -592,9 +624,9 @@
       (labeled-statement
        ((identifier : statement)
 	#unspecified)
-       ((CASE constant-expr : statement)
+       ((case constant-expr : statement)
 	#unspecified)
-       ((DEFAULT : statement)
+       ((default : statement)
 	#unspecified))
       
       (compound-statement
@@ -626,45 +658,45 @@
 	`(,expr ,SEMI-COMMA)))
       
       (selection-statement
-       ((IF PAR-OPEN expr PAR-CLO statement)
+       ((if PAR-OPEN expr PAR-CLO statement)
 	#unspecified)
-       ((IF PAR-OPEN expr PAR-CLO statement ELSE statement)
+       ((if PAR-OPEN expr PAR-CLO statement else statement)
 	#unspecified)
-       ((SWITCH PAR-OPEN expr PAR-CLO statement)
+       ((switch PAR-OPEN expr PAR-CLO statement)
 	#unspecified))
       
       (iteration-statement
-       ((WHILE PAR-OPEN expr PAR-CLO statement)
+       ((while PAR-OPEN expr PAR-CLO statement)
 	#unspecified)
-       ((DO statement WHILE PAR-OPEN expr PAR-CLO SEMI-COMMA)
+       ((do statement while PAR-OPEN expr PAR-CLO SEMI-COMMA)
 	#unspecified)
-       ((FOR PAR-OPEN SEMI-COMMA SEMI-COMMA PAR-CLO statement)
+       ((for PAR-OPEN SEMI-COMMA SEMI-COMMA PAR-CLO statement)
 	#unspecified)
-       ((FOR PAR-OPEN SEMI-COMMA SEMI-COMMA expr PAR-CLO statement)
+       ((for PAR-OPEN SEMI-COMMA SEMI-COMMA expr PAR-CLO statement)
 	#unspecified)
-       ((FOR PAR-OPEN SEMI-COMMA expr SEMI-COMMA PAR-CLO statement)
+       ((for PAR-OPEN SEMI-COMMA expr SEMI-COMMA PAR-CLO statement)
 	#unspecified)
-       ((FOR PAR-OPEN SEMI-COMMA expr SEMI-COMMA expr PAR-CLO statement)
+       ((for PAR-OPEN SEMI-COMMA expr SEMI-COMMA expr PAR-CLO statement)
 	#unspecified)
-       ((FOR PAR-OPEN expr SEMI-COMMA SEMI-COMMA PAR-CLO statement)
+       ((for PAR-OPEN expr SEMI-COMMA SEMI-COMMA PAR-CLO statement)
 	#unspecified)
-       ((FOR PAR-OPEN expr SEMI-COMMA SEMI-COMMA expr PAR-CLO statement)
+       ((for PAR-OPEN expr SEMI-COMMA SEMI-COMMA expr PAR-CLO statement)
 	#unspecified)
-       ((FOR PAR-OPEN expr SEMI-COMMA expr SEMI-COMMA PAR-CLO statement)
+       ((for PAR-OPEN expr SEMI-COMMA expr SEMI-COMMA PAR-CLO statement)
 	#unspecified)
-       ((FOR PAR-OPEN expr SEMI-COMMA expr SEMI-COMMA expr PAR-CLO statement)
+       ((for PAR-OPEN expr SEMI-COMMA expr SEMI-COMMA expr PAR-CLO statement)
 	#unspecified))
       
       (jump-statement
-       ((GOTO identifier SEMI-COMMA)
+       ((goto identifier SEMI-COMMA)
 	#unspecified)
-       ((CONTINUE SEMI-COMMA)
+       ((continue SEMI-COMMA)
 	#unspecified)
-       ((BREAK SEMI-COMMA)
+       ((break SEMI-COMMA)
 	#unspecified)
-       ((RETURN SEMI-COMMA)
+       ((return SEMI-COMMA)
 	#unspecified)
-       ((RETURN expr SEMI-COMMA)
+       ((return expr SEMI-COMMA)
 	#unspecified))
 
       (function-definition
@@ -678,9 +710,9 @@
 	(ast-fun-def #f #f declaration-specifiers declarator function-body)))
 
       (inline-specifier
-       ((INLINE)
+       ((inline)
 	#unspecified)
-       ((__INLINE__)
+       ((__inline__)
 	#unspecified))
 	
       (function-body
@@ -696,7 +728,7 @@
       ;; This is not an ANSI syntax. This is a Gcc specific syntax in order
       ;; to be able to parse the __attribute__ expression. 
       (gcc-attribute
-       ((__ATTRIBUTE__ PAR-OPEN gcc-attribute-values PAR-CLO)
+       ((__attribute__ PAR-OPEN gcc-attribute-values PAR-CLO)
 	#unspecified))
 
       (gcc-attribute-values
