@@ -1,41 +1,9 @@
-;*---------------------------------------------------------------------*/
-;*   A pratical implementation for the Scheme programming language     */
-;*                                                                     */
-;*                                    ,--^,                            */
-;*                              _ ___/ /|/                             */
-;*                          ,;'( )__, ) '                              */
-;*                         ;;  //   L__.                               */
-;*                         '   \\   /  '                               */
-;*                              ^   ^                                  */
-;*                                                                     */
-;*   Copyright (c) 1992-1999 Manuel Serrano                            */
-;*                                                                     */
-;*     Bug descriptions, use reports, comments or suggestions are      */
-;*     welcome. Send them to                                           */
-;*       bigloo-request@kaolin.unice.fr                                */
-;*       http://kaolin.unice.fr/bigloo                                 */
-;*                                                                     */
-;*   This program is free software; you can redistribute it            */
-;*   and/or modify it under the terms of the GNU General Public        */
-;*   License as published by the Free Software Foundation; either      */
-;*   version 2 of the License, or (at your option) any later version.  */
-;*                                                                     */
-;*   This program is distributed in the hope that it will be useful,   */
-;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
-;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
-;*   GNU General Public License for more details.                      */
-;*                                                                     */
-;*   You should have received a copy of the GNU General Public         */
-;*   License along with this program; if not, write to the Free        */
-;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
-;*   MA 02111-1307, USA.                                               */
-;*---------------------------------------------------------------------*/
 ;*=====================================================================*/
-;*    serrano/prgm/project/bigloo/comptime/Ast/alphatize.scm           */
+;*    serrano/prgm/project/bigloo2.3/comptime/Ast/alphatize.scm        */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Fri Jan  6 11:09:14 1995                          */
-;*    Last change :  Sun Apr 26 21:19:28 1998 (serrano)                */
+;*    Last change :  Mon Jul 17 18:00:36 2000 (serrano)                */
 ;*    -------------------------------------------------------------    */
 ;*    The substitution tools module                                    */
 ;*=====================================================================*/
@@ -86,12 +54,20 @@
 (define *location* #f)
 
 ;*---------------------------------------------------------------------*/
-;*    get-location ...                                                 */
+;*    get-inline-location ...                                          */
+;*    -------------------------------------------------------------    */
+;*    This function controls how location are propagate inside inline  */
+;*    body. The default behaviour is to propagate the location of the  */
+;*    caller. To change this, simply inverse the order of the tests.   */
 ;*---------------------------------------------------------------------*/
-(define (get-location node)
-   (if (location? (node-loc node))
-       (node-loc node)
-       *location*))
+(define (get-inline-location node)
+   (cond
+      ((location? *location*)
+       *location*)
+      ((location? (node-loc node))
+       (node-loc node))
+      (else
+       #f)))
 
 ;*---------------------------------------------------------------------*/
 ;*    do-alphatize ...                                                 */
@@ -103,7 +79,7 @@
 ;*---------------------------------------------------------------------*/
 (define-method (do-alphatize node::atom)
    (duplicate::atom node
-      (loc (get-location node))))
+      (loc (get-inline-location node))))
 
 ;*---------------------------------------------------------------------*/
 ;*    do-alphatize ::var ...                                           */
@@ -114,23 +90,25 @@
       (cond
 	 ((eq? alpha #unspecified)
 	  (use-variable! var (node-loc node) 'value)
-	  (duplicate::var node (loc (get-location node))))
+	  (duplicate::var node (loc (get-inline-location node))))
 	 ((variable? alpha)
 	  (use-variable! alpha (node-loc node) 'value)
 	  (if (fun? (variable-value alpha))
 	      (instantiate::closure
-		 (loc (get-location node))
+		 (loc (get-inline-location node))
 		 (type (variable-type alpha))
 		 (variable alpha))
 	      (duplicate::var node
-		 (loc (get-location node))
+		 (loc (get-inline-location node))
 		 (variable alpha))))
 	 ((atom? alpha)
 	  (duplicate::atom alpha))
+	 ((kwote? alpha)
+	  (duplicate::kwote alpha))
 	 (else
 	  (internal-error "alphatize"
-			  "Illegal alphatization"
-			  (shape node))))))
+			  "Illegal alphatization (var)"
+			  (cons (shape node) (shape alpha)))))))
 
 ;*---------------------------------------------------------------------*/
 ;*    do-alphatize ::closure ...                                       */
@@ -141,29 +119,29 @@
       (cond
 	 ((eq? alpha #unspecified)
 	  (use-variable! var (node-loc node) 'value)
-	  (duplicate::closure node (loc (get-location node))))
+	  (duplicate::closure node (loc (get-inline-location node))))
 	 ((variable? alpha)
 	  (use-variable! alpha (node-loc node) 'value)
 	  (duplicate::closure node
-	     (loc (get-location node))
+	     (loc (get-inline-location node))
 	     (variable alpha)))
 	 (else
 	  (internal-error "alphatize"
-			  "Illegal alphatization"
+			  "Illegal alphatization (closure)"
 			  (shape node))))))
 
 ;*---------------------------------------------------------------------*/
 ;*    do-alphatize ::kwote ...                                         */
 ;*---------------------------------------------------------------------*/
 (define-method (do-alphatize node::kwote)
-   (duplicate::kwote node (loc (get-location node))))
+   (duplicate::kwote node (loc (get-inline-location node))))
        
 ;*---------------------------------------------------------------------*/
 ;*    do-alphatize ::sequence ...                                      */
 ;*---------------------------------------------------------------------*/
 (define-method (do-alphatize node::sequence)
    (duplicate::sequence node
-      (loc (get-location node))
+      (loc (get-inline-location node))
       (nodes (map do-alphatize (sequence-nodes node)))))
 
 ;*---------------------------------------------------------------------*/
@@ -173,7 +151,7 @@
    ;; we have to enforce here a variable and not a closure (that
    ;; why the duplicate::var of the fun field).
    (duplicate::app node
-      (loc (get-location node))
+      (loc (get-inline-location node))
       (fun (let ((var (do-alphatize (app-fun node))))
 	      (if (closure? var)
 		  (duplicate::var var)
@@ -181,19 +159,19 @@
       (args (map do-alphatize (app-args node)))))
 
 ;*---------------------------------------------------------------------*/
-;*    do-alphatize ::app-ly ...                                         */
+;*    do-alphatize ::app-ly ...                                        */
 ;*---------------------------------------------------------------------*/
 (define-method (do-alphatize node::app-ly)
    (let ((fun (do-alphatize (app-ly-fun node)))
 	 (arg (do-alphatize (app-ly-arg node))))
       (if (closure? fun)
 	  (known-app-ly->node '()
-			     (get-location node)
+			     (get-inline-location node)
 			     (duplicate::var fun)
 			     arg
 			     'value)
 	  (duplicate::app-ly node
-	     (loc (get-location node))
+	     (loc (get-inline-location node))
 	     (fun fun)
 	     (arg arg)))))
 
@@ -210,15 +188,15 @@
 	  (if (correct-arity-app? (var-variable fun)
 				  (cdr args))
 	      (make-app-node '()
-			     (get-location node)
+			     (get-inline-location node)
 			     (duplicate::var fun)
 			     (cdr args))
-	      (user-error/location (get-location node)
+	      (user-error/location (get-inline-location node)
 				   "Illegal application"
 				   "wrong number of argument(s)"
 				   (shape node)))
 	  (duplicate::funcall node
-	     (loc (get-location node))
+	     (loc (get-inline-location node))
 	     (fun fun)
 	     (args args)))))
 
@@ -227,15 +205,78 @@
 ;*---------------------------------------------------------------------*/
 (define-method (do-alphatize node::pragma)
    (duplicate::pragma node
-      (loc (get-location node))
-      (args (map do-alphatize (pragma-args node)))))
+      (loc (get-inline-location node))
+      (expr* (map do-alphatize (pragma-expr* node)))))
+
+;*---------------------------------------------------------------------*/
+;*    do-alphatize ::getfield ...                                      */
+;*---------------------------------------------------------------------*/
+(define-method (do-alphatize node::getfield)
+   (duplicate::getfield node
+      (loc (get-inline-location node))
+      (expr* (map do-alphatize (getfield-expr* node)))))
+
+;*---------------------------------------------------------------------*/
+;*    do-alphatize ::setfield ...                                      */
+;*---------------------------------------------------------------------*/
+(define-method (do-alphatize node::setfield)
+   (duplicate::setfield node
+      (loc (get-inline-location node))
+      (expr* (map do-alphatize (setfield-expr* node)))))
+
+;*---------------------------------------------------------------------*/
+;*    do-alphatize ::new ...                                           */
+;*---------------------------------------------------------------------*/
+(define-method (do-alphatize node::new)
+   (duplicate::new node
+      (loc (get-inline-location node))))
+
+;*---------------------------------------------------------------------*/
+;*    do-alphatize ::vlength ...                                       */
+;*---------------------------------------------------------------------*/
+(define-method (do-alphatize node::vlength)
+   (duplicate::vlength node
+      (loc (get-inline-location node))
+      (expr* (map do-alphatize (vlength-expr* node)))))
+
+;*---------------------------------------------------------------------*/
+;*    do-alphatize ::vref ...                                          */
+;*---------------------------------------------------------------------*/
+(define-method (do-alphatize node::vref)
+   (duplicate::vref node
+      (loc (get-inline-location node))
+      (expr* (map do-alphatize (vref-expr* node)))))
+
+;*---------------------------------------------------------------------*/
+;*    do-alphatize ::vset! ...                                         */
+;*---------------------------------------------------------------------*/
+(define-method (do-alphatize node::vset!)
+   (duplicate::vset! node
+      (loc (get-inline-location node))
+      (expr* (map do-alphatize (vset!-expr* node)))))
+
+;*---------------------------------------------------------------------*/
+;*    do-alphatize ::valloc ...                                        */
+;*---------------------------------------------------------------------*/
+(define-method (do-alphatize node::valloc)
+   (duplicate::valloc node
+      (loc (get-inline-location node))
+      (expr* (map do-alphatize (valloc-expr* node)))))
+
+;*---------------------------------------------------------------------*/
+;*    do-alphatize ::isa ...                                           */
+;*---------------------------------------------------------------------*/
+(define-method (do-alphatize node::isa)
+   (duplicate::isa node
+      (expr* (list (do-alphatize (car (isa-expr* node)))))
+      (loc (get-inline-location node))))
 
 ;*---------------------------------------------------------------------*/
 ;*    do-alphatize ::cast ...                                          */
 ;*---------------------------------------------------------------------*/
 (define-method (do-alphatize node::cast)
    (duplicate::cast node
-      (loc (get-location node))
+      (loc (get-inline-location node))
       (arg (do-alphatize (cast-arg node)))))
 
 ;*---------------------------------------------------------------------*/
@@ -249,20 +290,20 @@
 	 ((eq? alpha #unspecified)
 	  (use-variable! var (node-loc node) 'set!)
 	  (duplicate::setq node
-	     (loc (get-location node))
-	     (var (duplicate::var v (loc (get-location node))))
+	     (loc (get-inline-location node))
+	     (var (duplicate::var v (loc (get-inline-location node))))
 	     (value (do-alphatize (setq-value node)))))
 	 ((variable? alpha)
 	  (use-variable! alpha (node-loc node) 'set!)
 	  (duplicate::setq node
-	     (loc (get-location node))
+	     (loc (get-inline-location node))
 	     (var (duplicate::var v
-		     (loc (get-location node))
+		     (loc (get-inline-location node))
 		     (variable alpha)))
 	     (value (do-alphatize (setq-value node)))))
 	 (else
 	  (internal-error "alphatize"
-			  "Illegal alphatization"
+			  "Illegal alphatization (setq)"
 			  (shape node))))))
 
 ;*---------------------------------------------------------------------*/
@@ -270,7 +311,7 @@
 ;*---------------------------------------------------------------------*/
 (define-method (do-alphatize node::conditional)
    (duplicate::conditional node
-      (loc (get-location node))
+      (loc (get-inline-location node))
       (test (do-alphatize (conditional-test node)))
       (true (do-alphatize (conditional-true node)))
       (false (do-alphatize (conditional-false node)))))
@@ -280,7 +321,7 @@
 ;*---------------------------------------------------------------------*/
 (define-method (do-alphatize node::fail)
    (duplicate::fail node
-      (loc (get-location node))
+      (loc (get-inline-location node))
       (proc (do-alphatize (fail-proc node)))
       (msg  (do-alphatize (fail-msg node)))
       (obj  (do-alphatize (fail-obj node)))))
@@ -290,7 +331,7 @@
 ;*---------------------------------------------------------------------*/
 (define-method (do-alphatize node::select)
    (duplicate::select node
-      (loc (get-location node))
+      (loc (get-inline-location node))
       (test (do-alphatize (select-test node)))
       (clauses (map (lambda (clause)
 		       (cons (car clause)
@@ -302,7 +343,7 @@
 ;*---------------------------------------------------------------------*/
 (define-method (do-alphatize node::make-box)
    (duplicate::make-box node
-      (loc (get-location node))
+      (loc (get-inline-location node))
       (value (do-alphatize (make-box-value node)))))
 
 ;*---------------------------------------------------------------------*/
@@ -310,7 +351,7 @@
 ;*---------------------------------------------------------------------*/
 (define-method (do-alphatize node::box-ref)
    (duplicate::box-ref node
-      (loc (get-location node))
+      (loc (get-inline-location node))
       (var (do-alphatize (box-ref-var node)))))
 
 ;*---------------------------------------------------------------------*/
@@ -318,7 +359,7 @@
 ;*---------------------------------------------------------------------*/
 (define-method (do-alphatize node::box-set!)
    (duplicate::box-set! node
-      (loc (get-location node))
+      (loc (get-inline-location node))
       (var (do-alphatize (box-set!-var node)))
       (value (do-alphatize (box-set!-value node)))))
 
@@ -342,20 +383,21 @@
 			  (old-body (sfun-body old-sfun))
 			  (new-body (alphatize (append old-locals old-args)
 					       (append new-locals new-args)
-					       (get-location node)
+					       (get-inline-location node)
 					       old-body))
 			  (new-sfun (duplicate::sfun old-sfun
 				       (args new-args)
 				       (body new-body))))
+		      (local-user?-set! new (local-user? old))
 		      (local-value-set! new new-sfun)))
 		old-locals
 		new-locals)
       (duplicate::let-fun node
-	 (loc (get-location node))
+	 (loc (get-inline-location node))
 	 (locals new-locals)
 	 (body (alphatize old-locals
 			  new-locals
-			  (get-location node)
+			  (get-inline-location node)
 			  (let-fun-body node))))))
 
 ;*---------------------------------------------------------------------*/
@@ -368,18 +410,21 @@
 				;; all local variables must be allocated
 				;; using the `make-local-svar' form
 				;; (for the key attribution).
-				(make-local-svar (local-id l) (local-type l)))
+				(let ((var (make-local-svar (local-id l)
+							    (local-type l))))
+				   (local-user?-set! var (local-user? l))
+				   var))
 			     old-locals))
 	  (new-bindings (map (lambda (binding new-local)
 				(cons new-local (do-alphatize (cdr binding))))
 			     (let-var-bindings node)
 			     new-locals)))
       (duplicate::let-var node
-	 (loc (get-location node))
+	 (loc (get-inline-location node))
 	 (bindings new-bindings)
 	 (body (alphatize old-locals
 			  new-locals
-			  (get-location node)
+			  (get-inline-location node)
 			  (let-var-body node))))))
 
 ;*---------------------------------------------------------------------*/
@@ -396,14 +441,15 @@
 					   (handler alpha-hdlg))))
 								 
 	  (old-body   (set-ex-it-body node)))
+      (local-user?-set! new-var (local-user? old-var))
       (duplicate::set-ex-it node
-	 (loc (get-location node))
+	 (loc (get-inline-location node))
 	 (var (duplicate::var (set-ex-it-var node)
-		 (loc (get-location node))
+		 (loc (get-inline-location node))
 		 (variable new-var)))
 	 (body (alphatize (list old-var)
 			  (list new-var)
-			  (get-location node)
+			  (get-inline-location node)
 			  old-body)))))
 
 ;*---------------------------------------------------------------------*/
@@ -411,7 +457,7 @@
 ;*---------------------------------------------------------------------*/
 (define-method (do-alphatize node::jump-ex-it)
    (duplicate::jump-ex-it node
-      (loc (get-location node))
+      (loc (get-inline-location node))
       (exit (do-alphatize (jump-ex-it-exit node)))
       (value (do-alphatize (jump-ex-it-value node)))))
 
