@@ -1,41 +1,9 @@
-;*---------------------------------------------------------------------*/
-;*   A pratical implementation for the Scheme programming language     */
-;*                                                                     */
-;*                                    ,--^,                            */
-;*                              _ ___/ /|/                             */
-;*                          ,;'( )__, ) '                              */
-;*                         ;;  //   L__.                               */
-;*                         '   \\   /  '                               */
-;*                              ^   ^                                  */
-;*                                                                     */
-;*   Copyright (c) 1992-1999 Manuel Serrano                            */
-;*                                                                     */
-;*     Bug descriptions, use reports, comments or suggestions are      */
-;*     welcome. Send them to                                           */
-;*       bigloo-request@kaolin.unice.fr                                */
-;*       http://kaolin.unice.fr/bigloo                                 */
-;*                                                                     */
-;*   This program is free software; you can redistribute it            */
-;*   and/or modify it under the terms of the GNU General Public        */
-;*   License as published by the Free Software Foundation; either      */
-;*   version 2 of the License, or (at your option) any later version.  */
-;*                                                                     */
-;*   This program is distributed in the hope that it will be useful,   */
-;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
-;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
-;*   GNU General Public License for more details.                      */
-;*                                                                     */
-;*   You should have received a copy of the GNU General Public         */
-;*   License along with this program; if not, write to the Free        */
-;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
-;*   MA 02111-1307, USA.                                               */
-;*---------------------------------------------------------------------*/
 ;*=====================================================================*/
 ;*    serrano/prgm/project/bigloo/comptime/Ast/sexp.scm                */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Fri May 31 15:05:39 1996                          */
-;*    Last change :  Sun May 24 08:06:46 1998 (serrano)                */
+;*    Last change :  Sat Jul  7 09:26:05 2001 (serrano)                */
 ;*    -------------------------------------------------------------    */
 ;*    We build an `ast node' from a `sexp'                             */
 ;*---------------------------------------------------------------------*/
@@ -58,6 +26,7 @@
 	    type_env
 	    type_cache
 	    type_typeof
+	    engine_param
 	    ast_ident
 	    ast_env
 	    ast_var
@@ -68,10 +37,12 @@
 	    ast_let
 	    ast_exit
 	    ast_app
-	    ast_apply)
+	    ast_apply
+	    ast_private)
    
    (export  (top-level-sexp->node::node    <sexp> ::obj)
 	    (sexp->node::node              ::obj <obj> ::obj ::symbol)
+	    (sexp*->node::pair-nil         ::pair-nil <obj> ::obj ::symbol)
 	    (define-primop-ref->node::node ::global ::node)
 	    (define-primop->node::node     ::global)
 	    (location->node::node          ::global)
@@ -99,7 +70,7 @@
 ;*---------------------------------------------------------------------*/
 (define (sexp->node exp stack loc site)
    [assert (site) (memq site *sites*)]
-   (trace (ast 2) "sexp->node: " exp #\Newline)
+   (trace (ast 2) "sexp->node(" loc "): " exp #\Newline)
    (match-case exp
 ;*--- () --------------------------------------------------------------*/
       (()
@@ -156,10 +127,15 @@
 		 (else
 		  (use-variable! global loc site)
 		  (if (and (not (eq? site 'app)) (fun? (global-value global)))
-		      (instantiate::closure
-			 (loc loc)
-			 (type (global-type global))
-			 (variable global))
+		      (if (and (cfun? (global-value global))
+			       (eq? *target-language* 'jvm)
+			       (not (eq? site 'apply)))
+			  (error-sexp->node "Java method cannot be used as value"
+					    exp loc)
+			  (instantiate::closure
+			     (loc loc)
+			     (type (global-type global))
+			     (variable global)))
 		      (instantiate::var 
 			 (loc loc)
 			 (type (global-type global))
@@ -169,7 +145,7 @@
        (let ((loc (find-location/loc exp loc)))
 	  (match-case exp
 	     ((@ (and (? symbol?) ?name) (and (? symbol?) ?module))
-	      (let ((global (find-global name module)))
+	      (let ((global (find-global/module name module)))
 		 (cond
 		    ((not (global? global))
 		     (error-sexp->node "Unbound variable" exp loc))
@@ -231,9 +207,6 @@
 				    (nodes nodes)))))
 ;*--- if --------------------------------------------------------------*/
       ((if . ?-)
-;*        (print "*** IF ********: " exp)                              */
-;*        (print "            loc: " (find-location/loc exp #f))       */
-;*        (print "        old-loc: " loc)                              */
        (match-case exp
           ((?- ?si ?alors ?sinon)
            (match-case si
@@ -251,12 +224,6 @@
 			  (si      (sexp->node si stack l-si 'value))
 			  (alors   (sexp->node alors stack l-alors 'value))
 			  (sinon   (sexp->node sinon stack l-sinon 'value))
-;* 			  (dummy   (begin (print "Un if: " exp)        */
-;* 					  (print "lif: " (find-location/loc exp */
-;* 									    #f)) */
-;* 					  (print "loc: " loc)          */
-;* 					  (print "oloc: " oloc)        */
-;* 					  (print "lsi: " (node-loc si)))) */
 			  (loc     (let ((loc (find-location/loc
 					       exp
 					       (node-loc si))))
@@ -287,11 +254,17 @@
 		  (val     (sexp->node val stack val-loc 'value)))
 	      (let ((ast (sexp->node var stack loc 'set!)))
 		 (if (var? ast)
-		     (instantiate::setq
-			(loc   loc)
-			(type  *unspec*)
-			(var   ast)
-			(value val))
+		     (with-access::var ast (variable)
+			(if (and (global? variable)
+				 (global-read-only? variable))
+			    (error-sexp->node "Read-only variable"
+					      exp
+					      loc)
+			    (instantiate::setq
+			       (loc   loc)
+			       (type  *unspec*)
+			       (var   ast)
+			       (value val))))
 		     (error-sexp->node "illegal `set!' expression"
 				       exp
 				       loc)))))
@@ -392,8 +365,8 @@
       ((lambda . ?-)
        (match-case exp
           ((?- ?args . ?body) 
-           (let* ((loc (find-location/loc exp loc))
-		  (fun (gensym 'lambda)))
+           (let ((loc  (find-location/loc exp loc))
+		 (fun  (make-anonymous-name loc)))
               (sexp->node `(labels ((,fun ,args ,(normalize-progn body))) ,fun)
 			  stack
 			  loc
@@ -430,6 +403,12 @@
 			     (find-location/loc exp loc)))))
 ;*--- case ------------------------------------------------------------*/
       ((case . ?-)
+       ;; former versions of the compiler used to make side effect
+       ;; one the list that hold the clauses. This is a bad idea.
+       ;; Because of macro expansion it could be that the very same
+       ;; case form is seen twice by the compiler. If we compile case
+       ;; by the means of side effects, the second time the compiler
+       ;; will see the case form it will think of it as incorrect.
        (match-case exp
           ((?- ?test . ?clauses)
            (let* ((loc  (find-location/loc exp loc))
@@ -437,28 +416,30 @@
 				    stack
 				    (find-location/loc test loc)
 				    'value)))
-	      (let loop ((cls clauses))
+	      (let loop ((cls clauses)
+			 (nclauses '()))
 		 (if (null? cls)
 		     (instantiate::select
 			(loc loc)
 			(type *_*)
 			(test test)
 			(item-type (typeof-atom (car (car (car clauses)))))
-			(clauses clauses))
-		     (let ((clause (car cls)))
-			;; we build the ast for the then part of the clause
-			(set-cdr! clause
-				  (sexp->node (normalize-progn (cdr clause))
-					      stack
-					      (find-location/loc clause loc)
-					      'value))
+			(clauses (reverse! nclauses)))
+		     (let* ((clause (car cls))
+			    (body   (sexp->node (normalize-progn (cdr clause))
+						stack
+						(find-location/loc clause loc)
+						'value))
+			    (nclause (cons (car clause) body)))
 			;; we check that it is not an illegal `else' clause
 			(if (and (eq? (car clause) 'else)
 				 (not (null? (cdr cls))))
 			    (error-sexp->node "Illegal `case' form"
 					      (find-location/loc exp loc)
 					      exp)
-			    (loop (cdr cls))))))))
+			    (loop (cdr cls)
+				  (cons (epairify nclause clause)
+					nclauses))))))))
           (else
 	   (error-sexp->node "Illegal `case' form"
 			     (find-location/loc exp loc)
@@ -470,8 +451,11 @@
       ((jump-exit . ?-)
        (jump-exit->node exp stack loc site))
 ;*--- apply -----------------------------------------------------------*/
-      ((apply . ?-)
+      ((apply ?- ?-)
        (applycation->node exp stack loc site))
+;*--- private ---------------------------------------------------------*/
+      ((? private-sexp?)
+       (private->node exp stack loc site))
 ;*--- app -------------------------------------------------------------*/
       (else
        ;; this expression can be a function call or a typed pragma
@@ -479,10 +463,11 @@
        ;; we compile a function call. This check is required by the
        ;; form (pragma::??? ...) (because we can't add a branch in the
        ;; match-case to check the node `pragma::???').
-       (let ((caller (car exp)))
+       (let ((caller (car exp))
+	     (loc    (find-location/loc exp loc)))
 	  (if (symbol? caller)
 	      ;; it might be a typed special forms (such as pragma or lambda)
-	      (let* ((pid  (parse-id caller))
+	      (let* ((pid  (parse-id caller loc))
 		     (id   (car pid))
 		     (type (cdr pid)))
 		 (case id
@@ -495,9 +480,7 @@
 			((?- ?args . ?body)
 			 (let* ((loc  (find-location/loc exp loc))
 				(fun  (mark-symbol-non-user! (gensym 'lambda)))
-				(tfun (symbol-append fun
-						     4dots
-						     (type-id type))))
+				(tfun (make-typed-ident fun (type-id type))))
 			    (sexp->node
 			     `(labels ((,tfun ,args ,(normalize-progn body)))
 				 ,fun)
@@ -507,7 +490,7 @@
 			(else
 			 (error-sexp->node "Illegal `lambda' form"
 					   exp
-					   (find-location/loc exp loc)))))
+					   loc))))
 		    (else
 		     (application->node exp stack loc site))))
 	      (application->node exp stack loc site))))))
@@ -515,18 +498,20 @@
 ;*---------------------------------------------------------------------*/
 ;*    sexp*->node ...                                                  */
 ;*---------------------------------------------------------------------*/
-(define (sexp*->node exp* stack loc site)
-   (let loop ((exps     exp*)
-	      (res      '())
-	      (loc      loc))
+(define (sexp*->node::pair-nil exp*::pair-nil stack loc site)
+   (let loop ((exps exp*)
+	      (res  '())
+	      (loc  loc))
       (cond
 	 ((null? exps)
 	  (error-sexp->node "Illegal empty sequence" exps loc))
 	 ((null? (cdr exps))
-	  (let ((loc (find-location/loc (car exps) loc)))
+	  (let ((loc (find-location/loc (car exps)
+					(find-location/loc exps loc))))
 	     (reverse! (cons (sexp->node (car exps) stack loc site) res))))
 	 (else
-	  (let ((loc (find-location/loc (car exps) loc)))
+	  (let ((loc (find-location/loc (car exps)
+					(find-location/loc exps loc))))
 	     (loop (cdr exps)
 		   (cons (sexp->node (car exps) stack loc 'value) res)
 		   loc))))))
@@ -629,3 +614,20 @@
 			    (args (list (sexp->node global '() #f 'value))))
 	  fun)))
    
+;*---------------------------------------------------------------------*/
+;*    make-anonymous-name ...                                          */
+;*---------------------------------------------------------------------*/
+(define (make-anonymous-name::symbol loc)
+   (if (location? loc)
+       (let ((file (location-fname loc))
+	     (line (location-lnum loc))
+	     (base (symbol->string (gensym "<anonymous:"))))
+	  (cond
+	     ((>=fx *bdb-debug* 1)
+	      (string->symbol
+	       (string-append base ":"
+			      file ":" (number->string line)
+			      ">")))
+	     (else
+	      (string->symbol (string-append base ">")))))
+       (symbol-append (gensym '<anonymous-) '>)))
