@@ -1,41 +1,9 @@
-;*---------------------------------------------------------------------*/
-;*   A pratical implementation for the Scheme programming language     */
-;*                                                                     */
-;*                                    ,--^,                            */
-;*                              _ ___/ /|/                             */
-;*                          ,;'( )__, ) '                              */
-;*                         ;;  //   L__.                               */
-;*                         '   \\   /  '                               */
-;*                              ^   ^                                  */
-;*                                                                     */
-;*   Copyright (c) 1992-1999 Manuel Serrano                            */
-;*                                                                     */
-;*     Bug descriptions, use reports, comments or suggestions are      */
-;*     welcome. Send them to                                           */
-;*       bigloo-request@kaolin.unice.fr                                */
-;*       http://kaolin.unice.fr/bigloo                                 */
-;*                                                                     */
-;*   This program is free software; you can redistribute it            */
-;*   and/or modify it under the terms of the GNU General Public        */
-;*   License as published by the Free Software Foundation; either      */
-;*   version 2 of the License, or (at your option) any later version.  */
-;*                                                                     */
-;*   This program is distributed in the hope that it will be useful,   */
-;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
-;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
-;*   GNU General Public License for more details.                      */
-;*                                                                     */
-;*   You should have received a copy of the GNU General Public         */
-;*   License along with this program; if not, write to the Free        */
-;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
-;*   MA 02111-1307, USA.                                               */
-;*---------------------------------------------------------------------*/
 ;*=====================================================================*/
 ;*    serrano/prgm/project/bigloo/comptime/Ast/unit.scm                */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Mon Jun  3 08:35:53 1996                          */
-;*    Last change :  Sun Nov 29 21:09:00 1998 (serrano)                */
+;*    Last change :  Thu Nov 23 13:20:46 2000 (serrano)                */
 ;*    -------------------------------------------------------------    */
 ;*    A module is composed of several unit (for instance, the user     */
 ;*    unit (also called the toplevel unit), the foreign unit, the      */
@@ -60,9 +28,9 @@
 	    ast_env
 	    ast_local
 	    ast_sexp
+	    object_class
 	    object_generic
 	    object_method
-	    object_inline
 	    tools_progn
 	    tools_args
 	    tools_misc
@@ -86,12 +54,9 @@
 ;*    unit-sexp*-add! ...                                              */
 ;*---------------------------------------------------------------------*/
 (define (unit-sexp*-add! unit sexp)
-   (let ((sexp (if (and (fixnum? *compiler-debug*) (>=fx *compiler-debug* 3))
-		   (cons `(set! *debug* ,*compiler-debug*) sexp)
-		   sexp)))
-      (if (null? (unit-sexp* unit))
-	  (unit-sexp*-set! unit sexp)
-	  (set-cdr! (last-pair (unit-sexp* unit)) sexp))))
+   (if (null? (unit-sexp* unit))
+       (unit-sexp*-set! unit sexp)
+       (set-cdr! (last-pair (unit-sexp* unit)) sexp)))
 
 ;*---------------------------------------------------------------------*/
 ;*    unit->defs ...                                                   */
@@ -113,9 +78,8 @@
 		 (def*  '()))
 	 (if (null? aexp*)
 	     (if (pair? init*)
-		 (let ((init (def-global-sfun! (symbol-append
-						(unit-initializer-id id)
-						'::obj)
+		 (let ((init (def-global-sfun-no-warning!
+				(symbol-append (unit-initializer-id id) '::obj)
 				'()
 				'()
 				*module*
@@ -177,7 +141,7 @@
 ;*---------------------------------------------------------------------*/
 (define (unit-initializers)
    (map (lambda (unit)
-	   (find-global (unit-initializer-id (car unit)) *module*))
+	   (find-global/module (unit-initializer-id (car unit)) *module*))
 	*unit-list*))
 
 ;*---------------------------------------------------------------------*/
@@ -205,12 +169,14 @@
 ;*---------------------------------------------------------------------*/
 (define (toplevel->ast sexp gdefs)
    (match-case sexp
+      ((begin)
+       (list sexp))
       ((begin . ?nsexp*)
        (reverse! (toplevel*->ast nsexp* gdefs)))
       ((define (?var . ?args) . ?exp)
-       (let* ((id     (id-of-id var))
+       (let* ((id     (id-of-id var (find-location sexp)))
 	      (def    (assq id gdefs))
-	      (global (find-global id *module*)))
+	      (global (find-global/module id *module*)))
 	  ;; exported variable are set to be written hence, we don't
 	  ;; have to check here if the variable has been declared has
 	  ;; exported (as a variable vs a function). We just have to
@@ -232,9 +198,9 @@
 		 (replace! sexp new-sexp)
 		 (make-svar-definition var sexp)))))
       ((define ?var (lambda ?args . ?exp))
-       (let* ((id     (id-of-id var))
+       (let* ((id     (id-of-id var (find-location sexp)))
 	      (def    (assq id gdefs))
-	      (global (find-global id *module*)))
+	      (global (find-global/module id *module*)))
 	  ;; same remark as in the previous match (variables vs functions)
 	  (if (and (eq? (car (cdr def)) 'read)
 		   (or (not (global? global))
@@ -250,7 +216,7 @@
        (set-car! (cddr sexp) 1-exp)
        (toplevel->ast sexp gdefs))
       ((define ?var (and (? symbol?) ?var2))
-       (let ((def (assq (id-of-id var) gdefs)))
+       (let ((def (assq (id-of-id var (find-location sexp)) gdefs)))
 	  (if (eq? (car (cdr def)) 'read)
 	      (let ((arity (get-global-arity var2 #f gdefs)))
 		 (if (fixnum? arity)
@@ -259,7 +225,7 @@
 		     (make-svar-definition var sexp)))
 	      (make-svar-definition var sexp))))
       ((define ?var (@ (and (? symbol?) ?var2) (and (? symbol?) ?module)))
-       (let ((def (assq (id-of-id var) gdefs)))
+       (let ((def (assq (id-of-id var (find-location sexp)) gdefs)))
 	  (if (eq? (car (cdr def)) 'read)
 	      (let ((arity (get-global-arity var2 module gdefs)))
 		 (if (fixnum? arity)
@@ -315,7 +281,7 @@
 ;*---------------------------------------------------------------------*/
 (define (get-global-arity id module gdefs)
    (let ((global (if (symbol? module)
-		     (find-global id module)
+		     (find-global/module id module)
 		     (find-global id))))
       (if (not (global? global))
 	  #f
@@ -353,23 +319,24 @@
 		 (apply ,id2 (cons* ,@(args*->args-list args)))))))
       (match-case sexp
 	 ((define ?var1 ?var2)
-	  (let ((id2 (id-of-id var2)))
+	  (let ((id2 (id-of-id var2 (find-location sexp))))
 	     (replace! sexp (do-eta-expanse var1 id2))))
 	 ((define ?var1 (@ ?var2 ?module2))
-	  (let* ((id2 (id-of-id var2)))
+	  (let* ((id2 (id-of-id var2 (find-location sexp))))
 	     (replace! sexp (do-eta-expanse/module var2 id2 module2)))))))
 	  
 ;*---------------------------------------------------------------------*/
 ;*    make-sfun-definition ...                                         */
 ;*---------------------------------------------------------------------*/
 (define (make-sfun-definition id::symbol module::symbol args body src class)
-   (let* ((locals   (let loop ((args  args)
+   (let* ((loc      (find-location src))
+	  (locals   (let loop ((args  args)
 			       (res   '()))
 		       (cond
 			  ((null? args)
 			   (reverse! res))
 			  ((not (pair? args)) 
-			   (let* ((pid  (check-id (parse-id args) src))
+			   (let* ((pid  (check-id (parse-id args loc) src))
 				  (id   (car pid))
 				  (type (cdr pid)))
 			      ;; there is no need to check the last
@@ -384,11 +351,14 @@
 				   (cons (make-user-local-svar arg *obj*) res))
 				  (reverse! res))))
 			  (else
-			   (let* ((pid  (check-id (parse-id (car args)) src))
+			   (let* ((pid  (check-id (parse-id (car args) loc)
+						  src))
 				  (id   (car pid))
 				  (type (cdr pid)))
 			      (loop (cdr args)
-				    (cons (make-user-local-svar id type)
+				    (cons (if (user-symbol? id)
+					      (make-user-local-svar id type)
+					      (make-local-svar id type))
 					  res)))))))
 	  (body     (make-dsssl-function-prelude id args body user-error)))
       (list (def-global-sfun! id args locals module class src 'now body))))
@@ -400,7 +370,7 @@
    (def-global-svar! id *module* src 'now)
    ;; without the Inline global variable optimization once should except
    ;; to find here `(set-car! src 'set!)'
-   (set-car! (cdr src) (car (check-id (parse-id id) src)))
+   (set-car! (cdr src) (car (check-id (parse-id id (find-location src)) src)))
    (list src))
 
 ;*---------------------------------------------------------------------*/
@@ -411,14 +381,13 @@
    (let* ((default-id   name)
 	  (default-tid  (if (eq? type *_*)
 			    default-id
-			    (symbol-append default-id
-					   4dots
-					   (type-id type))))
+			    (make-typed-ident default-id (type-id type))))
 	  (default-body (if (pair? body)
 			    (normalize-progn body)
 			    `(error ',name
 				    "No method for this object"
-				    ,(id-of-id (car args)))))
+				    ,(id-of-id (car args)
+					       (find-location src)))))
 	  (form (let ((tmp `(define ,(cons default-tid args) ,default-body)))
 		   (if (epair? *module-clause*)
 		       (econs (car tmp) (cdr tmp) (cer *module-clause*))
@@ -428,7 +397,10 @@
 	 (trace (ast 2) "  l'ast: " ast #\Newline)
 	 (if (pair? ast)
 	     (if (global? (car ast))
-		 (global-user?-set! (car ast) #f)))
+		 ;; we mark the function as a user one so the default
+		 ;; function will appears as is in the profiler or the
+		 ;; debugger.
+		 (global-user?-set! (car ast) #t)))
 	 ast)))
 
 ;*---------------------------------------------------------------------*/
@@ -437,28 +409,34 @@
 (define (make-sgfun-definition id module args body src gdefs)
    (trace ast "make-sgfun-definition: " id " " module " " args " " body
 	  #\Newline)
-   (let* ((locals (let loop ((args args)
-			     (res  '()))
-		     (cond
-			((null? args)
-			 (reverse! res))
-			((not (pair? args))
-			 (let* ((pid  (check-id (parse-id args) src))
-				(id   (car pid))
-				(type (cdr pid)))
-			    ;; there is no need to check the last
-			    ;; n-ary formal argument because it will
-			    ;; be checked when defining the global variable
-			    (reverse! (cons (make-user-local-svar id type)
-					    res))))
-			(else
-			 (let* ((pid  (check-id (parse-id (car args)) src))
-				(id   (car pid))
-				(type (cdr pid)))
-			    (loop (cdr args)
-				  (cons (make-user-local-svar id type)
-					res)))))))
-	  (pid     (check-id (parse-id id) src))
+   (let* ((loc    (find-location src))
+	  (locals (if (null? args)
+		      (user-error id
+				  "Illegal generic definition (first argument missing)"
+				  src)
+		      (let loop ((args args)
+				 (res  '()))
+			 (cond
+			    ((null? args)
+			     (reverse! res))
+			    ((not (pair? args))
+			     (let* ((pid  (check-id (parse-id args loc) src))
+				    (id   (car pid))
+				    (type (cdr pid)))
+				;; there is no need to check the last
+				;; n-ary formal argument because it will
+				;; be checked when defining the global variable
+				(reverse! (cons (make-user-local-svar id type)
+						res))))
+			    (else
+			     (let* ((pid  (check-id (parse-id (car args) loc)
+						    src))
+				    (id   (car pid))
+				    (type (cdr pid)))
+				(loop (cdr args)
+				      (cons (make-user-local-svar id type)
+					    res))))))))
+	  (pid     (check-id (parse-id id loc) src))
 	  (name    (car pid))
 	  (type    (cdr pid))
 	  (dname   (gensym (symbol-append name '-default)))
@@ -475,14 +453,6 @@
 		      'now
 		      body)))
       (trace (ast 2) "  body: " body #\Newline)
-      
-      ;; When optim-inline-method is set, the only situtations
-      ;; we declare a generic is when it is exported
-      ;; When it is imported, it has already been declared in the module
-      ;; containing its definition. When it is static, thanks to the
-      ;; inline optimisation, method are inlined and don't use the general
-      ;; dispatch mechanisme.
-      (add-generic-for-method-inlining! generic)
       (let ((o-unit (get-method-unit))
 	    (sexp*  (cons generic
 			  (append (list `(add-generic! (@ ,(global-id generic)
@@ -499,25 +469,26 @@
 ;*    make-method-definition ...                                       */
 ;*---------------------------------------------------------------------*/
 (define (make-method-definition id args body src)
-   (let ((locals (let loop ((args args)
-			    (res  '()))
-		    (cond
-		       ((null? args)
-			(reverse! res))
-		       ((not (pair? args))
-			(let* ((pid  (check-id (parse-id args) src))
-			       (id   (car pid))
-			       (type (cdr pid)))
-			   ;; there is no need to check the last
-			   ;; n-ary formal argument because it will
-			   ;; be checked when defining the global variable
-			   (reverse! (cons (make-local-svar id type) res))))
-		       (else
-			(let* ((pid  (check-id (parse-id (car args)) src))
-			       (id   (car pid))
-			       (type (cdr pid)))
-			   (loop (cdr args)
-				 (cons (make-local-svar id type) res))))))))
+   (let* ((loc    (find-location src))
+	  (locals (let loop ((args args)
+			     (res  '()))
+		     (cond
+			((null? args)
+			 (reverse! res))
+			((not (pair? args))
+			 (let* ((pid  (check-id (parse-id args loc) src))
+				(id   (car pid))
+				(type (cdr pid)))
+			    ;; there is no need to check the last
+			    ;; n-ary formal argument because it will
+			    ;; be checked when defining the global variable
+			    (reverse! (cons (make-local-svar id type) res))))
+			(else
+			 (let* ((pid  (check-id (parse-id (car args) loc) src))
+				(id   (car pid))
+				(type (cdr pid)))
+			    (loop (cdr args)
+				  (cons (make-local-svar id type) res))))))))
       (if (not (check-method-definition id args	locals src))
 	  (list #unspecified)
 	  (let ((o-unit (get-method-unit))
