@@ -1,41 +1,10 @@
-;*---------------------------------------------------------------------*/
-;*   A pratical implementation for the Scheme programming language     */
-;*                                                                     */
-;*                                    ,--^,                            */
-;*                              _ ___/ /|/                             */
-;*                          ,;'( )__, ) '                              */
-;*                         ;;  //   L__.                               */
-;*                         '   \\   /  '                               */
-;*                              ^   ^                                  */
-;*                                                                     */
-;*   Copyright (c) 1992-1999 Manuel Serrano                            */
-;*                                                                     */
-;*     Bug descriptions, use reports, comments or suggestions are      */
-;*     welcome. Send them to                                           */
-;*       bigloo-request@kaolin.unice.fr                                */
-;*       http://kaolin.unice.fr/bigloo                                 */
-;*                                                                     */
-;*   This program is free software; you can redistribute it            */
-;*   and/or modify it under the terms of the GNU General Public        */
-;*   License as published by the Free Software Foundation; either      */
-;*   version 2 of the License, or (at your option) any later version.  */
-;*                                                                     */
-;*   This program is distributed in the hope that it will be useful,   */
-;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
-;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
-;*   GNU General Public License for more details.                      */
-;*                                                                     */
-;*   You should have received a copy of the GNU General Public         */
-;*   License along with this program; if not, write to the Free        */
-;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
-;*   MA 02111-1307, USA.                                               */
-;*---------------------------------------------------------------------*/
 ;*=====================================================================*/
-;*    serrano/prgm/project/bigloo/comptime1.9b/Cfa/closure.scm         */
+;*    serrano/prgm/project/bigloo/comptime/Cfa/closure.scm             */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Thu Jun 27 11:35:13 1996                          */
-;*    Last change :  Wed Apr 16 12:22:32 1997 (serrano)                */
+;*    Last change :  Sat Jul  7 08:37:02 2001 (serrano)                */
+;*    Copyright   :  1996-2001 Manuel Serrano, see LICENSE file        */
 ;*    -------------------------------------------------------------    */
 ;*    The closure optimization described in:                           */
 ;*                                                                     */
@@ -64,11 +33,10 @@
 	    ast_node
 	    ast_env
 	    cfa_info
+	    cfa_info2
 	    cfa_cfa
 	    cfa_approx
-	    cfa_set
-	    cfa_info
-	    cfa_stack)
+	    cfa_set)
    (export  (closure-optimization!)
 	    (closure-optimization?)
 	    (add-procedure-ref!  ::node)
@@ -282,11 +250,9 @@
 	       (else
 		(var-variable-set! fun *make-el-procedure*))))
 	 (set! args (cddr args))
-	 (if (stack-optimization?)
-	     (stack! app)
-	     app)))
+	 app))
    (define (make-light-procedure-app app)
-      (with-access::make-procedure-app app (fun args stackable?)
+      (with-access::make-procedure-app app (fun args)
 	 (let* ((size (get-node-atom-value (caddr args)))
 		(ffun (var-variable (car args)))
 		(sfun (variable-value ffun)))
@@ -297,9 +263,7 @@
 				  'sefun))
 	    (var-variable-set! fun *make-l-procedure*)
 	    (set-cdr! args (cddr args))
-	    (if (stack-optimization?)
-		(stack! app)
-		app))))
+	    app)))
    ;; we change the procedure allocation sites
    (for-each (lambda (app)
 		(with-access::make-procedure-app app (X T)
@@ -445,17 +409,17 @@
 ;*    start-cache ...                                                  */
 ;*---------------------------------------------------------------------*/
 (define (start-cache)
-   (set! *procedure-ref* (find-global 'procedure-ref 'foreign))
-   (set! *procedure-set!* (find-global 'procedure-set! 'foreign))
-   (set! *procedure-l-ref* (find-global 'procedure-l-ref 'foreign))
-   (set! *procedure-l-set!* (find-global 'procedure-l-set! 'foreign))
-   (set! *procedure-el-ref* (find-global 'procedure-el-ref 'foreign))
-   (set! *procedure-el-set!* (find-global 'procedure-el-set! 'foreign))
-   (set! *procedure-1-el-ref* (find-global 'procedure-1-el-ref 'foreign))
-   (set! *procedure-1-el-set!* (find-global 'procedure-1-el-set! 'foreign))
-   (set! *make-el-procedure* (find-global 'make-el-procedure 'foreign))
-   (set! *make-el-procedure-1* (find-global 'make-el-procedure-1 'foreign))
-   (set! *make-l-procedure* (find-global 'make-l-procedure 'foreign)))
+   (set! *procedure-ref* (find-global/module 'procedure-ref 'foreign))
+   (set! *procedure-set!* (find-global/module 'procedure-set! 'foreign))
+   (set! *procedure-l-ref* (find-global/module 'procedure-l-ref 'foreign))
+   (set! *procedure-l-set!* (find-global/module 'procedure-l-set! 'foreign))
+   (set! *procedure-el-ref* (find-global/module 'procedure-el-ref 'foreign))
+   (set! *procedure-el-set!* (find-global/module 'procedure-el-set! 'foreign))
+   (set! *procedure-1-el-ref* (find-global/module 'procedure-1-el-ref 'foreign))
+   (set! *procedure-1-el-set!* (find-global/module 'procedure-1-el-set! 'foreign))
+   (set! *make-el-procedure* (find-global/module 'make-el-procedure 'foreign))
+   (set! *make-el-procedure-1* (find-global/module 'make-el-procedure-1 'foreign))
+   (set! *make-l-procedure* (find-global/module 'make-l-procedure 'foreign)))
 
 ;*---------------------------------------------------------------------*/
 ;*    stop-cache ...                                                   */
