@@ -1,41 +1,10 @@
-;*---------------------------------------------------------------------*/
-;*   A pratical implementation for the Scheme programming language     */
-;*                                                                     */
-;*                                    ,--^,                            */
-;*                              _ ___/ /|/                             */
-;*                          ,;'( )__, ) '                              */
-;*                         ;;  //   L__.                               */
-;*                         '   \\   /  '                               */
-;*                              ^   ^                                  */
-;*                                                                     */
-;*   Copyright (c) 1992-1999 Manuel Serrano                            */
-;*                                                                     */
-;*     Bug descriptions, use reports, comments or suggestions are      */
-;*     welcome. Send them to                                           */
-;*       bigloo-request@kaolin.unice.fr                                */
-;*       http://kaolin.unice.fr/bigloo                                 */
-;*                                                                     */
-;*   This program is free software; you can redistribute it            */
-;*   and/or modify it under the terms of the GNU General Public        */
-;*   License as published by the Free Software Foundation; either      */
-;*   version 2 of the License, or (at your option) any later version.  */
-;*                                                                     */
-;*   This program is distributed in the hope that it will be useful,   */
-;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
-;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
-;*   GNU General Public License for more details.                      */
-;*                                                                     */
-;*   You should have received a copy of the GNU General Public         */
-;*   License along with this program; if not, write to the Free        */
-;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
-;*   MA 02111-1307, USA.                                               */
-;*---------------------------------------------------------------------*/
 ;*=====================================================================*/
-;*    serrano/prgm/project/bigloo/comptime/Cfa/setup.scm               */
+;*    serrano/prgm/project/bigloo2.3/comptime/Cfa/setup.scm            */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Tue Jun 25 14:08:53 1996                          */
-;*    Last change :  Wed Jun  3 15:06:45 1998 (serrano)                */
+;*    Last change :  Thu Jul 20 07:34:06 2000 (serrano)                */
+;*    Copyright   :  1996-2000 Manuel Serrano, see LICENSE file        */
 ;*    -------------------------------------------------------------    */
 ;*    We setup the ast for the Cfa.                                    */
 ;*=====================================================================*/
@@ -54,6 +23,8 @@
 	    ast_var
 	    ast_node
 	    cfa_info
+	    cfa_info2
+	    cfa_info3
 	    cfa_approx)
    (export  (set-initial-approx! globals)
 	    (generic node-setup! ::node)
@@ -292,15 +263,54 @@
       (approx-set-top! (funcall/Cinfo-va-approx node))))
 
 ;*---------------------------------------------------------------------*/
-;*    node-setup! ::pragma ...                                         */
+;*    node-setup! ::extern ...                                         */
 ;*---------------------------------------------------------------------*/
 (define-method (node-setup! node::pragma)
-   (with-access::pragma node (args type)
-      (node-setup*! args)
+   (with-access::pragma node (expr* type)
+      (node-setup*! expr*)
       (widen!::pragma/Cinfo node
 	 (approx (make-type-approx type)))
       (approx-set-top! (pragma/Cinfo-approx node))))
 
+;*---------------------------------------------------------------------*/
+;*    node-setup! ::getfield ...                                       */
+;*---------------------------------------------------------------------*/
+(define-method (node-setup! node::getfield)
+   (with-access::getfield node (expr* type)
+      (node-setup*! expr*)
+      (widen!::getfield/Cinfo node
+	 (approx (make-type-approx type)))
+      (approx-set-top! (getfield/Cinfo-approx node))))
+
+;*---------------------------------------------------------------------*/
+;*    node-setup! ::setfield ...                                       */
+;*---------------------------------------------------------------------*/
+(define-method (node-setup! node::setfield)
+   (with-access::setfield node (expr* type)
+      (node-setup*! expr*)
+      (widen!::setfield/Cinfo node
+	 (approx (make-type-approx type)))
+      (approx-set-top! (setfield/Cinfo-approx node))))
+
+;*---------------------------------------------------------------------*/
+;*    node-setup! ::new ...                                            */
+;*---------------------------------------------------------------------*/
+(define-method (node-setup! node::new)
+   (with-access::new node (type)
+      (widen!::new/Cinfo node
+	 (approx (make-type-approx type)))
+      (approx-set-top! (new/Cinfo-approx node))))
+
+;*---------------------------------------------------------------------*/
+;*    node-setup! ::isa ...                                            */
+;*---------------------------------------------------------------------*/
+(define-method (node-setup! node::isa)
+   (with-access::isa node (type expr*)
+      (node-setup*! expr*)
+      (widen!::isa/Cinfo node
+	 (approx (make-type-approx type)))
+      (approx-set-top! (isa/Cinfo-approx node))))
+
 ;*---------------------------------------------------------------------*/
 ;*    node-setup! ::cast ...                                           */
 ;*---------------------------------------------------------------------*/
@@ -383,6 +393,7 @@
 		      ;; if the variable is read-only we set it a binding
 		      ;; value to improve the approximations which require
 		      ;; offset (such as make-procedure, procedure-ref)
+		      (node-setup! val)
 		      (widen!::reshaped-local var
 			 (binding-value (if (eq? (local-access var) 'read)
 					    val
@@ -390,7 +401,6 @@
 		      (trace (cfa 3) "Je vais descendre dans val: " (shape val)
 			     " " (find-runtime-type val)
 			     #\Newline)
-		      (node-setup! val)
 		      (trace (cfa 3) "  " (shape var) " ... " #\Newline)))
 		bindings)
       (node-setup! body)))
