@@ -1,41 +1,10 @@
-;*---------------------------------------------------------------------*/
-;*   A pratical implementation for the Scheme programming language     */
-;*                                                                     */
-;*                                    ,--^,                            */
-;*                              _ ___/ /|/                             */
-;*                          ,;'( )__, ) '                              */
-;*                         ;;  //   L__.                               */
-;*                         '   \\   /  '                               */
-;*                              ^   ^                                  */
-;*                                                                     */
-;*   Copyright (c) 1992-1999 Manuel Serrano                            */
-;*                                                                     */
-;*     Bug descriptions, use reports, comments or suggestions are      */
-;*     welcome. Send them to                                           */
-;*       bigloo-request@kaolin.unice.fr                                */
-;*       http://kaolin.unice.fr/bigloo                                 */
-;*                                                                     */
-;*   This program is free software; you can redistribute it            */
-;*   and/or modify it under the terms of the GNU General Public        */
-;*   License as published by the Free Software Foundation; either      */
-;*   version 2 of the License, or (at your option) any later version.  */
-;*                                                                     */
-;*   This program is distributed in the hope that it will be useful,   */
-;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
-;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
-;*   GNU General Public License for more details.                      */
-;*                                                                     */
-;*   You should have received a copy of the GNU General Public         */
-;*   License along with this program; if not, write to the Free        */
-;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
-;*   MA 02111-1307, USA.                                               */
-;*---------------------------------------------------------------------*/
 ;*=====================================================================*/
 ;*    serrano/prgm/project/bigloo/comptime/Cgen/cgen.scm               */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Tue Jul  2 13:17:04 1996                          */
-;*    Last change :  Sun Nov 29 17:15:45 1998 (serrano)                */
+;*    Last change :  Sat Jul  7 09:18:45 2001 (serrano)                */
+;*    Copyright   :  1996-2001 Manuel Serrano, see LICENSE file        */
 ;*    -------------------------------------------------------------    */
 ;*    The C production code.                                           */
 ;*=====================================================================*/
@@ -86,6 +55,7 @@
       [assert (sh) (string? (global-name global))])
    (let* ((sfun (widen!::sfun/C (global-value global)
 		   (label (instantiate::clabel
+			     (loc (sfun-loc (global-value global)))
 			     (name (global-name global))))
 		   (integrated #t)))
 	  (loc  (sfun-loc sfun))
@@ -95,13 +65,15 @@
 			       *return-kont*))))
       (reset-bdb-loc!)
       (newline *c-port*)
+      (newline *c-port*)
       (display "/* " *c-port*)
       (display (shape global) *c-port*)
       (display " */" *c-port*)
-;*       (newline *c-port*)                                            */
-      (emit-bdb-loc (if (location? loc)
-			loc
-			(cop-loc cop)))
+      ;; we have to emit a dummy location otherwise gdb get confused
+      ;; with the function arguments! Thus all functions looks like
+      ;; starting at the module definition site instead of there correct
+      ;; definition site
+      (emit-bdb-loc #f)
       ;; we have to reset the loc because we must not emit location
       ;; before the first c expression otherwise gdb get confused
       (reset-bdb-loc!)
@@ -109,7 +81,7 @@
       (sfun->c sfun global)
       (let ((cop (block-kont (sfun/C-label sfun) loc)))
 	 ;; we have to remove first location otherwise gdb get confused
- 	 (if (>fx *bdb-debug* 0)
+ 	 '(if (>fx *bdb-debug* 0)
 	     (let loop ((cop cop))
 		(cond
 		   ((block? cop)
@@ -127,9 +99,22 @@
 			   (begin
 			      (set! loc #unspecified)
 			      (loop body))))))))
+	 ;; we define a local variable that acts as a temporary variable
+	 (display "{ AN_OBJECT;" *c-port*)
+	 ;; when compiling for debugging, we have to insert a dummy
+	 ;; statement otherwise gdb get confused
+	 (if (and (> *bdb-debug* 0) (location? loc))
+	     (begin
+		(emit-bdb-loc loc)
+		(display "{ obj_t ___ = BUNSPEC; } /* bdb dummy init stmt */"
+			 *c-port*)))
 	 ;; we now may emit the body
-	 (emit-cop cop))
-      (newline *c-port*)
+	 (emit-cop cop)
+	 ;; emit the current location before the closing bracket
+	 (emit-bdb-loc (get-current-bdb-loc))
+	 ;; and then clause the function body
+	 (fprint *c-port* "}"))
+      (no-bdb-newline)
       (leave-function)))
   
 ;*---------------------------------------------------------------------*/
@@ -144,23 +129,26 @@
 ;*---------------------------------------------------------------------*/
 (define (sfun->c sfun variable)
    (with-access::variable variable (type id name)
-      (display (make-typed-declaration type name) *c-port*)
-      (if (null? (sfun-args sfun))
-	  (begin
-	     (display "()" *c-port*)
-	     (emit-newline))
-	  (begin
-	     (display #\( *c-port*)
-	     (let loop ((args (sfun-args sfun)))
-		(if (null? (cdr args))
-		    (with-access::local (car args) (name id type)
-		       (display (make-typed-declaration type name) *c-port*)
-		       (write-char #\) *c-port*)
-		       (emit-newline))
-		    (with-access::local (car args) (name id type)
-		       (display (make-typed-declaration type name) *c-port*)
-		       (display ", " *c-port*)
-		       (loop (cdr args)))))))))
+      (display (make-typed-declaration
+		type
+		(string-append
+		 name
+		 (if (null? (sfun-args sfun))
+		     "()"
+		     (string-append
+		      "("
+		      (let loop ((args (sfun-args sfun)))
+			 (if (null? (cdr args))
+			     (with-access::local (car args) (name id type)
+				(string-append
+				 (make-typed-declaration type name) ")"))
+			     (with-access::local (car args) (name id type)
+				(string-append
+				 (make-typed-declaration type name)
+				 ", "
+				 (loop (cdr args))))))))))
+	       *c-port*))
+   (no-bdb-newline))
 
 ;*---------------------------------------------------------------------*/
 ;*    *return-kont* ...                                                */
@@ -172,6 +160,7 @@
 	 (value (cond
 		   ((csetq? cop)
 		    (instantiate::csequence
+		       (loc    (cop-loc cop))
 		       (c-exp? #t)
 		       (cops (list cop (instantiate::catom
 					  (value #unspecified))))))
@@ -231,9 +220,11 @@
 		   (value (cond
 			     ((csetq? cop)
 			      (instantiate::csequence
+				 (loc (cop-loc cop))
 				 (c-exp? #t)
 				 (cops (list cop
 					       (instantiate::catom
+						  (loc (cop-loc cop))
 						  (value #unspecified))))))
 			     (else
 			      cop)))
@@ -248,7 +239,8 @@
 ;*    node->cop ...                                                    */
 ;*---------------------------------------------------------------------*/
 (define-method (node->cop node::atom kont)
-   (trace (cgen 3) "(node->cop node::atom kont): " (shape node) #\Newline
+   (trace (cgen 3)
+	  "(node->cop node::atom kont): " (shape node) #\Newline
 	  "  kont: " kont #\Newline)
    (with-access::atom node (value loc)
       (kont (instantiate::catom
@@ -259,7 +251,8 @@
 ;*    node->cop ...                                                    */
 ;*---------------------------------------------------------------------*/
 (define-method (node->cop node::kwote kont)
-   (trace (cgen 3) "(node->cop node::kwote kont): " (shape node) #\Newline
+   (trace (cgen 3)
+	  "(node->cop node::kwote kont): " (shape node) #\Newline
 	  "  kont: " kont #\Newline)
    (internal-error "node->cop" "Unexpected `kwote' node" (shape node)))
 
@@ -267,8 +260,12 @@
 ;*    node->cop ...                                                    */
 ;*---------------------------------------------------------------------*/
 (define-method (node->cop node::var kont)
-   (trace (cgen 3) "(node->cop node::var kont): " (shape node) #\Newline
+   (trace (cgen 3)
+	  "(node->cop node::var kont): " (shape node) #\Newline
 	  "  kont: " kont #\Newline)
+   (trace (cgen 4) "(node->cop node::var kont): " (shape node) #\Newline
+	  "  var-variable-name: " (variable-name (var-variable node))
+	  #\Newline)
    (with-access::var node (variable loc)
       (kont (instantiate::varc
 	       (variable variable)
@@ -278,7 +275,8 @@
 ;*    node->cop ::closure ...                                          */
 ;*---------------------------------------------------------------------*/
 (define-method (node->cop node::closure kont)
-   (trace (cgen 3) "(node->cop node::closure kont): " (shape node) #\Newline
+   (trace (cgen 3)
+	  "(node->cop node::closure kont): " (shape node) #\Newline
 	  "  kont: " kont #\Newline)
    (internal-error "node->cop" "Unexpected `closure' node" (shape node)))
 
@@ -286,13 +284,15 @@
 ;*    node->cop ::sequence ...                                         */
 ;*---------------------------------------------------------------------*/
 (define-method (node->cop node::sequence kont)
-   (trace (cgen 3) "(node->cop node::sequence kont): " (shape node) #\Newline
+   (trace (cgen 3)
+	  "(node->cop node::sequence kont): " (shape node) #\Newline
 	  "  kont: " kont #\Newline)
-   (with-access::sequence node (nodes)
+   (with-access::sequence node (nodes loc)
       (let ((exp nodes))
 	 (cond
 	    ((null? exp)
-	     (kont (instantiate::nop)))
+	     (kont (instantiate::nop
+		      (loc loc))))
 	    ((null? (cdr exp))
 	     (let ((cop (node->cop (car exp) kont)))
 		(instantiate::stop (value cop))))
@@ -300,85 +300,130 @@
 	     (let loop ((exp exp)
 			(new '()))
 		(if (null? (cdr exp))
-		    (instantiate::csequence
-		       (cops (reverse! (cons (node->cop (car exp) kont) new))))
+		    (let ((cop (node->cop (car exp) kont)))
+		       (instantiate::csequence
+			  (loc  (cop-loc cop))
+			  (cops (reverse! (cons cop new)))))
 		    (begin
 		       (if (not (side-effect? (car exp)))
 			   (loop (cdr exp) new)
 			   (loop (cdr exp)
 				 (cons (node->cop (car exp) *stop-kont*)
 				       new)))))))))))
+
+;*---------------------------------------------------------------------*/
+;*    extern->cop ...                                                  */
+;*---------------------------------------------------------------------*/
+(define (extern->cop format::bstring node::extern kont)
+   (trace (cgen 3)
+	  "(extern->cop node::extern kont): " (shape node) #\Newline
+	  "  kont: " kont #\Newline)
+   (with-access::extern node (expr* loc)
+      (node-args->cop expr*
+		      loc
+		      (lambda (new-args)
+			 (kont (instantiate::cpragma
+				  (loc loc)
+				  (format format)
+				  (args new-args)))))))
    
 ;*---------------------------------------------------------------------*/
 ;*    node->cop ::pragma ...                                           */
 ;*---------------------------------------------------------------------*/
 (define-method (node->cop node::pragma kont)
-   (trace (cgen 3) "(node->cop node::pragma kont): " (shape node) #\Newline
+   (trace (cgen 3)
+	  "(node->cop node::pragma kont): " (shape node) #\Newline
 	  "  kont: " kont #\Newline)
-   (with-access::pragma node (format args loc)
-      (let loop ((old-actuals  args)
-		 (new-actuals  '())
-		 (aux          (make-local-svar/name 'aux *obj*))
-		 (auxs         '())
-		 (exps         '()))
-	 (if (null? old-actuals)
-	     (if (null? auxs)
-		 (kont (instantiate::cpragma
-			  (loc    loc)
-			  (format format)
-			  (args (reverse! new-actuals))))
-		 (instantiate::block
-		    (body (instantiate::csequence
-			     (cops
-			      (list
-			       (instantiate::local-var
-				  (vars auxs)
-				  (loc  loc))
-			       (instantiate::csequence (cops exps))
-			       (kont (instantiate::cpragma
-					(loc    loc)
-					(format format)
-					(args (reverse! new-actuals))))))))))
-	     (let ((cop (node->cop (node-setq aux (car old-actuals))
-				   *id-kont*)))
-		(if (and (csetq? cop)
-			 (eq? (varc-variable (csetq-var cop)) aux)
-			 (or (catom? (csetq-value cop))
-			     (varc? (csetq-value cop))
-			     (cpragma? (csetq-value cop))))
-		    (loop (cdr old-actuals)
-			  (cons (csetq-value cop) new-actuals)
-			  aux
-			  auxs
-			  exps)
-		    (begin
-		       (local-type-set! aux (typeof (car old-actuals)))
-		       (loop (cdr old-actuals)
-			     (cons (instantiate::varc
-				      (variable aux)
-				      (loc      loc))
-				   new-actuals)
-			     (make-local-svar/name 'aux *obj*)
-			     (cons aux auxs)
-			     (cons cop exps)))))))))
+   (with-access::pragma node (format)
+      (extern->cop format node kont)))
 
+;*---------------------------------------------------------------------*/
+;*    node->cop ::getfield ...                                         */
+;*---------------------------------------------------------------------*/
+(define-method (node->cop node::getfield kont)
+   (trace (cgen 3)
+	  "(node->cop node::getfield kont): " (shape node) #\Newline
+	  "  kont: " kont #\Newline)
+   (with-access::getfield node (fname otype)
+      (let ((fmt (string-append "(((" (type-name otype) ")CREF($1))->"
+				fname ")")))
+	 (extern->cop fmt node kont))))
+   
+;*---------------------------------------------------------------------*/
+;*    node->cop ::setfield ...                                         */
+;*---------------------------------------------------------------------*/
+(define-method (node->cop node::setfield kont)
+   (trace (cgen 3)
+	  "(node->cop node::setfield kont): " (shape node) #\Newline
+	  "  kont: " kont #\Newline)
+   (with-access::setfield node (fname ftype otype)
+      (let ((fmt (string-append "((((" (type-name otype) ")CREF($1))->"
+				fname ") = ((" (type-name ftype)
+				")$2), BUNSPEC)")))
+	 (extern->cop fmt node kont))))
+
+;*---------------------------------------------------------------------*/
+;*    node->cop ::vlength ...                                          */
+;*---------------------------------------------------------------------*/
+(define-method (node->cop node::vlength kont)
+   (with-access::vlength node (c-format)
+      (trace (cgen 3)
+	     "(node->cop node::vlength kont): " (shape node) #\Newline
+	     "  kont: " kont " " c-format #\Newline)
+      (extern->cop c-format node kont)))
+   
+;*---------------------------------------------------------------------*/
+;*    node->cop ::vref ...                                             */
+;*---------------------------------------------------------------------*/
+(define-method (node->cop node::vref kont)
+   (with-access::vref node (c-format)
+      (trace (cgen 3)
+	     "(node->cop node::vref kont): " (shape node) #\Newline
+	     "  kont: " kont " " c-format #\Newline)
+      (extern->cop c-format node kont)))
+   
+;*---------------------------------------------------------------------*/
+;*    node->cop ::vset! ...                                            */
+;*---------------------------------------------------------------------*/
+(define-method (node->cop node::vset! kont)
+   (with-access::vset! node (c-format)
+      (trace (cgen 3)
+	     "(node->cop node::vset! kont): " (shape node) #\Newline
+	     "  kont: " kont " " c-format #\Newline)
+      (extern->cop c-format node kont)))
+   
+;*---------------------------------------------------------------------*/
+;*    node->cop ::valloc ...                                           */
+;*---------------------------------------------------------------------*/
+(define-method (node->cop node::valloc kont)
+   (with-access::valloc node (c-heap-format)
+      (trace (cgen 3)
+	     "(node->cop node::vallocate kont): " (shape node) #\Newline
+	     "  kont: " kont " " c-heap-format " " #\Newline) 
+      (extern->cop c-heap-format node kont)))
+   
 ;*---------------------------------------------------------------------*/
 ;*    node->cop ::cast ...                                             */
 ;*---------------------------------------------------------------------*/
 (define-method (node->cop node::cast kont)
-   (trace (cgen 3) "(node->cop node::cast kont): " (shape node) #\Newline
+   (trace (cgen 3)
+	  "(node->cop node::cast kont): " (shape node) #\Newline
 	  "  kont: " kont #\Newline)
    (with-access::cast node (arg type loc)
-      (kont (instantiate::ccast
-	       (type type)
-	       (loc loc)
-	       (arg (node->cop arg *id-kont*))))))
+      (node-args->cop (list arg)
+		      loc
+		      (lambda (new-args)
+			 (kont (instantiate::ccast
+				  (type type)
+				  (loc loc)
+				  (arg (car new-args))))))))
 
 ;*---------------------------------------------------------------------*/
 ;*    node->cop ::setq ...                                             */
 ;*---------------------------------------------------------------------*/
 (define-method (node->cop node::setq kont)
-   (trace (cgen 3) "(node->cop node::setq kont): " (shape node) #\Newline
+   (trace (cgen 3)
+	  "(node->cop node::setq kont): " (shape node) #\Newline
 	  "  kont: " kont #\Newline)
    (with-access::setq node (value loc)
       (let ((var (var-variable (setq-var node))))
@@ -409,6 +454,7 @@
 		(loc loc)
 		(body
 		 (instantiate::csequence
+		    (loc loc)
 		    (cops
 		     (list
 		      (instantiate::local-var
@@ -416,7 +462,9 @@
 			 (loc  loc))
 		      ctest
 		      (instantiate::cif
-			 (test (instantiate::varc (variable aux)))
+			 (test (instantiate::varc
+				  (variable aux)
+				  (loc loc)))
 			 (false (block-kont (node->cop false kont) loc))
 			 (true (block-kont (node->cop true kont) loc))
 			 (loc  loc)))))))))))
@@ -425,61 +473,26 @@
 ;*    node->cop ::fail ...                                             */
 ;*---------------------------------------------------------------------*/
 (define-method (node->cop node::fail kont)
-   (trace (cgen 3) "(node->cop node::fail kont): " (shape node) #\Newline
+   (trace (cgen 3)
+	  "(node->cop node::fail kont): " (shape node) #\Newline
 	  "  kont: " kont #\Newline)
    (with-access::fail node (proc msg obj loc)
-      (let loop ((old-actuals  (list proc msg obj))
-		 (new-actuals  '())
-		 (aux          (make-local-svar/name 'aux *obj*))
-		 (auxs         '())
-		 (exps         '()))
-	 (if (null? old-actuals)
-	     (if (null? auxs)
-		 (*fail-kont* (instantiate::cfail
-				 (loc  loc)
-				 (proc (caddr new-actuals))
-				 (msg  (cadr new-actuals))
-				 (obj  (car new-actuals))))
-		 (instantiate::block
-		    (loc  loc)
-		    (body (instantiate::csequence
-			     (cops (list (instantiate::local-var
-					    (loc  loc)
-					    (vars auxs))
-					 (instantiate::csequence
-					    (cops exps))
-					 (*fail-kont*
-					  (instantiate::cfail
-					     (loc  loc)
-					     (proc (caddr new-actuals))
-					     (msg (cadr new-actuals))
-					     (obj (car new-actuals))))))))))
-	     (let ((cop (node->cop (node-setq aux (car old-actuals))
-				   *id-kont*)))
-		(if (and (csetq? cop)
-			 (eq? (varc-variable (csetq-var cop)) aux))
-		    ;; the local is useless, we ignore it
-		    (loop (cdr old-actuals)
-			  (cons (csetq-value cop) new-actuals)
-			  aux
-			  auxs
-			  exps)
-		    (begin
-		       (local-type-set! aux *obj*)
-		       (loop (cdr old-actuals)
-			     (cons (instantiate::varc
-				      (variable aux)
-				      (loc loc))
-				   new-actuals)
-			     (make-local-svar/name 'aux *obj*)
-			     (cons aux auxs)
-			     (cons cop exps)))))))))
+      (node-args->cop (list proc msg obj)
+		      loc
+		      (lambda (new-args)
+			 (*fail-kont*
+			  (instantiate::cfail
+			     (loc loc)
+			     (proc (car new-args))
+			     (msg (cadr new-args))
+			     (obj (caddr new-args))))))))
 
 ;*---------------------------------------------------------------------*/
 ;*    node->cop ::select ...                                           */
 ;*---------------------------------------------------------------------*/
 (define-method (node->cop node::select kont)
-   (trace (cgen 3) "(node->cop node::select kont): " (shape node) #\Newline
+   (trace (cgen 3)
+	  "(node->cop node::select kont): " (shape node) #\Newline
 	  "  kont: " kont #\Newline)
    (with-access::select node (clauses test item-type loc)
       (for-each (lambda (clause)
@@ -494,7 +507,9 @@
 		   (test (csetq-value cop))
 		   (clauses clauses))
 		(instantiate::block
+		   (loc loc)
 		   (body (instantiate::csequence
+			    (loc loc)
 			    (cops (list (instantiate::local-var
 					   (loc  loc)
 					   (vars (list aux)))
@@ -510,7 +525,8 @@
 ;*    node->cop ::let-fun ...                                          */
 ;*---------------------------------------------------------------------*/
 (define-method (node->cop node::let-fun kont)
-   (trace (cgen 3) "(node->cop node::let-fun kont): " (shape node) #\Newline
+   (trace (cgen 3)
+	  "(node->cop node::let-fun kont): " (shape node) #\Newline
 	  "  kont: " kont #\Newline)
    (with-access::let-fun node (body locals loc)
       ;; local function are open-coded of their first call site.
@@ -524,6 +540,7 @@
 	     (block-kont
 	      (bdb-let-var
 	       (instantiate::csequence
+		  (loc loc)
 		  (cops (list (instantiate::local-var
 				 (loc  loc)
 				 (vars all-formals))
@@ -545,7 +562,8 @@
 ;*    node->cop ::let-var ...                                          */
 ;*---------------------------------------------------------------------*/
 (define-method (node->cop node::let-var kont)
-   (trace (cgen 3) "(node->cop node::let-var kont): " (shape node) #\Newline
+   (trace (cgen 3)
+	  "(node->cop node::let-var kont): " (shape node) #\Newline
 	  "  kont: " kont #\Newline)
    (with-access::let-var node (body bindings loc)
       (let ((decls (instantiate::local-var
@@ -561,6 +579,7 @@
 	 (block-kont
 	  (bdb-let-var
 	   (instantiate::csequence
+	      (loc loc)
 	      (cops (cons decls (append sets (list body)))))
 	   loc)
 	  loc))))
@@ -579,12 +598,14 @@
 ;*    node->cop ::set-ex-it ...                                        */
 ;*---------------------------------------------------------------------*/
 (define-method (node->cop node::set-ex-it kont)
-   (trace (cgen 3) "(node->cop node::set-ex-it kont): " (shape node) #\Newline
+   (trace (cgen 3)
+	  "(node->cop node::set-ex-it kont): " (shape node) #\Newline
 	  "  kont: " kont #\Newline)
    (with-access::set-ex-it node (var body loc)
       (let ((exit (var-variable var)))
 	 (set-variable-name! exit)
 	 (instantiate::csequence
+	    (loc loc)
 	    (cops (list
 		   (instantiate::cpragma
 		      (loc    loc)
@@ -595,14 +616,17 @@
 		      (vars (list (var-variable var))))
 		   (instantiate::cset-ex-it
 		      (loc loc)
-		      (exit (instantiate::varc (variable exit)))
+		      (exit (instantiate::varc
+			       (loc loc)
+			       (variable exit)))
 		      (jump-value (node->cop (instantiate::pragma
 						(loc loc)
 						(type *_*)
 						(format "_exit_value_")
-						(args '()))
+						(expr* '()))
 					     kont))
 		      (body (instantiate::csequence
+			       (loc loc)
 			       (cops
 				(list
 				 (node->cop
@@ -617,7 +641,7 @@
 					(string-sans-$
 					 (type-name (local-type exit)))
 					")jmpbuf"))
-				      (args '())))
+				      (expr* '())))
 				  *id-kont*)
 				 (node->cop body kont))))))))))))
 
@@ -625,7 +649,8 @@
 ;*    node->cop ::jump-ex-it ...                                       */
 ;*---------------------------------------------------------------------*/
 (define-method (node->cop node::jump-ex-it kont)
-   (trace (cgen 3) "(node->cop node::jump-ex-it kont): " (shape node) #\Newline
+   (trace (cgen 3)
+	  "(node->cop node::jump-ex-it kont): " (shape node) #\Newline
 	  "  kont: " kont #\Newline)
    (with-access::jump-ex-it node (exit value loc)
       (let* ((vaux  (make-local-svar/name 'aux *obj*))
@@ -645,10 +670,12 @@
 	     (instantiate::block
 		(loc loc)
 		(body (instantiate::csequence
+			 (loc loc)
 			 (cops (list (instantiate::local-var
 					(loc loc)
 					(vars (list eaux)))
 				       (instantiate::csequence
+					  (loc loc)
 					  (cops (list ecop)))
 				       (*exit-kont*
 					(instantiate::cjump-ex-it
@@ -661,21 +688,25 @@
 	     (instantiate::block
 		(loc loc)
 		(body (instantiate::csequence
+			 (loc loc)
 			 (cops (list (instantiate::local-var
 					(loc loc)
 					(vars (list vaux)))
 				     (instantiate::csequence
+					(loc loc)
 					(cops (list vcop)))
 				     (*exit-kont*
 				      (instantiate::cjump-ex-it
 					 (loc loc)
 					 (exit (csetq-value ecop))
 					 (value (instantiate::varc
+						   (loc loc)
 						   (variable vaux)))))))))))
 	    (else
 	     (instantiate::block
 		(loc loc)
 		(body (instantiate::csequence
+			 (loc loc)
 			 (cops
 			  (list
 			   (instantiate::local-var
@@ -696,7 +727,8 @@
 ;*    node->cop ::make-box ...                                         */
 ;*---------------------------------------------------------------------*/
 (define-method (node->cop node::make-box kont)
-   (trace (cgen 3) "(node->cop node::make-box kont): " (shape node) #\Newline
+   (trace (cgen 3)
+	  "(node->cop node::make-box kont): " (shape node) #\Newline
 	  "  kont: " kont #\Newline)
    (with-access::make-box node (value loc)
       (if (or (var? value) (atom? value) (kwote? value))
@@ -709,6 +741,7 @@
 	     (instantiate::block
 		(loc loc)
 		(body (instantiate::csequence
+			 (loc loc)
 			 (cops (list
 				(instantiate::local-var
 				   (loc loc)
@@ -725,7 +758,8 @@
 ;*    node->cop ::box-ref ...                                          */
 ;*---------------------------------------------------------------------*/
 (define-method (node->cop node::box-ref kont)
-   (trace (cgen 3) "(node->cop node::box-ref kont): " (shape node) #\Newline
+   (trace (cgen 3)
+	  "(node->cop node::box-ref kont): " (shape node) #\Newline
 	  "  kont: " kont #\Newline)
    (with-access::box-ref node (var loc)
       (kont (node->cop var (lambda (v) (instantiate::cbox-ref
@@ -736,7 +770,8 @@
 ;*    node->cop ::box-set! ...                                         */
 ;*---------------------------------------------------------------------*/
 (define-method (node->cop node::box-set! kont)
-   (trace (cgen 3) "(node->cop node::box-set! kont): " (shape node) #\Newline
+   (trace (cgen 3)
+	  "(node->cop node::box-set! kont): " (shape node) #\Newline
 	  "  kont: " kont #\Newline)
    (with-access::box-set! node (var value loc)
       (let ((v (var-variable var)))
@@ -769,3 +804,57 @@
       (if (not (string? (local-name local)))
 	  (error "make-local-svar/name" "Illegal local name" local))
       local))
+
+;*---------------------------------------------------------------------*/
+;*    no-bdb-newline ...                                               */
+;*    -------------------------------------------------------------    */
+;*    Emit a newline only if not compiling for bdb.                    */
+;*---------------------------------------------------------------------*/
+(define (no-bdb-newline)
+   (if (eq? *bdb-debug* 0)
+       (newline *c-port*)))
+
+;*---------------------------------------------------------------------*/
+;*    node-args->cop ...                                               */
+;*---------------------------------------------------------------------*/
+(define (node-args->cop args loc kont)
+   (let loop ((old-actuals  args)
+	      (new-actuals  '())
+	      (aux          (make-local-svar/name 'aux *obj*))
+	      (auxs         '())
+	      (exps         '()))
+      (if (null? old-actuals)
+	  (if (null? auxs)
+	      (kont (reverse! new-actuals))
+	      (instantiate::block
+		 (body (instantiate::csequence
+			  (loc loc)
+			  (cops (list
+				 (instantiate::local-var
+				    (vars auxs)
+				    (loc  loc))
+				 (instantiate::csequence (cops exps))
+				 (kont (reverse! new-actuals))))))))
+	  (let ((cop (node->cop (node-setq aux (car old-actuals)) *id-kont*)))
+	     (if (and (csetq? cop)
+		      (eq? (varc-variable (csetq-var cop)) aux)
+		      (or (catom? (csetq-value cop))
+			  (varc? (csetq-value cop))
+			  (cpragma? (csetq-value cop))))
+		 (loop (cdr old-actuals)
+		       (cons (csetq-value cop) new-actuals)
+		       aux
+		       auxs
+		       exps)
+		 (begin
+		    (local-type-set! aux (typeof (car old-actuals)))
+		    (loop (cdr old-actuals)
+			  (cons (instantiate::varc
+				   (variable aux)
+				   (loc      loc))
+				new-actuals)
+			  (make-local-svar/name 'aux *obj*)
+			  (cons aux auxs)
+			  (cons cop exps))))))))
+   
+   
