@@ -1,41 +1,10 @@
-;*---------------------------------------------------------------------*/
-;*   A pratical implementation for the Scheme programming language     */
-;*                                                                     */
-;*                                    ,--^,                            */
-;*                              _ ___/ /|/                             */
-;*                          ,;'( )__, ) '                              */
-;*                         ;;  //   L__.                               */
-;*                         '   \\   /  '                               */
-;*                              ^   ^                                  */
-;*                                                                     */
-;*   Copyright (c) 1992-1999 Manuel Serrano                            */
-;*                                                                     */
-;*     Bug descriptions, use reports, comments or suggestions are      */
-;*     welcome. Send them to                                           */
-;*       bigloo-request@kaolin.unice.fr                                */
-;*       http://kaolin.unice.fr/bigloo                                 */
-;*                                                                     */
-;*   This program is free software; you can redistribute it            */
-;*   and/or modify it under the terms of the GNU General Public        */
-;*   License as published by the Free Software Foundation; either      */
-;*   version 2 of the License, or (at your option) any later version.  */
-;*                                                                     */
-;*   This program is distributed in the hope that it will be useful,   */
-;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
-;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
-;*   GNU General Public License for more details.                      */
-;*                                                                     */
-;*   You should have received a copy of the GNU General Public         */
-;*   License along with this program; if not, write to the Free        */
-;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
-;*   MA 02111-1307, USA.                                               */
-;*---------------------------------------------------------------------*/
 ;*=====================================================================*/
-;*    serrano/prgm/project/bigloo/comptime/Cgen/emit-cop.scm           */
+;*    serrano/prgm/project/bigloo2.3/comptime/Cgen/emit-cop.scm        */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Tue Jul  2 14:39:37 1996                          */
-;*    Last change :  Mon Sep 21 13:52:26 1998 (serrano)                */
+;*    Last change :  Thu Sep 14 08:07:14 2000 (serrano)                */
+;*    Copyright   :  1996-2000 Manuel Serrano, see LICENSE file        */
 ;*    -------------------------------------------------------------    */
 ;*    The emission of cop code.                                        */
 ;*=====================================================================*/
@@ -44,7 +13,9 @@
 ;*    The module                                                       */
 ;*---------------------------------------------------------------------*/
 (module cgen_emit-cop
-   (include "Tools/location.sch")
+   (include "Tools/location.sch"
+	    "Tools/fprint.sch"
+	    "Tools/trace.sch")
    (import  type_type
 	    type_tools
 	    type_cache
@@ -53,18 +24,22 @@
 	    ast_node
 	    ast_env
 	    cgen_emit
-	    cgen_cop)
+	    cgen_cop
+	    (*module-location* module_module))
    (export  (generic emit-cop::bool ::cop)
 	    (reset-bdb-loc!)
 	    (emit-bdb-loc ::obj)
-	    (emit-newline . ::obj)
-	    (untrigraph::bstring ::bstring)))
+	    (untrigraph::bstring ::bstring)
+	    (get-current-bdb-loc)))
 
 ;*---------------------------------------------------------------------*/
 ;*    emit-cop ...                                                     */
 ;*    -------------------------------------------------------------    */
 ;*    If emit-cop emit an expression with a `;' it returns #f,         */
 ;*    otherwise it returns #t.                                         */
+;*    -------------------------------------------------------------    */
+;*    The general idea of that printer is that no specific printer     */
+;*    ever emit \n because they are emitted by the location printer.   */
 ;*---------------------------------------------------------------------*/
 (define-generic (emit-cop::bool cop::cop))
  
@@ -77,8 +52,7 @@
 	  (begin
 	     (emit-bdb-loc loc)
 	     (display name *c-port*)
-	     (write-char #\: *c-port*)
-	     (emit-newline)))
+	     (write-char #\: *c-port*)))
       (emit-cop body)))
 
 ;*---------------------------------------------------------------------*/
@@ -88,7 +62,6 @@
    (with-access::cgoto cop (label loc)
       (emit-bdb-loc loc)
       (fprin *c-port* "goto " (clabel-name label) #\;)
-      (emit-newline)
       #f))
 
 ;*---------------------------------------------------------------------*/
@@ -96,18 +69,20 @@
 ;*---------------------------------------------------------------------*/
 (define-method (emit-cop cop::block)
    (with-access::block cop (body loc)
+      (emit-bdb-loc loc)
       (if (block? body)
 	  (emit-cop body)
 	  (begin
-	     (emit-bdb-loc loc)
-	     (write-char #\{ *c-port*)
-	     (emit-newline)
+	     (display #"{ " *c-port*)
+	     (trace cgen (display "/* cop-block */" *c-port*))
+	     (emit-bdb-loc-comment loc)
 	     (if (emit-cop body)
 		 (begin
-		    (write-char #\; *c-port*)
-		    (emit-newline)))
-	     (write-char #\} *c-port*)
-	     (emit-newline)
+		    (emit-bdb-loc (get-current-bdb-loc))
+		    (display "; " *c-port*)
+		    (trace cgen (display "/* cop-block */" *c-port*))))
+	     (display "} " *c-port*)
+	     (trace cgen (display "/* cop-block */" *c-port*))
 	     #f))))
 
 ;*---------------------------------------------------------------------*/
@@ -118,9 +93,7 @@
       (emit-bdb-loc loc)
       (display "return " *c-port*)
       (if (emit-cop value)
-	  (begin
-	     (write-char #\; *c-port*)
-	     (emit-newline)))
+	  (write-char #\; *c-port*))
       #f))
 
 ;*---------------------------------------------------------------------*/
@@ -195,22 +168,25 @@
 	     (if (null? cops)
 		 (emit-atom-value #unspecified)
 		 (begin
-		    (write-char #\( *c-port*)
+		    (display "( " *c-port*)
+		    (trace cgen (display "/* cop-csequence */" *c-port*))
 		    (let liip ((exp cops))
 		       (if (null? (cdr exp))
 			   (begin
 			      (emit-cop (car exp))
-			      (write-char #\) *c-port*)
+			      (display ") " *c-port*)
+			      (trace cgen "/* cop-csequence */" *c-port*)
 			      #t)
 			   (begin
 			      (emit-cop (car exp))
 			      (if (cfail? (car exp))
 				  (begin
-				     (write-char #\) *c-port*)
+				     (display ") " *c-port*)
+				     (trace cgen  "/* cop-csequence */" *c-port*)
 				     #t)
 				  (begin
-				     (write-char #\, *c-port*)
-				     (emit-newline)
+				     (display ", " *c-port*)
+				     (trace cgen "/* cop-csequence */" *c-port*)
 				     (liip (cdr exp))))))))))
 	  (let liip ((exp cops))
 	     (if (null? exp)
@@ -218,8 +194,9 @@
 		 (let ((e (car exp)))
 		    (if (emit-cop e)
 			(begin
-			   (write-char #\; *c-port*)
-			   (emit-newline)))
+			   (display "; " *c-port*)
+			   (trace cgen
+				  (display "/* cop-csequence */" *c-port*))))
 		    (if (cfail? e)
 			(liip '())
 			(liip (cdr exp)))))))))
@@ -228,9 +205,10 @@
 ;*    emit-cop ::nop ...                                               */
 ;*---------------------------------------------------------------------*/
 (define-method (emit-cop cop::nop)
-   (write-char #\; *c-port*)
-   (emit-newline)
-   #f)
+   (with-access::nop cop (loc)
+      (display "; " *c-port*)
+      (trace cgen (display "/* cop-nop */" *c-port*))
+      #f))
 
 ;*---------------------------------------------------------------------*/
 ;*    emit-cop ::stop ...                                              */
@@ -238,9 +216,11 @@
 (define-method (emit-cop cop::stop)
    (with-access::stop cop (value loc)
       (if (emit-cop value)
+	  ;; we don't have to emit a location here because the
+	  ;; location is held by the printed value
 	  (begin
-	     (write-char #\; *c-port*)
-	     (emit-newline)))
+	     (display "; " *c-port*)
+	     (trace cgen (display "/* cop-stop */" *c-port*))))
       #f))
 
 ;*---------------------------------------------------------------------*/
@@ -289,8 +269,7 @@
 					      "")
 					     ")BUNSPEC)")
 			      "")
-			  #\;)
-		   (emit-newline))
+			  #\;))
 		vars)
       #f))
 
@@ -300,11 +279,10 @@
 (define-method (emit-cop cop::bdb-block)
    (with-access::bdb-block cop (loc body)
       (emit-bdb-loc loc)
-      (fprin *c-port* "int bigloo_dummy_bdb; bigloo_dummy_bdb = 0; {")
-      (emit-bdb-loc loc)
+      (fprin *c-port* "int bigloo_dummy_bdb; bigloo_dummy_bdb = 0; { ")
       (emit-cop body)
-      (write-char #\} *c-port*)
-      (emit-newline)))
+      (display "} " *c-port*)
+      (trace cgen (display " /* cop-bdb-block */" *c-port*))))
    
 ;*---------------------------------------------------------------------*/
 ;*    emit-cop ::cfuncall ...                                          */
@@ -462,7 +440,7 @@
    (with-access::cfail cop (proc msg obj loc)
       (emit-bdb-loc loc)
       (if (not *bfalse*)
-	  (set! *bfalse* (find-global 'bfalse 'foreign)))
+	  (set! *bfalse* (find-global/module 'bfalse 'foreign)))
       (cond
 	 ((and (varc? proc) (eq? (varc-variable proc) *bfalse*)
 	       (varc? msg) (eq? (varc-variable msg) *bfalse*)
@@ -492,38 +470,37 @@
 (define-method (emit-cop cop::cswitch)
    (with-access::cswitch cop (test clauses loc)
       (emit-bdb-loc loc)
-      (display "switch " *c-port*)
-      (write-char #\( *c-port*)
+      (display "switch( " *c-port*)
       (emit-cop test)
-      (write-char #\) *c-port*)
-      (write-char #\{ *c-port*)
-      (emit-newline)
+      (display ") { " *c-port*)
+      (trace cgen (display "/* cop-cswitch */" *c-port*))
       (let loop ((clauses clauses))
 	 (let ((clause (car clauses)))
 	    (if (eq? (car clause) 'else)
-		(begin
+		(let ((loc (cop-loc (cdr clause))))
+		   (emit-bdb-loc loc)
 		   (display "default: " *c-port*)
-		   (emit-newline)
 		   (if (emit-cop (cdr clause))
 		       (begin
-			  (write-char #\; *c-port*)
-			  (emit-newline)))
-		   (write-char #\} *c-port*)
-		   (emit-newline)
+			  (display "; " *c-port*)
+			  (trace cgen
+				 (display "/* cswitch default */" *c-port*))))
+		   (display "} " *c-port*)
+		   (trace cgen (display "/* cswitch */" *c-port*))
 		   #f)
 		(begin
 		   (for-each (lambda (t)
 				(display "case " *c-port*)
 				(emit-atom-value t)
 				(display " : " *c-port*)
-				(emit-newline))
+				(newline *c-port*))
 			     (car clause))
 		   (if (emit-cop (cdr clause))
 		       (begin
-			  (write-char #\; *c-port*)
-			  (emit-newline)))
+			  (display "; " *c-port*)
+			  (trace cgen
+				 (display "/* cswitch clause */" *c-port*))))
 		   (display "break;" *c-port*)
-		   (emit-newline)
 		   (loop (cdr clauses))))))))
 
 ;*---------------------------------------------------------------------*/
@@ -569,15 +546,16 @@
       (emit-bdb-loc loc)
       (display "if( SET_EXIT(" *c-port*)
       (emit-cop exit)
-      (display ") ) {" *c-port*)
+      (display ") ) { " *c-port*)
+      (trace cgen (display "/* cop-cset-ex-it */" *c-port*))
       (display "RESTORE_TRACE(); " *c-port*)
       (emit-cop jump-value)
       (emit-bdb-loc loc)
       (display "} else {" *c-port*)
       (emit-cop body)
       (emit-bdb-loc loc)
-      (write-char #\} *c-port*)
-      (emit-newline)
+      (display "} " *c-port*)
+      (trace cgen (display "/* cop-cset-ex-it */" *c-port*))
       #f))
 
 ;*---------------------------------------------------------------------*/
@@ -597,6 +575,14 @@
 ;*    emit-atom-value ...                                              */
 ;*---------------------------------------------------------------------*/
 (define (emit-atom-value value)
+   (define (emit-llong value macro)
+      (let ((disp (open-output-string)))
+	 (display value disp)
+	 (display macro *c-port*)
+	 (display "( " *c-port*)
+	 (let ((str (close-output-port disp)))
+	    (display (substring str 2 (string-length str)) *c-port*)
+	    (display ")" *c-port*))))
    (cond
       ((boolean? value)
        (display "((" *c-port*)
@@ -656,23 +642,27 @@
        (display ")" *c-port*)
        (display value *c-port*)
        (display ")" *c-port*))
+      ((elong? value)
+       (emit-llong value "LONG_TO_BELONG"))
+      ((llong? value)
+       (emit-llong value "LLONG_TO_BLLONG"))
       (else
        (display value *c-port*))))
 
 ;*---------------------------------------------------------------------*/
 ;*    untrigraph ...                                                   */
 ;*    -------------------------------------------------------------    */
-;*    We remove ?? and replace it by \077\077\--- (the octal ascii     */
+;*    We remove ?? and replace it by \077\077 (the octal ascii         */
 ;*    code of ?) in order to avoir C trigraph confusions.              */
 ;*---------------------------------------------------------------------*/
 (define (untrigraph from)
    (let* ((len   (string-length from))
-	  (len-3 (-fx len 3)))
-      ;; first we count how many collision we have
+	  (len-2 (-fx len 2)))
+      ;; first we count how many collisions we have
       (let ((nb-col (let loop ((i      0)
 			       (nb-col 0))
 		       (cond
-			  ((>fx i len-3)
+			  ((>fx i len-2)
 			   nb-col)
 			  ((not (char=? (string-ref from i) #\?))
 			   (loop (+fx i 1) nb-col))
@@ -680,13 +670,13 @@
 			   (loop (+fx i 2) nb-col))
 			  ;; yes, we have one
 			  (else
-			   (loop (+fx i 3) (+fx nb-col 1)))))))
+			   (loop (+fx i 2) (+fx nb-col 1)))))))
 	 (if (=fx nb-col 0)
 	     ;; there is no trigraph clashes
 	     from
 	     ;; there is some, we allocate a new string. Each trigraph
 	     ;; require 4 times its size.
-	     (let ((res   (make-string (+fx len (*fx 3 (*fx nb-col 3)))))
+	     (let ((res   (make-string (+fx len (*fx 3 (*fx nb-col 2)))))
 		   (len-1 (-fx len 1)))
 		(let loop ((r 0)
 			   (w 0))
@@ -694,7 +684,7 @@
 		      ((=fx r len)
 		       res)
 		      ((or (not (char=? (string-ref from r) #\?))
-			   (>fx r len-3))
+			   (>fx r len-2))
 		       (string-set! res w (string-ref from r))
 		       (loop (+fx r 1) (+fx w 1)))
 		      ((not (char=? (string-ref from (+fx r 1)) #\?))
@@ -711,25 +701,7 @@
 		       (string-set! res (+fx w 5) #\0)
 		       (string-set! res (+fx w 6) #\7)
 		       (string-set! res (+fx w 7) #\7)
-		       (string-set! res (+fx w 8) #\\)
-		       (let ((code (integer->string
-				    (char->integer (string-ref from (+fx r 2)))
-				    8)))
-			  (if (=fx (string-length code) 4)
-			      (begin
-				 (string-set! res (+fx w 9) #\0)
-				 (string-set! res (+fx w 10)
-					      (string-ref code 2))
-				 (string-set! res (+fx w 11)
-					      (string-ref code 3)))
-			      (begin
-				 (string-set! res (+fx w 9)
-					      (string-ref code 2))
-				 (string-set! res (+fx w 10)
-					      (string-ref code 3))
-				 (string-set! res (+fx w 11)
-					      (string-ref code 4)))))
-		       (loop (+fx r 3) (+fx w 12))))))))))
+		       (loop (+fx r 2) (+fx w 8))))))))))
 
 ;*---------------------------------------------------------------------*/
 ;*    *bdb-loc* ...                                                    */
@@ -744,6 +716,12 @@
 (define (reset-bdb-loc!)
    (set! *bdb-loc* #unspecified))
 
+;*---------------------------------------------------------------------*/
+;*    get-current-bdb-loc ...                                          */
+;*---------------------------------------------------------------------*/
+(define (get-current-bdb-loc)
+   *bdb-loc*)
+
 ;*---------------------------------------------------------------------*/
 ;*    emit-bdb-loc ...                                                 */
 ;*    -------------------------------------------------------------    */
@@ -754,43 +732,37 @@
 ;*---------------------------------------------------------------------*/
 (define (emit-bdb-loc cur-loc)
    (cond
-      ((<=fx *bdb-debug* 0)
-       #unspecified)
-      ((not (location? cur-loc))
-       (if (location? *bdb-loc*)
-	   (emit-bdb-loc *bdb-loc*)))
-      (else
-       (let ((cur-fname  (location-fname cur-loc))
-	     (cur-line   (location-lnum cur-loc))
-	     (prev-loc   *bdb-loc*))
-	  (if (or (not (location? prev-loc))
-		  (let ((prev-fname (location-fname prev-loc)))
-		     (or (eq? cur-fname prev-fname)
-			 (string=? cur-fname prev-fname))))
-	      ;; we have to emit a # line information only if the code
-	      ;; we are emitting belongs to the current function (that is,
-	      ;; we do not have to emit line information for inlined
-	      ;; bodies).
-	      (begin
-		 (fprint *c-port* #"\n# " cur-line " \"" cur-fname #\")
-		 (set! *bdb-loc* cur-loc)))))))
-
-;*---------------------------------------------------------------------*/
-;*    emit-newline ...                                                 */
-;*---------------------------------------------------------------------*/
-(define (emit-newline . mandatory)
-   (cond
-      ((<=fx *bdb-debug* 0)
+      ((or (<=fx *bdb-debug* 0) *bdb-debug-no-line-directives?*)
        (newline *c-port*))
-      ((pair? mandatory)
-       (if (location? *bdb-loc*)
-	   (emit-bdb-loc *bdb-loc*)
-	   (newline *c-port*)))
+      ((not (location? cur-loc))
+       (cond
+	  ((location? *bdb-loc*)
+	   (emit-bdb-loc *bdb-loc*))
+	  ((location? *module-location*)
+	   ;; when no location is found, we use the location of
+	   ;; the module clause
+	   (emit-bdb-loc *module-location*))
+	  ((pair? *src-files*)
+	   ;; when no location at all is found, emit a dummy
+	   ;; "first file line" location
+	   (fprint *c-port* #"\n# " 1 " \"" (car *src-files*) #\"))))
       (else
-       (write-char #\space *c-port*))))
+       (let ((cur-fname (location-fname cur-loc))
+	     (cur-line  (location-lnum cur-loc)))
+	  (fprint *c-port* #"\n# " cur-line " \"" cur-fname #\")
+	  (set! *bdb-loc* cur-loc)))))
 
 ;*---------------------------------------------------------------------*/
-;*    fprin ...                                                        */
+;*    emit-bdb-loc-comment ...                                         */
+;*    -------------------------------------------------------------    */
+;*    For debug purposes this function write a location is a C         */
+;*    comment (iff CUR-LOC is a location).                             */
 ;*---------------------------------------------------------------------*/
-(define (fprin port . values)
-   (for-each (lambda (value) (display value port)) values))
+(define (emit-bdb-loc-comment cur-loc)
+   (if (location? cur-loc)
+       (begin
+	  (display "/* " *c-port*)
+	  (display (location-fname cur-loc) *c-port*)
+	  (display " " *c-port*)
+	  (display (location-lnum cur-loc) *c-port*)
+	  (display " */" *c-port*))))
