@@ -1,41 +1,10 @@
-;*---------------------------------------------------------------------*/
-;*   A pratical implementation for the Scheme programming language     */
-;*                                                                     */
-;*                                    ,--^,                            */
-;*                              _ ___/ /|/                             */
-;*                          ,;'( )__, ) '                              */
-;*                         ;;  //   L__.                               */
-;*                         '   \\   /  '                               */
-;*                              ^   ^                                  */
-;*                                                                     */
-;*   Copyright (c) 1992-1999 Manuel Serrano                            */
-;*                                                                     */
-;*     Bug descriptions, use reports, comments or suggestions are      */
-;*     welcome. Send them to                                           */
-;*       bigloo-request@kaolin.unice.fr                                */
-;*       http://kaolin.unice.fr/bigloo                                 */
-;*                                                                     */
-;*   This program is free software; you can redistribute it            */
-;*   and/or modify it under the terms of the GNU General Public        */
-;*   License as published by the Free Software Foundation; either      */
-;*   version 2 of the License, or (at your option) any later version.  */
-;*                                                                     */
-;*   This program is distributed in the hope that it will be useful,   */
-;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
-;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
-;*   GNU General Public License for more details.                      */
-;*                                                                     */
-;*   You should have received a copy of the GNU General Public         */
-;*   License along with this program; if not, write to the Free        */
-;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
-;*   MA 02111-1307, USA.                                               */
-;*---------------------------------------------------------------------*/
 ;*=====================================================================*/
 ;*    serrano/prgm/project/bigloo/comptime/Cnst/alloc.scm              */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Mon Feb  6 13:51:36 1995                          */
-;*    Last change :  Tue Aug  4 14:30:05 1998 (serrano)                */
+;*    Last change :  Sat Jul  7 11:36:59 2001 (serrano)                */
+;*    Copyright   :  1995-2001 Manuel Serrano, see LICENSE file        */
 ;*    -------------------------------------------------------------    */
 ;*    The constant allocations.                                        */
 ;*=====================================================================*/
@@ -60,7 +29,8 @@
 	    ast_glo-def
 	    coerce_coerce
 	    cnst_cache
-	    cnst_node)
+	    cnst_node
+	    ast_ident)
    (export  (start-cnst-alloc!)
 	    (stop-cnst-alloc!)
 	    (get-cnst-offset)
@@ -68,14 +38,15 @@
 	    (get-cnst-sexp)
 	    (get-cnst-table)
 	    (cnst-table-id)
-	    (cnst-alloc-string::node    ::bstring   <loc>)
-	    (cnst-alloc-symbol::node    ::symbol    <loc>)
-	    (cnst-alloc-keyword::node   ::keyword   <loc>)
-	    (cnst-alloc-procedure::node ::node      <loc>)
-	    (cnst-alloc-real::node      ::real      <loc>)
-	    (cnst-alloc-list::node      <list>      <loc>)
-	    (cnst-alloc-vector::node    ::vector    <loc>)
-	    (cnst-alloc-tvector::node   <a-tvector> <loc>)))
+	    (cnst-alloc-string::node ::bstring <loc>)
+	    (cnst-alloc-ucs2-string::node ::ucs2string <loc>)
+	    (cnst-alloc-symbol::node ::symbol <loc>)
+	    (cnst-alloc-keyword::node ::keyword <loc>)
+	    (cnst-alloc-procedure::node ::node <loc>)
+	    (cnst-alloc-real::node ::real <loc>)
+	    (cnst-alloc-list::node <list> <loc>)
+	    (cnst-alloc-vector::node ::vector <loc>)
+	    (cnst-alloc-tvector::node <a-tvector> <loc>)))
 
 ;*---------------------------------------------------------------------*/
 ;*    structures                                                       */
@@ -130,14 +101,15 @@
 ;*---------------------------------------------------------------------*/
 ;*    The local hash-tables                                            */
 ;*---------------------------------------------------------------------*/
-(define *string-env*    '())
-(define *real-env*      '())
-(define *symbol-env*    '())
-(define *keyword-env*   '())
-(define *list-env*      '())
-(define *vector-env*    '())
-(define *global-set*    '())
-(define *global-sexp*   '())
+(define *string-env*     '())
+(define *ucs2string-env* '())
+(define *real-env*       '())
+(define *symbol-env*     '())
+(define *keyword-env*    '())
+(define *list-env*       '())
+(define *vector-env*     '())
+(define *global-set*     '())
+(define *global-sexp*    '())
 
 ;*---------------------------------------------------------------------*/
 ;*    *old-debug*                                                      */
@@ -148,13 +120,13 @@
 ;*    cnst-table-id ...                                                */
 ;*---------------------------------------------------------------------*/
 (define (cnst-table-id)
-   '__CNSTS_TABLE)
+   '__cnsts_table)
 
 ;*---------------------------------------------------------------------*/
 ;*    typed-cnst-table-id ...                                          */
 ;*---------------------------------------------------------------------*/
 (define (typed-cnst-table-id)
-   (symbol-append (cnst-table-id) '::cnst*))
+   (make-typed-ident (cnst-table-id) 'cnst*))
 
 ;*---------------------------------------------------------------------*/
 ;*    start-cnst-alloc! ...                                            */
@@ -172,26 +144,14 @@
    ;; when compiling for bdb2, default global variable are exported. of
    ;; course this may lead to incorrect compilation if the global variable
    ;; (its name is not gensymed nor qualified) is exported.
-   (if (>fx *bdb-debug* 1)
+   (if (>=fx *bdb-debug* 3)
        (global-import-set! *cnst-table* 'static))
    (set! *cnst-offset* 0)
-   (set! *string-env* (make-hash-table 1024
-				       (lambda (s) (string->0..2^x-1 s 10))
-				       cnst-info-cnst
-				       string=?))
+   (set! *string-env* (make-hashtable))
+   (set! *ucs2string-env* (make-hashtable))
    (set! *real-env*   '())
-   (set! *symbol-env* (make-hash-table 1024
-				       (lambda (s) (string->0..2^x-1
-						    (symbol->string s)
-						    10))
-				       cnst-info-cnst
-				       eq?))
-   (set! *keyword-env* (make-hash-table 1024
-					(lambda (s) (string->0..2^x-1
-						     (keyword->string s)
-						     10))
-					cnst-info-cnst
-					eq?))
+   (set! *symbol-env* (make-hashtable))
+   (set! *keyword-env* (make-hashtable))
    #t)
 
 ;*---------------------------------------------------------------------*/
@@ -199,12 +159,13 @@
 ;*---------------------------------------------------------------------*/
 (define (stop-cnst-alloc!)
    (set! *compiler-debug* *old-debug*)
-   (set! *string-env*  #unspecified)
-   (set! *real-env*    #unspecified) 
-   (set! *symbol-env*  #unspecified)
-   (set! *keyword-env* #unspecified)
-   (set! *list-env*    #unspecified) 
-   (set! *vector-env*  #unspecified) 
+   (set! *string-env*     #unspecified)
+   (set! *ucs2string-env* #unspecified)
+   (set! *real-env*       #unspecified) 
+   (set! *symbol-env*     #unspecified)
+   (set! *keyword-env*    #unspecified)
+   (set! *list-env*       #unspecified) 
+   (set! *vector-env*     #unspecified) 
    #t)
 
 ;*---------------------------------------------------------------------*/
@@ -220,7 +181,7 @@
 	      (variable *cnst-table-ref*)))
       (args (list (instantiate::atom
 		     (loc loc)
-		     (type (get-default-type))
+		     (type *int*)
 		     (value offset))))))
 	 
 ;*---------------------------------------------------------------------*/
@@ -233,17 +194,19 @@
 (define (cnst-alloc-string string loc)
    (define (alloc-string)
       ;; in lib-mode string are statically allocated
-      (let ((var (def-global-scnst! (symbol-append (gensym 'string) '::bstring)
+      (let ((var (def-global-scnst! (make-typed-ident (gensym 'string)
+						      'bstring)
 		    *module*
 		    string
-		    'sstring)))
+		    'sstring
+		    loc)))
 	 (if *shared-cnst?*
-	     (put-hash! (cnst-info string var) *string-env*))
+	     (hashtable-put! *string-env* string (cnst-info string var)))
 	 (instantiate::var
 	    (loc loc)
 	    (type (variable-type var))
 	    (variable var))))
-   (let ((old (and *shared-cnst?* (get-hash string *string-env*))))
+   (let ((old (and *shared-cnst?* (hashtable-get *string-env* string))))
       (cond
 	 (old
 	  (instantiate::var
@@ -253,16 +216,69 @@
 	 (else
 	  (alloc-string)))))
 
+;*---------------------------------------------------------------------*/
+;*    cnst-alloc-ucs2-string ...                                       */
+;*---------------------------------------------------------------------*/
+(define (cnst-alloc-ucs2-string string loc)
+   (define (lib-alloc-ucs2string bstring)
+      (let ((var (def-global-svar! (make-typed-ident (gensym 'ucs2string)
+						     'ucs2string)
+		    *module*
+		    'an-ucs2-string
+		    'now)))
+	 (hashtable-put! *ucs2string-env* string (cnst-info string var))
+	 (add-cnst-sexp! `(set! (@ ,(global-id var) ,(global-module var))
+				,(coerce!
+				  (instantiate::app
+				     (loc loc)
+				     (type *ucs2string*)
+				     (fun (instantiate::var
+					     (loc loc)
+					     (type *ucs2string*)
+					     (variable *string->ucs2string*)))
+				     (args (list (instantiate::atom
+						    (loc loc)
+						    (type *ucs2string*)
+						    (value bstring)))))
+				  var
+				  *obj*)))
+	 (instantiate::var
+	    (loc loc)
+	    (type (variable-type var))
+	    (variable var))))
+   (define (read-alloc-ucs2string)
+      (let ((offset *cnst-offset*))
+	 (set! *cnst-offset* (+fx 1 *cnst-offset*))
+	 (set! *global-set* (cons string *global-set*))
+	 (hashtable-put! *ucs2string-env* string (cnst-info string offset))
+	 (make-cnst-table-ref offset loc)))
+   (let ((old (hashtable-get *ucs2string-env* string))
+	 (string-as-bstring (ucs2-string->utf8-string string)))
+      (cond
+	 (old
+	  (if (eq? *init-mode* 'lib)
+	      (instantiate::var
+		 (loc loc)
+		 (type (get-default-type))
+		 (variable (cnst-info-offset old)))
+	      (make-cnst-table-ref (cnst-info-offset old) loc)))
+	 ((or (eq? *init-mode* 'lib)
+	      (=fx (string-length string-as-bstring) 0)
+	      (char=? (string-ref string-as-bstring 0) #\;))
+	  (lib-alloc-ucs2string string-as-bstring))
+	 (else
+	  (read-alloc-ucs2string)))))
+
 ;*---------------------------------------------------------------------*/
 ;*    cnst-alloc-symbol ...                                            */
 ;*---------------------------------------------------------------------*/
 (define (cnst-alloc-symbol symbol loc)
    (define (lib-alloc-symbol)
-      (let ((var (def-global-svar! (symbol-append (gensym 'symbol) '::symbol)
+      (let ((var (def-global-svar! (make-typed-ident (gensym 'symbol) 'symbol)
 		    *module*
 		    'a-symbol
 		    'now)))
-	 (put-hash! (cnst-info symbol var) *symbol-env*)
+	 (hashtable-put! *symbol-env* symbol (cnst-info symbol var))
 	 (add-cnst-sexp! `(set! (@ ,(global-id var) ,(global-module var))
 				,(coerce!
 				  (instantiate::app
@@ -277,6 +293,7 @@
 						    (type *string*)
 						    (value (symbol->string
 							    symbol))))))
+				  var
 				  *obj*)))
 	 (instantiate::var
 	    (loc loc)
@@ -286,9 +303,9 @@
       (let ((offset *cnst-offset*))
 	 (set! *cnst-offset* (+fx 1 *cnst-offset*))
 	 (set! *global-set* (cons symbol *global-set*))
-	 (put-hash! (cnst-info symbol offset) *symbol-env*)
+	 (hashtable-put! *symbol-env* symbol (cnst-info symbol offset))
 	 (make-cnst-table-ref offset loc)))
-   (let ((old (get-hash symbol *symbol-env*))
+   (let ((old (hashtable-get *symbol-env* symbol))
 	 (symbol-as-string (symbol->string symbol)))
       (cond
 	 (old
@@ -310,11 +327,12 @@
 ;*---------------------------------------------------------------------*/
 (define (cnst-alloc-keyword keyword loc)
    (define (lib-alloc-keyword)
-      (let ((var (def-global-svar! (symbol-append (gensym 'keyword) '::keyword)
+      (let ((var (def-global-svar! (make-typed-ident (gensym 'keyword)
+						     'keyword)
 		    *module*
 		    'a-keyword
 		    'now)))
-	 (put-hash! (cnst-info keyword var) *keyword-env*)
+	 (hashtable-put! *keyword-env* keyword (cnst-info keyword var))
 	 (add-cnst-sexp! `(set! (@ ,(global-id var) ,(global-module var))
 				,(coerce!
 				  (instantiate::app
@@ -329,6 +347,7 @@
 						    (type *string*)
 						    (value (keyword->string
 							    keyword))))))
+				  var
 				  *obj*)))
 	 (instantiate::var
 	    (loc loc)
@@ -338,9 +357,9 @@
       (let ((offset *cnst-offset*))
 	 (set! *cnst-offset* (+fx 1 *cnst-offset*))
 	 (set! *global-set* (cons keyword *global-set*))
-	 (put-hash! (cnst-info keyword offset) *keyword-env*)
+	 (hashtable-put! *keyword-env* keyword (cnst-info keyword offset))
 	 (make-cnst-table-ref offset loc)))
-   (let ((old (get-hash keyword *keyword-env*))
+   (let ((old (hashtable-get *keyword-env* keyword))
 	 (keyword-as-string (keyword->string keyword)))
       (cond
 	 (old
@@ -361,10 +380,11 @@
 ;*    cnst-alloc-procedure ...                                         */
 ;*---------------------------------------------------------------------*/
 (define (cnst-alloc-procedure procedure loc)
-   (let ((var (def-global-scnst! (symbol-append (gensym 'proc) '::procedure)
+   (let ((var (def-global-scnst! (make-typed-ident (gensym 'proc) 'procedure)
 		 *module*
 		 procedure
-		 'sfun)))
+		 'sfun
+		 loc)))
       (instantiate::var
 	 (loc loc)
 	 (type (variable-type var))
@@ -375,10 +395,11 @@
 ;*---------------------------------------------------------------------*/
 (define (cnst-alloc-real real loc)
    (define (lib-alloc-real)
-      (let ((var (def-global-scnst! (symbol-append (gensym 'real) '::real)
+      (let ((var (def-global-scnst! (make-typed-ident (gensym 'real) 'real)
 		    *module*
 		    real
-		    'sreal)))
+		    'sreal
+		    loc)))
 	 (set! *real-env* (cons (cons real var) *real-env*))
 	 (instantiate::var
 	    (loc loc)
@@ -431,14 +452,14 @@
 				      (value (car pair))))
 			    (loop (cdr pair)))))))))
    (define (lib-alloc-list)
-      (let ((var (def-global-svar! (symbol-append (gensym 'list) '::pair)
+      (let ((var (def-global-svar! (make-typed-ident (gensym 'list) 'pair)
 		    *module*
 		    'cnst-list
 		    'now)))
 	 (if *shared-cnst?*
 	     (set! *list-env* (cons (cnst-info pair var) *list-env*)))
 	 (add-cnst-sexp! `(set! (@ ,(global-id var) ,(global-module var))
-				,(coerce! (cnst-list pair) *obj*)))
+				,(coerce! (cnst-list pair) var *obj*)))
 	 (instantiate::var
 	    (loc loc)
 	    (type (variable-type var))
@@ -527,6 +548,7 @@
 					     (value (vector-tag vec))))))
 				var-body)))
 			   var-body)))))
+	#unspecified
 	*obj*)))
    (define (read-alloc-vector)
       (let ((offset *cnst-offset*))
@@ -536,7 +558,7 @@
 	     (set! *vector-env* (cons (cnst-info vec offset) *vector-env*)))
 	 (make-cnst-table-ref offset loc)))
    (define (lib-alloc-vector)
-      (let ((var (def-global-svar! (symbol-append (gensym 'vector) '::vector)
+      (let ((var (def-global-svar! (make-typed-ident (gensym 'vector) 'vector)
 		    *module*
 		    'cnst-vector
 		    'now)))
@@ -575,10 +597,11 @@
 ;*---------------------------------------------------------------------*/
 (define (cnst-alloc-tvector tvec loc)
    (define (C-static-alloc-tvector)
-      (let ((var (def-global-scnst! (symbol-append (gensym 'tvec) '::tvector)
+      (let ((var (def-global-scnst! (make-typed-ident (gensym 'tvec) 'tvector)
 		    *module*
 		    tvec
-		    'stvector)))
+		    'stvector
+		    loc)))
 	 (let* ((id  (type-id (a-tvector-type tvec)))
 		(aid (cnst! (instantiate::kwote
 			       (loc loc)
