@@ -1,41 +1,10 @@
-;*---------------------------------------------------------------------*/
-;*   A pratical implementation for the Scheme programming language     */
-;*                                                                     */
-;*                                    ,--^,                            */
-;*                              _ ___/ /|/                             */
-;*                          ,;'( )__, ) '                              */
-;*                         ;;  //   L__.                               */
-;*                         '   \\   /  '                               */
-;*                              ^   ^                                  */
-;*                                                                     */
-;*   Copyright (c) 1992-1999 Manuel Serrano                            */
-;*                                                                     */
-;*     Bug descriptions, use reports, comments or suggestions are      */
-;*     welcome. Send them to                                           */
-;*       bigloo-request@kaolin.unice.fr                                */
-;*       http://kaolin.unice.fr/bigloo                                 */
-;*                                                                     */
-;*   This program is free software; you can redistribute it            */
-;*   and/or modify it under the terms of the GNU General Public        */
-;*   License as published by the Free Software Foundation; either      */
-;*   version 2 of the License, or (at your option) any later version.  */
-;*                                                                     */
-;*   This program is distributed in the hope that it will be useful,   */
-;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
-;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
-;*   GNU General Public License for more details.                      */
-;*                                                                     */
-;*   You should have received a copy of the GNU General Public         */
-;*   License along with this program; if not, write to the Free        */
-;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
-;*   MA 02111-1307, USA.                                               */
-;*---------------------------------------------------------------------*/
 ;*=====================================================================*/
 ;*    serrano/prgm/project/bigloo/comptime/Engine/param.scm            */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Tue Oct  3 12:44:17 1995                          */
-;*    Last change :  Thu Feb 11 07:20:54 1999 (serrano)                */
+;*    Last change :  Tue Nov 27 10:28:01 2001 (serrano)                */
+;*    Copyright   :  1995-2001 Manuel Serrano, see LICENSE file        */
 ;*    -------------------------------------------------------------    */
 ;*    Global control of the compiler                                   */
 ;*=====================================================================*/
@@ -46,10 +15,11 @@
 (module engine_param
    (import  ;; tools_date is a generated file and hence, it can't be
 	    ;; set in the .afile file.
-	    (tools_date "Tools/date.scm"))
+	    (tools_date "Tools/date.scm")
+	    tools_misc
+	    engine_configure)
    (export  *bigloo-version*
 	    *bigloo-name*
-	    *bigloo-level*
 	    *bigloo-cmd-name*
 	    *bigloo-args*
 	    *rest-args*
@@ -60,6 +30,7 @@
 	    *bigloo-licensing?*
 	    *lib-mode*
 	    *init-mode*
+	    *dlopen-init*
 	    *max-c-token-length*
 	    *max-c-foreign-arity*
 	    *verbose*
@@ -70,30 +41,47 @@
 	    *unsafe-arity*
 	    *unsafe-version*
 	    *unsafe-library*
+	    *warning-overriden-slots*
 	    *profile-library*
 	    *trace-name*
 	    *trace-write-length*
 	    *additional-traces*
 	    *inlining?*
+	    *user-inlining?*
 	    *inlining-kfactor*
 	    *inlining-reduce-kfactor*
 	    *optim*
 	    *optim-stack?*
-	    *optim-inline-method?*
 	    *optim-unroll-loop?*
 	    *optim-loop-inlining?*
 	    *optim-O-macro?*
+	    *optim-cfa-arithmetic?*
+	    *optim-dataflow?*
+	    *optim-reduce-beta?*
+	    *optim-jvm-inlining*
+	    *optim-jvm-constructor-inlining*
+	    *optim-jvm-peephole*
+	    *optim-jvm-branch*
+	    *optim-jvm-fasteq*
+	    *jvm-purify*
+	    *jvm-env*
 	    *max-stack-alloc-size*
 	    *genericity*
 	    *shared-cnst?*
 	    ;; -------------------------------------------------------------
 	    ;; warning, any change about this variable name must be reported
 	    ;; in the no-trace-no-check macro of Llib/error.scm file
-	    *compiler-debug*            
+ 	    *compiler-debug*
+	    *compiler-sharing-debug?*
 	    ;; -------------------------------------------------------------
-	    *c-debug*                   
+	    *debug-module*
+	    *c-debug*
 	    *c-debug-option*
+	    *jvm-debug*
 	    *bdb-debug*
+	    *heap-debug*
+	    *heap-debug-copt*
+	    *bdb-debug-no-line-directives?*
 	    *profile-mode*
 	    *prof-table-name*
 	    *module-shape?*             
@@ -106,54 +94,73 @@
 	    *dest*
 	    *shell*
 	    *cc*
+	    *cflags-optim*
+	    *cflags-prof*
 	    *stdc*               
 	    *cc-options*       
 	    *rm-c-files*       
-	    *ld-options*       
+	    *ld-options*
+	    *ld-relative*
 	    *strip*            
 	    *bigloo-lib*
 	    *bigloo-lib-base-name*
 	    *gc-lib*
 	    *static-bigloo?*
+	    *double-ld-libs?*
 	    *bigloo-user-lib*
 	    *additional-bigloo-libraries*
+	    *additional-bigloo-zips*
 	    *default-lib-dir*
 	    *lib-dir*
 	    *lib-src-dir*
 	    *include-foreign*  
 	    *additional-include-foreign*
 	    *indent*
-	    *access-file*      
-	    *access-table*     
-	    *src-files*              
+	    *access-file*
+	    *access-file-default*
+	    *access-table*
+	    *qualified-type-file*
+	    *qualified-type-file-default*
+	    *src-files*
 	    *o-files*          
+	    *c-files*          
 	    *with-files*
+	    *early-with-modules*
 	    *interpreter*      
 	    *startup-file*     
 	    *call/cc?*
 	    *reflection?*
 	    *garbage-collector*
 	    *pass*
+	    *jvm-jar?*
+	    *jvm-shell*
+	    *jvm-java*
+	    *jvm-options*
+	    *jvm-classpath*
+	    *jvm-jarpath*
 	    *module-checksum-object?*
 	    *heap-base-name*
 	    *heap-name*
+	    *heap-jvm-name*
+	    *jvm-foreign-class-id*
+	    *jvm-foreign-class-name*
 	    *additional-heap-name*
 	    *additional-heap-names*
 	    *extend-entry*
 	    *auto-mode*
 	    *src-suffix*
+	    *c-suffix*
 	    *obj-suffix*
 	    *mco-suffix*
-	    *case-sensitive*
+	    *mco-include-path*
+	    *ast-case-sensitive*
 	    *user-heap-size*
 	    *reader*
+	    *target-language*
+	    *use-private?*
+	    *builtin-allocators*
+	    *eval-options*
 	    (bigloo-variables-usage ::bool))
-   (extern  (macro shell::string "SHELL")
-	    (macro c-compiler::string "C_COMPILER")
-	    (macro library-directory::string "LIBRARY_DIRECTORY")
-	    (macro user-libraries::string "USER_LIBRARIES")
-	    (macro c-beautifier::string "C_BEAUTIFIER")
-	    (macro library-base-name::string "LIBRARY_BASE_NAME"))
    (eval    (export-all)))
 
 ;*---------------------------------------------------------------------*/
@@ -215,20 +222,16 @@
 ;; the bigloo version
 (doc-define *bigloo-version*
 	    "The Bigloo major release number"
-	    "2.0")
+	    bgl-configure-release-number)
 ;; the bigloo name
 (doc-define *bigloo-name*
 	    "The Bigloo name"
 	    (string-append "Bigloo (" *bigloo-version* ")"))
-;; the bigloo minor release number
-(doc-define *bigloo-level*
-	    "The Bigloo minor release number (#f or a char)"
-	    #\a)
 (define *bigloo-cmd-name*   'nothing-yet)
 (define *bigloo-args*       'nothing-yet)
 (define *rest-args*         '())
 (define *bigloo-author*     "Manuel Serrano")
-(define *bigloo-email*      "Manuel.Serrano@unice.fr")
+(define *bigloo-email*      "Manuel.Serrano@inria.fr")
 (define *bigloo-date*       (bigloo-date))
 ;; the tmp directory
 (doc-define *bigloo-tmp*
@@ -271,12 +274,24 @@
 ;*---------------------------------------------------------------------*/
 ;; the shell
 (doc-define *shell*
-	    "The C compiler"
-	    shell)
+	    "The shell to exec C compilations"
+	    bgl-configure-shell)
 ;; the c compiler
 (doc-define *cc*
 	    "The C compiler"
-	    c-compiler)
+	    bgl-configure-c-compiler)
+;; the c compiler  option
+(doc-define *cflags*
+	    "The C compiler option"
+	    bgl-configure-c-flag)
+;; the c compiler optimization option
+(doc-define *cflags-optim*
+	    "The C compiler optimization option"
+	    bgl-configure-c-optim-flag)
+;; the c compiler profile option
+(doc-define *cflags-prof*
+	    "The C compiler profiling option"
+	    bgl-configure-c-prof-flag)
 ;; The C production type
 (doc-define *stdc*
 	    "Shall we produced ISO C?"
@@ -284,7 +299,7 @@
 ;; the CC option
 (doc-define *cc-options*
 	    "cc options"
-	    "")
+	    bgl-configure-c-flag)
 ;; shall we remove the C produced file?
 (doc-define *rm-c-files*
 	    "Shall we remove the C produced file?"
@@ -292,7 +307,11 @@
 ;; ld options
 (doc-define *ld-options*
 	    "ld options"
-	    ""))
+	    "")
+;; library link mode
+(doc-define *ld-relative*
+	    "Relative or absolute path names for libraries"
+	    #f)
 ;; strip ?
 (doc-define *strip*
 	    "Shall we strip the executable?"
@@ -300,14 +319,14 @@
 ;; the default lib dir path
 (doc-define *default-lib-dir*
 	    "The default lib dir path (without version)"
-	    library-directory)
+	    bgl-configure-library-directory)
 ;; the lib dir path
 (doc-define *lib-dir*
 	    "The lib dir path"
-	    (let ((lib-env (getenv "BIGLOOLIB")))
-	       (if (not (string? lib-env))
-		   (list *default-lib-dir*)
-		   (list lib-env))))
+	    (let ((lib-env (build-path-from-shell-variable "BIGLOOLIB")))
+	       (if (not (pair? lib-env))
+		   (list "." *default-lib-dir*)
+		   (cons "." lib-env))))
 ;; the lib source dir path
 (doc-define *lib-src-dir*
 	    "The lib dir path"
@@ -315,15 +334,11 @@
 ;; the bigloo library base name
 (doc-define *bigloo-lib-base-name*
 	    "The Bigloo library base name"
-	    library-base-name)
+	    bgl-configure-library-base-name)
 ;; the bigloo library
 (doc-define *bigloo-lib*
 	    "The Bigloo library"
-	    (string-append *bigloo-lib-base-name*
-			   *bigloo-version*
-			   (if (char? *bigloo-level*)
-			       (make-string 1 *bigloo-level*)
-			       "")))
+	    *bigloo-lib-base-name*)
 ;; the gc library
 (doc-define *gc-lib*
 	    "The Gc library"
@@ -332,25 +347,28 @@
 (doc-define *static-bigloo?*
 	    "Do we use the static Bigloo library"
 	    #f)
+;; does we include twice the additional user libraries?
+(doc-define *double-ld-libs?*
+	    "Do we include twice the additional user libraries"
+	    #t)
 ;; the user C libraries
 (doc-define *bigloo-user-lib*
 	    "The user extra C libraries"
-	    (list user-libraries))
+	    (list bgl-configure-user-libraries))
 ;; the user Bigloo libraries
 (doc-define *additional-bigloo-libraries*
 	    "The user extra Bigloo libraries"
 	    '())
+;; the user Bigloo zip files
+(doc-define *additional-bigloo-zips*
+	    "The user extra Bigloo Zip files"
+	    '())
 ;; the load path
 (set! *load-path* (append *lib-dir* *load-path*))
 ;; the C include files
 (doc-define *include-foreign*
 	    "The C included files"
-	    (list (string-append "bigloo"
-				 *bigloo-version*
-				 (if (char? *bigloo-level*)
-				     (make-string 1 *bigloo-level*)
-				     "")
-				 ".h")))
+	    (list "bigloo.h"))
 ;; the additional C include files
 (doc-define *additional-include-foreign*
 	    "The additional C included files"
@@ -362,12 +380,19 @@
 ;; the heap name
 (doc-define *heap-name*
 	    "The Bigloo heap file name"
-	    (string-append *heap-base-name*
-			   *bigloo-version*
-			   (if (char? *bigloo-level*)
-			       (make-string 1 *bigloo-level*)
-			       "")
-			   ".heap"))
+	    (string-append *heap-base-name* ".heap"))
+;; the jvm heap name
+(doc-define *heap-jvm-name*
+	    "The Bigloo heap file name for the JVM backend"
+	    (string-append *heap-base-name* ".jheap"))
+;; the jvm foreign class id
+(doc-define *jvm-foreign-class-id*
+	    "The identifier of the Jlib foreign class"
+	    'foreign)
+;; the jvm foreign class name
+(doc-define *jvm-foreign-class-name*
+	    "The name of the Jlib foreign class"
+	    "bigloo.foreign")
 ;; the additional heap name
 (doc-define *additional-heap-name*
 	    "A name of an additional heap file name to be build"
@@ -379,12 +404,19 @@
 ;; indent
 (doc-define *indent*
 	    "The name of the C beautifier"
-	    c-beautifier)
+	    bgl-configure-c-beautifier)
 ;; debugging level
 (doc-define *compiler-debug*
 	    "Debugging level"
 	    0)
-;; C debugging mode ?
+(doc-define *compiler-sharing-debug?*
+	    "Compiler self sharing debug"
+	    #f)
+;; debugging level
+(doc-define *debug-module*
+	    "Module initilazation debugging"
+	    0)
+;; C debugging mode?
 (doc-define *c-debug*
 	    "C debugging mode?"
 	    #f)
@@ -392,10 +424,23 @@
 (doc-define *c-debug-option*
 	    "cc debugging option"
 	    "-g")
+;; jvm debuggin mode?
+(doc-define *jvm-debug*
+	    "JVM debugging mode?"
+	    #f)
 ;; The bdb debugging option
 (doc-define *bdb-debug*
 	    "Bdb debugging mode"
 	    0)
+;; The heap debugging option
+(doc-define *heap-debug*
+	    "Heap debugging mode"
+	    0)
+;; The heap debugging option
+(doc-define *heap-debug-copt*
+	    "Heap debugging C flags"
+	    bgl-configure-heap-debug-copt)
+(define *bdb-debug-no-line-directives?* #f)
 ;; The Bigloo profiling option
 (doc-define *profile-mode*
 	    "Bigloo profile mode"
@@ -406,18 +451,36 @@
 	    "bmon.out")
 
 ;*---------------------------------------------------------------------*/
-;*    Les fichier de link                                              */
+;*    Access and qualifed-type                                         */
 ;*---------------------------------------------------------------------*/
 (doc-define *access-file*
 	    "The access file name"
 	    #f)
+(doc-define *access-file-default*
+	    "The default access file name"
+	    ".afile")
 (define *access-table* '())
+
+(doc-define *qualified-type-file*
+	    "The qualifed-type association file name"
+	    #f)
+(doc-define *qualified-type-file-default*
+	    "The qualifed-type association file name"
+	    ".jfile")
+
+;*---------------------------------------------------------------------*/
+;*    Link files                                                       */
+;*---------------------------------------------------------------------*/
 (doc-define *o-files*
 	    "The additional obect files"
 	    '())
+(doc-define *c-files*
+	    "The C source files"
+	    '())
 (doc-define *with-files*
 	    "The additional modules"
 	    '())
+(define *early-with-modules* '())
    
 ;*---------------------------------------------------------------------*/
 ;*    Des variables de controle sur `comment on doit compiler'         */
@@ -437,6 +500,24 @@
 (doc-define *pass*
 	    "Stop after the pass"
 	    'ld)
+(doc-define *jvm-jar?*
+	    "Enable/disable a JAR file production for the JVM back-end"
+	    #f)
+(doc-define *jvm-shell*
+	    "Shell to be used when producing JVM run scripts"
+	    bgl-configure-java-shell)
+(doc-define *jvm-java*
+	    "JVM to be used to run Java programs"
+	    bgl-configure-java)
+(doc-define *jvm-options*
+	    "JVM options"
+	    "")
+(doc-define *jvm-classpath*
+	    "JVM classpath"
+	    #f)
+(doc-define *jvm-jarpath*
+	    "JVM jarpath"
+	    #f)
 (doc-define *module-checksum-object?*
 	    "Produce a module checksum object (.mco)"
 	    #f)
@@ -465,6 +546,9 @@
 (doc-define *unsafe-library*
 	    "Use the unsafe library version"
 	    #f)
+(doc-define *warning-overriden-slots*
+	    "Set to #t to warn about virtual slot overriding"
+	    #t)
 (doc-define *profile-library*
 	    "Use the profiled library version"
 	    #f)
@@ -487,6 +571,9 @@
 (doc-define *init-mode*
 	    "Module initialization mode"
 	    'read)
+(doc-define *dlopen-init*
+	    "Emit a standard Bigloo dynamic loading init entry point"
+	    #f)
 (doc-define *max-c-token-length*
 	    "Max c token length"
 	    1024)
@@ -510,9 +597,6 @@
 (doc-define *optim-stack?*
 	    "Stack allocation optimization"
 	    #unspecified)
-(doc-define *optim-inline-method?*
-	    "Method inlining optimization"
-	    #unspecified)
 (doc-define *optim-unroll-loop?*
 	    "Loop unrolling optimization"
 	    #unspecified)
@@ -521,10 +605,46 @@
 	    #t)
 (doc-define *optim-O-macro?*
 	    "Enable optimization by macro-expansion"
-	    #t)
+	    #f)
+(doc-define *optim-jvm-inlining*
+	    "Enable JVM inlining"
+	    0)
+(doc-define *optim-jvm-constructor-inlining*
+	    "Enable JVM inlining for constructors"
+	    0)
+(doc-define *optim-jvm-peephole*
+	    "Enable JVM peephole optimization"
+	    0)
+(doc-define *optim-jvm-branch*
+	    "Enable JVM branch tensioning"
+	    0)
+(doc-define *optim-jvm-fasteq*
+	    "EQ? no longer works on integers (use =FX instead)"
+	    #f)
+(doc-define *jvm-purify*
+	    "Produce byte code verifier compliant JVM code"
+	    #f)
+(doc-define *jvm-env*
+	    "List of environment variables to be available in the compiled code"
+	    '())
+(doc-define *optim-jvm*
+	    "Enable optimization by inlining jvm code"
+	    0)
+(doc-define *optim-cfa-arithmetic?*
+	    "Enable refined arithmetic specialization"
+	    #f)
+(doc-define *optim-dataflow?*
+	    "Enable simple dataflow optimization"
+	    #f)
+(doc-define *optim-reduce-beta?*
+	    "Enable simple beta reduction"
+	    #f)
 (doc-define *inlining?*
 	    "Inlining optimization"
 	    #t)
+(doc-define *user-inlining?*
+	    "User inlining optimization"
+	    #t)
 (doc-define *inlining-kfactor*
 	    "Inlining growth factor"
 	    (lambda (olevel) (*fx 2 olevel)))
@@ -542,12 +662,21 @@
 ;*---------------------------------------------------------------------*/
 ;*    *src-suffix* ...                                                 */
 ;*    -------------------------------------------------------------    */
-;*    The list of suffix recognize by the compiler and the linker.     */
+;*    The list of suffix recognized by the compiler and the linker.    */
 ;*---------------------------------------------------------------------*/
 (doc-define *src-suffix*
 	    "Scheme legal suffixes"
 	    '("scm" "bgl"))
 
+;*---------------------------------------------------------------------*/
+;*    *c-suffix* ...                                                   */
+;*    -------------------------------------------------------------    */
+;*    The list of C suffix recognized by the compiler and the linker.  */
+;*---------------------------------------------------------------------*/
+(doc-define *c-suffix*
+	    "C legal suffixes"
+	    '("c"))
+
 ;*---------------------------------------------------------------------*/
 ;*    *obj-suffix* ...                                                 */
 ;*    -------------------------------------------------------------    */
@@ -566,6 +695,13 @@
 	    "Module checksum object legal suffixes"
 	    '("mco"))
 
+;*---------------------------------------------------------------------*/
+;*    *mco-include-path* ...                                           */
+;*---------------------------------------------------------------------*/
+(doc-define *mco-include-path*
+	    "Module checksum C include path"
+	    '("."))
+
 ;*---------------------------------------------------------------------*/
 ;*    Les auto-modes (comme en emacs)                                  */
 ;*---------------------------------------------------------------------*/
@@ -576,11 +712,11 @@
 	      ("oon" . "meroon")))
 
 ;*---------------------------------------------------------------------*/
-;*    *case-sensitive* ...                                             */
+;*    *ast-case-sensitive* ...                                         */
 ;*---------------------------------------------------------------------*/
-(doc-define *case-sensitive*
+(doc-define *ast-case-sensitive*
 	    "Case sensitivity"
-	    #f)
+	    #t)
 
 ;*---------------------------------------------------------------------*/
 ;*    *user-heap-size*                                                 */
@@ -596,6 +732,40 @@
 	    "The way the reader reads input file ('plain or 'intern)"
 	    'plain)
 
+;*---------------------------------------------------------------------*/
+;*    *target-language* ...                                            */
+;*---------------------------------------------------------------------*/
+(doc-define *target-language*
+	    "The target language (either C or JVM)"
+	    (string->symbol bgl-configure-default-back-end))
+
+;*---------------------------------------------------------------------*/
+;*    *use-private?* ...                                               */
+;*---------------------------------------------------------------------*/
+(doc-define *use-private?*
+	    "Use private construction instead of pragma"
+	    #f)
+
+;*---------------------------------------------------------------------*/
+;*    *builtin-allocators* ...                                         */
+;*    -------------------------------------------------------------    */
+;*    The builtin allocators (used only for Kprof).                    */
+;*---------------------------------------------------------------------*/
+(define *builtin-allocators*
+   '(("CONS" . "make_pair")
+     ("%STRING->SYMBOL" . "make_symbol")
+     ("%MAKE-STRING" . "string_to_bstring_len")
+     ("%MAKE-OUTPUT-PORT" . "make_output_port")
+     ("%MAKE-INPUT-PORT" . "make_input_port")
+     ("%MAKE-ERROR-PORT" . "make_error_port")))
+
+;*---------------------------------------------------------------------*/
+;*    *eval-options* ...                                               */
+;*---------------------------------------------------------------------*/
+(doc-define *eval-options*
+	    "A user variable to store dynamic command line options"
+	    '())
+
 ;*---------------------------------------------------------------------*/
 ;*    Other variables that are defined inside the interpreter...       */
 ;*---------------------------------------------------------------------*/
