@@ -1,41 +1,10 @@
-;*---------------------------------------------------------------------*/
-;*   A pratical implementation for the Scheme programming language     */
-;*                                                                     */
-;*                                    ,--^,                            */
-;*                              _ ___/ /|/                             */
-;*                          ,;'( )__, ) '                              */
-;*                         ;;  //   L__.                               */
-;*                         '   \\   /  '                               */
-;*                              ^   ^                                  */
-;*                                                                     */
-;*   Copyright (c) 1992-1999 Manuel Serrano                            */
-;*                                                                     */
-;*     Bug descriptions, use reports, comments or suggestions are      */
-;*     welcome. Send them to                                           */
-;*       bigloo-request@kaolin.unice.fr                                */
-;*       http://kaolin.unice.fr/bigloo                                 */
-;*                                                                     */
-;*   This program is free software; you can redistribute it            */
-;*   and/or modify it under the terms of the GNU General Public        */
-;*   License as published by the Free Software Foundation; either      */
-;*   version 2 of the License, or (at your option) any later version.  */
-;*                                                                     */
-;*   This program is distributed in the hope that it will be useful,   */
-;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
-;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
-;*   GNU General Public License for more details.                      */
-;*                                                                     */
-;*   You should have received a copy of the GNU General Public         */
-;*   License along with this program; if not, write to the Free        */
-;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
-;*   MA 02111-1307, USA.                                               */
-;*---------------------------------------------------------------------*/
 ;*=====================================================================*/
 ;*    serrano/prgm/project/bigloo/comptime/Expand/object.scm           */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Fri May  3 10:13:58 1996                          */
-;*    Last change :  Sat Jan  2 15:37:29 1999 (serrano)                */
+;*    Last change :  Sun Jul 22 10:41:05 2001 (serrano)                */
+;*    Copyright   :  1996-2001 Manuel Serrano, see LICENSE file        */
 ;*    -------------------------------------------------------------    */
 ;*    The Object expanders                                             */
 ;*=====================================================================*/
@@ -44,22 +13,25 @@
 ;*    The module                                                       */
 ;*---------------------------------------------------------------------*/
 (module expand_object
-   (include "Object/class.sch")
    (import  tools_args
 	    tools_progn
 	    tools_misc
+	    tools_location
 	    type_type
 	    type_env
 	    expand_eps
 	    engine_param
 	    ast_var
 	    ast_ident
-	    object_class)
-   (export  (expand-with-access ::obj ::procedure)
-	    (expand-instantiate ::obj ::procedure)
-	    (expand-duplicate   ::obj ::procedure)
-	    (expand-widen!      ::obj ::procedure)
-	    (expand-shrink!     ::obj ::procedure)))
+	    ast_private
+	    object_class
+	    object_slots)
+   (export  (expand-with-access    ::obj ::procedure)
+	    (expand-instantiate    ::obj ::procedure)
+	    (expand-co-instantiate ::obj ::procedure)
+	    (expand-duplicate      ::obj ::procedure)
+	    (expand-widen!         ::obj ::procedure)
+	    (expand-shrink!        ::obj ::procedure)))
 
 ;*---------------------------------------------------------------------*/
 ;*    expand-with-access ...                                           */
@@ -67,38 +39,123 @@
 (define (expand-with-access x e)
    (match-case x
       ((?with-access ?instance (and (? pair?) ?slots) . (and (? pair?) ?body))
-       (let ((class (type-of-id with-access)))
-	  (if (not (class? class))
-	      (error #f "Illegal `with-access' form" x)
+       (let* ((loc   (find-location x))
+	      (class (type-of-id with-access loc)))
+	  (cond
+	     ((not (tclass? class))
+	      (error #f "Illegal `with-access' form" x))
+	     (else
 	      (let loop ((s slots))
 		 (cond
 		    ((null? s)
 		     (let* ((aux (mark-symbol-non-user! (gensym 'instance)))
 			    (mark aux)
-			    (instance (e instance e)))
-			(replace! x
-				  (with-lexical
-				   slots
-				   mark
-				   (lambda () 
-				      (let ((e (with-access-expander
-						e
-						mark
-						aux
-						(type-id class)
-						slots)))
-					 `(let ((,aux ,instance))
-					     ,(e (normalize-progn body)
-						 e))))))))
+			    (instance (e instance e))
+			    (class-id (type-id class)))
+			(multiple-value-bind (direct indexed)
+			   (class-split-slots x class slots)
+			   (replace! x
+				     (with-lexical
+				      slots
+				      mark
+				      (find-location x)
+				      (lambda () 
+					 (let ((e (with-access-expander
+						   e
+						   mark
+						   aux
+						   class-id
+						   direct)))
+					    (make-with-access-body aux
+								   instance
+								   class-id
+								   indexed
+								   body
+								   e))))))))
 		    ((not (pair? s))
 		     (error #f "Illegal `with-access' form" x))
 		    ((symbol? (car s))
 		     (loop (cdr s)))
 		    (else
-		     (error #f "Illegal `with-access' form" x)))))))
+		     (error #f
+			    (string-append
+			     "Illegal `with-access' form. Illegal slot identifier `"
+			     (with-output-to-string
+				(lambda () (display (car s))))
+			     "'")
+			    x))))))))
       (else
        (error #f "Illegal `with-access' form" x))))
 
+;*---------------------------------------------------------------------*/
+;*    class-split-slots ...                                            */
+;*    -------------------------------------------------------------    */
+;*    Split a slot list in two lists: the direct slots and the         */
+;*    indexed slots.                                                   */
+;*---------------------------------------------------------------------*/
+(define (class-split-slots x class slots)
+   (let ((mark (gensym 'with-access)))
+      (for-each (lambda (slot)
+		   (with-access::slot slot (id)
+		      (putprop! id mark slot)))
+		(tclass-all-slots class))
+      (let loop ((slots slots)
+		 (direct '())
+		 (indexed '()))
+	 (if (null? slots)
+	     (values direct indexed)
+	     (let ((slot (getprop (car slots) mark)))
+		(if (not (slot? slot))
+		    (error #f
+			   (string-append
+			    "Illegal `with-access' form. Unknown slot `"
+			    (symbol->string (car slots))
+			    "'")
+			   x)
+		    (if (slot-indexed slot)
+			(loop (cdr slots)
+			      direct
+			      (cons slot indexed))
+			(loop (cdr slots)
+			      (cons (car slots) direct)
+			      indexed))))))))
+
+;*---------------------------------------------------------------------*/
+;*    make-with-access-body ...                                        */
+;*---------------------------------------------------------------------*/
+(define (make-with-access-body aux instance class-id indexed body e)
+   (define (make-indexed-body)
+      (let loop ((indexed indexed))
+	 (cond
+	    ((null? indexed)
+	     (e (normalize-progn body) e))
+	    ((slot-read-only? (car indexed))
+	     (let* ((slot (car indexed))
+		    (sref (symbol-append (slot-id slot) '-ref))
+		    (full-sref (symbol-append class-id '- sref))
+		    (slen (symbol-append (slot-id slot) '-len))
+		    (full-slen (symbol-append class-id '- slen))
+		    (i (mark-symbol-non-user! (gensym 'index))))
+		`(let ((,slen (,full-slen ,aux)))
+		    (labels ((,sref (,i) (,full-sref ,aux ,i)))
+		       ,(loop (cdr indexed))))))
+	    (else
+	     (let* ((slot (car indexed))
+		    (sref (symbol-append (slot-id slot) '-ref))
+		    (full-sref (symbol-append class-id '- sref))
+		    (sset (symbol-append (slot-id slot) '-set!))
+		    (full-sset (symbol-append class-id '- sset))
+		    (slen (symbol-append (slot-id slot) '-len))
+		    (full-slen (symbol-append class-id '- slen))
+		    (i (mark-symbol-non-user! (gensym 'index)))
+		    (v (mark-symbol-non-user! (gensym 'value))))
+		`(let ((,slen (,full-slen ,aux)))
+		    (labels ((,sref (,i) (,full-sref ,aux ,i))
+			     (,sset (,i ,v) (,full-sset ,aux ,i ,v)))
+		       ,(loop (cdr indexed)))))))))
+   `(let ((,aux ,instance))
+       ,(make-indexed-body)))
+
 ;*---------------------------------------------------------------------*/
 ;*    with-access-expander ...                                         */
 ;*---------------------------------------------------------------------*/
@@ -116,7 +173,7 @@
 	     (if (and (memq var slots)
 		      (let ((cell (assq var (lexical-stack))))
 			 (and (pair? cell) (eq? (cdr cell) mark))))
-		 (epairify
+		 (object-epairify
 		  `(,(symbol-append class '- var '-set!) ,instance ,val)
 		  x)
 		 (begin
@@ -130,41 +187,40 @@
 ;*---------------------------------------------------------------------*/
 (define (expand-instantiate x e)
    (match-case x
-      ((?instantiate . ?provided)
-       (let ((class (type-of-id instantiate)))
-	  (if (not (class? class))
-	      (error #f "Illegal `instantiate' form" x)
-	      (replace! x (e (instantiate->make x class provided) e)))))
+      ((?instantiate . ?-)
+       (let ((class (type-of-id instantiate (find-location x))))
+	  (cond
+	     ((not (tclass? class))
+	      (error #f "Illegal `instantiate' form" x))
+	     ((tclass-abstract? class)
+	      (error #f "Abstract classes can't be instantiated" x))
+	     (else
+	      (replace! x (instantiate->make x class e))))))
       (else
        (error #f "Illegal `instantiate' form" x))))
 
 ;*---------------------------------------------------------------------*/
 ;*    instantiate->make ...                                            */
 ;*---------------------------------------------------------------------*/
-(define (instantiate->make form class provided)
-   (let* ((slots            (if (not (class-widening class))
-				(class-slots class)
-				(append (class-slots (class-its-super class))
-					(class-slots class))))
-	  (len              (length slots))
-	  (vargs            (make-vector (length slots)))
-	  (new              (gensym 'new))
-	  (find-slot-offset (lambda (s-name)
-			       (let loop ((slots slots)
-					  (i     0))
-				  (cond
-				     ((null? slots)
-				      (error
-				       #f
-				       (string-append
-					"Illegal `instantiate' form (unknown slot "
-					(symbol->string s-name)
-					")")
-				       form))
-				     ((eq? (slot-id (car slots)) s-name)
-				      i)
-				     (else
-				      (loop (cdr slots) (+fx i 1))))))))
+(define (instantiate->make form class e)
+   (let ((make-name (symbol-append 'make- (type-id class))))
+      (instantiate->fill form
+			 class
+			 (lambda (largs)
+			    (if (epair? form)
+				(econs make-name largs (cer form))
+				(cons make-name largs)))
+			 e)))
+
+;*---------------------------------------------------------------------*/
+;*    instantiate->fill ...                                            */
+;*---------------------------------------------------------------------*/
+(define (instantiate->fill form class alloc e)
+   (let* ((slots    (tclass-all-slots class))
+	  (len      (length slots))
+	  (vargs    (make-vector (length slots)))
+	  (new      (gensym 'new))
+	  (provided (cdr form)))    
       ;; we collect the default values
       (let loop ((i     0)
 		 (slots slots))
@@ -184,19 +240,28 @@
 	     'done
 	     (let ((p (car provided)))
 		(match-case p
-		   ((?s-name ?value)
-		    (let ((pval (vector-ref vargs (find-slot-offset s-name))))
+		   (((and (? symbol?) ?s-name) ?value)
+		    ;; plain slot
+		    (let ((pval (vector-ref vargs
+					    (find-slot-offset slots
+							      s-name
+							      "instantiate"
+							      p))))
 		       (set-car! pval #t)
-		       (set-cdr! pval (epairify value p))))
-		   ((?s-name ?len ?value)
-		    (let* ((snum (find-slot-offset s-name))
+		       (set-cdr! pval (object-epairify value p))))
+		   (((and (? symbol?) ?s-name) ?len ?value)
+		    ;; indexed slot
+		    (let* ((snum (find-slot-offset slots
+						   s-name
+						   "instantiate"
+						   p))
 			   (slot (list-ref slots snum)))
-		       (if (not (slot-dyna-indexed? slot))
+		       (if (not (slot-indexed slot))
 			   (error #f "Illegal `instantiate' form" form)
 			   (let* ((pval (vector-ref vargs snum))
 				  (plen (vector-ref vargs (-fx snum 1))))
 			      (set-car! pval #t)
-			      (set-cdr! pval (epairify value p))
+			      (set-cdr! pval (object-epairify value p))
 			      (set-car! plen #t)
 			      (set-cdr! plen len)))))
 		   (else
@@ -225,14 +290,12 @@
 		 (largs    '())
 		 (virtuals '()))
 	 (if (=fx i len)
-	     (let* ((make-name (symbol-append 'make- (type-id class)))
-		    (alloc     `(,make-name ,@(reverse! largs))))
-		(if (null? virtuals)
-		    alloc
-		    `(let ((,new ,alloc))
-			,@virtuals
-			,new)))
-	     (let ((value (cdr (vector-ref vargs i))))
+	     (if (null? virtuals)
+		 (alloc (reverse! largs))
+		 `(let ((,new ,(alloc (reverse! largs))))
+		     ,@(reverse! virtuals)
+		     ,new))
+	     (let ((value (e (cdr (vector-ref vargs i)) e)))
 		(cond
 		   ((slot-virtual? (car slots))
 		    (loop (+fx i 1)
@@ -240,34 +303,148 @@
 			  largs
 			  (if (eq? value #unspecified)
 			      virtuals
-			      (cons (make-virtual-set (car slots)
-						      value
-						      new
-						      class
-						      form
-						      "instantiate")
-				    virtuals))))
+			      (if (epair? value)
+				  (econs (make-virtual-set (car slots)
+							   value
+							   new
+							   class
+							   form
+							   "instantiate")
+					 virtuals
+					 (cer value))
+				  (cons (make-virtual-set (car slots)
+							  value
+							  new
+							  class
+							  form
+							  "instantiate")
+					virtuals)))))
 		   (else
 		    (loop (+fx i 1)
 			  (cdr slots)
-			  (cons value largs)
+			  (if (epair? value)
+			      (econs value largs (cer value))
+			      (cons value largs))
 			  virtuals))))))))
    
+;*---------------------------------------------------------------------*/
+;*    expand-co-instantiate ...                                        */
+;*---------------------------------------------------------------------*/
+(define (expand-co-instantiate x e)
+   (match-case x
+      ((co-instantiate ?bindings . ?body)
+       (replace! x (co-instantiate->let bindings body x e)))
+      (else
+       (error #f "Illegal `co-instantiate' form" x))))
+
+;*---------------------------------------------------------------------*/
+;*    co-instantiate->let ...                                          */
+;*---------------------------------------------------------------------*/
+(define (co-instantiate->let bindings body x e)
+   (define (find-instantiate-class expr bdg loc)
+      (match-case expr
+	 ((?instantiate . ?body)
+	  (let* ((loc (find-location/loc body loc))
+		 (id.type (parse-id instantiate loc))
+		 (kclass (cdr id.type)))
+	     (cond
+		((not (eq? (car id.type) 'instantiate))
+		 (error #f
+			"co-instantiate:Illegal binding"
+			bdg))
+		((not (tclass? kclass))
+		 (error #f
+			"co-instantiate:Illegal class"
+			bdg))
+		((tclass-abstract? kclass)
+		 (error #f
+			"co-instantiate:Abstract classes can't be instantiated"
+			bdg))
+		(else
+		 kclass))))
+	 (else
+	  (error #f
+		 "co-instantiate:Illegal binding"
+		 bdg))))
+   (let ((loc (find-location x)))
+      (let loop ((bindings bindings)
+		 (user-variables '())
+		 (private-variables '())
+		 (rewrite-bindings '())
+		 (classes '())
+		 (instantiates '()))
+	 (if (null? bindings)
+	     (letrec ((e1 (lambda (x _)
+			     (if (symbol? x)
+				 (let ((cell (assq x rewrite-bindings)))
+				    (if (pair? cell)
+					(cdr cell)
+					x))
+				 (match-case x
+				    ((begin ?x)
+				     (e1 x e))
+				    (else
+				     (e x e)))))))
+		`(let ,(map (lambda (id class)
+			       (let* ((cid (tclass-id class))
+				      (tv (make-typed-ident id cid)))
+				  `(,tv (,(symbol-append 'allocate- cid)))))
+			    private-variables
+			    classes)
+		    (let ,(reverse!
+			   (map (lambda (user private class instantiate)
+				   (let* ((cid (tclass-id class))
+					  (fill (symbol-append 'fill- cid '!))
+					  (alloc (lambda (args)
+						    `(,fill ,private ,@args))))
+				      `(,user ,(instantiate->fill instantiate
+								  class
+								  alloc
+								  e1))))
+				user-variables
+				private-variables
+				classes
+				instantiates))
+		       ,(e `(begin ,@body) e))))
+	     (match-case (car bindings)
+		(((and ?var (? symbol?)) ?expr)
+		 (let* ((bdg (car bindings))
+			(loc (find-location/loc bdg x))
+			(id.type (parse-id var loc))
+			(id (car id.type))
+			(t (cdr id.type))
+			(priv (gensym))
+			(kclass (find-instantiate-class expr bdg loc)))
+		    (loop (cdr bindings)
+			  (cons var user-variables)
+			  (cons priv private-variables)
+			  (cons (cons id priv) rewrite-bindings)
+			  (cons kclass classes)
+			  (cons expr instantiates))))
+		(else
+		 (error #f
+			"co-instantiate:Illegal binding"
+			(car bindings))))))))
+
 ;*---------------------------------------------------------------------*/
 ;*    expand-duplicate ...                                             */
 ;*---------------------------------------------------------------------*/
 (define (expand-duplicate  x e)
    (match-case x
       ((?duplicate ?dup . ?prov)
-       (let* ((id.type (parse-id duplicate))
+       (let* ((id.type (parse-id duplicate (find-location x)))
 	      (id      (car id.type))
 	      (class   (cdr id.type)))
-	  (if (not (class? class))
+	  (cond
+	     ((not (tclass? class))
 	      (error #f
 		     (string-append "duplicate:Illegal class type:"
 				    (symbol->string id))
-		     x)
-	      (replace! x (e (duplicate->make x class dup prov) e)))))
+		     x))
+	     ((tclass-abstract? class)
+	      (error #f "Abstract classes can't be duplicated" x))
+	     (else
+	      (replace! x (e (duplicate->make x class dup prov) e))))))
       (else
        (error #f "Illegal `duplicate' form" x))))
 
@@ -281,51 +458,39 @@
 ;*    proper fields).                                                  */
 ;*---------------------------------------------------------------------*/
 (define (duplicate->make form class duplicated provided)
-   (let* ((slots            (if (not (class-widening class))
-				(class-slots class)
-				(append (class-slots (class-its-super class))
-					(class-slots class))))
+   (let* ((slots            (tclass-all-slots class))
 	  (len              (length slots))
 	  (dup-var          (mark-symbol-non-user! (gensym 'duplicated)))
-	  (dup-var-typed    (symbol-append dup-var 4dots (type-id class)))
+	  (dup-var-typed    (make-typed-ident dup-var (type-id class)))
 	  (new              (gensym 'new))
-	  (vargs            (make-vector (length slots)))
-	  (find-slot-offset (lambda (s-name)
-			       (let loop ((slots slots)
-					  (i     0))
-				  (cond
-				     ((null? slots)
-				      (error
-				       #f
-				       (string-append
-					"Illegal `duplicate' form (unknown slot "
-					(symbol->string s-name)
-					")")
-				       form))
-				     ((eq? (slot-id (car slots)) s-name)
-				      i)
-				     (else
-				      (loop (cdr slots) (+fx i 1))))))))
+	  (vargs            (make-vector (length slots))))
       ;; we collect the provided values
       (let loop ((provided provided))
 	 (if (null? provided)
 	     'done
 	     (let ((p (car provided)))
 		(match-case p
-		   ((?s-name ?value)
+		   (((and (? symbol?) ?s-name) ?value)
+		    ;; plain slot
 		    (vector-set! vargs
-				 (find-slot-offset s-name)
-				 (cons #t (epairify value p))))
-		   ((?s-name ?len ?value)
-		    (let* ((snum (find-slot-offset s-name))
+				 (find-slot-offset slots
+						   s-name
+						   "duplicate"
+						   p)
+				 (cons #t (object-epairify value p))))
+		   (((and (? symbol?) ?s-name) ?len ?value)
+		    ;; index slot
+		    (let* ((snum (find-slot-offset slots
+						   s-name
+						   "duplicate"
+						   p))
 			   (slot (list-ref slots snum)))
-		       (if (not (or (slot-stat-indexed? slot)
-				    (slot-dyna-indexed? slot)))
+		       (if (not (slot-indexed slot))
 			   (error #f "Illegal `duplicate' form" form)
 			   (begin
 			      (vector-set! vargs
 					   snum
-					   (cons #t (epairify value p)))
+					   (cons #t (object-epairify value p)))
 			      (vector-set! vargs (-fx snum 1)
 					   (cons #f len))))))
 		   (else
@@ -341,10 +506,7 @@
 		    ;; a value is already provided for this object
 		    'nothing
 		    (let ((slot (car slots)))
-		       ;; no value is provided for this object we pick
-		       ;; one from this duplicated object.
-		       (if (or (slot-dyna-indexed? slot)
-			       (slot-stat-indexed? slot))
+		       (if (slot-indexed slot)
 			   ;; for indexed slot, we pick the first value
 			   ;; (which is suposed to be existing).
 			   (let ((a-name (symbol-append (type-id class)
@@ -354,6 +516,8 @@
 			      (vector-set! vargs
 					   i
 					   (cons #t `(,a-name ,dup-var 0))))
+			   ;; no value is provided for this object we pick
+			   ;; one from this duplicated object.
 			   (let ((a-name (symbol-append (type-id class)
 							'-
 							(slot-id slot))))
@@ -372,7 +536,7 @@
 		    (alloc     `(,make-name ,@(reverse! largs))))
 		`(let* ((,dup-var-typed ,duplicated)
 			(,new           ,alloc))
-		    ,@virtuals
+		    ,@(reverse! virtuals)
 		    ,new))
 	     (let ((value (cdr (vector-ref vargs i))))
 		(cond
@@ -399,8 +563,8 @@
 (define (expand-widen! x e)
    (match-case x
       ((?widen! ?obj . ?provided)
-       (let ((class (type-of-id widen!)))
-	  (if (and (class? class) (class-widening class))
+       (let ((class (type-of-id widen! (find-location x))))
+	  (if (and (tclass? class) (tclass-widening class))
 	      (replace! x (e (make-widening x class obj provided) e))
 	      (error #f
 		     (string-append "widen!:Illegal class type:"
@@ -413,30 +577,14 @@
 ;*    make-widening ...                                                */
 ;*---------------------------------------------------------------------*/
 (define (make-widening form class obj provided)
-   (let* ((slots            (class-slots class))
-	  (holder           (class-holder class))
-	  (len              (length slots))
-	  (vargs            (make-vector (length slots)))
-	  (dup-var          (mark-symbol-non-user! (gensym 'obj)))
-	  (dup-var-typed    (symbol-append dup-var 4dots (type-id class)))
-	  (pragma-typed     (symbol-append 'pragma 4dots (type-id class)))
-	  (pragma-format    (string-append "((" (type-name class) ")($1))"))
-	  (find-slot-offset (lambda (s-name)
-			       (let loop ((slots slots)
-					  (i     0))
-				  (cond
-				     ((null? slots)
-				      (error
-				       #f
-				       (string-append
-					"Illegal `widen!' form (unknown slot "
-					(symbol->string s-name)
-					")")
-				       form))
-				     ((eq? (slot-id (car slots)) s-name)
-				      i)
-				     (else
-				      (loop (cdr slots) (+fx i 1))))))))
+   (let* ((slots         (tclass-slots class))
+	  (holder        (tclass-holder class))
+	  (len           (length slots))
+	  (vargs         (make-vector (length slots)))
+	  (dup-var       (mark-symbol-non-user! (gensym 'obj)))
+	  (tid           (type-id class))
+	  (dup-var-typed (make-typed-ident dup-var tid))
+	  (cast          (make-private-sexp 'cast tid obj)))
       ;; we collect the default values
       (let loop ((i     0)
 		 (slots slots))
@@ -454,20 +602,25 @@
 	     'done
 	     (let ((p (car provided)))
 		(match-case p
-		   ((?s-name ?value)
+		   (((and (? symbol?) ?s-name) ?value)
 		    (vector-set! vargs
-				 (find-slot-offset s-name)
-				 (cons #t (epairify value p))))
-		   ((?s-name ?len ?value)
-		    (let* ((snum (find-slot-offset s-name))
+				 (find-slot-offset slots
+						   s-name
+						   "widen!"
+						   p)
+				 (cons #t (object-epairify value p))))
+		   (((and (? symbol?) ?s-name) ?len ?value)
+		    (let* ((snum (find-slot-offset slots
+						   s-name
+						   "widen!"
+						   p))
 			   (slot (list-ref slots snum)))
-		       (if (not (or (slot-stat-indexed? slot)
-				    (slot-dyna-indexed? slot)))
+		       (if (not (slot-indexed slot))
 			   (error #f "Illegal `widen!' form" form)
 			   (begin
 			      (vector-set! vargs
 					   snum
-					   (cons #t (epairify value p)))
+					   (cons #t (object-epairify value p)))
 			      (vector-set! vargs (-fx snum 1)
 					   (cons #f len))))))
 		   (else
@@ -496,12 +649,11 @@
 		 (largs    '())
 		 (virtuals '()))
 	 (if (=fx i len)
-	     (let ((widening (symbol-append (class-widening class)
+	     (let ((widening (symbol-append (tclass-widening class)
 					    '-
 					    (type-id class))))
 		(if *unsafe-type*
-		    `(let ((,dup-var-typed (,pragma-typed ,pragma-format
-							  ,obj)))
+		    `(let ((,dup-var-typed ,cast))
 			(object-widening-set!
 			 ,dup-var
 			 (,widening ,@(reverse! largs)))
@@ -511,10 +663,11 @@
 			 ,dup-var
 			 ((@ class-num __object)
 			  (@ ,(global-id holder) ,(global-module holder))))
-			,@virtuals
+			;; in backtracing mode (-gbdb2) we mark the allocation
+			((@ object-mark-producer! __object) ,dup-var)
+			,@(reverse! virtuals)
 			,dup-var)
-		    `(let ((,dup-var-typed (,pragma-typed ,pragma-format
-							  ,obj)))
+		    `(let ((,dup-var-typed ,cast))
 			(if (object-widening ,dup-var)
 			    (shrink! ,dup-var))
 			(if (eq? ((@ class-super __object)
@@ -531,7 +684,9 @@
 				((@ class-num __object)
 				 (@ ,(global-id holder)
 				    ,(global-module holder))))
-			       ,@virtuals
+			       ;; in backtracing mode, we mark the allocation
+			       ((@ object-mark-producer! __object) ,dup-var)
+			       ,@(reverse! virtuals)
 			       ,dup-var)
 			    (error
 			     "widen!"
@@ -606,17 +761,36 @@
 			     (symbol->string (slot-id slot))
 			     ")")
 	      form)
-       (epairify `(,(symbol-append (class-id class) '- (slot-id slot) '-set!)
-		   ,var
-		   ,value)
-		 value)))
+       (object-epairify
+	`(,(symbol-append (tclass-id class) '- (slot-id slot) '-set!)
+	  ,var
+	  ,value)
+	value)))
 
 ;*---------------------------------------------------------------------*/
-;*    epairify ...                                                     */
+;*    object-epairify ...                                              */
 ;*---------------------------------------------------------------------*/
-(define (epairify obj epair)
+(define (object-epairify obj epair)
    (if (epair? epair)
        (if (pair? obj)
 	   (econs (car obj) (cdr obj) (cer epair))
-	   (epairify `(begin ,obj) epair))
+	   (object-epairify `(begin ,obj) epair))
        obj))
+
+;*---------------------------------------------------------------------*/
+;*    find-slot-offset ...                                             */
+;*---------------------------------------------------------------------*/
+(define (find-slot-offset slots::pair-nil name::symbol form::bstring sexp)
+   (let loop ((slots slots)
+	      (i 0))
+      (cond
+	 ((null? slots)
+	  (error #f
+		 (string-append "Illegal `" form "' form (unknown slot "
+				(symbol->string name)
+				")")
+		 sexp))
+	 ((eq? (slot-id (car slots)) name)
+	  i)
+	 (else   
+	  (loop (cdr slots) (+fx i 1))))))
