@@ -1,41 +1,10 @@
-;*---------------------------------------------------------------------*/
-;*   A pratical implementation for the Scheme programming language     */
-;*                                                                     */
-;*                                    ,--^,                            */
-;*                              _ ___/ /|/                             */
-;*                          ,;'( )__, ) '                              */
-;*                         ;;  //   L__.                               */
-;*                         '   \\   /  '                               */
-;*                              ^   ^                                  */
-;*                                                                     */
-;*   Copyright (c) 1992-1999 Manuel Serrano                            */
-;*                                                                     */
-;*     Bug descriptions, use reports, comments or suggestions are      */
-;*     welcome. Send them to                                           */
-;*       bigloo-request@kaolin.unice.fr                                */
-;*       http://kaolin.unice.fr/bigloo                                 */
-;*                                                                     */
-;*   This program is free software; you can redistribute it            */
-;*   and/or modify it under the terms of the GNU General Public        */
-;*   License as published by the Free Software Foundation; either      */
-;*   version 2 of the License, or (at your option) any later version.  */
-;*                                                                     */
-;*   This program is distributed in the hope that it will be useful,   */
-;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
-;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
-;*   GNU General Public License for more details.                      */
-;*                                                                     */
-;*   You should have received a copy of the GNU General Public         */
-;*   License along with this program; if not, write to the Free        */
-;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
-;*   MA 02111-1307, USA.                                               */
-;*---------------------------------------------------------------------*/
 ;*=====================================================================*/
-;*    serrano/prgm/project/bigloo/comptime/Globalize/node.scm          */
+;*    serrano/prgm/project/bigloo2.3/comptime/Globalize/node.scm       */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Fri Jan 27 14:12:58 1995                          */
-;*    Last change :  Mon Sep 21 08:51:39 1998 (serrano)                */
+;*    Last change :  Thu Jul 13 11:11:28 2000 (serrano)                */
+;*    Copyright   :  1995-2000 Manuel Serrano, see LICENSE file        */
 ;*    -------------------------------------------------------------    */
 ;*    We transforme the ast in order to fix the free variables, to     */
 ;*    remove the useless local functions (globalized or integrated     */
@@ -284,11 +253,11 @@
       node))
 
 ;*---------------------------------------------------------------------*/
-;*    glo! ::pragma ...                                                */
+;*    glo! ::extern ...                                                */
 ;*---------------------------------------------------------------------*/
-(define-method (glo! node::pragma integrator)
-   (with-access::pragma node (args)
-      (glo*! args integrator)
+(define-method (glo! node::extern integrator)
+   (with-access::extern node (expr*)
+      (glo*! expr* integrator)
       node))
 
 ;*---------------------------------------------------------------------*/
@@ -548,7 +517,7 @@
 	 (fun (instantiate::var
 		 (loc loc)
 		 (type *_*)
-		 (variable (find-global make-p 'foreign))))
+		 (variable (find-global/module make-p 'foreign))))
 	 (args (list (instantiate::var
 			(loc loc)
 			(type *_*)
@@ -600,7 +569,7 @@
       (fun (instantiate::var
 	      (loc loc)
 	      (type *_*)
-	      (variable (find-global 'procedure-set! 'foreign))))
+	      (variable (find-global/module 'procedure-set! 'foreign))))
       (args (list (instantiate::var
 		     (loc loc)
 		     (type *_*)
