@@ -1,41 +1,10 @@
-;*---------------------------------------------------------------------*/
-;*   A pratical implementation for the Scheme programming language     */
-;*                                                                     */
-;*                                    ,--^,                            */
-;*                              _ ___/ /|/                             */
-;*                          ,;'( )__, ) '                              */
-;*                         ;;  //   L__.                               */
-;*                         '   \\   /  '                               */
-;*                              ^   ^                                  */
-;*                                                                     */
-;*   Copyright (c) 1992-1999 Manuel Serrano                            */
-;*                                                                     */
-;*     Bug descriptions, use reports, comments or suggestions are      */
-;*     welcome. Send them to                                           */
-;*       bigloo-request@kaolin.unice.fr                                */
-;*       http://kaolin.unice.fr/bigloo                                 */
-;*                                                                     */
-;*   This program is free software; you can redistribute it            */
-;*   and/or modify it under the terms of the GNU General Public        */
-;*   License as published by the Free Software Foundation; either      */
-;*   version 2 of the License, or (at your option) any later version.  */
-;*                                                                     */
-;*   This program is distributed in the hope that it will be useful,   */
-;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
-;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
-;*   GNU General Public License for more details.                      */
-;*                                                                     */
-;*   You should have received a copy of the GNU General Public         */
-;*   License along with this program; if not, write to the Free        */
-;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
-;*   MA 02111-1307, USA.                                               */
-;*---------------------------------------------------------------------*/
 ;*=====================================================================*/
-;*    serrano/prgm/project/bigloo/comptime/Integrate/node.scm          */
+;*    serrano/prgm/project/bigloo2.3/comptime/Integrate/node.scm       */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Tue Mar 14 17:30:55 1995                          */
-;*    Last change :  Wed Jun  3 17:02:02 1998 (serrano)                */
+;*    Last change :  Thu Jul 13 11:25:06 2000 (serrano)                */
+;*    Copyright   :  1995-2000 Manuel Serrano, see LICENSE file        */
 ;*    -------------------------------------------------------------    */
 ;*    The computation of K and K* properties.                          */
 ;*=====================================================================*/
@@ -217,7 +186,7 @@
 	     (app-fun-set! node
 			   (instantiate::var
 			      (loc loc)
-			      (type type)
+			      (type (variable-type (the-global fun)))
 			      (variable (the-global fun)))))
 	 ;; we globalize the actuals before adding new one
 	 ;; otherwise, we could produce illegal `cell-ref'
@@ -249,7 +218,7 @@
 	 node)))
 	  
 ;*---------------------------------------------------------------------*/
-;*    glo! ::app-ly ...                                                 */
+;*    glo! ::app-ly ...                                                */
 ;*---------------------------------------------------------------------*/
 (define-method (glo! node::app-ly integrator)
    (with-access::app-ly node (fun arg)
@@ -267,11 +236,11 @@
       node))
 
 ;*---------------------------------------------------------------------*/
-;*    glo! ::pragma ...                                                */
+;*    glo! ::extern ...                                                */
 ;*---------------------------------------------------------------------*/
-(define-method (glo! node::pragma integrator)
-   (with-access::pragma node (args)
-      (glo*! args integrator)
+(define-method (glo! node::extern integrator)
+   (with-access::extern node (expr*)
+      (glo*! expr* integrator)
       node))
 
 ;*---------------------------------------------------------------------*/
