@@ -1,41 +1,10 @@
-;*---------------------------------------------------------------------*/
-;*   A pratical implementation for the Scheme programming language     */
-;*                                                                     */
-;*                                    ,--^,                            */
-;*                              _ ___/ /|/                             */
-;*                          ,;'( )__, ) '                              */
-;*                         ;;  //   L__.                               */
-;*                         '   \\   /  '                               */
-;*                              ^   ^                                  */
-;*                                                                     */
-;*   Copyright (c) 1992-1999 Manuel Serrano                            */
-;*                                                                     */
-;*     Bug descriptions, use reports, comments or suggestions are      */
-;*     welcome. Send them to                                           */
-;*       bigloo-request@kaolin.unice.fr                                */
-;*       http://kaolin.unice.fr/bigloo                                 */
-;*                                                                     */
-;*   This program is free software; you can redistribute it            */
-;*   and/or modify it under the terms of the GNU General Public        */
-;*   License as published by the Free Software Foundation; either      */
-;*   version 2 of the License, or (at your option) any later version.  */
-;*                                                                     */
-;*   This program is distributed in the hope that it will be useful,   */
-;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
-;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
-;*   GNU General Public License for more details.                      */
-;*                                                                     */
-;*   You should have received a copy of the GNU General Public         */
-;*   License along with this program; if not, write to the Free        */
-;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
-;*   MA 02111-1307, USA.                                               */
-;*---------------------------------------------------------------------*/
 ;*=====================================================================*/
 ;*    serrano/prgm/project/bigloo/comptime/Module/class.scm            */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Wed Jun  5 10:52:20 1996                          */
-;*    Last change :  Fri Feb  5 10:01:08 1999 (serrano)                */
+;*    Last change :  Fri Nov  2 12:07:26 2001 (serrano)                */
+;*    Copyright   :  1996-2001 Manuel Serrano, see LICENSE file        */
 ;*    -------------------------------------------------------------    */
 ;*    The class clause handling                                        */
 ;*=====================================================================*/
@@ -45,7 +14,6 @@
 ;*---------------------------------------------------------------------*/
 (module module_class
    (include "Ast/unit.sch"
-	    "Object/class.sch"
 	    "Tools/trace.sch")
    (import  module_module
 	    module_impuse
@@ -53,6 +21,7 @@
 	    engine_param
 	    tools_shape
 	    tools_error
+	    tools_location
 	    type_type
 	    type_env
 	    ast_ident
@@ -61,12 +30,13 @@
 	    object_class
 	    object_plain-access
 	    object_wide-access)
-   (export  (declare-class!      cdef mod::symbol imp::symbol fin::bool ::obj)
-	    (declare-wide-class! cdef mod::symbol imp::symbol ::obj)
-	    (get-class-hash      class-id::symbol fields)
+   (export  (declare-class! ::pair ::symbol ::symbol ::bool ::bool ::obj ::obj)
+	    (declare-wide-class! ::pair ::symbol ::symbol ::obj ::obj)
+	    (get-class-hash ::symbol ::pair-nil)
 	    (get-object-unit)
 	    (get-method-unit)
-	    (class-finalizer)))
+	    (class-finalizer)
+	    (delay-class-accessors!::type ::type delayed)))
 
 ;*---------------------------------------------------------------------*/
 ;*    Object units ...                                                 */
@@ -89,117 +59,111 @@
 ;*---------------------------------------------------------------------*/
 ;*    declare-class! ...                                               */
 ;*---------------------------------------------------------------------*/
-(define (declare-class! class-def module import final? src-def)
-   (trace (ast 2) "declare-class!: " src-def #\Newline)
-   ;; We create the class holder
-   ;; and we create a type for this class
-   (let* ((class-var (car class-def))
-	  (class-id  (id-of-id class-var))
-	  (holder    (if (or (eq? import 'import) (eq? import 'use))
-			 (import-parser module class-id)
-			 (begin
-			    (produce-module-clause! `(,import ,class-id))
-			    (find-global class-id module))))
-	  (type      (declare-class-type! class-def holder #f final? src-def)))
-      ;; type can be something else than a class if an error has been found
-      (if (class? type)
-	  (begin
-	     (cond
-		((memq import '(export static))
-		 ;; this is a domestic class, we have to declare the
-		 ;; global variable that holds the class object
-		 (make-add-class! holder type src-def)
-		 ;; we also have to declare the accessors.
-		 (add-declaration!
-		  type
-		  (delay
-		     (make-plain-class-accessors! class-def
-						  type
-						  src-def
-						  import))))
-		((and (>=fx *optim* 2) (<=fx *bdb-debug* 0))
-		 ;; when compiling for optimization (and not debugging)
-		 ;; we inline the class accessors, that it, we make as if
-		 ;; class accessors where locals.
-		 (add-declaration!
-		  type
-		  (delay
-		     (make-plain-class-accessors! class-def
-						  type
-						  src-def
-						  import))))
-		(else
-		 ;; when importing a class, we import the accessors...
-		 (add-declaration!
-		  type
-		  (delay
-		     (import-plain-class-accessors! class-def
-						    type
-						    src-def
-						    module)))))
-	     type)
-	  #unspecified)))
+(define (declare-class! class-def module import final? abstract? def-src decl-src)
+   (cond
+      ((memq import '(export static))
+       (declare-export-class! gen-plain-class-accessors!
+			      class-def module
+			      (if final? 'final 'plain)
+			      abstract?
+			      def-src decl-src import))
+      (else
+       (declare-import-class! import-plain-class-accessors!
+			      class-def module
+			      (if final? 'final 'plain)
+			      abstract?
+			      def-src decl-src))))
 
 ;*---------------------------------------------------------------------*/
 ;*    declare-wide-class! ...                                          */
 ;*---------------------------------------------------------------------*/
-(define (declare-wide-class! class-def module import src-def)
-   ;; We create the class holder
-   ;; and we create a type for this class
-   (let* ((class-var (car class-def))
-	  (class-id  (id-of-id class-var))
-	  (holder    (if (or (eq? import 'import) (eq? import 'use))
-			 (import-parser module class-id)
-			 (begin
-			    (produce-module-clause! `(,import ,class-id))
-			    (find-global class-id module))))
-	  (type      (declare-class-type! class-def
-					  holder
-					  (gensym 'widening)
-					  #f
-					  src-def)))
-      ;; type can be something else than a class is an error has been found
-      (if (class? type)
-	  ;; then, we declare the class when it is a class definition
-	  (cond
-	     ((memq import '(export static))
-	      ;; this is a domestic class, we have to declare the
-	      ;; global variable that holds the class object
-	      (make-add-class! holder type src-def)
-	      ;; we also have to declare the accessors.
-	      (add-declaration!
-	       type
-	       (delay
-		  (make-wide-class-accessors! class-def
-					      type
-					      src-def
-					      import))))
-	     ((and (>=fx *optim* 2) (<=fx *bdb-debug* 0))
-	      ;; when compiling for optimization (and not debugging)
-	      ;; we inline the class accessors, that it, we make as if
-	      ;; class accessors where locals.
-	      (add-declaration!
-	       type
-	       (delay
-		  (make-wide-class-accessors! class-def
-					      type
-					      src-def
-					      import))))
-	     (else
-	      ;; when importing a class, we import the accessors...
-	      (add-declaration!
-	       type
-	       (delay
-		  (import-wide-class-accessors! class-def
-						type
-						src-def
-						module))))))))
+(define (declare-wide-class! class-def module import def-src decl-src)
+   (cond
+      ((memq import '(export static))
+       (declare-export-class! gen-wide-class-accessors!
+			      class-def module 'wide #f
+			      def-src decl-src import))
+      (else
+       (declare-import-class! import-wide-class-accessors!
+			      class-def module 'wide #f
+			      def-src decl-src))))
 
 ;*---------------------------------------------------------------------*/
-;*    add-declaration! ...                                             */
+;*    declare-export-class! ...                                        */
 ;*---------------------------------------------------------------------*/
-(define (add-declaration! type delayed)
-   (set! *class-accesses* (cons (cons type delayed) *class-accesses*)))
+(define (declare-export-class! gen cdef module kind abstract? src-def src-decl import)
+   (trace (ast 2) "declare-export-class!: " src-def #\Newline)
+   ;; We create the class holder
+   ;; and we create a type for this class
+   (let* ((loc (find-location src-def))
+	  (class-var (car cdef))
+	  (class-id  (id-of-id class-var loc))
+	  (holder (begin
+	 	     (produce-module-clause! `(,import ,class-id))
+		     (find-global/module class-id module)))
+	  (final? (eq? kind 'final))
+	  (wide (if (eq? kind 'wide) 'widening #f))
+	  (tclass (declare-class-type! cdef holder wide
+				       final? abstract?
+				       src-def)))
+      ;; some paranoid checking
+      [assert (tclass) (tclass? tclass)]
+      ;; we mark that the holder is read-only
+      (global-set-read-only! holder)
+      ;; we store the src-import location in order to print a nice error
+      ;; message if that tclass is not defined
+      (type-import-location-set! tclass (find-location/loc src-decl loc))
+      ;; tclass can be something else than a class if an error has been found
+      (delay-class-accessors!
+       tclass
+       (delay
+	  (multiple-value-bind (concretes virtuals)
+	     (gen cdef tclass src-def import)
+	     (trace (ast 3)
+		    "declare-class! (export static): "#\Newline
+		    "    concretes: " concretes #\Newline
+		    "     virtuals: " virtuals #\Newline)
+	     ;; this is a domestic class, we have to declare the
+	     ;; global variable that holds the class object
+	     (make-add-class! holder tclass src-def virtuals)
+	     ;; we return the list of concretes
+	     concretes)))))
+   
+;*---------------------------------------------------------------------*/
+;*    declare-import-class! ...                                        */
+;*---------------------------------------------------------------------*/
+(define (declare-import-class! gen cdef module kind abstract? src-def src-decl)
+   (trace (ast 2) "declare-import-class!: " src-def #\Newline)
+   ;; We create the class holder
+   ;; and we create a type for this class
+   (let* ((loc (find-location src-def))
+	  (class-var (car cdef))
+	  (class-id (id-of-id class-var loc))
+	  (holder (import-parser module class-id))
+	  (final? (eq? kind 'final))
+	  (wide (if (eq? kind 'wide) 'widening #f))
+	  (tclass (declare-class-type! cdef holder wide
+				       final? abstract?
+				       src-def)))
+      ;; some paranoid checking
+      [assert (tclass) (tclass? tclass)]
+      ;; we store the src-import location in order to print a nice error
+      ;; message if that tclass is not defined
+      (type-import-location-set! tclass (find-location/loc src-decl loc))
+      ;; when importing a class, we import the accessors...
+      (delay-class-accessors!
+       tclass
+       (delay
+	  (multiple-value-bind (concretes virtuals)
+	     (gen cdef tclass src-def module)
+	     concretes)))))
+
+;*---------------------------------------------------------------------*/
+;*    delay-class-accessors! ...                                       */
+;*---------------------------------------------------------------------*/
+(define (delay-class-accessors!::type type::type delayed)
+   (set! *class-accesses* (cons (cons type delayed) *class-accesses*))
+   type)
 
 ;*---------------------------------------------------------------------*/
 ;*    *class-accesses* ...                                             */
@@ -214,39 +178,61 @@
 ;*---------------------------------------------------------------------*/
 ;*    make-add-class! ...                                              */
 ;*---------------------------------------------------------------------*/
-(define (make-add-class! holder class src-def)
-   (let* ((super          (class-its-super class))
+(define (make-add-class! holder class src-def virtuals)
+   (let* ((super          (tclass-its-super class))
 	  (holder-id      (global-id holder))
 	  (class-id       (type-id class))
 	  (class-module   (global-module holder))
+	  (class-make-id  (symbol-append 'make- class-id))
 	  (class-alloc-id (symbol-append 'allocate- class-id))
-	  (class-alloc    `(@ ,class-alloc-id ,class-module))
+	  (class-make     (if (tclass-abstract? class)
+			      `(lambda (x)
+				  (error
+				   ',class-make-id
+				   "Can't make instance of abstract classes"
+				   ',class-id))
+			      `(@ ,class-make-id ,class-module)))
+	  (class-alloc    (if (tclass-abstract? class)
+			      `(lambda (x)
+				  (error
+				   ',class-alloc-id
+				   "Can't allocate instance of abstract classes"
+				   ',class-id))
+			      `(@ ,class-alloc-id ,class-module)))
 	  (hash           (get-class-hash class-id (cddr src-def)))
-	  (constr         (class-constructor class)))
-      (let ((decl (if (not (class? super))
+	  (constr         (tclass-constructor class))
+	  (loc            (find-location src-def))
+	  (fields         (make-class-fields class-id (cddr src-def) loc)))
+      (trace (ast 2) "make-add-class!: " (shape class) " " virtuals #\Newline)
+      (let ((decl (if (not (tclass? super))
 		      `(define ,holder-id
-			  ((@ add-class! __object) ',class-id
-						   #f
-						   ,class-alloc
-						   ,hash
-						   ,(make-class-fields
-						     class-id
-						     (cddr src-def))
-						   ,constr))
-		      (let* ((sholder        (class-holder super))
+			  ((@ register-class! __object)
+			   ',class-id #f
+			   ,class-make ,class-alloc ,hash
+			   ,fields ,constr
+			   (vector ,@(map (lambda (v)
+					     `(cons ,(car v)
+						    (cons ,(cadr v)
+							  ,(caddr v))))
+					  virtuals))))
+		      (let* ((sholder        (tclass-holder super))
 			     (sholder-id     (global-id sholder))
 			     (sholder-module (global-module sholder)))
 			 `(define ,holder-id
-			     ((@ add-class! __object) ',class-id
-						      (@ ,sholder-id
-							 ,sholder-module)
-						      ,class-alloc
-						      ,hash
-						      ,(make-class-fields
-							class-id
-							(cddr src-def))
-						      ,constr))))))
-	 (set! *declared-classes* (cons decl *declared-classes*)))))
+			     ((@ register-class! __object)
+			      ',class-id (@ ,sholder-id ,sholder-module)
+			      ,class-make ,class-alloc ,hash
+			      ,fields ,constr
+			      (vector ,@(map (lambda (v)
+						`(cons ,(car v)
+						       (cons ,(cadr v)
+							     ,(caddr v))))
+					     virtuals))))))))
+	 ;; we have to inject a source positioning into the build declaration
+	 (let ((loc-decl (if (epair? src-def)
+			     (econs (car decl) (cdr decl) (cer src-def))
+			     decl)))
+	    (set! *declared-classes* (cons loc-decl *declared-classes*))))))
 
 ;*---------------------------------------------------------------------*/
 ;*    get-hash-class ...                                               */
@@ -275,24 +261,6 @@
 				  "* "
 				  (symbol->string id))
 				 16))))
-		((+ (and ?integer ?len) (and ?id (? symbol?)) . ?att)
-		 (loop (cdr fields)
-		       (bit-xor hash
-				(string->0..2^x-1
-				 (string-append
-				  "+ "
-				  (integer->string len)
-				  (symbol->string id))
-				 16))))
-		((+ (and ?string ?len) (and ?id (? symbol?)) . ?att)
-		 (loop (cdr fields)
-		       (bit-xor hash
-				(string->0..2^x-1
-				 (string-append
-				  "+ "
-				  len
-				  (symbol->string id))
-				 16))))
 		(((and ?id (? symbol?)) . ?att)
 		 (loop (cdr fields)
 		       (bit-xor hash
@@ -307,56 +275,69 @@
 ;*    definition (according to module_prototype and object_slots)      */
 ;*    to produce the correct proper list for the class declaration.    */
 ;*---------------------------------------------------------------------*/
-(define (make-class-fields class-id slot-defs)
+(define (make-class-fields class-id slot-defs loc)
    (define (read-only? attr)
       (let loop ((attr attr))
 	 (cond
 	    ((null? attr)
 	     #f)
-	    ((memq (car attr) '(read-only))
+	    ((eq? (car attr) 'read-only)
 	     #t)
 	    (else
 	     (loop (cdr attr))))))
+   (define (virtual? attr)
+      (let loop ((attr attr))
+	 (cond
+	    ((null? attr)
+	     #f)
+	    ((and (pair? (car attr)) (eq? (caar attr) 'get))
+	     #t)
+	    (else
+	     (loop (cdr attr))))))
+   (define (find-info-attribute attr)
+      (if (not (pair? attr))
+	  #f
+	  (match-case (car attr)
+	     ((info ?value)
+	      value)
+	     (else
+	      (find-info-attribute (cdr attr))))))
    (define (make-slot-field slot)
       (match-case slot
 	 ((? symbol?)
-	  (let ((id (fast-id-of-id slot)))
-	     `(vector ',slot
-		      ,(symbol-append class-id '- id)
-		      ,(symbol-append class-id '- id '-set!)
-		      #unspecified)))
+	  ;; simple slot without any attributes
+	  (let ((id (fast-id-of-id slot loc)))
+	     `((@ make-class-field __object)
+	       ',slot
+	       ,(symbol-append class-id '- id)
+	       ,(symbol-append class-id '- id '-set!)
+	       #unspecified
+	       #f
+	       #f)))
 	 ((* (and ?id (? symbol?)) . ?att)
-	  (let ((id (fast-id-of-id id)))
-	     `(vector ',id
-		      ,(symbol-append class-id '- id '-ref)
-		      ,(if (not (read-only? att))
-			   (symbol-append class-id '- id '-set!)
-			   '#unspecified)
-		      ,(symbol-append class-id '- id '-len))))
-	 ((+ (and ?integer ?len) (and ?id (? symbol?)) . ?att)
-	  (let ((id (fast-id-of-id id)))
-	     `(vector ',id
-		      ,(symbol-append class-id '- id '-ref)
-		      ,(if (not (read-only? att))
-			   (symbol-append class-id '- id '-set!)
-			   '#unspecified)
-		      (lambda (object) ,len))))
-	 ((+ (and ?string ?len) (and ?id (? symbol?)) . ?att)
-	  (let ((id (fast-id-of-id id)))
-	     `(vector ',id
-		      ,(symbol-append class-id '- id '-ref)
-		      ,(if (not (read-only? att))
-			   (symbol-append class-id '- id '-set!)
-			   '#unspecified)
-		      (lambda (object) (pragma::long ,len)))))
+	  ;; indexed slot with possible attributes
+	  (let ((id (fast-id-of-id id loc)))
+	     `((@ make-class-field __object)
+	       ',id
+	       ,(symbol-append class-id '- id '-ref)
+	       ,(if (not (read-only? att))
+		    (symbol-append class-id '- id '-set!)
+		    '#unspecified)
+	       ,(symbol-append class-id '- id '-len)
+	       #f
+	       ,(find-info-attribute slot))))
 	 (((and ?id (? symbol?)) . ?att)
-	  (let ((id (fast-id-of-id id)))
-	     `(vector ',id
-		      ,(symbol-append class-id '- id)
-		      ,(if (not (read-only? att))
-			   (symbol-append class-id '- id '-set!)
-			   '#unspecified)
-		      #unspecified)))
+	  ;; simple slot with attributes
+	  (let ((id (fast-id-of-id id loc)))
+	     `((@ make-class-field __object)
+	       ',id
+	       ,(symbol-append class-id '- id)
+	       ,(if (not (read-only? att))
+		    (symbol-append class-id '- id '-set!)
+		    '#unspecified)
+	       #unspecified
+	       ,(virtual? att)
+	       ,(find-info-attribute slot))))
 	 (else
 	  (internal-error "make-class-fields"
 			  "Illegal slot definition"
@@ -366,9 +347,7 @@
 			rest)
 		       (else
 			slot-defs))))
-      (if *reflection?*
-	  `(list ,@(map make-slot-field slot-defs))
-	  #unspecified)))
+      `(list ,@(map make-slot-field slot-defs))))
    
 ;*---------------------------------------------------------------------*/
 ;*    class-finalizer ...                                              */
@@ -377,7 +356,7 @@
 ;*---------------------------------------------------------------------*/
 (define (class-finalizer)
    (cond
-      ((and (null? *declared-classes*) (null? *class-accesses*))
+      ((null? *class-accesses*)
        'void)
       (else
        ;; class definitions and method definitions are splitted because
@@ -389,8 +368,19 @@
        ;; to the unit number for importations. The consequence of that
        ;; initialization order is that it is not possible inside a module M1
        ;; that defines a class C to import a module M2 that uses C.
-       (let ((body (append (reverse! *declared-classes*)
-			   (force-class-accesses))))
+       ;; We start forcing class accessors creation. This will produce
+       ;; class declarations that have to be added to the module object
+       ;; initialization
+       (let* ((body (force-class-accesses))
+	      (body (append (reverse! *declared-classes*) body))
+	      (body (if (>fx *debug-module* 0)
+			`((begin
+			    (pragma::int
+			     ,(string-append "puts(\"    Module init (object): "
+					     (symbol->string *module*)
+					     "\")"))
+			    ,@body))
+			body)))
 	  (set! *object-unit* (unit 'object
 				    19
 				    (if (pair? body) body '(#unspecified))
@@ -412,7 +402,7 @@
 ;*---------------------------------------------------------------------*/
 ;*    force-class-accesses ...                                         */
 ;*    -------------------------------------------------------------    */
-;*    Class accessors computation is tricky. We have to sord the class */
+;*    Class accessors computation is tricky. We have to sort the class */
 ;*    before creating the accessors. That is, the classes have to be   */
 ;*    defined from the root of the tree to the node.                   */
 ;*---------------------------------------------------------------------*/
@@ -426,7 +416,9 @@
 	      access
 	      (loop next '() access))
 	  (let* ((class (car (car cur)))
-		 (super (class-its-super class)))
+		 (super (if (tclass? class)
+			    (tclass-its-super class)
+			    (jclass-its-super class))))
 	     (cond
 		((eq? super class)
 		 ;; this is the root we proceed now
@@ -434,12 +426,12 @@
 		       next
 		       (append (force (cdr (car cur)))
 			       access)))
-		((not (class? super))
+		((and (tclass? class) (not (tclass? super)))
 		 (if (type? super)
 		     ;; this is and error that will be pointed out later. for
 		     ;; the moment we simply ignore it.
 		     (begin
-			(class-slots-set! class '())
+			(tclass-slots-set! class '())
 			(loop (cdr cur)
 			      next
 			      access))
@@ -447,7 +439,25 @@
 			   next
 			   (append (force (cdr (car cur)))
 				   access))))
-		((eq? (class-slots super) #unspecified)
+		((and (jclass? class) (not (jclass? super)))
+		 (if (type? super)
+		     ;; this is and error that will be pointed out later. for
+		     ;; the moment we simply ignore it.
+		     (begin
+			(jclass-slots-set! class '())
+			(loop (cdr cur)
+			      next
+			      access))
+		     (loop (cdr cur)
+			   next
+			   (append (force (cdr (car cur)))
+				   access))))
+		((and (tclass? class) (eq? (tclass-slots super) #unspecified))
+		 ;; we have not yet seen the super, we delay again
+		 (loop (cdr cur)
+		       (cons (car cur) next)
+		       access))
+		((and (jclass? class) (eq? (jclass-slots super) #unspecified))
 		 ;; we have not yet seen the super, we delay again
 		 (loop (cdr cur)
 		       (cons (car cur) next)
