@@ -1,41 +1,10 @@
-;*---------------------------------------------------------------------*/
-;*   A pratical implementation for the Scheme programming language     */
-;*                                                                     */
-;*                                    ,--^,                            */
-;*                              _ ___/ /|/                             */
-;*                          ,;'( )__, ) '                              */
-;*                         ;;  //   L__.                               */
-;*                         '   \\   /  '                               */
-;*                              ^   ^                                  */
-;*                                                                     */
-;*   Copyright (c) 1992-1999 Manuel Serrano                            */
-;*                                                                     */
-;*     Bug descriptions, use reports, comments or suggestions are      */
-;*     welcome. Send them to                                           */
-;*       bigloo-request@kaolin.unice.fr                                */
-;*       http://kaolin.unice.fr/bigloo                                 */
-;*                                                                     */
-;*   This program is free software; you can redistribute it            */
-;*   and/or modify it under the terms of the GNU General Public        */
-;*   License as published by the Free Software Foundation; either      */
-;*   version 2 of the License, or (at your option) any later version.  */
-;*                                                                     */
-;*   This program is distributed in the hope that it will be useful,   */
-;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
-;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
-;*   GNU General Public License for more details.                      */
-;*                                                                     */
-;*   You should have received a copy of the GNU General Public         */
-;*   License along with this program; if not, write to the Free        */
-;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
-;*   MA 02111-1307, USA.                                               */
-;*---------------------------------------------------------------------*/
 ;*=====================================================================*/
 ;*    serrano/prgm/project/bigloo/comptime/Module/impuse.scm           */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Tue Jun  4 12:25:53 1996                          */
-;*    Last change :  Fri Feb  5 09:55:00 1999 (serrano)                */
+;*    Last change :  Thu Apr 12 11:23:53 2001 (serrano)                */
+;*    Copyright   :  1996-2001 Manuel Serrano, see LICENSE file        */
 ;*    -------------------------------------------------------------    */
 ;*    The import/usr/from clauses compilation                          */
 ;*=====================================================================*/
@@ -53,10 +22,12 @@
 	    module_prototype
 	    module_class
 	    module_include
+	    module_checksum
 	    tools_speek
 	    tools_error
-	    tools_file
+	    tools_location
 	    type_type
+	    object_class
 	    read_access
 	    read_inline
 	    ast_var
@@ -70,8 +41,8 @@
 	    (make-use-compiler)
 	    (make-from-compiler)
 	    (get-imported-modules)
-	    (import-with-module! ::symbol)
-	    (import-parser       ::symbol prototype)))
+	    (import-with-module! ::symbol loc)
+	    (import-parser       ::symbol prototype . import-src)))
 
 ;*---------------------------------------------------------------------*/
 ;*    make-import-compiler ...                                         */
@@ -105,34 +76,44 @@
    (let ((mode (car clause)))
       (match-case clause
 	 ((?- . ?protos)
-	  (for-each (lambda (proto) (impuse-parser proto mode)) protos))
+	  (for-each (lambda (proto) (impuse-parser proto mode clause)) protos))
 	 (else
-	  (user-error "Parse error"
-		      (string-append "Illegal `"
-				     (string-downcase (symbol->string mode))
-				     "' clause")
-		      clause
-		      '())))))
+	  (user-error/location (find-location *module-clause*)
+			       "Parse error"
+			       (string-append
+				"Illegal `"
+				(string-downcase (symbol->string mode))
+				"' clause")
+			       clause
+			       '())))))
    
 ;*---------------------------------------------------------------------*/
 ;*    import-all-module ...                                            */
 ;*---------------------------------------------------------------------*/
-(define (import-all-module module::symbol mode)
-   (let ((b (assq module *import-module-list*)))
+(define (import-all-module module::symbol mode src)
+   (let ((b (assq module *import-module-list*))
+	 (loc (find-location/loc src (find-location *module-clause*))))
       (if (pair? b)
 	  (imode-vars-set! (cdr b) 'all)
 	  (set! *import-module-list*
-		(cons (cons module (imode module mode 'all #unspecified))
-		      *import-module-list*)))))
+		(cons
+		 (cons module (imode module mode 'all #unspecified loc src))
+		 *import-module-list*)))))
 
 ;*---------------------------------------------------------------------*/
 ;*    import-1-module ...                                              */
 ;*---------------------------------------------------------------------*/
-(define (import-1-module module::symbol var mode)
-   (let ((cell (assq module *import-module-list*)))
+(define (import-1-module module::symbol var mode src)
+   (let ((cell (assq module *import-module-list*))
+	 (loc (find-location/loc src (find-location *module-clause*))))
       (if (not (pair? cell))
 	  (set! *import-module-list*
-		(cons (cons module (imode module mode (list var) #unspecified))
+		(cons (cons module (imode module
+					  mode
+					  (list var)
+					  #unspecified
+					  loc
+					  src))
 		      *import-module-list*))
 	  (let ((b (cdr cell)))
 	     (case (imode-vars b)
@@ -147,11 +128,12 @@
 ;*---------------------------------------------------------------------*/
 ;*    import-with-module! ...                                          */
 ;*---------------------------------------------------------------------*/
-(define (import-with-module! module)
-   (let ((b (assq module *import-module-list*)))
+(define (import-with-module! module src)
+   (let ((b (assq module *import-module-list*))
+	 (loc (find-location/loc src (find-location *module-clause*))))
       (if (not (pair? b))
 	  (set! *import-module-list*
-		(cons (cons module (imode module 'with '() 0))
+		(cons (cons module (imode module 'with '() 0 loc src))
 		      *import-module-list*)))))
    
 ;*---------------------------------------------------------------------*/
@@ -163,7 +145,7 @@
 ;*            |  (variable module-name)                                */
 ;*            |  (variable module-name "file-name" *)                  */
 ;*---------------------------------------------------------------------*/
-(define (impuse-parser prototype mode)
+(define (impuse-parser prototype mode import-src)
    (trace (ast 2) "impuse-parser: " prototype " " mode #\Newline)
    (match-case prototype
       (((and ?module (? symbol?)) (and ?fname (? string?)) . ?rest)
@@ -173,18 +155,18 @@
 	  (cond
 	     ((null? rest)
 	      (add-access! module (reverse! fnames))
-	      (import-all-module module mode))
+	      (import-all-module module mode prototype))
 	     ((string? (car rest))
 	      (loop (cdr rest)
 		    (cons (car rest) fnames)))
 	     (else
-	     (user-error "Parse error"
-			 "Illegal imported clause"
-			 prototype
-			 '())))))
+	      (user-error "Parse error"
+			  "Illegal imported clause"
+			  prototype
+			  '())))))
       (((and ?var (? symbol?)) (and ?module (? symbol?)))
        ;; (variable module-name)
-       (import-1-module module var mode))
+       (import-1-module module var mode prototype))
       (((and (? symbol?) ?v) (and (? symbol?) ?m) (and ?f (? string?)) . ?rest)
        ;; (variable module-name "file-name" ...) 
        (let loop ((rest   rest)
@@ -192,27 +174,29 @@
 	  (cond
 	     ((null? rest)
 	      (add-access! m (reverse! fnames))
-	      (import-1-module m v mode))
+	      (import-1-module m v mode prototype))
 	     ((string? (car rest))
 	      (loop (cdr rest) (cons (car rest) fnames)))
 	     (else
-	     (user-error "Parse error"
-			 "Illegal imported clause"
-			 prototype
-			 '())))))
+	      (user-error "Parse error"
+			  "Illegal imported clause"
+			  prototype
+			  '())))))
       ((and ?module (? symbol?))
        ;; module-name
-       (import-all-module module mode))
+       (import-all-module module mode import-src))
       (else
-       (user-error "Parse error"
-		   "Illegal `import/use' clause"
-		   prototype
-		   '()))))
+       (user-error/location (find-location/loc import-src
+					       (find-location *module-clause*))
+			    "Parse error"
+			    "Illegal `import/use' clause"
+			    prototype
+			    '()))))
 
 ;*---------------------------------------------------------------------*/
 ;*    imode ...                                                        */
 ;*---------------------------------------------------------------------*/
-(define-struct imode id mode vars checksum)
+(define-struct imode id mode vars checksum loc src)
 
 ;*---------------------------------------------------------------------*/
 ;*    *import-module-list* ...                                         */
@@ -258,7 +242,16 @@
 		     ;; the unit number of importation must be smaller
 		     ;; than the one for classes. See module_class module
 		     ;; (@ref class.scm:class unit@).
-		     (list (unit 'imported-modules 12 init-call* #t))
+		     (let ((body (if (>fx *debug-module* 0)
+				     `((begin
+					 (pragma::int
+					  ,(string-append
+					    "puts(\"    Module init (import): "
+					    (symbol->string *module*)
+					    "\")"))
+					 ,@init-call*))
+				     init-call*)))
+			(list (unit 'imported-modules 12 body #t)))
 		     (let* ((id          (imode-id (car init*)))
 			    (checksum    (imode-checksum (car init*)))
 			    (init-fun-id (module-initialization-id id)))
@@ -302,10 +295,17 @@
 ;*---------------------------------------------------------------------*/
 ;*    import-parser ...                                                */
 ;*---------------------------------------------------------------------*/
-(define (import-parser module::symbol prototype)
-   (let ((proto (parse-prototype prototype)))
+(define (import-parser module::symbol prototype . import-src)
+   (let ((proto (parse-prototype prototype))
+	 (src   (if (pair? import-src)
+		    (car import-src)
+		    #f)))
       (if (not (pair? proto))
-	  (user-error "Parse error" "Illegal prototype" prototype '())
+	  (user-error/location (find-location *module-clause*)
+			       "Parse error"
+			       "Illegal prototype"
+			       prototype
+			       '())
 	  (case (car proto)
 	     ((sfun sifun sgfun)
 	      (declare-global-sfun! (cadr proto)
@@ -313,15 +313,18 @@
 				    module
 				    'import
 				    (car proto)
-				    prototype))
+				    prototype
+				    src))
 	     ((svar)
-	      (declare-global-svar! (cadr proto) module	'import	prototype))
+	      (declare-global-svar! (cadr proto) module	'import	prototype src))
 	     ((class)
-	      (declare-class! (cdr proto) module 'import #f prototype))
+	      (declare-class! (cdr proto) module 'import #f #f prototype src))
+	     ((abstract-class)
+	      (declare-class! (cdr proto) module 'import #f #t prototype src))
 	     ((final-class)
-	      (declare-class! (cdr proto) module 'import #t prototype))
+	      (declare-class! (cdr proto) module 'import #t #f prototype src))
 	     ((wide-class)
-	      (declare-wide-class! (cdr proto) module 'import prototype))
+	      (declare-wide-class! (cdr proto) module 'import prototype src))
 	     (else
 	      (user-error "Parse error" "Illegal prototype" prototype '()))))))
    
@@ -337,23 +340,27 @@
 	       (if (eq? (imode-mode imode) 'use) "used" "imported")
 	       " module " module "]" #\Newline)
       (if (not b)
-	  (user-error "read-imported-module"
-		      "Can't find such module"
-		      module
-		      '())
+	  (user-error/location (imode-loc imode)
+			       "read-imported-module"
+			       "Can't find such module"
+			       module
+			       '())
 	  (let* ((fnames (cdr b))
 		 (fname  (find-file/path (car fnames) *load-path*)))
 	     (if (not (string? fname))
-		 (user-error "read-imported-module"
-			     "Can't open such file"
-			     (car fnames)
-			     '())
+		 (user-error/location (imode-loc imode)
+				      "read-imported-module"
+				      "Can't open such file"
+				      (car fnames)
+				      '())
 		 (let ((port (open-input-file fname)))
+		    (reader-reset!)
 		    (if (not (input-port? port))
-			(user-error "read-imported-module"
-				    "Can't open such file"
-				    (car fnames)
-				    '())
+			(user-error/location (imode-loc imode)
+					     "read-imported-module"
+					     "Can't open such file"
+					     (car fnames)
+					     '())
 			(let ((handler
 			       (lambda (escape proc mes obj)
 				  (notify-error proc mes obj)
@@ -369,12 +376,13 @@
 					      (consume-module! module mdecl))
 					   (get-include-consumed-directive)))
 				   (code   (get-include-consumed-code))
-				   (check  (checksum-module mdecl)))
+				   (check  (module-checksum mdecl))
+				   (src    (imode-src imode)))
 			       (imode-checksum-set! imode check)
 			       (look-for-inline
 				(if (not (pair? wanted))
-				    (import-everything prov module)
-				    (import-wanted prov wanted module))
+				    (import-everything prov module src)
+				    (import-wanted prov wanted module src))
 				code
 				port
 				(cdr fnames)
@@ -385,12 +393,12 @@
 ;*---------------------------------------------------------------------*/
 ;*    import-everything ...                                            */
 ;*---------------------------------------------------------------------*/
-(define (import-everything provided module::symbol)
+(define (import-everything provided module::symbol src)
    (let loop ((provided provided)
 	      (inline   '()))
       (if (null? provided)
 	  inline
-	  (let ((p (import-parser module (car provided))))
+	  (let ((p (import-parser module (car provided) src)))
 	     (if (global? p)
 		 (let ((val (global-value p)))
 		    (loop (cdr provided)
@@ -407,7 +415,7 @@
 ;*---------------------------------------------------------------------*/
 ;*    import-wanted ...                                                */
 ;*---------------------------------------------------------------------*/
-(define (import-wanted provided wanted module::symbol)
+(define (import-wanted provided wanted module::symbol src)
    (let loop ((provided provided)
 	      (inline   '())
 	      (wanted   wanted))
@@ -417,19 +425,21 @@
 	 ((null? wanted) 
 	  inline)
 	 ((null? provided)
-	  (user-error module
-		      "Can't find exportation for these identifiers"
-		      wanted
-		      '()))
+	  (user-error/location (find-location *module-clause*)
+			       module
+			       "Can't find exportation for these identifiers"
+			       wanted
+			       '()))
 	 (else
 	  (let ((proto (parse-prototype (car provided))))
 	     (if (pair? proto)
-		 (let ((id (fast-id-of-id (cadr proto))))
+		 (let ((id (fast-id-of-id (cadr proto)
+					  (find-location (car provided)))))
 		    (if (not (memq id wanted))
 			(loop (cdr provided)
 			      inline
 			      wanted)
-			(let ((p (import-parser module (car provided))))
+			(let ((p (import-parser module (car provided) src)))
 			   (cond
 			      ((global? p)
 			       (loop (cdr provided)
