@@ -1,41 +1,10 @@
-;*---------------------------------------------------------------------*/
-;*   A pratical implementation for the Scheme programming language     */
-;*                                                                     */
-;*                                    ,--^,                            */
-;*                              _ ___/ /|/                             */
-;*                          ,;'( )__, ) '                              */
-;*                         ;;  //   L__.                               */
-;*                         '   \\   /  '                               */
-;*                              ^   ^                                  */
-;*                                                                     */
-;*   Copyright (c) 1992-1999 Manuel Serrano                            */
-;*                                                                     */
-;*     Bug descriptions, use reports, comments or suggestions are      */
-;*     welcome. Send them to                                           */
-;*       bigloo-request@kaolin.unice.fr                                */
-;*       http://kaolin.unice.fr/bigloo                                 */
-;*                                                                     */
-;*   This program is free software; you can redistribute it            */
-;*   and/or modify it under the terms of the GNU General Public        */
-;*   License as published by the Free Software Foundation; either      */
-;*   version 2 of the License, or (at your option) any later version.  */
-;*                                                                     */
-;*   This program is distributed in the hope that it will be useful,   */
-;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
-;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
-;*   GNU General Public License for more details.                      */
-;*                                                                     */
-;*   You should have received a copy of the GNU General Public         */
-;*   License along with this program; if not, write to the Free        */
-;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
-;*   MA 02111-1307, USA.                                               */
-;*---------------------------------------------------------------------*/
 ;*=====================================================================*/
 ;*    serrano/prgm/project/bigloo/comptime/Reduce/1occ.scm             */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Thu Jul 13 10:29:17 1995                          */
-;*    Last change :  Wed Mar  4 11:26:24 1998 (serrano)                */
+;*    Last change :  Thu Nov  9 15:31:05 2000 (serrano)                */
+;*    Copyright   :  1995-2000 Manuel Serrano, see LICENSE file        */
 ;*    -------------------------------------------------------------    */
 ;*    The removal of the local variables appearing just once.          */
 ;*    The only goal of this pass is to prune the ast.                  */
@@ -81,7 +50,6 @@
    (verbose 2 "(removed : " *variable-removed* #\) #\newline)
    globals)
 
-
 ;*---------------------------------------------------------------------*/
 ;*    Statitics ...                                                    */
 ;*---------------------------------------------------------------------*/
@@ -156,14 +124,13 @@
 	 (values reset' node))))
 
 ;*---------------------------------------------------------------------*/
-;*    node-1occ! ::pragma ...                                          */
+;*    node-1occ! ::extern ...                                          */
 ;*    -------------------------------------------------------------    */
-;*    We don't remove single occurrences for pragma constructions      */
+;*    We don't remove single occurrences for extern constructions      */
 ;*    because it may introduce typing pbm.                             */
 ;*---------------------------------------------------------------------*/
-(define-method (node-1occ! node::pragma 1-exp*)
-   (with-access::pragma node (args)
-      (values (side-effect? node) node)))
+(define-method (node-1occ! node::extern 1-exp*)
+   (values (side-effect? node) node))
 
 ;*---------------------------------------------------------------------*/
 ;*    node-1occ! ::cast ...                                            */
