@@ -1,41 +1,10 @@
-;*---------------------------------------------------------------------*/
-;*   A pratical implementation for the Scheme programming language     */
-;*                                                                     */
-;*                                    ,--^,                            */
-;*                              _ ___/ /|/                             */
-;*                          ,;'( )__, ) '                              */
-;*                         ;;  //   L__.                               */
-;*                         '   \\   /  '                               */
-;*                              ^   ^                                  */
-;*                                                                     */
-;*   Copyright (c) 1992-1999 Manuel Serrano                            */
-;*                                                                     */
-;*     Bug descriptions, use reports, comments or suggestions are      */
-;*     welcome. Send them to                                           */
-;*       bigloo-request@kaolin.unice.fr                                */
-;*       http://kaolin.unice.fr/bigloo                                 */
-;*                                                                     */
-;*   This program is free software; you can redistribute it            */
-;*   and/or modify it under the terms of the GNU General Public        */
-;*   License as published by the Free Software Foundation; either      */
-;*   version 2 of the License, or (at your option) any later version.  */
-;*                                                                     */
-;*   This program is distributed in the hope that it will be useful,   */
-;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
-;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
-;*   GNU General Public License for more details.                      */
-;*                                                                     */
-;*   You should have received a copy of the GNU General Public         */
-;*   License along with this program; if not, write to the Free        */
-;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
-;*   MA 02111-1307, USA.                                               */
-;*---------------------------------------------------------------------*/
 ;*=====================================================================*/
 ;*    serrano/prgm/project/bigloo/comptime/Type/tenv.scm               */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Sun Dec 25 11:32:49 1994                          */
-;*    Last change :  Fri Dec 25 14:56:39 1998 (serrano)                */
+;*    Last change :  Fri Jul  6 14:03:43 2001 (serrano)                */
+;*    Copyright   :  1994-2001 Manuel Serrano, see LICENSE file        */
 ;*    -------------------------------------------------------------    */
 ;*    The Type environment manipulation                                */
 ;*=====================================================================*/
@@ -50,32 +19,41 @@
    
    (import  tools_shape
 	    tools_error
+	    tools_location
 	    ast_ident
 	    ast_var
 	    engine_param
 	    module_module
+	    module_class
 	    type_type
 	    type_tools
 	    type_cache
-	    object_class)
+	    foreign_ctype
+	    object_class
+	    object_wide-access
+	    object_plain-access
+	    foreign_access
+	    module_foreign)
 
-   (static  (bind-type!::type         ::symbol ::bool)
+   (static  (bind-type!::type ::symbol ::bool loc)
 	    (uninitialized-types))
 
    (export  (initialize-Tenv!)
-	    (set-Tenv!                <Tenv>)
-	    (add-Tenv!                <Tenv>)
-	    (get-Tenv)
-	    (find-type::type          ::symbol)
-	    (use-type!::type          ::symbol)
-	    (use-foreign-type!::type  ::symbol)
-	    (type-exists?::bool       ::symbol)
-	    (declare-type!::type      ::symbol ::bstring ::symbol)
-	    (declare-subtype!::type   ::symbol ::bstring symbol* ::symbol)
+	    (set-tenv! <Tenv>)
+	    (add-tenv! <Tenv>)
+	    (get-tenv)
+	    (find-type::type ::symbol)
+	    (use-type!::type ::symbol loc)
+	    (use-type/import-loc!::type ::symbol loc loci)
+	    (use-foreign-type!::type ::symbol loc)
+	    (use-foreign-type/import-loc!::type ::symbol loc loci)
+	    (type-exists?::bool ::symbol)
+	    (declare-type!::type ::symbol ::bstring ::symbol)
+	    (declare-subtype!::type ::symbol ::bstring symbol* ::symbol)
 	    (declare-aliastype!::type ::symbol ::bstring ::symbol ::type)
-	    (for-each-type!           ::procedure)
+	    (for-each-type! ::procedure)
 	    (check-types)
-	    (sub-type?::bool          ::type ::type)))
+	    (sub-type?::bool ::type ::type)))
 
 ;*---------------------------------------------------------------------*/
 ;*    *Tenv* ...                                                       */
@@ -85,25 +63,25 @@
 (define *Tenv* 'the-global-environment)
 
 ;*---------------------------------------------------------------------*/
-;*    get-hash-number ...                                              */
+;*    set-tenv! ...                                                    */
 ;*---------------------------------------------------------------------*/
-(define (get-hash-number o)
-   (string->0..2^x-1 (symbol->string o) 10))
+(define (set-tenv! Tenv)
+   (set! *Tenv* Tenv))
 
 ;*---------------------------------------------------------------------*/
-;*    set-Tenv! ...                                                    */
+;*    add-tenv! ...                                                    */
 ;*---------------------------------------------------------------------*/
-(define (set-Tenv! Tenv)
-   (set! *Tenv* Tenv)
-   (struct-set! *Tenv* 2 get-hash-number)
-   (struct-set! *Tenv* 3 type-id)
-   (struct-set! *Tenv* 5 eq?))
-
-;*---------------------------------------------------------------------*/
-;*    add-Tenv! ...                                                    */
-;*---------------------------------------------------------------------*/
-(define (add-Tenv! Tenv)
+(define (add-tenv! Tenv)
    (define (adjust-type-coercers! type)
+      ;; fix aliased type
+      (if (type? (type-alias type))
+	  (type-alias-set! type (find-type (type-id (type-alias type)))))
+      ;; fix subtypes
+      (type-parents-set! type
+			 (map (lambda (parent)
+				 (find-type (type-id parent)))
+			      (type-parents type)))
+      ;; fix coercers
       (for-each (lambda (coercer)
 		   (let ((from (coercer-from coercer))
 			 (to   (coercer-to coercer)))
@@ -134,46 +112,70 @@
    ;; in a first stage, we bind all new types (not rebinding already binded
    ;; types)
    (let ((remember-list '()))
-      ;; the remember list is used to store class type. In have to make
-      ;; two traversals over classes. A first one for defining them,
+      
+      ;; the remember list is used to store class type. We have to make
+      ;; three traversals over classes. A first one for defining them,
       ;; a second one, when all classes are correctly set up, to
       ;; fix the its-super class fields. Without the fix, new installed
       ;; classes would have a its-super field that points to the old
       ;; hash table (the one restored, not the current compiler's one).
-      (for-each-hash (lambda (new)
-			(let* ((id  (type-id new))
-			       (old (get-hash id *Tenv*)))
-			   (cond
-			      ((not (type? old))
-			       (put-hash! new *Tenv*)
-			       (if (class? new)
-				   (begin
-				      (heap-add-class! new)
-				      (set! remember-list
-					    (cons new remember-list)))))
-			      ((not (type-init? old))
-			       (error "add-Tenv"
-				      "Illegal type heap redefinition"
-				      id)
-			       (bigloo-exit 255))
-			      (else
-			       ;; we have to store the new coercers 
-			       ;; for the old type
-			       (add-type-coercers! old new)))))
-		     Tenv)
+      ;; The last traversal used to construct the class accessors
+      (hashtable-for-each Tenv
+			  (lambda (k new)
+			     (let* ((id  (type-id new))
+				    (old (hashtable-get *Tenv* id)))
+				(cond
+				   ((not (type? old))
+				    (hashtable-put! *Tenv* id new)
+				    (if (tclass? new)
+					(begin
+					   (heap-add-class! new)
+					   (set! remember-list
+						 (cons new remember-list)))))
+				   ((not (type-init? old))
+				    (error "add-Tenv!"
+					   "Illegal type heap redefinition"
+					   id)
+				    (exit 255))
+				   (else
+				    ;; we have to store the new coercers 
+				    ;; for the old type
+				    (add-type-coercers! old new))))))
+      (hashtable-for-each Tenv
+			  (lambda (k new)
+			     (let* ((id  (type-id new))
+				    (old (hashtable-get *Tenv* id)))
+				(if (ctype? old)
+				    (let ((l (type-location old)))
+				       (foreign-accesses-add!
+					(make-ctype-accesses! old old l)))))))
       ;; we have to walk thru the remember list in order to
       ;; setup the correct super class fields
       (for-each (lambda (new)
-		   (if (class? new)
-		       (let ((super (class-its-super new)))
-			  (if (class? super)
-			      (let* ((super-id (class-id super))
+		   (if (tclass? new)
+		       (let ((super (tclass-its-super new)))
+			  (if (tclass? super)
+			      (let* ((super-id (tclass-id super))
 				     (old-s    (find-type super-id)))
-				 (if (not (class? old-s))
+				 (if (not (tclass? old-s))
 				     (error "add-Tenv"
 					    "Can't find super class of"
-					    (class-name new))
-				     (class-its-super-set! new old-s)))))))
+					    (tclass-name new))
+				     (tclass-its-super-set! new old-s)))))))
+		remember-list)
+      ;; the last walk to construct the class accessors
+      ;; When we load an additional heap that contains classes
+      ;; definition, the accessors are build when compiling the module
+      ;; that uses the library. These accessors have to be added to
+      ;; the other classes accessors.
+      (for-each (lambda (new)
+		   (if (tclass? new)
+		       (delay-class-accessors!
+			new
+			(delay
+			   (if (tclass-widening new)
+			       (heap-wide-class-accessors! new)
+			       (heap-plain-class-accessors! new))))))
 		remember-list))
    ;; in a second stage, we have to reset the all coercer for _all_ types.
    ;; This is mandatory because some types have not been rebound and
@@ -181,17 +183,14 @@
    ;; we have to walk thru all types, not only the freshly defined ones
    ;; because old types may have coercion to new types (for instance, for
    ;; fresh classes).
-   (for-each-hash (lambda (new)
-		     (adjust-type-coercers! new))
-		  *Tenv*))
+   (hashtable-for-each *Tenv*
+		       (lambda (k new)
+			  (adjust-type-coercers! new))))
 		 
 ;*---------------------------------------------------------------------*/
-;*    get-Tenv ...                                                     */
+;*    get-tenv ...                                                     */
 ;*---------------------------------------------------------------------*/
-(define (get-Tenv)
-   (struct-set! *Tenv* 2 'get-hash-number)
-   (struct-set! *Tenv* 3 'type-id)
-   (struct-set! *Tenv* 5 'eq?)
+(define (get-tenv)
    *Tenv*)
 
 ;*---------------------------------------------------------------------*/
@@ -199,13 +198,13 @@
 ;*---------------------------------------------------------------------*/
 (define (initialize-Tenv!)
    ;; the global environment
-   (set! *Tenv* (make-hash-table 1024 get-hash-number type-id eq? 256)))
+   (set! *Tenv* (make-hashtable)))
 
 ;*---------------------------------------------------------------------*/
 ;*    find-type ...                                                    */
 ;*---------------------------------------------------------------------*/
 (define (find-type::type id::symbol)
-   (let ((type (get-hash id *Tenv*)))
+   (let ((type (hashtable-get *Tenv* id)))
       (if (not (type? type))
 	  (error "find-type" "Can't find type" id)
 	  type)))
@@ -216,7 +215,7 @@
 ;*    Returns #t if the type exists _and_ is initialized.              */
 ;*---------------------------------------------------------------------*/
 (define (type-exists?::bool id::symbol)
-   (let ((type (get-hash id *Tenv*)))
+   (let ((type (hashtable-get *Tenv* id)))
       (if (not (type? type))
 	  #f
 	  (type-init? type))))
@@ -224,8 +223,8 @@
 ;*---------------------------------------------------------------------*/
 ;*    bind-type! ...                                                   */
 ;*---------------------------------------------------------------------*/
-(define (bind-type!::type id::symbol init?::bool)
-   (let ((type (get-hash id *Tenv*)))
+(define (bind-type!::type id::symbol init?::bool loc)
+   (let ((type (hashtable-get *Tenv* id)))
       (if (type? type)
 	  (if (and (not *lib-mode*) (type-init? type))
 	      (user-error "bind-type!" "Type redefinition" (shape type))
@@ -235,24 +234,41 @@
 		 (if init? (type-init?-set! type #t))
 		 ;; and we return it.
 		 type))
-	  (let ((new (instantiate::type (id id) (init? init?))))
-	     (put-hash! new *Tenv*)
+	  (let ((new (instantiate::type (id id) (init? init?) (location loc))))
+	     (hashtable-put! *Tenv* id new)
 	     new))))
 
 ;*---------------------------------------------------------------------*/
 ;*    use-type! ...                                                    */
 ;*---------------------------------------------------------------------*/
-(define (use-type!::type id::symbol)
-   (trace (ast 2) "~~~ use-type!: " id #\Newline)
-   (let ((type (get-hash id *Tenv*)))
+(define (use-type!::type id::symbol loc)
+   (trace (ast 2) "~~~ use-type!: " id " loc: " loc #\Newline)
+   (let ((type (hashtable-get *Tenv* id)))
       (cond
 	 ((type? type)
 	  type)
 	 (*types-already-checked?*
-	  (error "use-type!" "Can't find type" id))
+	  (user-error/location loc "use-type!" "Can't find type" id))
 	 (else
 	  (trace (ast 3) "    TYPE BOUND " id #\Newline)
-	  (bind-type! id #f)))))
+	  (bind-type! id #f loc)))))
+
+;*---------------------------------------------------------------------*/
+;*    use-type/import-loc! ...                                         */
+;*---------------------------------------------------------------------*/
+(define (use-type/import-loc!::type id::symbol loc loci)
+   (trace (ast 2) "~~~ use-type!: " id " loc: " loc #\Newline)
+   (let ((type (hashtable-get *Tenv* id)))
+      (cond
+	 ((type? type)
+	  type)
+	 (*types-already-checked?*
+	  (user-error/location loc "use-type!" "Can't find type" id))
+	 (else
+	  (trace (ast 3) "    TYPE BOUND " id #\Newline)
+	  (let ((type (bind-type! id #f loc)))
+	     (type-import-location-set! type loci)
+	     type)))))
 
 ;*---------------------------------------------------------------------*/
 ;*    use-foreign-type! ...                                            */
@@ -263,9 +279,9 @@
 ;*    Since I also want a backward compatibility Bigloo accepts the    */
 ;*    two syntaxes. This function implement the compatibility.         */
 ;*---------------------------------------------------------------------*/
-(define (use-foreign-type!::type id::symbol)
+(define (use-foreign-type!::type id::symbol loc)
    (trace (ast 2) "~~~ use-foreign-type!: " id #\Newline)
-   (let ((tid (parse-id id)))
+   (let ((tid (parse-id id loc)))
       ;; parse-id calls  use-type! so, here we have to call use-type!
       ;; if and only if parse-id did do it with a real type.
       ;; That is, if the cdr of the result of parse-id is not
@@ -273,7 +289,29 @@
       (if (eq? (cdr tid) (get-default-type))
 	  ;; This works only because the default type is not a legal type
 	  ;; that one can use in a foreign clause.
-	  (use-type! (car tid))
+	  (use-type! (car tid) loc)
+	  (cdr tid))))
+
+;*---------------------------------------------------------------------*/
+;*    use-foreign-type/import-loc! ...                                 */
+;*    -------------------------------------------------------------    */
+;*    I have changed the syntax for the foreign declaration. In order  */
+;*    to be consisten now, extern clauses have to be written using     */
+;*    the :: notation (e.g. (print::int (::string ::int) "printf")).   */
+;*    Since I also want a backward compatibility Bigloo accepts the    */
+;*    two syntaxes. This function implement the compatibility.         */
+;*---------------------------------------------------------------------*/
+(define (use-foreign-type/import-loc!::type id::symbol loc loci)
+   (trace (ast 2) "~~~ use-foreign-type!: " id #\Newline)
+   (let ((tid (parse-id id loc)))
+      ;; parse-id calls  use-type! so, here we have to call use-type!
+      ;; if and only if parse-id did do it with a real type.
+      ;; That is, if the cdr of the result of parse-id is not
+      ;; the default type.
+      (if (eq? (cdr tid) (get-default-type))
+	  ;; This works only because the default type is not a legal type
+	  ;; that one can use in a foreign clause.
+	  (use-type/import-loc! (car tid) loc loci)
 	  (cdr tid))))
 
 ;*---------------------------------------------------------------------*/
@@ -281,11 +319,11 @@
 ;*---------------------------------------------------------------------*/
 (define (declare-type!::type id::symbol name::bstring class::symbol)
    (trace (ast 2) "~~~ declare-type!: " id #\Newline)
-   (if (not (memq class '(bigloo C _)))
+   (if (not (memq class '(bigloo C _ java)))
        (user-error "declare-type!"
 		   "Illegal type class"
 		   class)
-       (let ((type (bind-type! id #t)))
+       (let ((type (bind-type! id #t #unspecified)))
 	  (type-name-set!   type name)
 	  (type-$-set!      type ($-in-name? name))
 	  (type-class-set!  type class)
@@ -299,7 +337,7 @@
 (define (declare-subtype!::type id::symbol name::bstring parents class::symbol)
    (trace (ast 2) "~~~ declare-subtype!: " id #\Newline)
    [assert (parents) (list? parents)]
-   (let ((type    (bind-type! id #t))
+   (let ((type    (bind-type! id #t #unspecified))
 	 (parents (map find-type parents)))
       (type-name-set!    type name)
       (type-$-set!       type ($-in-name? name))
@@ -320,7 +358,7 @@
 ;*    for-each-type! ...                                               */
 ;*---------------------------------------------------------------------*/
 (define (for-each-type! proc)
-   (for-each-hash proc *Tenv*))
+   (hashtable-for-each *Tenv* (lambda (k x) (proc x))))
 
 ;*---------------------------------------------------------------------*/
 ;*    uninitialized-types ...                                          */
@@ -351,10 +389,39 @@
 (define (check-types)
    (let ((ut (uninitialized-types)))
       (if (pair? ut)
-	  (error *module*
-		 "These types are used but not defined"
-		 (shape ut))
-	  (set! *types-already-checked?* #t))))
+	  (begin
+	     (newline (current-error-port))
+	     (fprint (current-error-port)
+		     (length ut)
+		     " type(s) used but not defined.")
+	     (let loop ((ut ut))
+		(cond
+		   ((null? ut)
+		    (fprint (current-error-port) "Stopping compilation...")
+		    (exit -1))
+		   ((type? (car ut))
+		    (try (if (type-import-location (car ut))
+			     (user-error/location (type-import-location (car ut))
+						  *module*
+						  "Undefined type used in export clause"
+						  (shape (car ut)))
+			     (user-error/location (type-location (car ut))
+						  *module*
+						  "Undefined used type"
+						  (shape (car ut))))
+			 (lambda (escape obj proc msg)
+			    (notify-error obj proc msg)
+			    (escape #f)))
+		    (loop (cdr ut)))
+		   (else
+		    (try (error *module*
+				"Undefined type"
+				(shape (car ut)))
+			 (lambda (escape obj proc msg)
+			    (notify-error obj proc msg)
+			    (escape #f)))
+		    (loop (cdr ut)))))))
+      (set! *types-already-checked?* #t)))
 
 ;*---------------------------------------------------------------------*/
 ;*    sub-type? ...                                                    */
