@@ -1,42 +1,10 @@
-;*---------------------------------------------------------------------*/
-;*   A pratical implementation for the Scheme programming language     */
-;*                                                                     */
-;*                                    ,--^,                            */
-;*                              _ ___/ /|/                             */
-;*                          ,;'( )__, ) '                              */
-;*                         ;;  //   L__.                               */
-;*                         '   \\   /  '                               */
-;*                              ^   ^                                  */
-;*                                                                     */
-;*   Copyright (c) 1992-1999 Manuel Serrano                            */
-;*                                                                     */
-;*     Bug descriptions, use reports, comments or suggestions are      */
-;*     welcome. Send them to                                           */
-;*       bigloo-request@kaolin.unice.fr                                */
-;*       http://kaolin.unice.fr/bigloo                                 */
-;*                                                                     */
-;*   This program is free software; you can redistribute it            */
-;*   and/or modify it under the terms of the GNU General Public        */
-;*   License as published by the Free Software Foundation; either      */
-;*   version 2 of the License, or (at your option) any later version.  */
-;*                                                                     */
-;*   This program is distributed in the hope that it will be useful,   */
-;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
-;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
-;*   GNU General Public License for more details.                      */
-;*                                                                     */
-;*   You should have received a copy of the GNU General Public         */
-;*   License along with this program; if not, write to the Free        */
-;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
-;*   MA 02111-1307, USA.                                               */
-;*---------------------------------------------------------------------*/
 ;<font size="-3"><pre>
 ;*=====================================================================*/
 ;*    .../prgm/project/bigloo/examples/Yacc2bigloo/yacc2bigloo.scm     */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Mon Nov 27 09:54:26 1995                          */
-;*    Last change :  Mon Jan 18 21:02:22 1999 (serrano)                */
+;*    Last change :  Mon Jul 17 08:13:12 2000 (serrano)                */
 ;*    -------------------------------------------------------------    */
 ;*    The `Yacc grammar -> Bigloo grammar' translator                  */
 ;*=====================================================================*/
@@ -314,7 +282,7 @@
 
       ;; |
       (#\|
-       (list '|))
+       (list ''or))
 
       ;; "???"
       ((: #\" (* (or #\" (out #\" #\Newline ))) #\")
@@ -367,7 +335,7 @@
 (define parser
    (lalr-grammar
       ;; tokens
-      (ID : | SEMI-COMMA)
+      (ID : OR SEMI-COMMA)
       ;; rules
       (rules
        (()
@@ -380,7 +348,7 @@
 	`(,(car ID) ,matches)))
 
       (matches
-       ((match | matches)
+       ((match OR matches)
 	`(,match ,@matches))
        ((match SEMI-COMMA)
 	`(,match)))
