@@ -1,41 +1,9 @@
-;*---------------------------------------------------------------------*/
-;*   A pratical implementation for the Scheme programming language     */
-;*                                                                     */
-;*                                    ,--^,                            */
-;*                              _ ___/ /|/                             */
-;*                          ,;'( )__, ) '                              */
-;*                         ;;  //   L__.                               */
-;*                         '   \\   /  '                               */
-;*                              ^   ^                                  */
-;*                                                                     */
-;*   Copyright (c) 1992-1999 Manuel Serrano                            */
-;*                                                                     */
-;*     Bug descriptions, use reports, comments or suggestions are      */
-;*     welcome. Send them to                                           */
-;*       bigloo-request@kaolin.unice.fr                                */
-;*       http://kaolin.unice.fr/bigloo                                 */
-;*                                                                     */
-;*   This program is free software; you can redistribute it            */
-;*   and/or modify it under the terms of the GNU General Public        */
-;*   License as published by the Free Software Foundation; either      */
-;*   version 2 of the License, or (at your option) any later version.  */
-;*                                                                     */
-;*   This program is distributed in the hope that it will be useful,   */
-;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
-;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
-;*   GNU General Public License for more details.                      */
-;*                                                                     */
-;*   You should have received a copy of the GNU General Public         */
-;*   License along with this program; if not, write to the Free        */
-;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
-;*   MA 02111-1307, USA.                                               */
-;*---------------------------------------------------------------------*/
 ;*=====================================================================*/
 ;*    serrano/prgm/project/bigloo/recette/rgc.scm                      */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Thu Sep  8 11:03:03 1994                          */
-;*    Last change :  Mon Sep 21 13:51:44 1998 (serrano)                */
+;*    Last change :  Tue Nov  9 10:12:42 1999 (serrano)                */
 ;*    -------------------------------------------------------------    */
 ;*    Quelques tests d'rgc                                             */
 ;*=====================================================================*/
@@ -95,6 +63,12 @@
        (cons (string->integer (the-submatch 1))
 	     (string->integer (the-submatch 2))))))
 
+(define *number2*
+   (regular-grammar ()
+      ((: (submatch (* digit)) "." (submatch (* digit)))
+       (cons (string->integer (the-submatch 1))
+	     (string->integer (the-submatch 2))))))
+
 (define (recette-suffix string)
    (string-case string
       ((: (* all) "." (submatch (+ (out "."))))
@@ -102,12 +76,97 @@
       (else
        "")))
 
+(define (test-rgc= str)
+   (let ((port (open-input-string str))
+	 (gram (regular-grammar ()
+             ((= 2 (: #\; (* all) #\newline))
+	      (the-string))
+	     (else
+	      ""))))
+      (read/rp gram port)))
+
+(define (test-rgc>= str)
+   (let ((port (open-input-string str))
+	 (gram (regular-grammar ()
+             ((>= 2 (: #\; (* all) #\newline))
+	      (the-string))
+	     (else
+	      ""))))
+      (read/rp gram port)))
+
+(define (rgc-and str)
+   (let ((port (open-input-string str))
+	 (gram (regular-grammar ()
+		  ((+ (and (#\a #\b) "09abcd")) (the-string))
+		  (else
+		   ""))))
+      (read/rp gram port)))
+
+(define (rgc-and-2 str)
+   (let ((port (open-input-string str))
+	 (gram (regular-grammar ()
+		  ((+ (and "am" "nz")) (the-string))
+		  (else
+		   ""))))
+      (read/rp gram port)))
+
+(define (rgc-but str)
+   (let ((port (open-input-string str))
+	 (gram (regular-grammar ()
+		  ((+ (but ("09ad") ("ce"))) (the-string))
+		  (else
+		   ""))))
+      (read/rp gram port)))
+
+(define (rgc-** str)
+   (let ((port (open-input-string str))
+	 (gram (regular-grammar ()
+		  ((** 3 6 #\a) (the-string)))))
+      (read/rp gram port)))
+
+(define (rgc-... str)
+   (let ((port (open-input-string str))
+	 (gram (regular-grammar ()
+		  ((... 3 "begin") (the-string)))))
+      (read/rp gram port)))
+
+(define (rgc-submatch str)
+   (let ((port (open-input-string str))
+	 (gram (regular-grammar ()
+		  ((: (submatch (* #\space))
+		      (submatch (+ #\+))
+		      (submatch (* #\space)))
+		   (string-append (the-submatch 1) (the-submatch 3))))))
+      (read/rp gram port)))
+   
 ;*---------------------------------------------------------------------*/
 ;*    test-rgc ...                                                     */
 ;*---------------------------------------------------------------------*/
 (define (test-rgc)
    (test-module "rgc" "rgc.scm")
-   (test "submatch"
+   (test "submatch+" 
 	 (read/rp *number* (open-input-string "3.1415"))
 	 '(3 . 1415))
-   (test "string-case" (recette-suffix "toto.org.scm") "scm"))
+   (test "submatch*"
+	 (read/rp *number2* (open-input-string "3.1415"))
+	 '(3 . 1415))
+   (test "string-case" (recette-suffix "toto.org.scm") "scm")
+   (test "rgc ="
+	 (test-rgc= #";1line\n;2line\n;3line\n;4line\n")
+	 #";1line\n;2line\n")
+   (let ((str #";1line\n;2line\n;3line\n;4line\n"))
+      (test "rgc >="
+	    (test-rgc>= str)
+	    str))
+   (test "rgc and" (rgc-and "aaaabbbbccc") "aaaabbbb")
+   (test "rgc and" (rgc-and-2 "aaaabbbbccc") "")
+   (test "rgc but" (rgc-but "aaaabbbbccc") "aaaabbbb")
+   (test "rgc **" (rgc-** "aaaaaaaaaaabbbbccc") "aaaaaa")
+   (test "rgc ..." (rgc-... "begin") "beg")
+   (test "rgc submatch" (rgc-submatch "   +++   ") "      ")
+   (test "fixnum" ((regular-grammar () ((: digit) (the-fixnum)))
+		   (open-input-string "1234"))
+	 1)
+   (test "fixnum" ((regular-grammar () ((+ digit) (the-fixnum)))
+		   (open-input-string "1234"))
+	 1234))
