@@ -1,41 +1,9 @@
-/*---------------------------------------------------------------------*/
-/*   A pratical implementation for the Scheme programming language     */
-/*                                                                     */
-/*                                    ,--^,                            */
-/*                              _ ___/ /|/                             */
-/*                          ,;'( )__, ) '                              */
-/*                         ;;  //   L__.                               */
-/*                         '   \\   /  '                               */
-/*                              ^   ^                                  */
-/*                                                                     */
-/*   Copyright (c) 1992-1999 Manuel Serrano                            */
-/*                                                                     */
-/*     Bug descriptions, use reports, comments or suggestions are      */
-/*     welcome. Send them to                                           */
-/*       bigloo-request@kaolin.unice.fr                                */
-/*       http://kaolin.unice.fr/bigloo                                 */
-/*                                                                     */
-/*   This program is free software; you can redistribute it            */
-/*   and/or modify it under the terms of the GNU General Public        */
-/*   License as published by the Free Software Foundation; either      */
-/*   version 2 of the License, or (at your option) any later version.  */
-/*                                                                     */
-/*   This program is distributed in the hope that it will be useful,   */
-/*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
-/*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
-/*   GNU General Public License for more details.                      */
-/*                                                                     */
-/*   You should have received a copy of the GNU General Public         */
-/*   License along with this program; if not, write to the Free        */
-/*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
-/*   MA 02111-1307, USA.                                               */
-/*---------------------------------------------------------------------*/
 /*=====================================================================*/
-/*    .../serrano/prgm/project/bigloo/runtime/Clib/cports.c            */
+/*    serrano/prgm/project/bigloo/runtime/Clib/cports.c                */
 /*    -------------------------------------------------------------    */
 /*    Author      :  Manuel Serrano                                    */
 /*    Creation    :  Thu Jul 23 15:34:53 1992                          */
-/*    Last change :  Fri Oct 30 11:10:40 1998 ()                       */
+/*    Last change :  Thu Nov  8 09:29:57 2001 (serrano)                */
 /*    -------------------------------------------------------------    */
 /*    Input ports handling                                             */
 /*=====================================================================*/
@@ -46,26 +14,16 @@
 #include <dirent.h>
 #include <string.h>
 #if( !(defined( NeXT ) && (defined( mc68000 ) || defined( i386 ))) )
-#   include <termio.h>
+#   ifdef HAVE_TERMIO
+#      include <termio.h>
+#   endif
 #endif
 #if( !defined( sony_news ) && \
      !(defined( NeXT ) && (defined( mc68000 ) || defined( i386 ))) )
 #   include <unistd.h>
 #endif
 #include <sys/file.h>
-/*---------------------------------------------------------------------*/
-/*    The file `gc_private.h' must not be included into this file. To  */
-/*    prevent this inclusion, we define the GC_PRIVATE_H macro. We     */
-/*    have to protect this file, because `gc_private.h' use an         */
-/*    incorrect prototype for `sbrk' that is incompatible we some      */
-/*    operating system (such as Linux). Linux uses a prototype in      */
-/*    the file `unisys.h' which is not compatible with the one used    */
-/*    in file `gc_private.h'.                                          */
-/*---------------------------------------------------------------------*/
-#if( defined( i386 ) )
-#   define GC_PRIVATE_H
-#endif
-#include <bigloo2.0a.h>
+#include <bigloo.h>
 #if( defined( sony_news ) || (defined( NeXT ) && defined( mc68000 )) )
 #   include <ctype.h>
 #endif
@@ -83,33 +41,38 @@
 obj_t current_output_port, current_input_port, current_error_port;
 long default_io_bufsiz;
 
-/*---------------------------------------------------------------------*/
-/*    Is Bigloo case sensitive ?                                       */
-/*---------------------------------------------------------------------*/
-extern obj_t bigloo_case_sensitive;
-
 /*---------------------------------------------------------------------*/
 /*    External definitions.                                            */
 /*---------------------------------------------------------------------*/
-extern obj_t  string_to_bstring();
-extern obj_t  string_to_symbol();
-extern obj_t  string_to_keyword();
-extern obj_t  make_real();
-extern void   c_error();
-extern obj_t  make_input_port( char *, FILE *, obj_t, long );
+extern obj_t string_to_bstring();
+extern obj_t string_to_keyword();
+extern obj_t make_real();
+extern void  c_error();
+extern obj_t make_input_port( char *, FILE *, obj_t, long );
+extern obj_t make_string();
+
+extern int bgl_fgetc( FILE * );
+extern int bgl_feof( FILE * );
+
+#if( !defined( fgetc ) )
+#  define  bgl_fgetc fgetc
+#endif
+
+#if( !defined( feof ) )
+#  define  bgl_feof feof
+#endif
 
 /*---------------------------------------------------------------------*/
 /*    Prototypes                                                       */
 /*---------------------------------------------------------------------*/
 static bool_t pipe_name_p( char * );
-static char  *pipe_name( char * );
+static char *pipe_name( char * );
 
 /*---------------------------------------------------------------------*/
 /*     make_output_port ...                                            */
 /*---------------------------------------------------------------------*/
 obj_t
-make_output_port( char *name, FILE *file, obj_t kindof )
-{
+make_output_port( char *name, FILE *file, obj_t kindof ) {
    obj_t new_output_port;
 
    new_output_port = GC_MALLOC( OUTPUT_PORT_SIZE );
@@ -119,32 +82,34 @@ make_output_port( char *name, FILE *file, obj_t kindof )
    new_output_port->output_port_t.name   = name;
    new_output_port->output_port_t.kindof = kindof;
    
-   return BREF( new_output_port );
+   return BGL_HEAP_DEBUG_MARK_OBJ( BREF( new_output_port ) );
 }
 
 /*---------------------------------------------------------------------*/
 /*    open_output_file ...                                             */
 /*---------------------------------------------------------------------*/
 obj_t
-open_output_file( obj_t name )
-{
+open_output_file( obj_t name ) {
    FILE *file;
 
 #if( HAVE_PIPE )
-   if( pipe_name_p( BSTRING_TO_STRING( name ) ) )
-   {
-      if( !(file = popen( pipe_name( BSTRING_TO_STRING( name ) ), "w" )) )
+   if( pipe_name_p( (char *)BSTRING_TO_STRING( name ) ) ) {
+      if( !(file = popen( pipe_name( (char *)BSTRING_TO_STRING( name ) ),
+			  "w" )) )
 	 return BFALSE;
       
-      return make_output_port( BSTRING_TO_STRING( name ), file, KINDOF_PIPE );
-   }
-   else
+      return make_output_port( (char *)BSTRING_TO_STRING( name ),
+			       file,
+			       KINDOF_PIPE );
+   } else
 #endif
    {
-      if( !(file = fopen( BSTRING_TO_STRING( name ), "w" )) )
+      if( !(file = fopen( (char *)BSTRING_TO_STRING( name ), "wb" )) )
 	 return BFALSE;
       
-      return make_output_port( BSTRING_TO_STRING( name ), file, KINDOF_FILE );
+      return make_output_port( (char *)BSTRING_TO_STRING( name ),
+			       file,
+			       KINDOF_FILE );
    }
 }
 
@@ -152,14 +117,15 @@ open_output_file( obj_t name )
 /*    append_output_file ...                                           */
 /*---------------------------------------------------------------------*/
 obj_t
-append_output_file( obj_t name )
-{
+append_output_file( obj_t name ) {
    FILE *file;
    
-   if( !(file = fopen( BSTRING_TO_STRING( name ), "a+" )) )
+   if( !(file = fopen( (char *)BSTRING_TO_STRING( name ), "a+b" )) )
       return BFALSE;
 
-   return make_output_port( BSTRING_TO_STRING( name ), file, KINDOF_FILE );
+   return make_output_port( (char *)BSTRING_TO_STRING( name ),
+			    file,
+			    KINDOF_FILE );
 }
  
 /*---------------------------------------------------------------------*/
@@ -167,15 +133,15 @@ append_output_file( obj_t name )
 /*    open_output_string ...                                           */
 /*---------------------------------------------------------------------*/
 obj_t
-open_output_string()
-{
+open_output_string() {
    char *buffer;
    obj_t port;
 
    port = GC_MALLOC( OUTPUT_STRING_PORT_SIZE );
 
    /* We allocate a buffer filled of zero */
-   buffer = (char *)(GC_MALLOC_ATOMIC( OUTPUT_STRING_PORT_BUFFER_SIZE + 1 ));
+   buffer = BGL_HEAP_DEBUG_MARK_STR(
+      (char *)(GC_MALLOC_ATOMIC( OUTPUT_STRING_PORT_BUFFER_SIZE + 1 )) );
    bzero( buffer, OUTPUT_STRING_PORT_BUFFER_SIZE + 1 );
 
    port->output_string_port_t.header = MAKE_HEADER( OUTPUT_STRING_PORT_TYPE,0);
@@ -183,48 +149,68 @@ open_output_string()
    port->output_string_port_t.size   = OUTPUT_STRING_PORT_BUFFER_SIZE;
    port->output_string_port_t.offset = 0;
    
-   return BREF( port );
+   return BGL_HEAP_DEBUG_MARK_OBJ( BREF( port ) );
 }
- 
+
+/*---------------------------------------------------------------------*/
+/*    obj_t                                                            */
+/*    get_output_string ...                                            */
+/*---------------------------------------------------------------------*/
+obj_t
+get_output_string( obj_t port ) {
+   if( OUTPUT_STRING_PORTP( port ) ) {
+      if( OUTPUT_STRING_PORT( port ).size == 0 )
+	 return make_string( 0, ' ' );
+      else
+	 return string_to_bstring_len( OUTPUT_STRING_PORT( port ).buffer,
+				       OUTPUT_STRING_PORT( port ).offset );
+   }
+   else
+      C_FAILURE( "get-output-string", "Not a string port", port );
+}
+
 /*---------------------------------------------------------------------*/
 /*    close_output_port ...                                            */
 /*---------------------------------------------------------------------*/
 obj_t
-close_output_port( obj_t port )
-{
-   if( OUTPUT_STRING_PORTP( port ) )
-   {
-      if( OUTPUT_STRING_PORT( port ).size == 0 )
-      {
+close_output_port( obj_t port ) {
+   if( OUTPUT_STRING_PORTP( port ) ) {
+      if( OUTPUT_STRING_PORT( port ).size == 0 ) {
 	 C_FAILURE( "close-output-port", "port already closed", port );
       
 	 return port;
-      }
-      else
-      {
+      } else {
 	 obj_t res;
 
-	 res = string_to_bstring( OUTPUT_STRING_PORT( port ).buffer );
+	 res = string_to_bstring_len( OUTPUT_STRING_PORT( port ).buffer,
+				      OUTPUT_STRING_PORT( port ).offset );
 	 OUTPUT_STRING_PORT( port ).buffer = 0L;
 	 OUTPUT_STRING_PORT( port ).size = 0;
 
 	 return res;
       }
-   }
-   else
-   {
+   } else {
       /* We do not close console ports (e.g. stdout, stderr) */
-      switch( (long)(OUTPUT_PORT( port ).kindof) )
-      {
-	 case KINDOF_FILE:
+      switch( (long)(OUTPUT_PORT( port ).kindof) ) {
+	 case (long)KINDOF_FILE:
+	    OUTPUT_PORT( port ).kindof = KINDOF_CLOSED;
 	    fclose( OUTPUT_PORT( port ).file );
 	    break;
 	    
 #if( HAVE_PIPE )
-	 case KINDOF_PIPE:
+	 case (long)KINDOF_PROCPIPE:
+	    OUTPUT_PORT( port ).kindof = KINDOF_CLOSED;
+	    fclose( OUTPUT_PORT( port ).file );
+	    break;
+	    
+	 case (long)KINDOF_PIPE:
+	    OUTPUT_PORT( port ).kindof = KINDOF_CLOSED;
 	    pclose( OUTPUT_PORT( port ).file );
 	    break;
 #endif
+	    
+	 case (long)KINDOF_CLOSED:
+	    break;
       }
       
       return port;
@@ -236,10 +222,8 @@ close_output_port( obj_t port )
 /*    make_input_port ...                                              */
 /*---------------------------------------------------------------------*/
 obj_t
-make_input_port( char *name, FILE *file, obj_t kindof, long bufsiz )
-{
+make_input_port( char *name, FILE *file, obj_t kindof, long bufsiz ) {
    obj_t new_input_port;
-   long  buflen;
 
    /* An input port cannot be allocated as an atomic object    */
    /* because it holds a buffer and a name that are GC objects */
@@ -249,7 +233,7 @@ make_input_port( char *name, FILE *file, obj_t kindof, long bufsiz )
    new_input_port->input_port_t.kindof      = kindof;
    new_input_port->input_port_t.name        = name;
    new_input_port->input_port_t.file        = file;
-   new_input_port->input_port_t.filepos     = 1;
+   new_input_port->input_port_t.filepos     = 0;
    new_input_port->input_port_t.bufsiz      = bufsiz;
    new_input_port->input_port_t.eof         = 0;
    new_input_port->input_port_t.matchstart  = 0;
@@ -257,18 +241,55 @@ make_input_port( char *name, FILE *file, obj_t kindof, long bufsiz )
    new_input_port->input_port_t.forward     = 0;
    new_input_port->input_port_t.abufsiz     = 1;
    new_input_port->input_port_t.lastchar    = '\n';
+   new_input_port->input_port_t.syseof      = bgl_feof;
+
+   switch( (long)kindof ) {
+      case (long)KINDOF_CONSOLE:
+      case (long)KINDOF_SOCKET:
+	 new_input_port->input_port_t.sysread = bgl_fgetc;
+	 break;
+
+      case (long)KINDOF_FILE:
+      case (long)KINDOF_PIPE:
+      case (long)KINDOF_PROCPIPE:
+	 new_input_port->input_port_t.sysread = (int (*)())fread;
+	 break;
+
+      default:
+	 new_input_port->input_port_t.sysread = (int (*)())fread;
+   }
+      
    
-   if( bufsiz > 0 )
-   {
-      new_input_port->input_port_t.buffer = (char *)GC_MALLOC_ATOMIC(bufsiz+1);
+   if( bufsiz > 0 ) {
+      new_input_port->input_port_t.buffer = BGL_HEAP_DEBUG_MARK_STR(
+	 (unsigned char *)GC_MALLOC_ATOMIC(bufsiz+1) );
       new_input_port->input_port_t.buffer[ 0 ] = '\0';
    }
    else
       new_input_port->input_port_t.buffer = '\0';
 
-   return BREF( new_input_port );
+   return BGL_HEAP_DEBUG_MARK_OBJ( BREF( new_input_port ) );
 }
-     
+
+/*---------------------------------------------------------------------*/
+/*    obj_t                                                            */
+/*    open_input_pipe ...                                              */
+/*---------------------------------------------------------------------*/
+obj_t
+open_input_pipe( obj_t name, obj_t bbufsiz ) {
+#if( HAVE_PIPE )
+   FILE *file;
+   char *cname = (char *)BSTRING_TO_STRING( name );
+
+   if( !(file = popen( cname, "r" )) )
+      return BFALSE;
+
+   return make_input_port( (char *)cname, file, KINDOF_PIPE, CINT( bbufsiz ) );
+#else
+   return BFLASE;
+#endif
+}
+
 /*---------------------------------------------------------------------*/
 /*    open_input_file ...                                              */
 /*    -------------------------------------------------------------    */
@@ -279,23 +300,20 @@ make_input_port( char *name, FILE *file, obj_t kindof, long bufsiz )
 /*    like pipes when the file name is something like "| ...".         */
 /*---------------------------------------------------------------------*/
 obj_t
-open_input_file( obj_t name, obj_t bbufsiz )
-{ 
+open_input_file( obj_t name, obj_t bbufsiz ) { 
    FILE *file;
-   char *cname = BSTRING_TO_STRING( name );
+   char *cname = (char *)BSTRING_TO_STRING( name );
    
 #if( HAVE_PIPE )
-   if( pipe_name_p( cname ) )
-   {
+   if( pipe_name_p( cname ) ) {
       if( !(file = popen( pipe_name( cname ), "r" )) )
 	 return BFALSE;
 
-      return make_input_port( cname, file, KINDOF_PIPE, CINT( bbufsiz ) );
-   }
-   else
+      return make_input_port( (char *)cname, file, KINDOF_PIPE, CINT( bbufsiz ) );
+   } else
 #endif
    {
-      if( !(file = fopen( cname, "r" )) )
+      if( !(file = fopen( (const char *)cname, "rb" )) )
 	 return BFALSE;
 
       return make_input_port( cname, file, KINDOF_FILE, CINT( bbufsiz ) );
@@ -306,33 +324,42 @@ open_input_file( obj_t name, obj_t bbufsiz )
 /*    open_input_console ...                                           */
 /*---------------------------------------------------------------------*/
 obj_t
-open_input_console()
-{
+open_input_console() {
    return make_input_port( "[stdin]",
 			   stdin,
 			   KINDOF_CONSOLE,
 			   default_io_bufsiz );
 }
 
+/*---------------------------------------------------------------------*/
+/*    obj_t                                                            */
+/*    file_to_buffered_input_port ...                                  */
+/*---------------------------------------------------------------------*/
+obj_t
+file_to_buffered_input_port( FILE *file, long bufsize ) {
+   if( file == stdin )
+      return open_input_console();
+   else {
+      long bsize = bufsize <= 0 ? default_io_bufsiz : bufsize;
+      
+      return make_input_port( "[file]", file, KINDOF_FILE, bsize );
+   }
+}
+
 /*---------------------------------------------------------------------*/
 /*    obj_t                                                            */
 /*    file_to_input_port ...                                           */
 /*---------------------------------------------------------------------*/
 obj_t
-file_to_input_port( FILE *file )
-{
-   if( file == stdin )
-      return open_input_console();
-   else
-      return make_input_port( "[file]", file, KINDOF_FILE, default_io_bufsiz );
+file_to_input_port( FILE *file ) {
+   return file_to_buffered_input_port( file, -1 );
 }
 
 /*---------------------------------------------------------------------*/
 /*    open_input_string ...                                            */
 /*---------------------------------------------------------------------*/
 obj_t
-open_input_string( obj_t string )
-{
+open_input_string( obj_t string ) {
    obj_t port;
    long  bufsiz = STRING_LENGTH( string );
 
@@ -340,7 +367,9 @@ open_input_string( obj_t string )
    
    CREF( port )->input_port_t.eof     = 1;
    CREF( port )->input_port_t.abufsiz = bufsiz + 1;
-   memcpy( &RGC_BUFFER( port )[ 0 ], BSTRING_TO_STRING( string ), bufsiz );
+   memcpy( &RGC_BUFFER( port )[ 0 ],
+	   (void *)BSTRING_TO_STRING( string ),
+	   bufsiz );
    RGC_BUFFER( port )[ bufsiz ] = '\0';
    
    return port;   
@@ -351,17 +380,14 @@ open_input_string( obj_t string )
 /*    open_input_c_string ...                                          */
 /*---------------------------------------------------------------------*/
 obj_t
-open_input_c_string( char *c_string )
-{
+open_input_c_string( char *c_string ) {
    obj_t port;
    long  bufsiz = strlen( c_string );
-   static char *new_string = 0;
-   static int   new_len = 0;
+   char *new_string = 0;
 
-   if( bufsiz > new_len ) {
-      new_string = (char *)GC_MALLOC_ATOMIC( bufsiz + 1 );
-      new_len    = bufsiz;
-   }
+   if( bufsiz > 0 )
+      new_string = BGL_HEAP_DEBUG_MARK_STR(
+	 (char *)GC_MALLOC_ATOMIC( bufsiz + 1 ) );
 
    strcpy( new_string, c_string );
 
@@ -370,57 +396,151 @@ open_input_c_string( char *c_string )
    CREF( port )->input_port_t.eof     = 1;
    CREF( port )->input_port_t.bufsiz  = bufsiz + 1;
    CREF( port )->input_port_t.abufsiz = bufsiz + 1;
-   CREF( port )->input_port_t.buffer  = new_string;
+   CREF( port )->input_port_t.buffer  = (unsigned char *)new_string;
    
    return port;   
 }
-     
+
+/*---------------------------------------------------------------------*/
+/*    obj_t                                                            */
+/*    reopen_input_c_string ...                                        */
+/*    -------------------------------------------------------------    */
+/*    Simply changes the input buffer of an input string. Does not     */
+/*    re-allocate a brand new input-port.                              */
+/*---------------------------------------------------------------------*/
+obj_t
+reopen_input_c_string( obj_t port, char *c_string ) {
+   long bufsiz = strlen( c_string );
+
+   if( CREF( port )->input_port_t.bufsiz < (bufsiz + 1) ) {
+      CREF( port )->input_port_t.bufsiz = bufsiz + 1;
+      CREF( port )->input_port_t.buffer =
+	 (unsigned char *)BGL_HEAP_DEBUG_MARK_STR(
+	    (char *)GC_MALLOC_ATOMIC( bufsiz + 1 ) );
+   }
+
+   CREF( port )->input_port_t.abufsiz     = bufsiz + 1;
+   CREF( port )->input_port_t.matchstart  = 0;
+   CREF( port )->input_port_t.matchstop   = 0;
+   CREF( port )->input_port_t.forward     = 0;
+   CREF( port )->input_port_t.lastchar    = '\n';
+   strcpy( CREF( port )->input_port_t.buffer, (unsigned char *)c_string );
+
+   return port;
+}
+
 /*---------------------------------------------------------------------*/
 /*    close_input_port ...                                             */
 /*---------------------------------------------------------------------*/
 obj_t
-close_input_port( obj_t port )
-{
-   if( INPUT_PORTP( port ) )
-   {
+close_input_port( obj_t port ) {
+   if( INPUT_PORTP( port ) ) {
       /* We do not close the console port */
-      switch( (long)(INPUT_PORT( port ).kindof) )
-      {
-	 case KINDOF_FILE:
+      switch( (long)(INPUT_PORT( port ).kindof) ) {
+	 case (long)KINDOF_FILE:
+	    INPUT_PORT( port ).kindof = KINDOF_CLOSED;
 	    INPUT_PORT( port ).eof = 1;
 	    RGC_BUFFER( port ) = 0L;
 	    fclose( INPUT_PORT( port ).file );
 	    break;
 	    
 #if( HAVE_PIPE ) 
-	 case KINDOF_PIPE:
+	 case (long)KINDOF_PROCPIPE:
+	    INPUT_PORT( port ).kindof = KINDOF_CLOSED;
+	    INPUT_PORT( port ).eof = 1;
+	    RGC_BUFFER( port ) = 0L;
+	    fclose( INPUT_PORT( port ).file );
+	    break;
+	    
+	 case (long)KINDOF_PIPE:
+	    INPUT_PORT( port ).kindof = KINDOF_CLOSED;
 	    INPUT_PORT( port ).eof = 1;
 	    RGC_BUFFER( port ) = 0L;
 	    pclose( INPUT_PORT( port ).file );
 	    break;
 #endif
+	    
+	 case (long)KINDOF_STRING:
+	    INPUT_PORT( port ).kindof = KINDOF_CLOSED;
+	    break;
+	    
+	 case (long)KINDOF_CLOSED:
+	 case (long)KINDOF_CONSOLE:
+	    break;
+ 
+	 case (long)KINDOF_SOCKET:
+	    INPUT_PORT( port ).kindof = KINDOF_CLOSED;
+	    INPUT_PORT( port ).eof = 1;
+	    RGC_BUFFER( port ) = 0L;
+	    fclose( INPUT_PORT( port ).file );
+	    break;
+ 
+	 default:
+	    C_FAILURE( "close-input-port", "unknown input-port type", port );
+	    break;
       }
    } 
 
    return port;
 }
 
+/*---------------------------------------------------------------------*/
+/*    obj_t                                                            */
+/*    bgl_input_port_seek ...                                          */
+/*---------------------------------------------------------------------*/
+obj_t
+bgl_input_port_seek( obj_t port, long pos ) {
+   if( !INPUT_PORT_ON_FILEP( port ) )
+      return BFALSE;
+
+   fseek( INPUT_PORT( port ).file, pos, SEEK_SET );
+	   
+   INPUT_PORT( port ).filepos     = pos;
+   INPUT_PORT( port ).eof         = 0;
+   INPUT_PORT( port ).matchstart  = 0;
+   INPUT_PORT( port ).matchstop   = 0;
+   INPUT_PORT( port ).forward     = 0;
+   INPUT_PORT( port ).abufsiz     = 1;
+   INPUT_PORT( port ).lastchar    = '\n';
+   INPUT_PORT( port ).buffer[ 0 ] = '\0';
+   
+   return BTRUE;
+}
+
+/*---------------------------------------------------------------------*/
+/*    obj_t                                                            */
+/*    bgl_input_port_reopen ...                                        */
+/*---------------------------------------------------------------------*/
+obj_t
+bgl_input_port_reopen( obj_t port ) {
+   if( !INPUT_PORT_ON_FILEP( port ) )
+      return BFALSE;
+
+   freopen( INPUT_PORT( port ).name, "r", INPUT_PORT( port ).file );
+	   
+   INPUT_PORT( port ).filepos     = 0;
+   INPUT_PORT( port ).eof         = 0;
+   INPUT_PORT( port ).matchstart  = 0;
+   INPUT_PORT( port ).matchstop   = 0;
+   INPUT_PORT( port ).forward     = 0;
+   INPUT_PORT( port ).abufsiz     = 1;
+   INPUT_PORT( port ).lastchar    = '\n';
+   INPUT_PORT( port ).buffer[ 0 ] = '\0';
+   
+   return BTRUE;
+}
+
 /*---------------------------------------------------------------------*/
 /*    obj                                                              */
 /*    reset_console ...                                                */
 /*    -------------------------------------------------------------    */
-/*    We flush input port, for ^C works correctly within the           */
+/*    We flush input port, for ^C to work correctly within the         */
 /*    interpreter. The only place this function is called is in the    */
 /*    REPL (see Eva/eval.scm).                                         */
 /*---------------------------------------------------------------------*/
 obj_t
-reset_console( obj_t port )
-{
-   if( INPUT_PORT( port ).kindof == KINDOF_CONSOLE )
-   {
-      long size = INPUT_PORT( port ).bufsiz - 1;
-      int c;
-
+reset_console( obj_t port ) {
+   if( INPUT_PORT( port ).kindof == KINDOF_CONSOLE ) {
       INPUT_PORT( port ).matchstart  = 0;
       INPUT_PORT( port ).matchstop   = 0;
       INPUT_PORT( port ).abufsiz     = 1;
@@ -434,8 +554,7 @@ reset_console( obj_t port )
 /*---------------------------------------------------------------------*/
 /*     init_io ...                                                     */
 /*---------------------------------------------------------------------*/
-void init_io()
-{
+void init_io() {
 #if( !defined( _SBFSIZ ) )
 #   define _SBFSIZ 1
 #endif
@@ -451,9 +570,17 @@ void init_io()
 /*    fexists ...                                                      */
 /*---------------------------------------------------------------------*/
 bool_t
-fexists( char *name )
-{
-   return !access( name, R_OK );
+fexists( char *name ) {
+#if( HAVE_PIPE )
+   if( pipe_name_p( name ) ) {
+      return 1;
+   }
+#else
+   if( pipe_name_p( name ) ) {
+      return 0;
+   }
+#endif   
+   return !access( name, F_OK );
 }
 
 /*---------------------------------------------------------------------*/
@@ -464,12 +591,8 @@ fexists( char *name )
 /*    This allows, restart reading after a ^D.                         */
 /*---------------------------------------------------------------------*/
 bool_t
-reset_eof( obj_t port )
-{
-   if( INPUT_PORT( port ).kindof == KINDOF_CONSOLE )
-   {
-      long bufsiz = INPUT_PORT( port ).bufsiz;
-      
+reset_eof( obj_t port ) {
+   if( INPUT_PORT( port ).kindof == KINDOF_CONSOLE ) {
       /* we forget about EOF */
       INPUT_PORT( port ).eof = 0;
       
@@ -492,8 +615,7 @@ reset_eof( obj_t port )
 /*    On flush un string port.                                         */
 /*---------------------------------------------------------------------*/
 obj_t
-strport_flush( obj_t port )
-{
+strport_flush( obj_t port ) {
    obj_t res;
 
    res = string_to_bstring( OUTPUT_STRING_PORT( port ).buffer );
@@ -509,24 +631,21 @@ strport_flush( obj_t port )
 /*    strport_grow ...                                                 */
 /*---------------------------------------------------------------------*/
 obj_t
-strport_grow( obj_t p )
-{
+strport_grow( obj_t p ) {
    long  old_size, new_size;
    char *old_buffer, *new_buffer;
 	
    old_buffer = OUTPUT_STRING_PORT( p ).buffer;
    old_size   = OUTPUT_STRING_PORT( p ).size;
 
-   if( old_size == 0 )
-   {
+   if( old_size == 0 ) {
       C_FAILURE( "string-port", "close port", p );
       
       return p;
-   }
-   else
-   {
+   } else {
       new_size   = old_size * 2;
-      new_buffer = (char *)( GC_MALLOC_ATOMIC( new_size + 1 ) );
+      new_buffer = BGL_HEAP_DEBUG_MARK_STR(
+	 (char *)( GC_MALLOC_ATOMIC( new_size + 1 ) ) );
       bzero( &new_buffer[ old_size + 1 ], old_size );
       strcpy( new_buffer, old_buffer );
 
@@ -545,8 +664,7 @@ strport_grow( obj_t p )
 /*    sequence "| ".                                                   */
 /*---------------------------------------------------------------------*/
 static bool_t
-pipe_name_p( char *name )
-{
+pipe_name_p( char *name ) {
    return ((name[ 0 ] == '|') && (name[ 1 ] == ' '));
 }
 
@@ -557,8 +675,7 @@ pipe_name_p( char *name )
 /*    Pipe name to name translation.                                   */
 /*---------------------------------------------------------------------*/
 static char *
-pipe_name( char *pipe_name )
-{
+pipe_name( char *pipe_name ) {
    return (pipe_name + 1);
 }
 
@@ -568,8 +685,7 @@ pipe_name( char *pipe_name )
 /*    -------------------------------------------------------------    */
 /*    Is a file a directory?                                           */
 /*---------------------------------------------------------------------*/
-bool_t directoryp( char *name )
-{
+bool_t directoryp( char *name ) {
    struct stat buf;
 
    if( stat( name, &buf ) == -1 )
@@ -582,20 +698,17 @@ bool_t directoryp( char *name )
 /*    obj_t                                                            */
 /*    directory_to_list ...                                            */
 /*---------------------------------------------------------------------*/
-obj_t directory_to_list( char *name )
-{
+obj_t directory_to_list( char *name ) {
    DIR *dir;
    struct dirent *dirent;
    obj_t res = BNIL;
    
-   if( dir = opendir( name ) )
-   {
-      while( dirent = readdir( dir ) )
-      {
+   if( (dir = opendir( name )) ) {
+      while( (dirent = readdir( dir )) ) {
 	 char *fname = dirent->d_name;
 	 
 	 if( strcmp( fname, ".") && strcmp( fname, ".." ) )
-	    res = make_pair( string_to_bstring( fname ), res );
+	    res = MAKE_PAIR( string_to_bstring( fname ), res );
       }
       closedir( dir );
    }
