@@ -1,41 +1,9 @@
-/*---------------------------------------------------------------------*/
-/*   A pratical implementation for the Scheme programming language     */
-/*                                                                     */
-/*                                    ,--^,                            */
-/*                              _ ___/ /|/                             */
-/*                          ,;'( )__, ) '                              */
-/*                         ;;  //   L__.                               */
-/*                         '   \\   /  '                               */
-/*                              ^   ^                                  */
-/*                                                                     */
-/*   Copyright (c) 1992-1999 Manuel Serrano                            */
-/*                                                                     */
-/*     Bug descriptions, use reports, comments or suggestions are      */
-/*     welcome. Send them to                                           */
-/*       bigloo-request@kaolin.unice.fr                                */
-/*       http://kaolin.unice.fr/bigloo                                 */
-/*                                                                     */
-/*   This program is free software; you can redistribute it            */
-/*   and/or modify it under the terms of the GNU General Public        */
-/*   License as published by the Free Software Foundation; either      */
-/*   version 2 of the License, or (at your option) any later version.  */
-/*                                                                     */
-/*   This program is distributed in the hope that it will be useful,   */
-/*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
-/*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
-/*   GNU General Public License for more details.                      */
-/*                                                                     */
-/*   You should have received a copy of the GNU General Public         */
-/*   License along with this program; if not, write to the Free        */
-/*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
-/*   MA 02111-1307, USA.                                               */
-/*---------------------------------------------------------------------*/
 /*=====================================================================*/
 /*    serrano/prgm/project/bigloo/runtime/Clib/cprocess.c              */
 /*    -------------------------------------------------------------    */
 /*    Author      :  Erick Gallesio                                    */
 /*    Creation    :  Mon Jan 19 17:35:12 1998                          */
-/*    Last change :  Fri Sep 18 15:33:22 1998 (serrano)                */
+/*    Last change :  Thu Nov 22 06:39:58 2001 (serrano)                */
 /*    -------------------------------------------------------------    */
 /*    Process handling C part. This part is mostly compatible with     */
 /*    STK. This code is extracted from STK by Erick Gallesio.          */
@@ -47,7 +15,8 @@
 #include <sys/stat.h>
 #include <unistd.h>
 #include <signal.h>
-#include <bigloo2.0a.h>
+#include <string.h>
+#include <bigloo.h>
 
 #define MSG_SIZE 1024
 
@@ -64,12 +33,16 @@ extern obj_t make_input_port( char *, FILE *, obj_t, long );
 extern obj_t make_output_port( char *, FILE *, obj_t );
 extern int   default_io_bufsiz;
 extern obj_t close_input_port( obj_t );
-   
+extern obj_t close_output_port( obj_t );
+extern long  bgl_list_length( obj_t );
+extern char *strerror();
+
 /*---------------------------------------------------------------------*/
 /*    process table                                                    */
 /*---------------------------------------------------------------------*/
-#define MAX_PROC_NUM 40                    /* (simultaneous processes) */
-static obj_t proc_arr[ MAX_PROC_NUM ];     /* process table            */
+#define DEFAULT_MAX_PROC_NUM 255                   
+static int max_proc_num = 0;               /* (simultaneous processes) */
+static obj_t *proc_arr;                    /* process table            */
 
 static char *std_streams[ 3 ] = {
   "input",	
@@ -78,14 +51,14 @@ static char *std_streams[ 3 ] = {
 };
 
 /*---------------------------------------------------------------------*/
-/*    int                                                              */
+/*    bool_t                                                           */
 /*    c_process_alivep ...                                             */
 /*---------------------------------------------------------------------*/
-obj_t
+bool_t
 c_process_alivep( obj_t process )
 {
    if( PROCESS( process ).exited ) 
-      return BFALSE;
+      return 0;
    else
    {
       int info, res;
@@ -95,7 +68,7 @@ c_process_alivep( obj_t process )
       
       if( res == 0 ) 
 	 /* process is still running */
-	 return BTRUE;
+	 return 1;
       else
       {
 	 if( res == PROCESS_PID( process ) )
@@ -103,10 +76,10 @@ c_process_alivep( obj_t process )
 	    /* process has terminated and we must save this information */
 	    PROCESS(process).exited      = 1;
 	    PROCESS(process).exit_status = info;
-	    return BFALSE;
+	    return 0;
 	 }
 	 else
-	    return BFALSE;
+	    return 0;
       }
    }
 }
@@ -124,10 +97,10 @@ c_unregister_process( obj_t proc )
    {
       obj_t p = PROCESS( proc ).stream[ i ];
       
-      if( INPUT_PORTP( p ) )
+      if( INPUT_PORTP( p ) && (INPUT_PORT( p ).kindof != KINDOF_PROCPIPE) )
 	 close_input_port( p );
 
-      if( OUTPUT_PORTP( p ) )
+      if( OUTPUT_PORTP( p ) && (INPUT_PORT( p ).kindof != KINDOF_PROCPIPE) )
 	 close_output_port( p );
    }
    
@@ -163,10 +136,11 @@ process_terminate_handler( int sig )
      /*		  by PURGE_PROCESS_TABLE                              */
      /* Sometimes I think that life is a little bit complicated (ndrl */
      /* sic Erick Gallesio :-)                                        */
-     for( i = 0; i < MAX_PROC_NUM; i++ )
+     for( i = 0; i < max_proc_num; i++ )
      {
 	proc = proc_arr[ i ];
-	if( PROCESSP( proc ) && !c_process_alivep( proc ) )
+
+	if( PROCESSP( proc ) && (!c_process_alivep( proc )) )
 	   /* This process has exited. We can delete it from the table*/
 	   c_unregister_process( proc );
      }
@@ -186,16 +160,28 @@ process_terminate_handler( int sig )
 static void
 init_process()
 {
-   static initializedp = 0;
+   static int initializedp = 0;
    int i;
+   char *env;
    
    if( initializedp )
       return;
    else
       initializedp = 1;
 
+   env = getenv( "BIGLOOLIVEPROCESS" );
+   
+   if( env ) {
+      max_proc_num = atoi( env );
+      if( max_proc_num < 0 ) max_proc_num = DEFAULT_MAX_PROC_NUM;
+   } else {
+      max_proc_num = DEFAULT_MAX_PROC_NUM;
+   }
+   
+   proc_arr = calloc( max_proc_num + 1, sizeof( obj_t ) );
+
    /* we first initialize the process table */
-   for( i = 0; i < MAX_PROC_NUM; i++ ) proc_arr[ i ] = BUNSPEC;
+   for( i = 0; i < max_proc_num; i++ ) proc_arr[ i ] = BUNSPEC;
 
 #if( defined( HAVE_SIGCHLD ) )
    /* On systems which support SIGCHLD, the processes table is cleaned */
@@ -235,7 +221,7 @@ find_process()
 {
    int i;
 
-   for( i = 0; i < MAX_PROC_NUM; i++ )
+   for( i = 0; i < max_proc_num; i++ )
       if( BUNSPEC == proc_arr[ i ] ) return i;
    return (-1);
 }
@@ -278,7 +264,7 @@ make_process()
    if( i < 0 )
       C_FAILURE( "make-process", "too many processes", BUNSPEC );
 
-   a_proc = GC_MALLOC( PROCESS_SIZE );
+   a_proc                        = GC_MALLOC( PROCESS_SIZE );
    a_proc->process_t.header      = MAKE_HEADER( PROCESS_TYPE, 0 );
    a_proc->process_t.index       = i;
    a_proc->process_t.stream[ 0 ] = BFALSE;
@@ -290,7 +276,7 @@ make_process()
    /* Enter this process in the process table */
    proc_arr[ i ] = BREF( a_proc );
    
-   return BREF( a_proc );
+   return BGL_HEAP_DEBUG_MARK_OBJ( BREF( a_proc ) );
 }
   
 /*---------------------------------------------------------------------*/
@@ -300,13 +286,13 @@ make_process()
 obj_t
 c_run_process( obj_t bhost, obj_t bfork, obj_t bwaiting,
 	       obj_t binput, obj_t boutput, obj_t berror,
-	       obj_t bcommand, obj_t bargs )
+	       obj_t bcommand, obj_t bargs, obj_t benv )
 {
    bool_t     waiting;
-   int        pid, i, argc;
+   int        pid = -1, i, argc;
    obj_t      redirection[ 3 ];
    int        pipes[ 3 ][ 2 ];
-   char       msg[ MSG_SIZE ], **argv, **argv_start;
+   char       msg[ MSG_SIZE ], *name, **argv, **argv_start;
    obj_t      runner;
    obj_t      proc;
    
@@ -332,24 +318,24 @@ c_run_process( obj_t bhost, obj_t bfork, obj_t bwaiting,
       {
 	 /* redirection to a file */
 	 int j;
-	 char *ri = BSTRING_TO_STRING( redirection[ i ] );
+	 char *ri = (char *)BSTRING_TO_STRING( redirection[ i ] );
 	 
-	 for( j = 0; j < 3; j++ )
+	 for( j = 0; j < i; j++ )
 	 {
 	    if( j != i && STRINGP( redirection[ j ] ) )
 	    {
 	       struct stat stat_i, stat_j;
-	       char *rj = BSTRING_TO_STRING( redirection[ j ] );
+	       char *rj = (char *)BSTRING_TO_STRING( redirection[ j ] );
 	       /* Do a stat to see if we try to open the same file 2    */
 	       /* times. If stat == -1 this is probably because file    */
 	       /* doesn't exist yet.                                    */
 	       if( stat( ri, &stat_i ) == -1 )
 		  continue;
-	       if( stat( rj, &stat_j) == -1 )
+	       if( stat( rj, &stat_j ) == -1 )
 		  continue;
 		
 	       if( stat_i.st_dev==stat_j.st_dev &&
-		   stat_i.st_ino==stat_j.st_ino)
+		   stat_i.st_ino==stat_j.st_ino )
 	       {
 		  /* Same file was cited 2 times */
 		  if( i == 0 || j == 0 )
@@ -357,9 +343,12 @@ c_run_process( obj_t bhost, obj_t bfork, obj_t bwaiting,
 		     sprintf( msg, "read/write on the same file: %s", ri );
 		     cannot_run( pipes, bcommand, msg );
 		  }
-		  
+
 		  /* assert(i == 1 && j == 2 || i == 2 && j == 1); */
 		  pipes[ i ][ 0 ] = dup( pipes[ j ][ 0 ] );
+
+		  if( pipes[ i ][ 0 ] == -1 )
+		     printf( "ERROR: %s", strerror( errno ) );
 		  break;
 	       }
 	    }
@@ -368,7 +357,7 @@ c_run_process( obj_t bhost, obj_t bfork, obj_t bwaiting,
 	 /* Two cases are possible here:                                     */
 	 /* - we have stdout and stderr redirected on the same file (j != 3) */
 	 /* - we have not found current file in list of redirections (j == 3)*/
-	 if( j == 3 )
+	 if( j == i )
 	 {
 	    pipes[ i ][ 0 ] = open( ri,
 				    i==0 ? O_RDONLY:(O_WRONLY|O_CREAT|O_TRUNC),
@@ -377,6 +366,7 @@ c_run_process( obj_t bhost, obj_t bfork, obj_t bwaiting,
 	    
 	 if( pipes[ i ][ 0 ] < 0 )
 	 {
+	    printf( "error: i:%d  j:%d\n", i, j );
 	    sprintf( msg,
 		     "can't redirect standard %s to file %s",
 		     std_streams[ i ],
@@ -404,21 +394,22 @@ c_run_process( obj_t bhost, obj_t bfork, obj_t bwaiting,
    /* command + arguments initializations    */
    /* 4 = null + rsh + host + command + args */
    argc = 0;
-   argv_start = (char **)GC_MALLOC_ATOMIC( (list_length(bargs) + 4) *
-					   sizeof(char *) );
+   argv_start = (char **)BGL_HEAP_DEBUG_MARK_STR(
+      GC_MALLOC_ATOMIC( (bgl_list_length( bargs ) + 4) * sizeof(char *) ) );
    argv = argv_start + 2;
 
-   argv[ argc++ ] = BSTRING_TO_STRING( bcommand );
+   argv[ argc++ ] = (char *)BSTRING_TO_STRING( bcommand );
    for( runner = bargs; PAIRP( runner ); runner = CDR( runner ) ) 
-      argv[ argc++ ] = BSTRING_TO_STRING( CAR( runner ) );
+      argv[ argc++ ] = (char *)BSTRING_TO_STRING( CAR( runner ) );
    argv[ argc ] = 0L;
    
    /* rsh initialization */
    if( STRINGP( bhost ) )
    {
       argc += 2;
+      argv = argv_start;
       argv[ 0 ] = "rsh";
-      argv[ 1 ] = BSTRING_TO_STRING( bhost );
+      argv[ 1 ] = (char *)BSTRING_TO_STRING( bhost );
    }
 
    /* proc object creation */
@@ -426,10 +417,6 @@ c_run_process( obj_t bhost, obj_t bfork, obj_t bwaiting,
 
    switch( CBOOL( bfork ) && (pid = fork()) )
    {
-      case -1:
-	 cannot_run( pipes, bcommand, "can't create child process" ); 
-	 break;
-
       case 0:
 	 /* The child process */
 	 for( i = 0; i < 3; i++ )
@@ -457,13 +444,44 @@ c_run_process( obj_t bhost, obj_t bfork, obj_t bwaiting,
 	 for( i = 3; i < NOFILE; i++ ) close( i );
 
 	 /* and now we do the exec */
-	 execvp( *argv, argv );
+	 if( PAIRP( benv ) ) {
+	    extern int bgl_envp_len;
+	    extern char **bgl_envp;
+	    int len = bgl_envp_len + bgl_list_length( benv );
+	    int i;
+	    char **envp, **crunner, **init_envp;
+
+	    envp = alloca( sizeof( char * ) * (len + 1) );
+
+	    if( bgl_envp ) {
+	       for( init_envp = bgl_envp, crunner = envp;
+		    *init_envp;
+		    init_envp++, crunner++ ) {
+		  *crunner = *init_envp;
+	       }
+	    }
+		
+	    for( runner = benv;
+		 PAIRP( runner );
+		 crunner++, runner = CDR( runner ) ) {
+	       *crunner = BSTRING_TO_STRING( CAR( runner ) );
+	    }
+	    *crunner = 0;
+	    execve( *argv, argv, envp );
+	 } else {
+	     execvp( *argv, argv );
+	 }
 
 	 /* if we are here, it is because we cannot exec */
 	 cannot_run( pipes, bcommand, "can't execute" ); 
 	 exit( 1 );
 
       default:
+	 if( pid == -1 ) {
+	    cannot_run( pipes, bcommand, "can't create child process" ); 
+	    return proc;
+	 }
+
 	 /* The is the parent process */
 	 PROCESS( proc ).pid = pid;
 	 for( i = 0; i < 3; i++ )
@@ -488,14 +506,22 @@ c_run_process( obj_t bhost, obj_t bfork, obj_t bwaiting,
 		  if( f == NULL )
 		     cannot_run( pipes, bcommand, "cannot fdopen" );
 
-		  sprintf( msg, "pipe-%s-%d", std_streams[ i ], pid);
+		  sprintf( msg, "pipe-%s-%d", std_streams[ i ], pid );
+
+		  /* port name must be heap allocated because it has */
+		  /* the same lifetime as the created port.          */
+		  name = BGL_HEAP_DEBUG_MARK_STR(
+		     (char *)GC_MALLOC_ATOMIC( strlen( msg ) + 1 ) );
+		  strcpy( name, msg );
 		  
-		  if( i != 1 )
+		  if( i == 0 )
 		     PROCESS( proc ).stream[ i ] =
-			make_output_port( msg, f, KINDOF_PIPE );
+			make_output_port( name, f, KINDOF_PROCPIPE );
 		  else
 		     PROCESS( proc ).stream[ i ] =
-			make_input_port( msg,f,KINDOF_PIPE,default_io_bufsiz );
+			make_input_port(name,f,
+					KINDOF_PROCPIPE,
+					default_io_bufsiz);
 	       }
 	    }
 	 }
@@ -522,8 +548,8 @@ c_process_list()
 
    PURGE_PROCESS_TABLE();
 
-   for( i = 0; i < MAX_PROC_NUM; i++ )
-      if( proc_arr[ i ] != BUNSPEC )
+   for( i = 0; i < max_proc_num; i++ )
+      if( (proc_arr[ i ] != BUNSPEC) && c_process_alivep( proc_arr[ i ] ) )
 	 lst = MAKE_PAIR( proc_arr[ i ], lst );
    
    return lst;
