@@ -1,41 +1,9 @@
-/*---------------------------------------------------------------------*/
-/*   A pratical implementation for the Scheme programming language     */
-/*                                                                     */
-/*                                    ,--^,                            */
-/*                              _ ___/ /|/                             */
-/*                          ,;'( )__, ) '                              */
-/*                         ;;  //   L__.                               */
-/*                         '   \\   /  '                               */
-/*                              ^   ^                                  */
-/*                                                                     */
-/*   Copyright (c) 1992-1999 Manuel Serrano                            */
-/*                                                                     */
-/*     Bug descriptions, use reports, comments or suggestions are      */
-/*     welcome. Send them to                                           */
-/*       bigloo-request@kaolin.unice.fr                                */
-/*       http://kaolin.unice.fr/bigloo                                 */
-/*                                                                     */
-/*   This program is free software; you can redistribute it            */
-/*   and/or modify it under the terms of the GNU General Public        */
-/*   License as published by the Free Software Foundation; either      */
-/*   version 2 of the License, or (at your option) any later version.  */
-/*                                                                     */
-/*   This program is distributed in the hope that it will be useful,   */
-/*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
-/*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
-/*   GNU General Public License for more details.                      */
-/*                                                                     */
-/*   You should have received a copy of the GNU General Public         */
-/*   License along with this program; if not, write to the Free        */
-/*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
-/*   MA 02111-1307, USA.                                               */
-/*---------------------------------------------------------------------*/
 /*=====================================================================*/
 /*    serrano/prgm/project/bigloo/runtime/Clib/crgc.c                  */
 /*    -------------------------------------------------------------    */
 /*    Author      :  Manuel Serrano                                    */
 /*    Creation    :  Sun Sep 13 11:58:32 1998                          */
-/*    Last change :  Thu Feb 11 21:28:15 1999 (serrano)                */
+/*    Last change :  Wed Nov  7 16:04:41 2001 (serrano)                */
 /*    -------------------------------------------------------------    */
 /*    Rgc runtime (mostly port handling).                              */
 /*=====================================================================*/
@@ -45,27 +13,18 @@
 #include <sys/stat.h>
 #include <dirent.h>
 #include <string.h>
+#include <sys/time.h>
 #if( !(defined( NeXT ) && (defined( mc68000 ) || defined( i386 ))) )
-#   include <termio.h>
+#   ifdef HAVE_TERMIO
+#      include <termio.h>
+#   endif
 #endif
 #if( !defined( sony_news ) && \
      !(defined( NeXT ) && (defined( mc68000 ) || defined( i386 ))) )
 #   include <unistd.h>
 #endif
 #include <sys/file.h>
-/*---------------------------------------------------------------------*/
-/*    The file `gc_private.h' must not be included into this file. To  */
-/*    prevent this inclusion, we define the GC_PRIVATE_H macro. We     */
-/*    have to protect this file, because `gc_private.h' use an         */
-/*    incorrect prototype for `sbrk' that is incompatible we some      */
-/*    operating system (such as Linux). Linux uses a prototype in      */
-/*    the file `unisys.h' which is not compatible with the one used    */
-/*    in file `gc_private.h'.                                          */
-/*---------------------------------------------------------------------*/
-#if( defined( i386 ) )
-#   define GC_PRIVATE_H
-#endif
-#include <bigloo2.0a.h>
+#include <bigloo.h>
 #if( defined( sony_news ) || (defined( NeXT ) && defined( mc68000 )) )
 #   include <ctype.h>
 #endif
@@ -77,36 +36,41 @@
 #   define isascii( c ) (!((c) & ~0177))
 #endif
 
-/*---------------------------------------------------------------------*/
-/*    DEBUG                                                            */
-/*---------------------------------------------------------------------*/
-#define DEBUG
-
-#if( !defined( DEBUG ) )
-#   define assert( exp ) (0)
-#endif
+#define RGC_DEBUG
+#undef RGC_DEBUG
 
 /*---------------------------------------------------------------------*/
 /*    C importations                                                   */
 /*---------------------------------------------------------------------*/
 extern obj_t bigloo_case_sensitive;
 extern obj_t string_to_bstring_len( char *, int );
-extern obj_t string_to_symbol( char * );
 extern obj_t string_to_keyword( char * );
+extern int toupper( int );
 
 /*---------------------------------------------------------------------*/
-/*    Stub code (for Tk)                                               */
+/*    int                                                              */
+/*    bgl_fgetc ...                                                    */
+/*    -------------------------------------------------------------    */
+/*    We explicitly defines this function because on some system       */
+/*    fgetc is a macro.                                                */
 /*---------------------------------------------------------------------*/
-int (*the_getc)(FILE *) = fgetc;
+int
+bgl_fgetc( FILE *stream ) {
+   return getc( stream );
+}
 
-#if( defined( feof) )
-int my_feof( FILE *stream ) {
+/*---------------------------------------------------------------------*/
+/*    int                                                              */
+/*    bgl_feof ...                                                     */
+/*    -------------------------------------------------------------    */
+/*    We explicitly defines this function because on some system       */
+/*    feof is a macro.                                                 */
+/*---------------------------------------------------------------------*/
+int
+bgl_feof( FILE *stream ) {
    return feof( stream );
 }
-int (*the_feof)(FILE *) = my_feof;
-#else
-int (*the_feof)(FILE *) = feof;
-#endif
+
 
 /*---------------------------------------------------------------------*/
 /*    static void                                                      */
@@ -131,7 +95,7 @@ rgc_enlarge_buffer( obj_t port )
    memcpy( buffer, RGC_BUFFER( port ), bufsize );
 
    INPUT_PORT( port ).bufsiz = 2 * bufsize;
-   RGC_BUFFER( port )      = buffer;
+   RGC_BUFFER( port )        = (unsigned char *)buffer;
 }
   
 /*---------------------------------------------------------------------*/
@@ -141,13 +105,15 @@ rgc_enlarge_buffer( obj_t port )
 /*    This function implements a non blocking fread on the console.    */
 /*---------------------------------------------------------------------*/
 static long
-fread_con( char *ptr, long size, long nmemb, FILE *stream )
+fread_con( obj_t port, char *ptr, long size, long nmemb )
 {
    long  num = size * nmemb;
    char *buf = ptr;
    int   c;
+   int (*a_getc)() = INPUT_PORT( port ).sysread;
+   FILE *stream = INPUT_PORT( port ).file;
    
-   while( ((c = the_getc( stream )) != EOF) )
+   while( ((c = a_getc( stream )) != EOF) )
    {
       *buf++ = c;
 
@@ -157,7 +123,7 @@ fread_con( char *ptr, long size, long nmemb, FILE *stream )
 
    return (long)(buf - ptr);
 }
-   
+     
 /*---------------------------------------------------------------------*/
 /*    static bool_t                                                    */
 /*    rgc_size_fill_con_buffer ...                                     */
@@ -169,23 +135,25 @@ rgc_size_fill_con_buffer( obj_t port, int abufsize, int size )
 
 #if defined( RGC_DEBUG )
    printf( "rgc_size_fill_console_buffer: abufsize: %d  size: %d\n", abufsize, size );
+   printf( "buffer[%s]\n", buffer );
    assert( (abufsize + size) == INPUT_PORT( port ).bufsiz );
    assert( size > 0 );
 #endif
 
    /* we start reading at ABUFSIZE - 1 because we have */
    /* to remove the '\0' sentinel that ends the buffer */
-   abufsize += fread_con( &buffer[ abufsize - 1 ],
+   abufsize += fread_con( port,
+			  (char *)&buffer[ abufsize - 1 ],
  			  1,
-			  size,
-			  INPUT_PORT( port ).file );
+			  size );
    buffer[ abufsize - 1 ] = '\0';
    
    INPUT_PORT( port ).abufsiz = abufsize;
    assert( INPUT_PORT( port ).abufsiz <= INPUT_PORT( port ).bufsiz );
    
-   if( the_feof( INPUT_PORT( port ).file ) )
+   if( INPUT_PORT( port ).syseof( INPUT_PORT( port ).file ) )
 #if defined( RGC_DEBUG )
+   if( INPUT_PORT( port ).kindof == KINDOF_SOCKET )
       printf( "rgc_size_fill_console_buffer: EOF\n"),
 #endif
       INPUT_PORT( port ).eof = 1;
@@ -215,7 +183,6 @@ rgc_fill_console_buffer( obj_t port )
 {
    long bufsize    = INPUT_PORT( port ).bufsiz;
    long abufsize   = INPUT_PORT( port ).abufsiz;
-   long forward    = INPUT_PORT( port ).forward;
    long matchstart = INPUT_PORT( port ).matchstart;
 
 #if defined( RGC_DEBUG )
@@ -276,15 +243,15 @@ rgc_size_fill_file_buffer( obj_t port, int abufsize, int size )
 
    /* we start reading at ABUFSIZE - 1 because we have */
    /* to remove the '\0' sentinel that ends the buffer */
-   abufsize += fread( &buffer[ abufsize - 1 ],
-		      1,
-		      size,
-		      INPUT_PORT(port).file );
+   abufsize += INPUT_PORT( port ).sysread( &buffer[ abufsize - 1 ],
+					   1,
+					   size,
+					   INPUT_PORT(port).file );
    
    INPUT_PORT( port ).abufsiz = abufsize;
    assert( INPUT_PORT( port ).abufsiz <= INPUT_PORT( port ).bufsiz );
    
-   if( feof( INPUT_PORT( port ).file ) )
+   if( INPUT_PORT( port ).syseof( INPUT_PORT( port ).file ) )
       INPUT_PORT( port ).eof = 1;
 
    if( ferror( INPUT_PORT( port ).file ) )
@@ -312,10 +279,10 @@ rgc_size_fill_file_buffer( obj_t port, int abufsize, int size )
 }
 
 /*---------------------------------------------------------------------*/
-/*    void *                                                           */
+/*    void                                                             */
 /*    movemem ...                                                      */
 /*---------------------------------------------------------------------*/
-void *movemem( char *dest, char *src, long n )
+void movemem( char *dest, char *src, long n )
 {
    while( n-- )
       *dest++ = *src++;
@@ -330,7 +297,6 @@ rgc_fill_file_buffer( obj_t port )
 {
    long bufsize    = INPUT_PORT( port ).bufsiz;
    long abufsize   = INPUT_PORT( port ).abufsiz;
-   long forward    = INPUT_PORT( port ).forward;
    long matchstart = INPUT_PORT( port ).matchstart;
    unsigned char *buffer = RGC_BUFFER( port );
 
@@ -347,7 +313,7 @@ rgc_fill_file_buffer( obj_t port )
       long movesize = abufsize - matchstart;
 	 
       /* we shift the buffer left and we fill the buffer */
-      movemem( &buffer[ 0 ], &buffer[ matchstart ], movesize );
+      movemem( (char *)&buffer[ 0 ], (char *)&buffer[ matchstart ], movesize );
 	 
       abufsize                     -= matchstart;
       INPUT_PORT( port ).matchstart = 0;
@@ -396,11 +362,43 @@ rgc_fill_buffer( obj_t port )
       return 0;
    else
    {
-      if( (INPUT_PORT( port ).kindof == KINDOF_FILE) ||
-	  (INPUT_PORT( port ).kindof == KINDOF_PIPE) )
-	 return rgc_fill_file_buffer( port );
-      else
-	 return rgc_fill_console_buffer( port );
+#if defined( RGC_DEBUG )
+      switch( (long)INPUT_PORT( port ).kindof ) {
+	 case (long)KINDOF_FILE:
+	    printf( "filling a FILE\n" );
+	    break;
+	 case (long)KINDOF_PIPE:
+	    printf( "filling a PIPE\n" );
+	    break;
+	 case (long)KINDOF_PROCPIPE:
+	    printf( "filling a PIPE\n" );
+	    break;
+	 case (long)KINDOF_CONSOLE:
+	    printf( "filling a CONSOLE\n" );
+	    break;
+	 case (long)KINDOF_SOCKET:
+	    printf( "filling a SOCKET\n" );
+	    break;
+	 case (long)KINDOF_CLOSED:
+	    printf( "filling a CLOSED\n" );
+	    break;
+	 case (long)KINDOF_STRING:
+	    printf( "filling a STRING\n" );
+	    break;
+      }
+#endif
+
+      switch( (long)INPUT_PORT( port ).kindof ) {
+	 case (long)KINDOF_FILE:
+	 case (long)KINDOF_PROCPIPE:
+	 case (long)KINDOF_PIPE:
+	    return rgc_fill_file_buffer( port );
+	 case (long)KINDOF_CONSOLE:
+	 case (long)KINDOF_SOCKET:
+	    return rgc_fill_console_buffer( port );
+	 default:
+	    return 0;
+      }
    }
 }
    
@@ -423,9 +421,25 @@ rgc_buffer_substring( obj_t ip, long offset, long end )
 	   INPUT_PORT( ip ).forward, INPUT_PORT( ip ).abufsiz );
 #endif
 
-   return string_to_bstring_len( &RGC_BUFFER( ip )[ start + offset ], len );
+   return string_to_bstring_len( (char *)&RGC_BUFFER( ip )[ start + offset ],
+				 len );
 }
 
+/*---------------------------------------------------------------------*/
+/*    CHEAT_BUFFER                                                     */
+/*---------------------------------------------------------------------*/
+#define CHEAT_BUFFER() \
+   long stop  = INPUT_PORT( ip ).matchstop; \
+   char bck; \
+   bck = RGC_BUFFER( ip )[ stop ]; \
+   RGC_BUFFER( ip )[ stop ] = '\0';
+
+/*---------------------------------------------------------------------*/
+/*    RESTORE_BUFFER                                                   */
+/*---------------------------------------------------------------------*/
+#define RESTORE_BUFFER() \
+   RGC_BUFFER( ip )[ stop ] = bck;
+
 /*---------------------------------------------------------------------*/
 /*    long                                                             */
 /*    rgc_buffer_fixnum ...                                            */
@@ -433,7 +447,15 @@ rgc_buffer_substring( obj_t ip, long offset, long end )
 long
 rgc_buffer_fixnum( obj_t ip )
 {
-   return atol( &RGC_BUFFER( ip )[ INPUT_PORT( ip ).matchstart ] );
+   long res;
+   
+   CHEAT_BUFFER();
+   
+   res = atol( &RGC_BUFFER( ip )[ INPUT_PORT( ip ).matchstart ] );
+   
+   RESTORE_BUFFER();
+   
+   return res;
 }
 
 /*---------------------------------------------------------------------*/
@@ -443,7 +465,15 @@ rgc_buffer_fixnum( obj_t ip )
 double
 rgc_buffer_flonum( obj_t ip )
 {
-   return strtod( &RGC_BUFFER( ip )[ INPUT_PORT( ip ).matchstart ], 0 );
+   double res;
+   
+   CHEAT_BUFFER();
+  
+   res = strtod( &RGC_BUFFER( ip )[ INPUT_PORT( ip ).matchstart ], 0 );
+   
+   RESTORE_BUFFER();
+   
+   return res;
 }
 
 /*---------------------------------------------------------------------*/
@@ -453,30 +483,72 @@ rgc_buffer_flonum( obj_t ip )
 obj_t
 rgc_buffer_symbol( obj_t ip )
 {
-   long size  = RGC_BUFFER_LENGTH( ip );
-   long start = INPUT_PORT( ip ).matchstart;
-   long stop  = INPUT_PORT( ip ).matchstop;
-   char bck;
-   char *aux;
+   unsigned char *aux;
    obj_t sym;
+   long start = INPUT_PORT( ip ).matchstart;
    
-   bck = RGC_BUFFER( ip )[ stop ];
-   RGC_BUFFER( ip )[ stop ] = '\0';
+   CHEAT_BUFFER();
    
    aux = &RGC_BUFFER( ip )[ start ];
    
-   if( (BUNSPEC == bigloo_case_sensitive) || !CBOOL( bigloo_case_sensitive ) )
-   {
-      unsigned char *walk;
+   sym = string_to_symbol( (char *)aux );
 
-      for( walk = aux; *walk; walk++ )
-	 if( isascii( *walk ) )
-	    *walk = toupper( *walk );
-   }
+   RESTORE_BUFFER();
+   
+   return sym;
+}
 
-   sym = string_to_symbol( aux );
-   RGC_BUFFER( ip )[ stop ] = bck;
+/*---------------------------------------------------------------------*/
+/*    obj_t                                                            */
+/*    rgc_buffer_upcase_symbol ...                                     */
+/*---------------------------------------------------------------------*/
+obj_t
+rgc_buffer_upcase_symbol( obj_t ip )
+{
+   unsigned char *aux;
+   obj_t sym;
+   long start = INPUT_PORT( ip ).matchstart;
+   unsigned char *walk;
+   
+   CHEAT_BUFFER();
+   
+   aux = &RGC_BUFFER( ip )[ start ];
+   
+   for( walk = aux; *walk; walk++ )
+      if( isascii( *walk ) )
+	 *walk = toupper( *walk );
 
+   sym = string_to_symbol( (char *)aux );
+
+   RESTORE_BUFFER();
+   
+   return sym;
+}
+
+/*---------------------------------------------------------------------*/
+/*    obj_t                                                            */
+/*    rgc_buffer_downcase_symbol ...                                   */
+/*---------------------------------------------------------------------*/
+obj_t
+rgc_buffer_downcase_symbol( obj_t ip )
+{
+   unsigned char *aux;
+   obj_t sym;
+   long start = INPUT_PORT( ip ).matchstart;
+   unsigned char *walk;
+   
+   CHEAT_BUFFER();
+   
+   aux = &RGC_BUFFER( ip )[ start ];
+   
+   for( walk = aux; *walk; walk++ )
+      if( isascii( *walk ) )
+	 *walk = tolower( *walk );
+
+   sym = string_to_symbol( (char *)aux );
+
+   RESTORE_BUFFER();
+   
    return sym;
 }
 
@@ -487,29 +559,72 @@ rgc_buffer_symbol( obj_t ip )
 obj_t
 rgc_buffer_keyword( obj_t ip )
 {
-   long size  = RGC_BUFFER_LENGTH( ip );
-   long start = INPUT_PORT( ip ).matchstart;
-   long stop  = INPUT_PORT( ip ).matchstop;
-   char bck;
-   char *aux;
+   unsigned char *aux;
    obj_t key;
+   long start = INPUT_PORT( ip ).matchstart;
+   unsigned char *walk;
    
-   bck = RGC_BUFFER( ip )[ stop ];
-   RGC_BUFFER( ip )[ stop ] = '\0';
+   CHEAT_BUFFER();
    
    aux = &RGC_BUFFER( ip )[ start ];
    
-   if( (BUNSPEC == bigloo_case_sensitive) || !CBOOL( bigloo_case_sensitive ) )
-   {
-      unsigned char *walk;
+   key = string_to_keyword( (char *)aux );
 
-      for( walk = aux; *walk; walk++ )
-	 if( isascii( *walk ) )
-	    *walk = toupper( *walk );
-   }
+   RESTORE_BUFFER();
 
-   key = string_to_keyword( aux );
-   RGC_BUFFER( ip )[ stop ] = bck;
+   return key;
+}
+ 
+/*---------------------------------------------------------------------*/
+/*    obj_t                                                            */
+/*    rgc_buffer_downcase_keyword ...                                  */
+/*---------------------------------------------------------------------*/
+obj_t
+rgc_buffer_downcase_keyword( obj_t ip )
+{
+   unsigned char *aux;
+   obj_t key;
+   long start = INPUT_PORT( ip ).matchstart;
+   unsigned char *walk;
+   
+   CHEAT_BUFFER();
+   
+   aux = &RGC_BUFFER( ip )[ start ];
+   
+   for( walk = aux; *walk; walk++ )
+      if( isascii( *walk ) )
+	 *walk = tolower( *walk );
+
+   key = string_to_keyword( (char *)aux );
+
+   RESTORE_BUFFER();
+
+   return key;
+}
+ 
+/*---------------------------------------------------------------------*/
+/*    obj_t                                                            */
+/*    rgc_buffer_upcase_keyword ...                                    */
+/*---------------------------------------------------------------------*/
+obj_t
+rgc_buffer_upcase_keyword( obj_t ip )
+{
+   unsigned char *aux;
+   obj_t key;
+   long start = INPUT_PORT( ip ).matchstart;
+   unsigned char *walk;
+   
+   CHEAT_BUFFER();
+   
+   aux = &RGC_BUFFER( ip )[ start ];
+   
+   for( walk = aux; *walk; walk++ )
+      if( isascii( *walk ) )
+	 *walk = toupper( *walk );
+
+   key = string_to_keyword( (char *)aux );
+
+   RESTORE_BUFFER();
 
    return key;
 }
@@ -528,7 +643,7 @@ rgc_buffer_unget_char( obj_t ip, int c )
       RGC_BUFFER( ip )[ 0 ] = c;
       if( INPUT_PORT( ip ).abufsiz == 0 )
       {
-	 INPUT_PORT( ip ).abufsiz == 1;
+	 INPUT_PORT( ip ).abufsiz = 1;
 	 RGC_BUFFER( ip )[ 1 ] = '\0';
       }
    }
@@ -545,7 +660,7 @@ rgc_buffer_unget_char( obj_t ip, int c )
 bool_t
 rgc_buffer_bol_p( obj_t ip )
 {
-#if( defined( RGC_DEBUG ) )   
+#if( defined( RGC_DEBUG ) )
    printf( "RGC_BUFFER_BOL_P: mstart: %d  [mstart]: %d  lastchar: %d  --> %d\n",
 	   INPUT_PORT( ip ).matchstart, 
 	   RGC_BUFFER( ip )[ INPUT_PORT( ip ).matchstart - 1 ],
@@ -677,7 +792,55 @@ rgc_buffer_eof_p( obj_t ip )
       return 0;
    }
 }
-      
-      
+
+/*---------------------------------------------------------------------*/
+/*    static int                                                       */
+/*    file_charready ...                                               */
+/*---------------------------------------------------------------------*/
+static int
+file_charready( FILE *f ) {
+#if( BGL_HAVE_SELECT )
+   fd_set readfds;
+   struct timeval timeout;
+   int fno = fileno( f ) + 1;
+
+   FD_ZERO( &readfds );
+   FD_SET( fno, &readfds );
+   timeout.tv_sec = 0; timeout.tv_usec = 0;
+
+   return select( fno, &readfds, NULL, NULL, &timeout );
+#else
+   return 0;
+#endif
+}
+
+/*---------------------------------------------------------------------*/
+/*    boot_t                                                           */
+/*    bgl_charready ...                                                */
+/*---------------------------------------------------------------------*/
+bool_t
+bgl_rgc_charready( obj_t port ) {
+   switch( (long)INPUT_PORT( port ).kindof ) {
+      case (long)KINDOF_CLOSED:
+	 return 0;
+	 
+      case (long)KINDOF_STRING:
+	 return ((INPUT_PORT( port ).forward+1) < INPUT_PORT( port ).abufsiz);
+	 
+      case (long)KINDOF_FILE:
+	 return ((INPUT_PORT( port ).forward+1) < INPUT_PORT( port ).abufsiz)
+	    || !bgl_feof( INPUT_PORT( port ).file );
+	 
+      case (long)KINDOF_PROCPIPE:
+      case (long)KINDOF_PIPE:
+      case (long)KINDOF_CONSOLE:
+      case (long)KINDOF_SOCKET:
+	 return ((INPUT_PORT( port ).forward+1) < INPUT_PORT( port ).abufsiz)
+	    || file_charready( INPUT_PORT( port ).file );
+	 
+      default:
+	 return 0;
+   }
+}
 
    
