@@ -1,41 +1,9 @@
-/*---------------------------------------------------------------------*/
-/*   A pratical implementation for the Scheme programming language     */
-/*                                                                     */
-/*                                    ,--^,                            */
-/*                              _ ___/ /|/                             */
-/*                          ,;'( )__, ) '                              */
-/*                         ;;  //   L__.                               */
-/*                         '   \\   /  '                               */
-/*                              ^   ^                                  */
-/*                                                                     */
-/*   Copyright (c) 1992-1999 Manuel Serrano                            */
-/*                                                                     */
-/*     Bug descriptions, use reports, comments or suggestions are      */
-/*     welcome. Send them to                                           */
-/*       bigloo-request@kaolin.unice.fr                                */
-/*       http://kaolin.unice.fr/bigloo                                 */
-/*                                                                     */
-/*   This program is free software; you can redistribute it            */
-/*   and/or modify it under the terms of the GNU General Public        */
-/*   License as published by the Free Software Foundation; either      */
-/*   version 2 of the License, or (at your option) any later version.  */
-/*                                                                     */
-/*   This program is distributed in the hope that it will be useful,   */
-/*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
-/*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
-/*   GNU General Public License for more details.                      */
-/*                                                                     */
-/*   You should have received a copy of the GNU General Public         */
-/*   License along with this program; if not, write to the Free        */
-/*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
-/*   MA 02111-1307, USA.                                               */
-/*---------------------------------------------------------------------*/
 /*=====================================================================*/
 /*    serrano/prgm/project/bigloo/runtime/Clib/cstring.c               */
 /*    -------------------------------------------------------------    */
 /*    Author      :  Manuel Serrano                                    */
 /*    Creation    :  Tue Sep  5 09:55:58 1995                          */
-/*    Last change :  Thu Oct 29 07:55:07 1998 (serrano)                */
+/*    Last change :  Mon Oct  1 07:28:07 2001 (serrano)                */
 /*    -------------------------------------------------------------    */
 /*    String management                                                */
 /*=====================================================================*/
@@ -43,13 +11,14 @@
 #include <stdio.h>
 #include <string.h>
 #include <errno.h>
-#include <bigloo2.0a.h>
+#include <bigloo.h>
 #include <varargs.h>
 
 /*---------------------------------------------------------------------*/
 /*    Les recuperations externes                                       */
 /*---------------------------------------------------------------------*/
 extern void perror();
+extern obj_t create_string_for_read( obj_t bstring, int symbolp );
 
 /*---------------------------------------------------------------------*/
 /*    obj_t                                                            */
@@ -59,10 +28,13 @@ obj_t
 string_to_bstring_len( char *c_string, int len )
 {
    obj_t string;
+   char *dst;
+   int   i;
    
    if( !c_string )
       c_string = "";
 
+   /* STRING_SIZE already includes 1 byte for one char */
    string = GC_MALLOC_ATOMIC( STRING_SIZE + len );
 
 #if( !defined( TAG_STRING ) )
@@ -70,10 +42,13 @@ string_to_bstring_len( char *c_string, int len )
 #endif	
    string->string_t.length = len;
 
-   strncpy( &(string->string_t.char0), c_string, len );
-   ((char *)&(string->string_t.char0))[ len ] = '\0';
+   dst = &(string->string_t.char0);
+   for( i = 0; i < len; i++ )
+      dst[ i ] = c_string[ i ];
+
+   dst[ i ] = '\0';
    
-   return BSTRING( string );
+   return BGL_HEAP_DEBUG_MARK_OBJ( BSTRING( string ) );
 }
    
 /*---------------------------------------------------------------------*/
@@ -82,14 +57,14 @@ string_to_bstring_len( char *c_string, int len )
 obj_t
 string_to_bstring( char *c_string )
 {
-   return string_to_bstring_len( c_string, strlen( c_string ) );
+   return string_to_bstring_len( c_string, c_string ? strlen( c_string ) : 0 );
 }
 
 /*---------------------------------------------------------------------*/
 /*    make_string ...                                                  */
 /*---------------------------------------------------------------------*/
 obj_t
-make_string( long len, char c )
+make_string( int len, unsigned char c )
 {
    obj_t string;
 
@@ -108,10 +83,31 @@ make_string( long len, char c )
       memset( &(string->string_t.char0), c, len );
       (&(string->string_t.char0))[ len ] = '\0';
 		
-      return BSTRING( string );
+      return BGL_HEAP_DEBUG_MARK_OBJ( BSTRING( string ) );
    }
 }
 
+/*---------------------------------------------------------------------*/
+/*    make_string_sans_fill ...                                        */
+/*    -------------------------------------------------------------    */
+/*    This is a private function that does not need to check the       */
+/*    correctness of the len argument.                                 */
+/*---------------------------------------------------------------------*/
+obj_t
+make_string_sans_fill( int len )
+{
+   obj_t string = GC_MALLOC_ATOMIC( STRING_SIZE + len );
+
+#if( !defined( TAG_STRING ) )
+   string->string_t.header = MAKE_HEADER( STRING_TYPE, 0 );
+#endif	
+   string->string_t.length = len;
+
+   (&(string->string_t.char0))[ len ] = '\0';
+		
+   return BGL_HEAP_DEBUG_MARK_OBJ( BSTRING( string ) );
+}
+
 /*---------------------------------------------------------------------*/
 /*    c_constant_string_to_string ...                                  */
 /*---------------------------------------------------------------------*/
@@ -127,7 +123,7 @@ c_constant_string_to_string( char *c_string )
 obj_t
 string_append( obj_t s1, obj_t s2 )
 {
-   long  l1, l2;
+   int  l1, l2;
    obj_t string;
    
    l1 = STRING( s1 ).length;
@@ -145,16 +141,44 @@ string_append( obj_t s1, obj_t s2 )
    memcpy( &((char *)(&(string->string_t.char0)))[ l1 ], &STRING_REF( s2, 0 ), l2 );
    ((char *)(&(string->string_t.char0)))[ l1 + l2 ] = '\0';
 	
-   return BSTRING( string );
+   return BGL_HEAP_DEBUG_MARK_OBJ( BSTRING( string ) );
+}
+ 
+/*---------------------------------------------------------------------*/
+/*    string_append_3 ...                                              */
+/*---------------------------------------------------------------------*/
+obj_t
+string_append_3( obj_t s1, obj_t s2, obj_t s3 )
+{
+   int  l1, l2, l3;
+   obj_t string;
+   
+   l1 = STRING( s1 ).length;
+   l2 = STRING( s2 ).length;
+   l3 = STRING( s3 ).length;
+	
+   string = GC_MALLOC_ATOMIC( STRING_SIZE + l1 + l2 + l3 );
+
+#if( !defined( TAG_STRING ) )
+   string->string_t.header = MAKE_HEADER( STRING_TYPE, 0 );
+#endif	
+   string->string_t.length = l1 + l2 + l3;
+
+   memcpy( &(string->string_t.char0), &STRING_REF( s1, 0 ), l1 );
+   memcpy( &((char *)(&(string->string_t.char0)))[ l1 ], &STRING_REF( s2, 0 ), l2 );
+   memcpy( &((char *)(&(string->string_t.char0)))[ l1 + l2 ], &STRING_REF( s3, 0 ), l3 );
+   ((char *)(&(string->string_t.char0)))[ l1 + l2 + l3 ] = '\0';
+	
+   return BGL_HEAP_DEBUG_MARK_OBJ( BSTRING( string ) );
 }
  
 /*---------------------------------------------------------------------*/
 /*    c_substring ...                                                  */
 /*---------------------------------------------------------------------*/
 obj_t
-c_substring( obj_t src_string, long min, long max )
+c_substring( obj_t src_string, int min, int max )
 {
-   long  len;
+   int  len;
    obj_t dst_string;
    
    len = max - min;
@@ -171,7 +195,7 @@ c_substring( obj_t src_string, long min, long max )
             len );
    (&(dst_string->string_t.char0))[ len ] = '\0';
 
-   return BSTRING( dst_string );
+   return BGL_HEAP_DEBUG_MARK_OBJ( BSTRING( dst_string ) );
 }
 
 /*---------------------------------------------------------------------*/
@@ -179,37 +203,63 @@ c_substring( obj_t src_string, long min, long max )
 /*    blit_string ...                                                  */
 /*---------------------------------------------------------------------*/
 obj_t
-blit_string( obj_t s1, obj_t offset1, obj_t s2, obj_t offset2, obj_t len )
+blit_string( obj_t s1, int offset1, obj_t s2, int offset2, int len )
 {
-   bcopy( &STRING_REF( s1, CINT( offset1 ) ),
-	  &STRING_REF( s2, CINT( offset2 ) ),
-	  CINT( len ) );
+   bcopy( &STRING_REF( s1, offset1 ), &STRING_REF( s2, offset2 ), len );
    return BUNSPEC;
 }
 	
 /*---------------------------------------------------------------------*/
 /*    integer_to_string ...                                            */
 /*---------------------------------------------------------------------*/
-char *
+obj_t
 integer_to_string( long x, long radix )
 {
-   char *aux;
-
-   aux = (char *)GC_MALLOC_ATOMIC( 1024 );
-
+   obj_t aux;
+   int bits = (x <= 0 ? 1 : 0);
+   long ax = abs( x );
+   
    switch( radix )
    {
+      case 2 :
+      {
+	 char *s;
+	 bits = ((x == 0) ? 1 : floor( log( ax ) / log( 2 )) + 1);
+	    
+	 aux = make_string_sans_fill( bits );
+
+	 s = BSTRING_TO_STRING( aux ) + bits;
+	 *s = '\0';
+
+	 for( s--; bits > 0; bits--, s--, x = x >> 1 )
+	    *s =  ((x & 1) == 0) ? '0' : '1';
+      }
+      break;
+	 
       case 8 :
-         sprintf( aux, "#o%o", x );
+	 while( ax > 0 ) bits++, ax /= 8;
+
+	 aux = make_string_sans_fill( bits );
+	 
+         sprintf( BSTRING_TO_STRING( aux ), "%lo", x );
          break;
          
       case 10 :
-         sprintf( aux, "%ld", x );
+	 while( ax > 0 ) bits++, ax /= 10;
+	 
+	 aux = make_string_sans_fill( bits );
+         sprintf( BSTRING_TO_STRING( aux ), "%ld", x );
          break;
          
       case 16 :
-         sprintf( aux, "#x%lx", x );
+	 while( ax > 0 ) bits++, ax /= 16;
+	 
+	 aux = make_string_sans_fill( bits );
+         sprintf( BSTRING_TO_STRING( aux ), "%lx", x );
          break;
+
+      default:
+	 aux = 0L;
    }
 
    return aux;
@@ -310,6 +360,7 @@ zero:
    } while( wp-- );
 
    if( dpt > 0 )
+   {
       if( (dpt > 4) && (exp > 6) )
       {
 	 d = (a[ 0 ]=='-'?2:1);
@@ -324,6 +375,7 @@ zero:
 	    a[ ch++ ] = '0';
 	 a[ ch++ ] = '.';
       }
+   }
    
    if( a[ ch-1 ]=='.' )
       /* trailing zero */
@@ -354,7 +406,7 @@ zero:
 /*    Many thanks to Raj Manandhar and Alain Mellan for providing us   */
 /*    with these codes.                                                */
 /*---------------------------------------------------------------------*/
-char *
+obj_t
 real_to_string( double x )
 {
 #if( !defined( DOUBLE_PRECISION ) )
@@ -368,9 +420,11 @@ real_to_string( double x )
 #   define FLONUM_LEN  20
 #   define FORMAT      "%#.12g"
 #endif
-   char new;
-
-   new = GC_MALLOC_ATOMIC( FLONUM_LEN + 1 );
+   char *new;
+   obj_t res;
+   
+   res = make_string_sans_fill( FLONUM_LEN + 1 );
+   new = BSTRING_TO_STRING( res );
       
    memset( new, '\0', FLONUM_LEN );
    sprintf( new, FORMAT, REAL( o ).real );
@@ -397,19 +451,149 @@ real_to_string( double x )
 	  new[ i ] = '\0';
        }
    }
-   return new;
+   return res;
 #else
-   char *new;
+   static char buf[ 80 ];
    int size;
 
-   new = (char *)GC_MALLOC_ATOMIC( 1024 );
-   size = internal_real_to_string( x, new );
-   new[size] = '\0';
+   size = internal_real_to_string( x, buf );
+   buf[ size ] = '\0';
    
-   return new;
+   return string_to_bstring_len( buf, size );
 #endif   
 }
 
+/*---------------------------------------------------------------------*/
+/*    obj_t                                                            */
+/*    bgl_double_to_ieee_string ...                                    */
+/*    -------------------------------------------------------------    */
+/*    A converted string is exactly sizeof(double)+1 actual characters */
+/*    long. The +1 is used to encoded the endianess of the machine     */
+/*    that has constructed the string.                                 */
+/*    -------------------------------------------------------------    */
+/*    This function encodes the number into a big endian               */
+/*    representation. This is mandatory in order to conform the        */
+/*    JVM specification                                                */
+/*---------------------------------------------------------------------*/
+obj_t 
+bgl_double_to_ieee_string( double o ) {
+   obj_t res = make_string_sans_fill( sizeof( double ) );
+   char *aux = BSTRING_TO_STRING( res );
+   char *tmp = (char *)(&o);
+
+#if( BGL_BIG_ENDIAN == 1 )
+   int i;
+   
+   for( i = 0; i < sizeof( double ); i++ ) {
+      aux[ i ] = tmp[ i ];
+   }
+   aux[ i ] = 0;
+#else
+   int i, j;
+   
+   for( j = 0, i = (sizeof( double ) - 1); i >= 0; i--, j++ ) {
+      aux[ j ] = tmp[ i ];
+   }
+   aux[ j ] = 0;
+#endif
+   
+   return res;
+}
+
+/*---------------------------------------------------------------------*/
+/*    double                                                           */
+/*    bgl_ieee_string_to_double ...                                    */
+/*    -------------------------------------------------------------    */
+/*    The number is stored using a big endian encoding                 */
+/*---------------------------------------------------------------------*/
+double
+bgl_ieee_string_to_double( obj_t s ) {
+   char *aux = BSTRING_TO_STRING( s );
+   double res;
+   char *tmp = (char *)(&res);
+
+#if( BGL_BIG_ENDIAN == 1 )
+   int i;
+   
+   for( i = 0; i < sizeof( double ); i++ ) {
+      tmp[ i ] = aux[ i ];
+   }
+#else
+   int i, j;
+   
+   for( j = 0, i = sizeof( double ) - 1;  i >= 0; i--, j++ ) {
+      tmp[ i ] = aux[ j ];
+   }
+#endif
+
+   return res;
+}
+
+/*---------------------------------------------------------------------*/
+/*    obj_t                                                            */
+/*    bgl_float_to_ieee_string ...                                     */
+/*    -------------------------------------------------------------    */
+/*    A converted string is exactly sizeof(float)+1 actual characters  */
+/*    long. The +1 is used to encoded the endianess of the machine     */
+/*    that has constructed the string.                                 */
+/*    -------------------------------------------------------------    */
+/*    This function encodes the number into a big endian               */
+/*    representation. This is mandatory in order to conform the        */
+/*    JVM specification                                                */
+/*---------------------------------------------------------------------*/
+obj_t 
+bgl_float_to_ieee_string( float o ) {
+   obj_t res = make_string_sans_fill( sizeof( float ) + 1 );
+   char *aux = BSTRING_TO_STRING( res );
+   char *tmp = (char *)(&o);
+
+#if( BGL_BIG_ENDIAN == 1 )
+   int i;
+   
+   for( i = 0; i < sizeof( float ); i++ ) {
+      aux[ i ] = tmp[ i ];
+   }
+   aux[ i ] = 0;
+#else
+   int i, j;
+   for( j = 0, i = (sizeof( float ) - 1); i >= 0; i--, j++ ) {
+      aux[ i ] = tmp[ i ];
+   }
+   aux[ j ] = 0;
+#endif
+   
+   return res;
+}
+
+/*---------------------------------------------------------------------*/
+/*    float                                                            */
+/*    bgl_ieee_string_to_float ...                                     */
+/*    -------------------------------------------------------------    */
+/*    The number is stored using a big endian encoding                 */
+/*---------------------------------------------------------------------*/
+float
+bgl_ieee_string_to_float( obj_t s ) {
+   char *aux = BSTRING_TO_STRING( s );
+   float res;
+   char *tmp = (char *)(&res);
+
+#if( BGL_BIG_ENDIAN == 1 )
+   int i;
+   
+   for( i = 0; i < sizeof( float ); i++ ) {
+      tmp[ i ] = aux[ i ];
+   }
+#else
+   int i, j;
+   
+   for( j = 0, i = sizeof( float ) - 1; i >= 0; i--, j++ ) {
+      tmp[ i ] = aux[ j ];
+   }
+#endif
+
+   return res;
+}
+
 /*---------------------------------------------------------------------*/
 /*    bool_t                                                           */
 /*    bigloo_strcmp ...                                                */
@@ -417,13 +601,14 @@ real_to_string( double x )
 bool_t
 bigloo_strcmp( obj_t o1, obj_t o2 )
 {
-   long l1, l2;
+   int l1, l2;
 
    l1 = STRING_LENGTH( o1 );
    l2 = STRING_LENGTH( o2 );
 
    if( l1 == l2 )
-      return !memcmp( BSTRING_TO_STRING( o1 ), BSTRING_TO_STRING( o2 ), l1 );
+      return !memcmp( (void *)BSTRING_TO_STRING( o1 ),
+		      (void *)BSTRING_TO_STRING( o2 ), l1 );
    else
       return 0;
 }
@@ -433,15 +618,16 @@ bigloo_strcmp( obj_t o1, obj_t o2 )
 /*    bigloo_strncmp ...                                               */
 /*---------------------------------------------------------------------*/
 bool_t
-bigloo_strncmp( obj_t o1, obj_t o2, long l )
+bigloo_strncmp( obj_t o1, obj_t o2, int l )
 {
-   long l1, l2;
+   int l1, l2;
 
    l1 = STRING_LENGTH( o1 );
    l2 = STRING_LENGTH( o2 );
 
    if( (l1 >= l) && (l2 >= l) )
-      return !memcmp( BSTRING_TO_STRING( o1 ), BSTRING_TO_STRING( o2 ), l );
+      return !memcmp( (void *)BSTRING_TO_STRING( o1 ),
+		      (void *)BSTRING_TO_STRING( o2 ), l );
    else
       return 0;
 }
@@ -451,17 +637,18 @@ bigloo_strncmp( obj_t o1, obj_t o2, long l )
 /*    bigloo_strncmp_ci ...                                            */
 /*---------------------------------------------------------------------*/
 bool_t
-bigloo_strncmp_ci( obj_t o1, obj_t o2, long l )
+bigloo_strncmp_ci( obj_t o1, obj_t o2, int l )
 {
-   long l1, l2;
+   int l1, l2;
 
    l1 = STRING_LENGTH( o1 );
    l2 = STRING_LENGTH( o2 );
 
    if( (l1 >= l) && (l2 >= l) )
    {
-      char *st1 = BSTRING_TO_STRING( o1 ), *st2 = BSTRING_TO_STRING( o2 );
-      long i;
+      char *st1 = (char *)BSTRING_TO_STRING( o1 );
+      char *st2 = (char *)BSTRING_TO_STRING( o2 );
+      int i;
       
       for( i = 0;
 	   (i < l) && (tolower( *st1 ) == tolower( *st2 ));
@@ -479,15 +666,16 @@ bigloo_strncmp_ci( obj_t o1, obj_t o2, long l )
 bool_t        
 strcicmp( obj_t bst1, obj_t bst2 )
 {
-   long l1, l2;
+   int l1, l2;
 
    l1 = STRING_LENGTH( bst1 );
    l2 = STRING_LENGTH( bst2 );
 
    if( l1 == l2 )
    {
-      char *st1 = BSTRING_TO_STRING( bst1 ), *st2 = BSTRING_TO_STRING( bst2 );
-      long i;
+      char *st1 = (char *)BSTRING_TO_STRING( bst1 );
+      char *st2 = (char *)BSTRING_TO_STRING( bst2 );
+      int i;
       
       for( i = 0;
 	   (i < l1) && (tolower( *st1 ) == tolower( *st2 ));
@@ -510,8 +698,8 @@ string_lt( obj_t bst1, obj_t bst2 )
 {
    unsigned char *st1 = (unsigned char *)BSTRING_TO_STRING( bst1 );
    unsigned char *st2 = (unsigned char *)BSTRING_TO_STRING( bst2 );
-   long l1, l2;
-   long i, min;
+   int l1, l2;
+   int i, min;
 
    l1 = STRING_LENGTH( bst1 );
    l2 = STRING_LENGTH( bst2 );
@@ -534,8 +722,8 @@ string_le( obj_t bst1, obj_t bst2 )
 {
    unsigned char *st1 = (unsigned char *)BSTRING_TO_STRING( bst1 );
    unsigned char *st2 = (unsigned char *)BSTRING_TO_STRING( bst2 );
-   long l1, l2;
-   long i, min;
+   int l1, l2;
+   int i, min;
 
    l1 = STRING_LENGTH( bst1 );
    l2 = STRING_LENGTH( bst2 );
@@ -558,8 +746,8 @@ string_gt( obj_t bst1, obj_t bst2 )
 {
    unsigned char *st1 = (unsigned char *)BSTRING_TO_STRING( bst1 );
    unsigned char *st2 = (unsigned char *)BSTRING_TO_STRING( bst2 );
-   long l1, l2;
-   long i, min;
+   int l1, l2;
+   int i, min;
 
    l1 = STRING_LENGTH( bst1 );
    l2 = STRING_LENGTH( bst2 );
@@ -582,8 +770,8 @@ string_ge( obj_t bst1, obj_t bst2 )
 {
    unsigned char *st1 = (unsigned char *)BSTRING_TO_STRING( bst1 );
    unsigned char *st2 = (unsigned char *)BSTRING_TO_STRING( bst2 );
-   long l1, l2;
-   long i, min;
+   int l1, l2;
+   int i, min;
 
    l1 = STRING_LENGTH( bst1 );
    l2 = STRING_LENGTH( bst2 );
@@ -606,8 +794,8 @@ string_cilt( obj_t bst1, obj_t bst2 )
 {
    unsigned char *st1 = (unsigned char *)BSTRING_TO_STRING( bst1 );
    unsigned char *st2 = (unsigned char *)BSTRING_TO_STRING( bst2 );
-   long l1, l2;
-   long i, min;
+   int l1, l2;
+   int i, min;
 
    l1 = STRING_LENGTH( bst1 );
    l2 = STRING_LENGTH( bst2 );
@@ -630,8 +818,8 @@ string_cile( obj_t bst1, obj_t bst2 )
 {
    unsigned char *st1 = (unsigned char *)BSTRING_TO_STRING( bst1 );
    unsigned char *st2 = (unsigned char *)BSTRING_TO_STRING( bst2 );
-   long l1, l2;
-   long i, min;
+   int l1, l2;
+   int i, min;
 
    l1 = STRING_LENGTH( bst1 );
    l2 = STRING_LENGTH( bst2 );
@@ -656,8 +844,8 @@ string_cigt( obj_t bst1, obj_t bst2 )
 {
    unsigned char *st1 = (unsigned char *)BSTRING_TO_STRING( bst1 );
    unsigned char *st2 = (unsigned char *)BSTRING_TO_STRING( bst2 );
-   long l1, l2;
-   long i, min;
+   int l1, l2;
+   int i, min;
 
    l1 = STRING_LENGTH( bst1 );
    l2 = STRING_LENGTH( bst2 );
@@ -682,8 +870,8 @@ string_cige( obj_t bst1, obj_t bst2 )
 {
    unsigned char *st1 = (unsigned char *)BSTRING_TO_STRING( bst1 );
    unsigned char *st2 = (unsigned char *)BSTRING_TO_STRING( bst2 );
-   long l1, l2;
-   long i, min;
+   int l1, l2;
+   int i, min;
 
    l1 = STRING_LENGTH( bst1 );
    l2 = STRING_LENGTH( bst2 );
@@ -713,19 +901,18 @@ string_cige( obj_t bst1, obj_t bst2 )
 /*    faire avec cette fonction.                                       */
 /*---------------------------------------------------------------------*/
 obj_t
-escape_C_string( unsigned char *src )
-{
+escape_C_string( unsigned char *src ) {
    /* on supprime un caractere de cette chaine car elle est rendue par le */
-   /* lecteur comme etant `"tototo'. Ceci est dut au fait qu'on utilise   */
+   /* lecteur comme etant `"tototo'. Ceci est du au fait qu'on utilise    */
    /* la fonction `the-small-string' qui supprime le premier et le        */
    /* dernier caractere de la chaine lu. Comme les chaines etrangeres     */
-   /* commence par 2 caracteres, on en supprime 1 autre maintenant.       */
+   /* commencent par 2 caracteres, on en supprime 1 autre maintenant.     */
 
-   long len = strlen( (const char *)(++src) );
+   int len = strlen( (const char *)(++src) );
    unsigned char *dst;
    obj_t string;
 
-   string = GC_MALLOC( STRING_SIZE + len );
+   string = GC_MALLOC_ATOMIC( STRING_SIZE + len );
 
 #if( !defined( TAG_STRING ) )
    string->string_t.header = MAKE_HEADER( STRING_TYPE, 0 );
@@ -733,73 +920,87 @@ escape_C_string( unsigned char *src )
 
    dst = ((unsigned char *)(&string->string_t.char0));
 
-   while( *src )
-   {
+   while( *src ) {
       if( *src != '\\' )
          *dst++ = *src++;
-      else
-      {
+      else {
 	 len--;
 	 
-         switch( *++src ) 
-         {
+         switch( *++src ) {
             case '\0' : *dst++ = '\\';
-                        break;
+	       break;
 
             case 'n'  : *dst++ = '\n';
-                        break;
+	       break;
                         
             case 't'  : *dst++ = '\t';
-                        break;
+	       break;
                         
             case 'b'  : *dst++ = '\b';
-                        break;
+	       break;
                         
             case 'r'  : *dst++ = '\r';
-                        break;
+	       break;
                         
             case 'f'  : *dst++ = '\f';
-                        break;
+	       break;
                         
             case 'v'  : *dst++ = '\v';
-                        break;
+	       break;
                         
             case '\\' : *dst++ = '\\';
-                        break;
+	       break;
                         
             case '\'' : *dst++ = '\'';
-                        break;
+	       break;
                         
             case '"'  : *dst++ = '\"';
-                        break;
+	       break;
 
 #if( defined( __STDC___ ) )                          
             case 'a'  : *dst++ = '\a';
-                        break;
+	       break;
 
             case '?'  : *dst++ = '\?';
-                        break;
+	       break;
 #endif                        
 
-            default   : if( isdigit( *(src) ) &&
-			    isdigit( *(src+1) ) &&
-			    isdigit( *(src+2) ) )
-	    /* C'est une representation alpha-numerique `\???' */
-	    {
-	       unsigned char aux;
+            default   : {
+	       char s0 = *src;
+	       char s1 = *(src+1);
+	       char s2 = *(src+2);
 	       
-	       aux = (*src     - '0')*64 +
-	             (*(src+1) - '0')*8 +
-		     (*(src+2) - '0');
-	       *dst++ = aux;
-	       src+=2;
+	       if( isdigit( s0 ) && isdigit( s1 ) && isdigit( s2 ) ) {
+		  unsigned char aux;
 	       
-	       len -= 2;
-	    }
-	    else
-	       *dst++ = *src;
+		  aux = (*src     - '0')*64 +
+   		        (*(src+1) - '0')*8 +
+		        (*(src+2) - '0');
+		  *dst++ = aux;
+		  src+=2;
+		  len -= 2;
+	       } else {
+		  if( ((s0 == 'x') || (s0 == 'X')) &&
+		      isxdigit( s1 ) &&
+		      isxdigit( s2 ) ) {
+		     unsigned char aux;
+		     char n1, n2;
+
+		     n1 = isdigit(s1) ? (s1 - '0') :
+			10 + ((s1 >= 'a') ?  (s1 - 'a') : (s1 - 'A'));
+		     n2 = isdigit(s2) ? (s2 - '0') :
+			10 + ((s2 >= 'a') ?  (s2 - 'a') : (s2 - 'A'));
+			
+		     *dst++ = n1*16 + n2;
+		     src+=2;
+		     len -= 2;
+		  }
+		  else
+		     *dst++ = *src;
+	       }
 		 
-	    break;
+	       break;
+	    }
          }
          src++;
       }
@@ -808,7 +1009,7 @@ escape_C_string( unsigned char *src )
    
    string->string_t.length = len;
 
-   return BSTRING( string );
+   return BGL_HEAP_DEBUG_MARK_OBJ( BSTRING( string ) );
 }
        
 /*---------------------------------------------------------------------*/
@@ -820,11 +1021,11 @@ escape_C_string( unsigned char *src )
 obj_t
 escape_scheme_string( char *src )
 {
-   long  len = strlen( src );
+   int  len = strlen( src );
    char *dst;
    obj_t string;
 
-   string = GC_MALLOC( STRING_SIZE + len );
+   string = GC_MALLOC_ATOMIC( STRING_SIZE + len );
 
 #if( !defined( TAG_STRING ) )
    string->string_t.header = MAKE_HEADER( STRING_TYPE, 0 );
@@ -838,27 +1039,17 @@ escape_scheme_string( char *src )
          *dst++ = *src++;
       else
       {
+	 char c = *(src + 1);
 	 len--;
-	 
-         switch( *++src )  
-         {
-            case '\\' : *dst++ = '\\';
-                        break;
-                        
-            case '"'  : *dst++ = '"';
-                        break;
-
-            default   : *dst++ = *src;
-                        break;
-         }
-         src++;
+	 *dst++ = ((c == 'n') ? '\n' : c);
+         src += 2;
       }
    }
    *dst = '\0';
    
    string->string_t.length = len;
 
-   return BSTRING( string );
+   return BGL_HEAP_DEBUG_MARK_OBJ( BSTRING( string ) );
 }
 
 /*---------------------------------------------------------------------*/
@@ -873,14 +1064,14 @@ int escape_char_found = 0;
 
 /*---------------------------------------------------------------------*/
 /*    obj_t                                                            */
-/*    string_for_read ...                                              */
+/*    create_string_for_read ...                                       */
 /*---------------------------------------------------------------------*/
 obj_t
-string_for_read( obj_t bstring )
+create_string_for_read( obj_t bstring, int symbolp )
 {
    unsigned char *dst;
-   unsigned char *src = (unsigned char *)(BSTRING_TO_STRING( bstring ));
-   long  r, w, len = STRING_LENGTH( bstring );
+   unsigned char *src = (unsigned char *)BSTRING_TO_STRING( bstring );
+   int  r, w, len = STRING_LENGTH( bstring );
    obj_t res;
 	
 #define BUFFER_SIZE 200
@@ -897,62 +1088,95 @@ string_for_read( obj_t bstring )
    for( r = 0, w = 0; r < len; r++ )
       switch( src[ r ] )
       {
-         case '\n' : dst[ w++ ] = '\\';
-                dst[ w++ ] = 'n';
-                escape_char_found = 1;
-                break;
+         case '\n' :
+	    dst[ w++ ] = '\\';
+	    dst[ w++ ] = 'n';
+	    escape_char_found = 1;
+	    break;
 
-         case '\t' : dst[ w++ ] = '\\';
-                dst[ w++ ] = 't';
-                escape_char_found = 1;
-                break;
+         case '\t' :
+	    dst[ w++ ] = '\\';
+	    dst[ w++ ] = 't';
+	    escape_char_found = 1;
+	    break;
 
-         case '\b' : dst[ w++ ] = '\\';
-                dst[ w++ ] = 'b';
-                escape_char_found = 1;
-                break;
+         case '\b' :
+	    dst[ w++ ] = '\\';
+	    dst[ w++ ] = 'b';
+	    escape_char_found = 1;
+	    break;
               
-         case '\r' : dst[ w++ ] = '\\';
-                dst[ w++ ] = 'r';
-                escape_char_found = 1;
-                break;
+         case '\r' :
+	    dst[ w++ ] = '\\';
+	    dst[ w++ ] = 'r';
+	    escape_char_found = 1;
+	    break;
               
-         case '\f' : dst[ w++ ] = '\\';
-                dst[ w++ ] = 'f';
-                escape_char_found = 1;
-                break;
+         case '\f' :
+	    dst[ w++ ] = '\\';
+	    dst[ w++ ] = 'f';
+	    escape_char_found = 1;
+	    break;
 
-         case '\v' : dst[ w++ ] = '\\';
-                dst[ w++ ] = 'v';
-                escape_char_found = 1;
-                break;
+         case '\v' :
+	    dst[ w++ ] = '\\';
+	    dst[ w++ ] = 'v';
+	    escape_char_found = 1;
+	    break;
 
-         case '"'  : dst[ w++ ] = '\\';
-                dst[ w++ ] = '"';
-                escape_char_found = 1;
-                break;
+         case '"'  :
+	    dst[ w++ ] = '\\';
+	    dst[ w++ ] = '"';
+	    escape_char_found = 1;
+	    break;
               
-         case '\\' : dst[ w++ ] = '\\';
-                dst[ w++ ] = '\\';
-                escape_char_found = 1;
-                break;
+         case '\\' :
+	    dst[ w++ ] = '\\';
+	    dst[ w++ ] = '\\';
+	    escape_char_found = 1;
+	    break;
 
-         default :   if( (src[r] < 128) && (isprint( src[ r ] )) )
-                        dst[ w++ ] = src[ r ];
-                     else
-           {
-              escape_char_found = 1;
-              sprintf( &dst[ w ], "\\%03o", ((unsigned char *)src)[ r ] );
-              w += 4;
-           }
+         case '|' :
+	    if( symbolp )
+	    {
+	       dst[ w++ ] = '\\';
+	       dst[ w++ ] = '|';
+	       escape_char_found = 1;
+	    }
+	    else
+	    {
+	       dst[ w++ ] = '|';
+	    }
+	    break;
+		    
+         default :
+	    if( (src[ r ] < 128) && (isprint( src[ r ] )) )
+	       dst[ w++ ] = src[ r ];
+	    else
+	    {
+	       escape_char_found = 1;
+	       sprintf( &dst[ w ], "\\%03o", ((unsigned char *)src)[ r ] );
+	       w += 4;
+	    }
       }
 
    dst[ w ] = '\0';
    
-   res = string_to_bstring( dst );
+   res = string_to_bstring( (char *)dst );
 
    if( dst != buffer )
       free( dst );
    
    return res;
 }
+
+/*---------------------------------------------------------------------*/
+/*    obj_t                                                            */
+/*    string_for_read ...                                              */
+/*---------------------------------------------------------------------*/
+obj_t
+string_for_read( obj_t bstring )
+{
+  return create_string_for_read( bstring, 0 );
+}
+
