@@ -1,41 +1,9 @@
-/*---------------------------------------------------------------------*/
-/*   A pratical implementation for the Scheme programming language     */
-/*                                                                     */
-/*                                    ,--^,                            */
-/*                              _ ___/ /|/                             */
-/*                          ,;'( )__, ) '                              */
-/*                         ;;  //   L__.                               */
-/*                         '   \\   /  '                               */
-/*                              ^   ^                                  */
-/*                                                                     */
-/*   Copyright (c) 1992-1999 Manuel Serrano                            */
-/*                                                                     */
-/*     Bug descriptions, use reports, comments or suggestions are      */
-/*     welcome. Send them to                                           */
-/*       bigloo-request@kaolin.unice.fr                                */
-/*       http://kaolin.unice.fr/bigloo                                 */
-/*                                                                     */
-/*   This program is free software; you can redistribute it            */
-/*   and/or modify it under the terms of the GNU General Public        */
-/*   License as published by the Free Software Foundation; either      */
-/*   version 2 of the License, or (at your option) any later version.  */
-/*                                                                     */
-/*   This program is distributed in the hope that it will be useful,   */
-/*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
-/*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
-/*   GNU General Public License for more details.                      */
-/*                                                                     */
-/*   You should have received a copy of the GNU General Public         */
-/*   License along with this program; if not, write to the Free        */
-/*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
-/*   MA 02111-1307, USA.                                               */
-/*---------------------------------------------------------------------*/
 /*=====================================================================*/
-/*    serrano/prgm/project/bigloo/runtime1.9c/Clib/cucs2.c             */
+/*    serrano/prgm/project/bigloo2.3/runtime/Clib/cucs2.c              */
 /*    -------------------------------------------------------------    */
 /*    Author      :  Manuel Serrano                                    */
 /*    Creation    :  Sun May 18 08:35:34 1997                          */
-/*    Last change :  Tue Aug  5 16:44:14 1997 (serrano)                */
+/*    Last change :  Thu Aug 10 12:59:51 2000 (serrano)                */
 /*    -------------------------------------------------------------    */
 /*    UCS-2 Characters management.                                     */
 /*    -------------------------------------------------------------    */
@@ -43,7 +11,7 @@
 /*    the Java 1.1.1 Character implementation (so, many thanks         */
 /*    to Lee Boynton, Guy Steele and Akira Tanaka).                    */
 /*=====================================================================*/
-#include <bigloo2.0a.h>
+#include <bigloo.h>
  
 /*---------------------------------------------------------------------*/
 /*    General Character category types                                 */
@@ -1354,9 +1322,9 @@ static unsigned int A[] = {
 /*    entry in the Unicode attribute table.                            */
 /*---------------------------------------------------------------------*/
 bool_t
-ucs2_definedp( ucs2_t ch )
+ucs2_definedp( int ch )
 {
-   return (A[ Y[ (X[ ch>>6 ]<<6) | (ch&0x3F) ] ] & 0x1F) != UNASSIGNED;
+   return (A[ (int)Y[ (X[ ch>>6 ]<<6) | (ch&0x3F) ] ] & 0x1F) != UNASSIGNED;
 }
 
 /*---------------------------------------------------------------------*/
@@ -1370,7 +1338,7 @@ ucs2_definedp( ucs2_t ch )
 bool_t
 ucs2_lowerp( ucs2_t ch )
 {
-   return (A[ Y[ (X[ ch>>6 ]<<6) | (ch & 0x3F) ] ] & 0x1F) == LOWERCASE_LETTER;
+   return (A[ (int)Y[ (X[ ch>>6 ]<<6) | (ch & 0x3F) ] ] & 0x1F) == LOWERCASE_LETTER;
 }
 
 /*---------------------------------------------------------------------*/
@@ -1380,7 +1348,7 @@ ucs2_lowerp( ucs2_t ch )
 bool_t
 ucs2_upperp( ucs2_t ch )
 {
-   return (A[ Y[ (X[ ch>>6 ]<<6) | (ch&0x3F) ] ] & 0x1F) == UPPERCASE_LETTER;
+   return (A[ (int)Y[ (X[ ch>>6 ]<<6) | (ch&0x3F) ] ] & 0x1F) == UPPERCASE_LETTER;
 }
 
 /*---------------------------------------------------------------------*/
@@ -1390,7 +1358,7 @@ ucs2_upperp( ucs2_t ch )
 bool_t
 ucs2_digitp( ucs2_t ch )
 {
-   return (A[ Y[ (X[ ch>>6 ]<<6) | (ch&0x3F) ] ] & 0x1F) == DECIMAL_NUMBER;
+   return (A[ (int)Y[ (X[ ch>>6 ]<<6) | (ch&0x3F) ] ] & 0x1F) == DECIMAL_NUMBER;
 }
 
 /*---------------------------------------------------------------------*/
@@ -1405,7 +1373,7 @@ ucs2_letterp( ucs2_t ch )
 	      (1 << TITLECASE_LETTER) |
 	      (1 << MODIFIER_LETTER)  |
 	      (1 << OTHER_LETTER))
-	     >> (A[ Y[ (X[ ch>>6 ]<<6) | (ch&0x3F) ] ] & 0x1F)) & 1) != 0);
+	     >> (A[ (int)Y[ (X[ ch>>6 ]<<6) | (ch&0x3F) ] ] & 0x1F)) & 1) != 0);
 }
 
 /*---------------------------------------------------------------------*/
@@ -1424,7 +1392,7 @@ ucs2_letterp( ucs2_t ch )
 bool_t
 ucs2_whitespacep( ucs2_t ch )
 {
-   return (A[ Y[ (X[ ch>>6 ]<<6) | (ch&0x3F) ] ] & 0x00070000) == 0x00040000;
+   return (A[ (int)Y[ (X[ ch>>6 ]<<6) | (ch&0x3F) ] ] & 0x00070000) == 0x00040000;
 }
 
 /*---------------------------------------------------------------------*/
@@ -1434,7 +1402,7 @@ ucs2_whitespacep( ucs2_t ch )
 ucs2_t
 ucs2_toupper( ucs2_t ch )
 {
-   unsigned int val = A[ Y[ (X[ ch>>6 ]<<6) | (ch&0x3F) ] ];
+   unsigned int val = A[ (int)Y[ (X[ ch>>6 ]<<6) | (ch&0x3F) ] ];
    
    if( (val & 0x00100000) != 0 )
       return (ucs2_t)(ch - (val >> 22));
@@ -1449,7 +1417,7 @@ ucs2_toupper( ucs2_t ch )
 ucs2_t
 ucs2_tolower( ucs2_t ch )
 {
-   unsigned int val = A[ Y[ (X[ ch>>6 ]<<6) | (ch & 0x3F) ] ];
+   unsigned int val = A[ (int)Y[ (X[ ch>>6 ]<<6) | (ch & 0x3F) ] ];
    
    if ((val & 0x00200000) != 0)
       return (char)(ch + (val >> 22));
