@@ -1,61 +1,33 @@
-/*---------------------------------------------------------------------*/
-/*   A pratical implementation for the Scheme programming language     */
-/*                                                                     */
-/*                                    ,--^,                            */
-/*                              _ ___/ /|/                             */
-/*                          ,;'( )__, ) '                              */
-/*                         ;;  //   L__.                               */
-/*                         '   \\   /  '                               */
-/*                              ^   ^                                  */
-/*                                                                     */
-/*   Copyright (c) 1992-1999 Manuel Serrano                            */
-/*                                                                     */
-/*     Bug descriptions, use reports, comments or suggestions are      */
-/*     welcome. Send them to                                           */
-/*       bigloo-request@kaolin.unice.fr                                */
-/*       http://kaolin.unice.fr/bigloo                                 */
-/*                                                                     */
-/*   This program is free software; you can redistribute it            */
-/*   and/or modify it under the terms of the GNU General Public        */
-/*   License as published by the Free Software Foundation; either      */
-/*   version 2 of the License, or (at your option) any later version.  */
-/*                                                                     */
-/*   This program is distributed in the hope that it will be useful,   */
-/*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
-/*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
-/*   GNU General Public License for more details.                      */
-/*                                                                     */
-/*   You should have received a copy of the GNU General Public         */
-/*   License along with this program; if not, write to the Free        */
-/*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
-/*   MA 02111-1307, USA.                                               */
-/*---------------------------------------------------------------------*/
 /*=====================================================================*/
 /*    serrano/prgm/project/bigloo/runtime/Clib/cunicode.c              */
 /*    -------------------------------------------------------------    */
 /*    Author      :  Manuel Serrano                                    */
 /*    Creation    :  Mon May 19 17:47:11 1997                          */
-/*    Last change :  Sun Oct  4 13:57:44 1998 (serrano)                */
+/*    Last change :  Wed Jun  6 14:11:47 2001 (serrano)                */
 /*    -------------------------------------------------------------    */
 /*    Unicode strings handling                                         */
 /*=====================================================================*/
 #include <string.h>
-#include <bigloo2.0a.h>
+#include <bigloo.h>
 
 /*---------------------------------------------------------------------*/
 /*    Importations                                                     */
 /*---------------------------------------------------------------------*/
 extern bool_t ucs2_lower( ucs2_t );
-extern char *integer_to_string();
+extern obj_t integer_to_string();
 extern char *real_to_string();
 extern obj_t make_string();
+extern ucs2_t ucs2_tolower( ucs2_t );
+extern char strputc( char, obj_t );
+extern obj_t lstrputs( char *, obj_t, int );
+extern obj_t strputs( char *, obj_t );
 
 /*---------------------------------------------------------------------*/
 /*    ucs2_string_t                                                    */
 /*    make_ucs2_string ...                                             */
 /*---------------------------------------------------------------------*/
 obj_t
-make_ucs2_string( long len, ucs2_t c )
+make_ucs2_string( int len, ucs2_t c )
 {
    obj_t   string;
    ucs2_t *cstring;
@@ -65,9 +37,9 @@ make_ucs2_string( long len, ucs2_t c )
 	       string_to_bstring( "Illegal string size" ),
 	       BINT( len ) );
    {
-      long i;
+      int i;
       
-      string  = GC_MALLOC_ATOMIC( UCS2_STRING_SIZE + len );
+      string  = GC_MALLOC_ATOMIC( UCS2_STRING_SIZE + (len * sizeof(ucs2_t)) );
       cstring = (&(string->ucs2_string_t.char0));
 
       string->ucs2_string_t.header = MAKE_HEADER( UCS2_STRING_TYPE, 0 );
@@ -77,7 +49,7 @@ make_ucs2_string( long len, ucs2_t c )
 	cstring[ i ] = c;
       cstring[ i ] = (ucs2_t)0;
 		
-      return BUCS2STRING( string );
+      return BGL_HEAP_DEBUG_MARK_OBJ( BUCS2STRING( string ) );
    }
 }
 
@@ -86,7 +58,7 @@ make_ucs2_string( long len, ucs2_t c )
 /*    ucs2cpy ...                                                      */
 /*---------------------------------------------------------------------*/
 void
-ucs2cpy( ucs2_t *u1, ucs2_t *u2, long len )
+ucs2cpy( ucs2_t *u1, ucs2_t *u2, int len )
 {
    for( len--; len >= 0; len-- )
       u1[ len ] = u2[ len ];
@@ -98,13 +70,13 @@ ucs2cpy( ucs2_t *u1, ucs2_t *u2, long len )
 obj_t
 ucs2_string_append( obj_t s1, obj_t s2 )
 {
-   long    l1, l2;
+   int    l1, l2;
    obj_t   ucs2_string;
    
    l1 = UCS2_STRING( s1 ).length;
    l2 = UCS2_STRING( s2 ).length;
-	
-   ucs2_string = GC_MALLOC_ATOMIC( UCS2_STRING_SIZE + l1 + l2 );
+
+   ucs2_string = GC_MALLOC_ATOMIC( UCS2_STRING_SIZE + ((l1 + l2) * sizeof( ucs2_t )) );
 
    ucs2_string->ucs2_string_t.header = MAKE_HEADER( UCS2_STRING_TYPE, 0 );
    ucs2_string->ucs2_string_t.length = l1 + l2;
@@ -117,7 +89,7 @@ ucs2_string_append( obj_t s1, obj_t s2 )
 	    l2 );
    ((ucs2_t *)(&(ucs2_string->ucs2_string_t.char0)))[ l1 + l2 ] = (ucs2_t)0;
 	
-   return BUCS2STRING( ucs2_string );
+   return BGL_HEAP_DEBUG_MARK_OBJ( BUCS2STRING( ucs2_string ) );
 }
  
 /*---------------------------------------------------------------------*/
@@ -126,14 +98,14 @@ ucs2_string_append( obj_t s1, obj_t s2 )
 obj_t
 c_subucs2_string( src_ucs2_string, min, max )
 obj_t src_ucs2_string;
-long  min, max;
+int  min, max;
 {
-   long  len;
+   int  len;
    obj_t dst_ucs2_string;
    
    len = max - min;
 
-   dst_ucs2_string = GC_MALLOC_ATOMIC( UCS2_STRING_SIZE + len );
+   dst_ucs2_string = GC_MALLOC_ATOMIC( UCS2_STRING_SIZE + (len * sizeof( ucs2_t ) ) );
 
    dst_ucs2_string->ucs2_string_t.header = MAKE_HEADER( UCS2_STRING_TYPE, 0 );
    dst_ucs2_string->ucs2_string_t.length = len;
@@ -143,7 +115,7 @@ long  min, max;
             len );
    (&(dst_ucs2_string->ucs2_string_t.char0))[ len ] = (ucs2_t)0;
 
-   return BUCS2STRING( dst_ucs2_string );
+   return BGL_HEAP_DEBUG_MARK_OBJ( BUCS2STRING( dst_ucs2_string ) );
 }
 
 /*---------------------------------------------------------------------*/
@@ -153,12 +125,12 @@ long  min, max;
 obj_t
 c_ucs2_string_copy( obj_t src )
 {
-   long    len = UCS2_STRING_LENGTH( src );
+   int    len = UCS2_STRING_LENGTH( src );
    obj_t   string;
    ucs2_t *cstring, *cstr;
-   long    i;
+   int    i;
 
-   string  = GC_MALLOC_ATOMIC( UCS2_STRING_SIZE + len );
+   string  = GC_MALLOC_ATOMIC( UCS2_STRING_SIZE + (len * sizeof( ucs2_t ) ) );
    cstring = (&(string->ucs2_string_t.char0));
    cstr    = &UCS2_STRING_REF( src, 0 );
 
@@ -169,7 +141,7 @@ c_ucs2_string_copy( obj_t src )
       cstring[ i ] = cstr[ i ];
    cstring[ i ] = (ucs2_t)0;
 		
-   return BUCS2STRING( string );
+   return BGL_HEAP_DEBUG_MARK_OBJ( BUCS2STRING( string ) );
 }
 
 /*---------------------------------------------------------------------*/
@@ -179,12 +151,12 @@ c_ucs2_string_copy( obj_t src )
 obj_t
 string_to_ucs2_string( char *c )
 {
-   long    len = strlen( c );
+   int    len = strlen( c );
    obj_t   string;
    ucs2_t *cstring;
-   long    i;
+   int    i;
 
-   string  = GC_MALLOC_ATOMIC( UCS2_STRING_SIZE + len );
+   string  = GC_MALLOC_ATOMIC( UCS2_STRING_SIZE + (len * sizeof( ucs2_t ) ) );
    cstring = (&(string->ucs2_string_t.char0));
 
    string->ucs2_string_t.header = MAKE_HEADER( UCS2_STRING_TYPE, 0 );
@@ -194,7 +166,7 @@ string_to_ucs2_string( char *c )
       cstring[ i ] = (ucs2_t)(c[ i ]);
    cstring[ i ] = (ucs2_t)0;
 		
-   return BUCS2STRING( string );
+   return BGL_HEAP_DEBUG_MARK_OBJ( BUCS2STRING( string ) );
 }
 
 /*---------------------------------------------------------------------*/
@@ -204,13 +176,13 @@ string_to_ucs2_string( char *c )
 obj_t
 bstring_to_ucs2_string( obj_t src )
 {
-   long    len = STRING_LENGTH( src );
+   int    len = STRING_LENGTH( src );
    obj_t   string;
    ucs2_t *cstring;
-   long    i;
-   char   *c = (&(src->string_t.char0));
+   int    i;
+   char   *c = (char *)(&(src->string_t.char0));
 
-   string  = GC_MALLOC_ATOMIC( UCS2_STRING_SIZE + len );
+   string  = GC_MALLOC_ATOMIC( UCS2_STRING_SIZE + (len * sizeof( ucs2_t ) ) );
    cstring = (&(string->ucs2_string_t.char0));
 
    string->ucs2_string_t.header = MAKE_HEADER( UCS2_STRING_TYPE, 0 );
@@ -220,7 +192,7 @@ bstring_to_ucs2_string( obj_t src )
       cstring[ i ] = (ucs2_t)(c[ i ]);
    cstring[ i ] = (ucs2_t)0;
 		
-   return BUCS2STRING( string );
+   return BGL_HEAP_DEBUG_MARK_OBJ( BUCS2STRING( string ) );
 }
 
 /*---------------------------------------------------------------------*/
@@ -230,7 +202,7 @@ obj_t
 integer_to_ucs2_string( x, radix )
 long x, radix;
 {
-   return string_to_ucs2_string( integer_to_string( x, radix ) );
+   return string_to_ucs2_string( BSTRING_TO_STRING( integer_to_string( x, radix ) ) );
 }
 
 /*---------------------------------------------------------------------*/
@@ -240,7 +212,7 @@ obj_t
 real_to_ucs2_string( x )
 double x;
 {
-   return string_to_ucs2_string( real_to_string( x ) );
+   return string_to_ucs2_string( BSTRING_TO_STRING( real_to_string( x ) ) );
 }
 
 /*---------------------------------------------------------------------*/
@@ -251,7 +223,7 @@ bool_t
 ucs2_strcmp( o1, o2 )
 obj_t o1, o2;
 {
-   long l1, l2;
+   int l1, l2;
    
    l1 = UCS2_STRING_LENGTH( o1 );
    l2 = UCS2_STRING_LENGTH( o2 );
@@ -280,7 +252,7 @@ bool_t
 ucs2_strcicmp( bst1, bst2 )
 obj_t bst1, bst2;
 {
-   long l1, l2;
+   int l1, l2;
 
    l1 = UCS2_STRING_LENGTH( bst1 );
    l2 = UCS2_STRING_LENGTH( bst2 );
@@ -289,7 +261,7 @@ obj_t bst1, bst2;
    {
       ucs2_t *st1 = BUCS2_STRING_TO_UCS2_STRING( bst1 );
       ucs2_t *st2 = BUCS2_STRING_TO_UCS2_STRING( bst2 );
-      long i;
+      int i;
    
       for( i = 0;
 	   ucs2_tolower( *st1 ) == ucs2_tolower( *st2 );
@@ -310,8 +282,8 @@ obj_t bst1, bst2;
 {
    ucs2_t *st1 = BUCS2_STRING_TO_UCS2_STRING( bst1 );
    ucs2_t *st2 = BUCS2_STRING_TO_UCS2_STRING( bst2 );
-   long l1, l2;
-   long i, min;
+   int l1, l2;
+   int i, min;
 
    l1 = UCS2_STRING_LENGTH( bst1 );
    l2 = UCS2_STRING_LENGTH( bst2 );
@@ -335,8 +307,8 @@ obj_t bst1, bst2;
 {
    ucs2_t *st1 = BUCS2_STRING_TO_UCS2_STRING( bst1 );
    ucs2_t *st2 = BUCS2_STRING_TO_UCS2_STRING( bst2 );
-   long l1, l2;
-   long i, min;
+   int l1, l2;
+   int i, min;
 
    l1 = UCS2_STRING_LENGTH( bst1 );
    l2 = UCS2_STRING_LENGTH( bst2 );
@@ -360,8 +332,8 @@ obj_t bst1, bst2;
 {
    ucs2_t *st1 = BUCS2_STRING_TO_UCS2_STRING( bst1 );
    ucs2_t *st2 = BUCS2_STRING_TO_UCS2_STRING( bst2 );
-   long l1, l2;
-   long i, min;
+   int l1, l2;
+   int i, min;
 
    l1 = UCS2_STRING_LENGTH( bst1 );
    l2 = UCS2_STRING_LENGTH( bst2 );
@@ -385,8 +357,8 @@ obj_t bst1, bst2;
 {
    ucs2_t *st1 = BUCS2_STRING_TO_UCS2_STRING( bst1 );
    ucs2_t *st2 = BUCS2_STRING_TO_UCS2_STRING( bst2 );
-   long l1, l2;
-   long i, min;
+   int l1, l2;
+   int i, min;
 
    l1 = UCS2_STRING_LENGTH( bst1 );
    l2 = UCS2_STRING_LENGTH( bst2 );
@@ -410,8 +382,8 @@ obj_t bst1, bst2;
 {
    ucs2_t *st1 = BUCS2_STRING_TO_UCS2_STRING( bst1 );
    ucs2_t *st2 = BUCS2_STRING_TO_UCS2_STRING( bst2 );
-   long l1, l2;
-   long i, min;
+   int l1, l2;
+   int i, min;
 
    l1 = UCS2_STRING_LENGTH( bst1 );
    l2 = UCS2_STRING_LENGTH( bst2 );
@@ -437,8 +409,8 @@ obj_t bst1, bst2;
 {
    ucs2_t *st1 = BUCS2_STRING_TO_UCS2_STRING( bst1 );
    ucs2_t *st2 = BUCS2_STRING_TO_UCS2_STRING( bst2 );
-   long l1, l2;
-   long i, min;
+   int l1, l2;
+   int i, min;
 
    l1 = UCS2_STRING_LENGTH( bst1 );
    l2 = UCS2_STRING_LENGTH( bst2 );
@@ -464,8 +436,8 @@ obj_t bst1, bst2;
 {
    ucs2_t *st1 = BUCS2_STRING_TO_UCS2_STRING( bst1 );
    ucs2_t *st2 = BUCS2_STRING_TO_UCS2_STRING( bst2 );
-   long l1, l2;
-   long i, min;
+   int l1, l2;
+   int i, min;
 
    l1 = UCS2_STRING_LENGTH( bst1 );
    l2 = UCS2_STRING_LENGTH( bst2 );
@@ -491,8 +463,8 @@ obj_t bst1, bst2;
 {
    ucs2_t *st1 = BUCS2_STRING_TO_UCS2_STRING( bst1 );
    ucs2_t *st2 = BUCS2_STRING_TO_UCS2_STRING( bst2 );
-   long l1, l2;
-   long i, min;
+   int l1, l2;
+   int i, min;
 
    l1 = UCS2_STRING_LENGTH( bst1 );
    l2 = UCS2_STRING_LENGTH( bst2 );
@@ -516,9 +488,9 @@ obj_t bst1, bst2;
 obj_t
 display_ucs2string( obj_t obj, obj_t port )
 {
-   long    len  = UCS2_STRING_LENGTH( obj );
+   int    len  = UCS2_STRING_LENGTH( obj );
    ucs2_t *ucs2 = BUCS2_STRING_TO_UCS2_STRING( obj );
-   long    i;
+   int    i;
    
    if( OUTPUT_STRING_PORTP( port ) )
    {
@@ -558,8 +530,8 @@ display_ucs2string( obj_t obj, obj_t port )
 obj_t
 write_utf8string( obj_t string, obj_t port )
 {
-   char *aux = BSTRING_TO_STRING( string );
-   long  len = STRING_LENGTH( string );
+   char *aux = (char *)BSTRING_TO_STRING( string );
+   int  len = STRING_LENGTH( string );
    
    if( OUTPUT_STRING_PORTP( port ) )
    {
@@ -609,12 +581,12 @@ utf8_size( ucs2_t ucs2 )
 obj_t
 ucs2_string_to_utf8_string( obj_t bucs2 )
 {
-   long           len      = UCS2_STRING_LENGTH( bucs2 );
-   long           utf8_len = 0;
-   ucs2_t        *cucs2    = BUCS2_STRING_TO_UCS2_STRING( bucs2 );
-   long           read, write;
-   obj_t          result;
-   unsigned char *cresult;
+   int    len      = UCS2_STRING_LENGTH( bucs2 );
+   int    utf8_len = 0;
+   ucs2_t *cucs2    = BUCS2_STRING_TO_UCS2_STRING( bucs2 );
+   int    read, write;
+   obj_t   result;
+   char   *cresult;
    
    /* First, we compute the size of the futur utf8 string */
    for( read = 0, utf8_len = 0; read < len; read++ )
@@ -622,7 +594,7 @@ ucs2_string_to_utf8_string( obj_t bucs2 )
 
    /* now, we allocate the new string */
    result  = make_string( utf8_len, '0' );
-   cresult = (unsigned char *)BSTRING_TO_STRING( result );
+   cresult = BSTRING_TO_STRING( result );
    
    /* and we fill it */
    for( read = 0, write = 0; read < len; read++ )
@@ -658,12 +630,12 @@ ucs2_string_to_utf8_string( obj_t bucs2 )
 obj_t
 utf8_string_to_ucs2_string( obj_t butf8 )
 {
-   long           len = STRING_LENGTH( butf8 );
-   ucs2_t        *aux = (ucs2_t *)alloca( len * sizeof( ucs2_t ) );
-   unsigned char *cutf8 = (unsigned char *)BSTRING_TO_STRING( butf8 );
-   long           read, write;
-   obj_t          string;
-   ucs2_t        *cstring;
+   int    len = STRING_LENGTH( butf8 );
+   ucs2_t *aux = (ucs2_t *)alloca( len * sizeof( ucs2_t ) );
+   char   *cutf8 = BSTRING_TO_STRING( butf8 );
+   int    read, write;
+   obj_t   string;
+   ucs2_t *cstring;
    
    for( read = 0, write = 0; read < len; write++ )
    {
@@ -712,13 +684,13 @@ utf8_string_to_ucs2_string( obj_t butf8 )
       }
    }
 	 
-   string  = GC_MALLOC_ATOMIC( UCS2_STRING_SIZE + len );
+   string  = GC_MALLOC_ATOMIC( UCS2_STRING_SIZE + (len * sizeof( ucs2_t ) ) );
    cstring = (&(string->ucs2_string_t.char0));
 
    string->ucs2_string_t.header = MAKE_HEADER( UCS2_STRING_TYPE, 0 );
    string->ucs2_string_t.length = write;
    ucs2cpy( cstring, aux, write );
 		
-   return BUCS2STRING( string );
+   return BGL_HEAP_DEBUG_MARK_OBJ( BUCS2STRING( string ) );
 }
    
