@@ -1,43 +1,15 @@
-;*---------------------------------------------------------------------*/
-;*   A pratical implementation for the Scheme programming language     */
-;*                                                                     */
-;*                                    ,--^,                            */
-;*                              _ ___/ /|/                             */
-;*                          ,;'( )__, ) '                              */
-;*                         ;;  //   L__.                               */
-;*                         '   \\   /  '                               */
-;*                              ^   ^                                  */
-;*                                                                     */
-;*   Copyright (c) 1992-1999 Manuel Serrano                            */
-;*                                                                     */
-;*     Bug descriptions, use reports, comments or suggestions are      */
-;*     welcome. Send them to                                           */
-;*       bigloo-request@kaolin.unice.fr                                */
-;*       http://kaolin.unice.fr/bigloo                                 */
-;*                                                                     */
-;*   This program is free software; you can redistribute it            */
-;*   and/or modify it under the terms of the GNU General Public        */
-;*   License as published by the Free Software Foundation; either      */
-;*   version 2 of the License, or (at your option) any later version.  */
-;*                                                                     */
-;*   This program is distributed in the hope that it will be useful,   */
-;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
-;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
-;*   GNU General Public License for more details.                      */
-;*                                                                     */
-;*   You should have received a copy of the GNU General Public         */
-;*   License along with this program; if not, write to the Free        */
-;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
-;*   MA 02111-1307, USA.                                               */
-;*---------------------------------------------------------------------*/
 ;*=====================================================================*/
 ;*    serrano/prgm/project/bigloo/runtime/Eval/eval.scm                */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Sat Oct 22 09:34:28 1994                          */
-;*    Last change :  Fri Nov 13 15:37:08 1998 (serrano)                */
+;*    Last change :  Mon Oct  1 07:51:40 2001 (serrano)                */
 ;*    -------------------------------------------------------------    */
 ;*    L'evaluateur de Bigloo                                           */
+;*    -------------------------------------------------------------    */
+;*    Source documentation:                                            */
+;*       @path ../../manuals/eval.texi@                                */
+;*       @node Eval@                                                   */
 ;*=====================================================================*/
 
 ;*---------------------------------------------------------------------*/
@@ -45,70 +17,88 @@
 ;*---------------------------------------------------------------------*/
 (module __eval
    
-   (export  (eval                       <expression> . <env>)
-	    (scheme-report-environment  <version>)
-	    (null-environment           <versino>)
+   (export  (eval <expression> . <env>)
+	    (scheme-report-environment <version>)
+	    (null-environment <version>)
 	    (interaction-environment)
-	    (load                       <string>)
-	    (loadq                      <string>)
-	    (loada                      <string>)
+	    *load-verbose*
+	    (load <string>)
+	    (loadq <string>)
+	    (loada <string>)
 	    (repl)
 	    (quit)
-	    (module-declaration!        decls)
-	    (expand-define-macro        <expression> <expander>)
+	    (module-declaration! decls)
+	    (expand-define-macro <expression> <expander>)
 	    (expand-define-hygien-macro <expression> <expander>)
-	    (expand-define-expander     <expression> <expander>)
-	    (expand-define-pattern      <expression>)
-	    (set-prompter!              ::procedure)
+	    (expand-define-expander <expression> <expander>)
+	    (expand-define-pattern <expression>)
+	    (set-prompter! ::procedure)
 	    (get-prompter::procedure)
 	    *load-path*
 	    *user-pass-name*
 	    *user-pass*
-	    (notify-assert-fail         vars body loc)
-	    *nil*)
+	    (notify-assert-fail vars body loc)
+	    *nil*
+	    (transcript-on ::bstring)
+	    (transcript-off)
+	    (default-repl-error-notifier proc obj msg)
+	    (set-repl-printer! ::procedure)
+	    (native-repl-printer::procedure)
+	    (set-repl-error-notifier! ::procedure)
+	    (get-repl-error-notifier::procedure))
    
-   (import  (__type                    "Llib/type.scm")
-	    (__error                   "Llib/error.scm")
-	    (__bigloo                  "Llib/bigloo.scm")
-	    (__tvector                 "Llib/tvector.scm")
-	    (__structure               "Llib/struct.scm")
-	    (__tvector                 "Llib/tvector.scm")
-	    (__bexit                   "Llib/bexit.scm")
-	    (__os                      "Llib/os.scm")
+   (import  __type
+	    __error
+	    __bigloo
+	    __tvector
+	    __structure
+	    __tvector
+	    __bexit
+	    __os
 	    
-	    (__reader                  "Read/reader.scm")
+	    __pp_circle
 	    
-	    (__r4_numbers_6_5          "Ieee/number.scm")
-	    (__r4_numbers_6_5_fixnum   "Ieee/fixnum.scm")
-	    (__r4_numbers_6_5_flonum   "Ieee/flonum.scm")
-	    (__r4_characters_6_6       "Ieee/char.scm")
-	    (__r4_equivalence_6_2      "Ieee/equiv.scm")
-	    (__r4_booleans_6_1         "Ieee/boolean.scm")
-	    (__r4_symbols_6_4          "Ieee/symbol.scm")
-	    (__r4_strings_6_7          "Ieee/string.scm")
-	    (__r4_pairs_and_lists_6_3  "Ieee/pair-list.scm")
-	    (__r4_input_6_10_2         "Ieee/input.scm")
-	    (__r4_control_features_6_9 "Ieee/control.scm")
-	    (__r4_vectors_6_8          "Ieee/vector.scm")
-	    (__r4_ports_6_10_1         "Ieee/port.scm")
-	    (__r4_output_6_10_3        "Ieee/output.scm")
-
-	    (__macro                   "Eval/macro.scm")
-	    (__install_expanders       "Eval/expanders.scm")
-	    (__progn                   "Eval/progn.scm")
-	    (__expand                  "Eval/expand.scm")
-	    (__evcompile               "Eval/evcompile.scm")
-	    (__evmeaning               "Eval/evmeaning.scm")
-	    (__evprimop                "Eval/evprimop.scm")
-	    (__evenv                   "Eval/evenv.scm")
-	    (extend.r.macro-env __match_normalize "Match/normalize.scm")
-   
-	    (__r5_macro_4_3_init       "R5rs/init5.scm"))
+	    __reader
+	    
+	    __r4_numbers_6_5
+	    __r4_numbers_6_5_fixnum
+	    __r4_numbers_6_5_flonum
+	    __r4_characters_6_6
+	    __r4_equivalence_6_2
+	    __r4_booleans_6_1
+	    __r4_symbols_6_4
+	    __r4_strings_6_7
+	    __r4_pairs_and_lists_6_3
+	    __r4_input_6_10_2
+	    __r4_control_features_6_9
+	    __r4_vectors_6_8
+	    __r4_ports_6_10_1
+	    __r4_output_6_10_3
+	    
+	    __macro
+	    __install_expanders
+	    __progn
+	    __expand
+	    __evcompile
+	    __evmeaning
+	    __evprimop
+	    __evenv
+	    (extend.r.macro-env __match_normalize)
+	    
+	    __r5_macro_4_3_init)
    
    (extern  (include "signal.h")
 	    (reset-console!::obj   (::obj) "reset_console")
 	    (macro sigsetmask::int (::int) "sigsetmask")
-	    (macro sigint::int             "SIGINT")))
+	    (macro sigint::int             "SIGINT"))
+   
+   (java    (class foreign
+	       (method static reset-console!::obj (::obj)
+		       "reset_console")
+	       (method static sigsetmask::int (::int)
+		       "sigsetmask")
+	       (field static sigint::int
+		      "SIGINT"))))
 
 ;*---------------------------------------------------------------------*/
 ;*    Expanders setup.                                                 */
@@ -167,6 +157,41 @@
 		    (display ":=> ")
 		    (flush-output-port (current-output-port))))
 
+;*---------------------------------------------------------------------*/
+;*    repl-notify-error ...                                            */
+;*---------------------------------------------------------------------*/
+(define (repl-notify-error proc msg obj)
+   ((if (procedure? *repl-error-notifier*)
+	*repl-error-notifier*
+	default-repl-error-notifier)
+    proc msg obj))
+
+;*---------------------------------------------------------------------*/
+;*    default-repl-error-notifier ...                                  */
+;*---------------------------------------------------------------------*/
+(define (default-repl-error-notifier proc msg obj)
+   (evmeaning-notify-error proc msg obj)
+   (flush-output-port (current-error-port)))
+   
+;*---------------------------------------------------------------------*/
+;*    *repl-error-notifier* ...                                        */
+;*---------------------------------------------------------------------*/
+(define *repl-error-notifier* default-repl-error-notifier)
+
+;*---------------------------------------------------------------------*/
+;*    set-repl-error-notifier! ...                                     */
+;*---------------------------------------------------------------------*/
+(define (set-repl-error-notifier! notifier::procedure)
+   (if (=fx (procedure-arity notifier) 3)
+       (set! *repl-error-notifier* notifier)
+       (error "set-repl-error-notifier!" "Illegal notifier" notifier)))
+
+;*---------------------------------------------------------------------*/
+;*    get-repl-error-notifier ...                                      */
+;*---------------------------------------------------------------------*/
+(define (get-repl-error-notifier::procedure)
+   *repl-error-notifier*)
+
 ;*---------------------------------------------------------------------*/
 ;*    set-prompter! ...                                                */
 ;*---------------------------------------------------------------------*/
@@ -187,7 +212,7 @@
 ;*    Global repl parameters ...                                       */
 ;*---------------------------------------------------------------------*/
 (define *repl-num* 0)
-(define *repl-quit* bigloo-exit)
+(define *repl-quit* (lambda (x) (%exit x)))
 
 ;*---------------------------------------------------------------------*/
 ;*    repl ...                                                         */
@@ -216,10 +241,7 @@
 	    (bind-exit (re-enter-internal-repl)
 	       ;; we setup ^C interupt
 	       (letrec ((intrhdl (lambda (n)
-				    (newline (current-error-port))
-				    (fprint (current-error-port)
-					    "*** INTERRUPT:bigloo:")
-				    (flush-output-port (current-error-port))
+				    (notify-interrupt n)
 				    ;; we flush current input port
 				    (reader-reset!)
 				    (reset-console! (current-input-port))
@@ -233,8 +255,9 @@
 	       (let loop ()
 		  (*prompt* *repl-num*)
 		  (let ((exp (try (read)
-				  (lambda (escape proc obj msg)
-				     (flush-output-port (current-error-port))
+				  (lambda (escape proc msg obj)
+				     (evmeaning-reset-error!)
+				     (repl-notify-error proc msg obj)
 				     (if (eof-object? obj)
 					 (reset-eof (current-input-port)))
 				     (sigsetmask 0)
@@ -242,18 +265,47 @@
 		     (if (eof-object? exp)
 			 (quit)
 			 (let ((v (try (eval exp)
-				       (lambda (escape proc obj msg)
-					  (evmeaning-notify-error proc obj msg)
-					  (flush-output-port (current-error-port))
+				       (lambda (escape proc msg obj)
+					  (repl-notify-error proc msg obj)
 					  (sigsetmask 0)
 					  (escape #unspecified)))))
-			    (print v)
+			    (if (not (eq? *transcript* (current-output-port)))
+				(fprint *transcript* ";; " exp))
+			    (*repl-printer* v *transcript*)
+			    (newline *transcript*)
 			    (set! exp #unspecified)
 			    (loop))))))
 	    (loop))
 	 (if (procedure? old-intrhdl)
 	     (signal sigint old-intrhdl)))))
 
+;*---------------------------------------------------------------------*/
+;*    default-repl-printer ...                                         */
+;*---------------------------------------------------------------------*/
+(define (default-repl-printer exp . port)
+   (apply display-circle exp port))
+
+;*---------------------------------------------------------------------*/
+;*    *repl-printer* ...                                               */
+;*---------------------------------------------------------------------*/
+(define *repl-printer* default-repl-printer)
+
+;*---------------------------------------------------------------------*/
+;*    set-repl-printer! ...                                            */
+;*---------------------------------------------------------------------*/
+(define (set-repl-printer! disp)
+   (if (not (correct-arity? disp -2))
+       (error "set-repl-printer!" "Illegal repl-printer (wrong arity)" disp)
+       (let ((old *repl-printer*))
+	  (set! *repl-printer* disp)
+	  old)))
+
+;*---------------------------------------------------------------------*/
+;*    native-repl-printer ...                                          */
+;*---------------------------------------------------------------------*/
+(define (native-repl-printer)
+   default-repl-printer)
+
 ;*---------------------------------------------------------------------*/
 ;*    quit ...                                                         */
 ;*---------------------------------------------------------------------*/
@@ -269,27 +321,35 @@
 ;*    find-file ...                                                    */
 ;*---------------------------------------------------------------------*/
 (define (find-file name)
-   (if (file-exists? name)
-       name
-       (let loop ((path *load-path*))
-	  (if (null? path)
-	      name
-	      (let ((try (string-append (car path) "/" name)))
-		 (if (file-exists? try)
-		     try
-		     (loop (cdr path))))))))
+   (if (not (string? name))
+       (error "find-file" "Illegal file name" name)
+       (if (file-exists? name)
+	   name
+	   (let loop ((path *load-path*))
+	      (if (null? path)
+		  name
+		  (let ((try (string-append (car path) "/" name)))
+		     (if (file-exists? try)
+			 try
+			 (loop (cdr path)))))))))
+
+;*---------------------------------------------------------------------*/
+;*    *load-verbose* ...                                               */
+;*---------------------------------------------------------------------*/
+(define *load-verbose* #t)
 
 ;*---------------------------------------------------------------------*/
 ;*    load ...                                                         */
 ;*---------------------------------------------------------------------*/
 (define (load file-name)
-   (loadv file-name #t))
+   (loadv file-name *load-verbose*))
 
 (define (loadq file-name)
    (loadv file-name #f))
 
 (define (loadv file-name v?)
    (let ((port (open-input-file (find-file file-name))))
+      (reader-reset!)
       (evmeaning-reset-error!)
       (if (input-port? port)
 	  (try (let loop ((sexp         (read port #t))
@@ -311,7 +371,9 @@
 			  (let ((main (assq 'main (cddr sexp))))
 			     (let ((v (eval sexp)))
 				(if v?
-				    (print v))
+				    (begin
+				       (display-circle v)
+				       (newline)))
 				(loop (read port #t)
 				      v
 				      #t
@@ -322,14 +384,16 @@
 		      (let ((v (eval sexp)))
 			 (evmeaning-reset-error!)
 			 (if v?
-			     (print v))
+			     (begin
+				(display-circle v)
+				(newline)))
 			 (loop (read port #t)
 			       v
 			       module-seen?
 			       main)))))
 	       (lambda (escape proc mes obj)
 		  ;; on imprime le message d'erreur
-		  (evmeaning-notify-error proc mes obj)
+		  (repl-notify-error proc mes obj)
 		  (error "load"
 			 "error occured when loading"
 			 file-name)))
@@ -370,7 +434,7 @@
 		      (error name "wrong number of argument for expand" macro)
 		      (try (expd-eval x e)
 			   (lambda (escape proc mes obj)
-			      (evmeaning-notify-error proc mes obj)
+			      (repl-notify-error proc mes obj)
 			      (error "expand"
 				     "error occured while expansing the call"
 				     x))))))))
@@ -396,7 +460,7 @@
 	   (lambda (x e)
 	      (try (expd-eval x e) 
 		   (lambda (escape proc mes obj)
-		      (evmeaning-notify-error proc mes obj)
+		      (repl-notify-error proc mes obj)
 		      (error "expand"
 			     "error occured while expansing the call"
 			     x))))))
@@ -422,7 +486,7 @@
 	      (lambda (x e)
 		 (try (expd-eval x e) 
 		      (lambda (escape proc mes obj)
-			 (evmeaning-notify-error proc mes obj)
+			 (repl-notify-error proc mes obj)
 			 (error "expand"
 				"error occured while expansing the call"
 				x))))))
@@ -448,7 +512,9 @@
 		    `(car ,arg)
 		    (destructure (cdr pat)
 				 `(cdr ,arg)
-				 bindings)))))
+				 bindings)))
+      (else
+       (error "expand" "Illegal macro parameter" pat))))
 
 ;*---------------------------------------------------------------------*/
 ;*    module-declaration! ...                                          */
@@ -536,9 +602,7 @@
 ;*    notify-assert-fail ...                                           */
 ;*---------------------------------------------------------------------*/
 (define (notify-assert-fail vars fail-body loc)
-   (let ((port (current-error-port))
-	 (old  (get-write-length)))
-      (set-write-length! 80)
+   (let ((port (current-error-port)))
       (try (if (pair? loc)
 	       (error/location "assert"
 			       "assertion failed"
@@ -553,9 +617,14 @@
 	      (escape #unspecified)))
       (fprint port "-----------------------")
       (fprint port "Variables' value are : ")
-      (for-each (lambda (f) (fprint port "   " f " : " (eval f))) vars)
+      (for-each (lambda (f)
+		   (display "   " port)
+		   (display f port)
+		   (display " : " port)
+		   (write-circle (eval f) port)
+		   (newline port))
+		vars)
       (fprint port "-----------------------")
-      (set-write-length! old)
       (let ((old-prompter (get-prompter)))
 	 (set-prompter! (lambda (num) (display "*:=> ")))
 	 (repl)
@@ -571,3 +640,33 @@
 ;*---------------------------------------------------------------------*/
 (define *user-pass*      (unspecified))  ;; l'eventuelle user passe 
 (define *user-pass-name* "User")         ;; le nom de la user pass
+
+;*---------------------------------------------------------------------*/
+;*    *transcript* ...                                                 */
+;*---------------------------------------------------------------------*/
+(define *transcript* (current-output-port))
+
+;*---------------------------------------------------------------------*/
+;*    transcript-on ...                                                */
+;*---------------------------------------------------------------------*/
+(define (transcript-on file::bstring)
+   (if (not (eq? *transcript* (current-output-port)))
+       (error "transcript-on" "A transcript is already in use" *transcript*)
+       (begin
+	  (set! *transcript* (append-output-file file))
+	  (fprint *transcript* ";; session started on " (date))
+	  #unspecified)))
+
+;*---------------------------------------------------------------------*/
+;*    transcript-off ...                                               */
+;*---------------------------------------------------------------------*/
+(define (transcript-off)
+   (if (eq? *transcript* (current-output-port))
+       (error "transcript-off"
+	      "No transcript is currently in use"
+	      *transcript*)
+       (begin
+	  (close-output-port *transcript*)
+	  (set! *transcript* (current-output-port))))
+   #unspecified)
+   
