@@ -1,41 +1,9 @@
-;*---------------------------------------------------------------------*/
-;*   A pratical implementation for the Scheme programming language     */
-;*                                                                     */
-;*                                    ,--^,                            */
-;*                              _ ___/ /|/                             */
-;*                          ,;'( )__, ) '                              */
-;*                         ;;  //   L__.                               */
-;*                         '   \\   /  '                               */
-;*                              ^   ^                                  */
-;*                                                                     */
-;*   Copyright (c) 1992-1999 Manuel Serrano                            */
-;*                                                                     */
-;*     Bug descriptions, use reports, comments or suggestions are      */
-;*     welcome. Send them to                                           */
-;*       bigloo-request@kaolin.unice.fr                                */
-;*       http://kaolin.unice.fr/bigloo                                 */
-;*                                                                     */
-;*   This program is free software; you can redistribute it            */
-;*   and/or modify it under the terms of the GNU General Public        */
-;*   License as published by the Free Software Foundation; either      */
-;*   version 2 of the License, or (at your option) any later version.  */
-;*                                                                     */
-;*   This program is distributed in the hope that it will be useful,   */
-;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
-;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
-;*   GNU General Public License for more details.                      */
-;*                                                                     */
-;*   You should have received a copy of the GNU General Public         */
-;*   License along with this program; if not, write to the Free        */
-;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
-;*   MA 02111-1307, USA.                                               */
-;*---------------------------------------------------------------------*/
 ;*=====================================================================*/
-;*    serrano/prgm/project/bigloo/runtime/Eval/evcompile.scm           */
+;*    .../prgm/project/bigloo/runtime.case1.2/Eval/evcompile.scm       */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Fri Mar 25 09:09:18 1994                          */
-;*    Last change :  Fri Sep 18 17:59:02 1998 (serrano)                */
+;*    Last change :  Wed Jul  4 15:05:42 2001 (serrano)                */
 ;*    -------------------------------------------------------------    */
 ;*    La pre-compilation des formes pour permettre l'interpretation    */
 ;*    rapide                                                           */
@@ -47,34 +15,34 @@
 (module __evcompile
    (include "Eval/byte-code.sch")
    
-   (import  (__type                    "Llib/type.scm")
-	    (__error                   "Llib/error.scm")
-	    (__bigloo                  "Llib/bigloo.scm")
-	    (__tvector                 "Llib/tvector.scm")
-	    (__structure               "Llib/struct.scm")
-	    (__tvector                 "Llib/tvector.scm")
-	    (__bexit                   "Llib/bexit.scm")
-	    (__os                      "Llib/os.scm")
-	    (__dsssl                   "Llib/dsssl.scm")
+   (import  __type
+	    __error
+	    __bigloo
+	    __tvector
+	    __structure
+	    __tvector
+	    __bexit
+	    __os
+	    __dsssl
 	    
-	    (__r4_numbers_6_5          "Ieee/number.scm")
-	    (__r4_numbers_6_5_fixnum   "Ieee/fixnum.scm")
-	    (__r4_numbers_6_5_flonum   "Ieee/flonum.scm")
-	    (__r4_characters_6_6       "Ieee/char.scm")
-	    (__r4_equivalence_6_2      "Ieee/equiv.scm")
-	    (__r4_booleans_6_1         "Ieee/boolean.scm")
-	    (__r4_symbols_6_4          "Ieee/symbol.scm")
-	    (__r4_strings_6_7          "Ieee/string.scm")
-	    (__r4_pairs_and_lists_6_3  "Ieee/pair-list.scm")
-	    (__r4_input_6_10_2         "Ieee/input.scm")
-	    (__r4_control_features_6_9 "Ieee/control.scm")
-	    (__r4_vectors_6_8          "Ieee/vector.scm")
-	    (__r4_ports_6_10_1         "Ieee/port.scm")
-	    (__r4_output_6_10_3        "Ieee/output.scm")
+	    __r4_numbers_6_5
+	    __r4_numbers_6_5_fixnum
+	    __r4_numbers_6_5_flonum
+	    __r4_characters_6_6
+	    __r4_equivalence_6_2
+	    __r4_booleans_6_1
+	    __r4_symbols_6_4
+	    __r4_strings_6_7
+	    __r4_pairs_and_lists_6_3
+	    __r4_input_6_10_2
+	    __r4_control_features_6_9
+	    __r4_vectors_6_8
+	    __r4_ports_6_10_1
+	    __r4_output_6_10_3
 
-   	    (__evenv                   "Eval/evenv.scm")
-	    (__eval                    "Eval/eval.scm")
-	    (__expand                  "Eval/expand.scm"))
+   	    __evenv
+	    __eval
+	    __expand)
    
    (export  (evcompile exp env Genv where named? loc)
 	    (find-loc  exp default)))
@@ -93,21 +61,19 @@
 ;*    La phase d'expansion a genere une syntaxe correcte. On n'a donc  */
 ;*    plus du tout a la tester maintenant.                             */
 ;*---------------------------------------------------------------------*/
-(define (evcompile exp env Genv where named? loc)
+(define (evcompile exp env genv where named? loc)
    (match-case exp
       (()
        (evcompile-error loc "eval" "Illegal expression" '()))
       ((module ?- . ?decls)
        (module-declaration! decls)
        (unspecified))
-      ((assert ?- ?- ?-)
-       (unspecified))
-      ((assert ?- ?-)
+      ((assert . ?-)
        (unspecified))
       ((atom ?atom)
        (cond
 	  ((symbol? atom)
-	   (evcompile-ref (variable loc atom env Genv) loc))
+	   (evcompile-ref (variable loc atom env genv) loc))
 	  ((or (vector? atom)
 	       (struct? atom))
 	   (evcompile-error loc
@@ -120,60 +86,64 @@
        (evcompile-cnst cnst (find-loc exp loc)))
       ((if ?si ?alors ?sinon)
        (let ((loc (find-loc exp loc)))
-	  (evcompile-if (evcompile si env Genv
+	  (evcompile-if (evcompile si env genv
 				   where #f (find-loc si loc))
-			(evcompile alors env Genv
+			(evcompile alors env genv
 				   where named? (find-loc alors loc))
-			(evcompile sinon env Genv
+			(evcompile sinon env genv
 				   where named? (find-loc sinon loc))
 			loc)))
       ((begin . ?rest)
-       (evcompile-begin rest env Genv where named? (find-loc exp loc)))
+       (evcompile-begin rest env genv where named? (find-loc exp loc)))
       ((define ?var (and (lambda . ?-) ?val))
        (if (and (eq? where 'nowhere)
-		(or (eq? Genv (scheme-report-environment 5))
-		    (eq? Genv (null-environment 5))))
+		(or (eq? genv (scheme-report-environment 5))
+		    (eq? genv (null-environment 5))))
 	   (evcompile-error loc
 			    "eval"
 			    "Illegal define form (sealed environment)"
 			    exp)
 	   (let ((loc (find-loc exp loc)))
 	      (evcompile-define-lambda var
-				       (delay (evcompile val '() Genv var #t
+				       (delay (evcompile val '() genv var #t
 							 (find-loc exp loc)))
 				       loc))))
       ((define ?var ?val)
        (if (and (eq? where 'nowhere)
-		(or (eq? Genv (scheme-report-environment 5))
-		    (eq? Genv (null-environment 5))))
+		(or (eq? genv (scheme-report-environment 5))
+		    (eq? genv (null-environment 5))))
 	   (evcompile-error loc
 			    "eval"
 			    "Illegal define form (sealed environment)"
 			    exp)
 	   (let ((loc (find-loc exp loc)))
 	      (evcompile-define-value var
-				      (evcompile val '() Genv where named?
+				      (evcompile val '() genv where named?
 						 (find-loc val loc))
 				      loc))))
-      ((set! ?var ?val)
-       (let ((loc (find-loc exp loc)))
-	  (evcompile-set (variable loc var env Genv)
-			 (evcompile val env Genv var #t (find-loc val loc))
-			 loc)))
+      ((set! . ?-)
+       (match-case exp
+	  ((?- (and (? symbol?) ?var) ?val)
+	   (let ((loc (find-loc exp loc)))
+	      (evcompile-set (variable loc var env genv)
+			     (evcompile val env genv var #t (find-loc val loc))
+			     loc)))
+	  (else
+	   (error "set!" "Illegal form" exp))))
       ((bind-exit ?escape ?body)
        (let ((loc (find-loc exp loc)))
 	  (evcompile-bind-exit (evcompile `(lambda ,escape ,body)
 					  env
-					  Genv
+					  genv
 					  escape
 					  #t
 					  (find-loc body loc))
 			       loc)))
       ((unwind-protect ?body . ?protect)
        (let ((loc (find-loc exp loc)))
-	  (evcompile-unwind-protect (evcompile body env Genv where named?
+	  (evcompile-unwind-protect (evcompile body env genv where named?
 					       (find-loc body loc))
-				    (evcompile-begin protect env Genv
+				    (evcompile-begin protect env genv
 						     where named?
 						     (find-loc protect loc))
 				    loc)))
@@ -194,18 +164,18 @@
 							     proc
 							     msg
 							     obj))))
-				       (extend-env scm-formals env) Genv
+				       (extend-env scm-formals env) genv
 				       where #f
 				       (find-loc body loc))
 			 where
 			 named? loc)))
       (((atom ?fun) . ?args)
        (let* ((loc     (find-loc exp loc))
-	      (actuals (map (lambda (a) (evcompile a env Genv where #f loc))
+	      (actuals (map (lambda (a) (evcompile a env genv where #f loc))
 			    args)))
 	  (cond
 	     ((symbol? fun)
-	      (let ((proc (variable loc fun env Genv)))
+	      (let ((proc (variable loc fun env genv)))
 		 (cond
 		    ((eval-global? proc)
 		     (evcompile-global-application proc actuals loc))
@@ -223,9 +193,9 @@
 			      (list "eval" "Not a procedure" fun))))))
       ((?fun . ?args)
        (let ((loc     (find-loc exp loc))
-	     (actuals (map (lambda (a) (evcompile a env Genv where #f loc))
+	     (actuals (map (lambda (a) (evcompile a env genv where #f loc))
 			   args))
-	     (proc    (evcompile fun env Genv where #f loc)))
+	     (proc    (evcompile fun env genv where #f loc)))
 	  (evcompile-application fun proc actuals loc)))
       (else
        (evcompile-error loc "eval" "Illegal form" exp))))
@@ -283,23 +253,23 @@
 ;*---------------------------------------------------------------------*/
 ;*    evcompile-begin ...                                              */
 ;*---------------------------------------------------------------------*/
-(define (evcompile-begin body env Genv where named? loc)
+(define (evcompile-begin body env genv where named? loc)
    (cond
       ((null? body)
-       (evcompile #unspecified env Genv where named? loc))
+       (evcompile #unspecified env genv where named? loc))
       ((null? (cdr body))
-       (evcompile (car body) env Genv where named? (find-loc (car body) loc)))
+       (evcompile (car body) env genv where named? (find-loc (car body) loc)))
       (else
        (let ((cbody (let loop ((rest body))
 		       (cond
 			  ((null? rest)
 			   '())
 			  ((null? (cdr rest))
-			   (cons (evcompile (car rest) env Genv where named?
+			   (cons (evcompile (car rest) env genv where named?
 					    (find-loc (car rest) loc))
 				 '()))
 			  (else
-			   (cons (evcompile (car rest) env Genv where #f
+			   (cons (evcompile (car rest) env genv where #f
 					    (find-loc (car rest) loc))
 				 (loop (cdr rest))))))))
 	  (make-byte-code 16 loc (list->vector cbody))))))
@@ -467,7 +437,7 @@
 ;*---------------------------------------------------------------------*/
 ;*    variable ...                                                     */
 ;*---------------------------------------------------------------------*/
-(define (variable loc symbol env Genv)
+(define (variable loc symbol env genv)
    (if (not (symbol? symbol))
        (evcompile-error loc "eval" "Illegal `set!' expression" symbol)
        (let ((offset (let loop ((env   env)
@@ -481,15 +451,15 @@
 			    (loop (cdr env) (+fx count 1)))))))
 	  (if offset
 	      offset
-	      (let ((global (if (or (eq? Genv (scheme-report-environment 5))
-				    (eq? Genv (interaction-environment)))
+	      (let ((global (if (or (eq? genv (scheme-report-environment 5))
+				    (eq? genv (interaction-environment)))
 				(eval-lookup symbol)
 				(evcompile-error loc
 						 "eval"
 						 "Unbound variable"
 						 symbol))))
 		 (if (not global)
-		     (if (eq? Genv (scheme-report-environment 5))
+		     (if (eq? genv (scheme-report-environment 5))
 			 (evcompile-error loc
 					  "eval"
 					  "Unbound variable"
