@@ -1,41 +1,9 @@
-;*---------------------------------------------------------------------*/
-;*   A pratical implementation for the Scheme programming language     */
-;*                                                                     */
-;*                                    ,--^,                            */
-;*                              _ ___/ /|/                             */
-;*                          ,;'( )__, ) '                              */
-;*                         ;;  //   L__.                               */
-;*                         '   \\   /  '                               */
-;*                              ^   ^                                  */
-;*                                                                     */
-;*   Copyright (c) 1992-1999 Manuel Serrano                            */
-;*                                                                     */
-;*     Bug descriptions, use reports, comments or suggestions are      */
-;*     welcome. Send them to                                           */
-;*       bigloo-request@kaolin.unice.fr                                */
-;*       http://kaolin.unice.fr/bigloo                                 */
-;*                                                                     */
-;*   This program is free software; you can redistribute it            */
-;*   and/or modify it under the terms of the GNU General Public        */
-;*   License as published by the Free Software Foundation; either      */
-;*   version 2 of the License, or (at your option) any later version.  */
-;*                                                                     */
-;*   This program is distributed in the hope that it will be useful,   */
-;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
-;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
-;*   GNU General Public License for more details.                      */
-;*                                                                     */
-;*   You should have received a copy of the GNU General Public         */
-;*   License along with this program; if not, write to the Free        */
-;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
-;*   MA 02111-1307, USA.                                               */
-;*---------------------------------------------------------------------*/
 ;*=====================================================================*/
 ;*    serrano/prgm/project/bigloo/runtime/Eval/evmeaning.scm           */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Wed Aug  4 10:48:41 1993                          */
-;*    Last change :  Fri Sep 18 17:58:53 1998 (serrano)                */
+;*    Last change :  Mon Dec 10 18:37:17 2001 (serrano)                */
 ;*    -------------------------------------------------------------    */
 ;*    L'interprete de bigloo (la version `lambda')                     */
 ;*=====================================================================*/
@@ -44,49 +12,70 @@
 ;*    Le module                                                        */
 ;*---------------------------------------------------------------------*/
 (module __evmeaning
-    
+   
    (include "Eval/byte-code.sch")
    
-   (import  (__type                    "Llib/type.scm")
-	    (__error                   "Llib/error.scm")
-	    (__bigloo                  "Llib/bigloo.scm")
-	    (__tvector                 "Llib/tvector.scm")
-	    (__structure               "Llib/struct.scm")
-	    (__tvector                 "Llib/tvector.scm")
-	    (__bexit                   "Llib/bexit.scm")
-	    (__os                      "Llib/os.scm")
+   (import  __type
+	    __error
+	    __bigloo
+	    __tvector
+	    __structure
+	    __tvector
+	    __bexit
+	    __os
 	    
-	    (__r4_numbers_6_5          "Ieee/number.scm")
-	    (__r4_numbers_6_5_fixnum   "Ieee/fixnum.scm")
-	    (__r4_numbers_6_5_flonum   "Ieee/flonum.scm")
-	    (__r4_characters_6_6       "Ieee/char.scm")
-	    (__r4_equivalence_6_2      "Ieee/equiv.scm")
-	    (__r4_booleans_6_1         "Ieee/boolean.scm")
-	    (__r4_symbols_6_4          "Ieee/symbol.scm")
-	    (__r4_strings_6_7          "Ieee/string.scm")
-	    (__r4_pairs_and_lists_6_3  "Ieee/pair-list.scm")
-	    (__r4_input_6_10_2         "Ieee/input.scm")
-	    (__r4_control_features_6_9 "Ieee/control.scm")
-	    (__r4_vectors_6_8          "Ieee/vector.scm")
-	    (__r4_ports_6_10_1         "Ieee/port.scm")
-	    (__r4_output_6_10_3        "Ieee/output.scm")
+	    __r4_numbers_6_5
+	    __r4_numbers_6_5_fixnum
+	    __r4_numbers_6_5_flonum
+	    __r4_characters_6_6
+	    __r4_equivalence_6_2
+	    __r4_booleans_6_1
+	    __r4_symbols_6_4
+	    __r4_strings_6_7
+	    __r4_pairs_and_lists_6_3
+	    __r4_input_6_10_2
+	    __r4_control_features_6_9
+	    __r4_vectors_6_8
+	    __r4_ports_6_10_1
+	    __r4_output_6_10_3
 	    
-	    (__progn                   "Eval/progn.scm")
-	    (__evenv                   "Eval/evenv.scm")
-	    (__evcompile               "Eval/evcompile.scm"))
+	    __progn
+	    __evenv
+	    __evcompile)
+   
+   (static  (update-eval-global! variable value))
    
    (export  (evmeaning exp env)
 	    (evmeaning-reset-error!)
-	    (evmeaning-notify-error proc mes obj))
+	    (evmeaning-notify-error proc mes obj)
+	    (evmeaning-location))
    
-   (foreign (obj funcall-0  (obj obj)                 "eval_funcall_0")
-	    (obj funcall-1  (obj obj obj)             "eval_funcall_1")
-	    (obj funcall-2  (obj obj obj obj)         "eval_funcall_2")
-	    (obj funcall-3  (obj obj obj obj obj)     "eval_funcall_3")
-	    (obj funcall-4  (obj obj obj obj obj obj) "eval_funcall_4")
-	    (obj eval-apply (obj obj obj)             "eval_apply"))
-	    
-   (static  (update-eval-global! variable value)))
+   (extern  (%funcall-0::obj  (::procedure)
+			      "eval_funcall_0")
+	    (%funcall-1::obj  (::procedure ::obj)
+			      "eval_funcall_1")
+	    (%funcall-2::obj  (::procedure ::obj ::obj)
+			      "eval_funcall_2")
+	    (%funcall-3::obj  (::procedure ::obj ::obj ::obj)
+			      "eval_funcall_3")
+	    (%funcall-4::obj  (::procedure ::obj ::obj ::obj ::obj)
+			      "eval_funcall_4")
+	    (%eval-apply::obj (::procedure ::obj)
+			      "eval_apply"))
+   
+   (java    (class foreign
+	       (method static %funcall-0::obj (::procedure)
+		       "eval_funcall_0")
+	       (method static %funcall-1::obj (::procedure ::obj)
+		       "eval_funcall_1")
+	       (method static %funcall-2::obj (::procedure ::obj ::obj)
+		       "eval_funcall_2")
+	       (method static %funcall-3::obj (::procedure ::obj ::obj ::obj)
+		       "eval_funcall_3")
+	       (method static %funcall-4::obj (::procedure ::obj ::obj ::obj ::obj)
+		       "eval_funcall_4")
+	       (method static %eval-apply::obj (::procedure ::obj)
+		       "eval_apply"))))
 
 ;*---------------------------------------------------------------------*/
 ;*    evmeaning ...                                                    */
@@ -277,99 +266,111 @@
 	      (let* ((value (value-of-byte-code exp))
 		     (name  (vector-ref value 0))
 		     (fun   (__evmeaning_address-ref (eval-global-value
-						      (vector-ref value 1)))))
-		 (funcall-1 name fun
-			    (evmeaning (vector-ref value 2) stack))))
+						      (vector-ref value 1))))
+		     (a0    (evmeaning (vector-ref value 2) stack)))
+		 (set! *current-bcode* exp)
+		 (funcall-1 name fun a0)))
 	     ((58)
 	      (let* ((value (value-of-byte-code exp))
 		     (name  (vector-ref value 0))
-		     (fun   (eval-global-value (vector-ref value 1))))
-		 (funcall-1 name fun
-			    (evmeaning (vector-ref value 2) stack))))
+		     (fun   (eval-global-value (vector-ref value 1)))
+		     (a0    (evmeaning (vector-ref value 2) stack)))
+		 (set! *current-bcode* exp)
+		 (funcall-1 name fun a0)))
 	     ;; l'appel de fonction d'arite 2
 	     ((21)
 	      (let* ((value (value-of-byte-code exp))
 		     (name  (vector-ref value 0))
 		     (fun   (__evmeaning_address-ref (eval-global-value
-						      (vector-ref value 1)))))
-		 (funcall-2 name fun
-			    (evmeaning (vector-ref value 2) stack)
-			    (evmeaning (vector-ref value 3) stack))))
+						      (vector-ref value 1))))
+		     (a0    (evmeaning (vector-ref value 2) stack))
+		     (a1    (evmeaning (vector-ref value 3) stack)))
+		 (set! *current-bcode* exp)
+		 (funcall-2 name fun a0 a1)))
 	     ((59)
 	      (let* ((value (value-of-byte-code exp))
 		     (name  (vector-ref value 0))
-		     (fun   (eval-global-value (vector-ref value 1))))
-		 (funcall-2 name fun
-			    (evmeaning (vector-ref value 2) stack)
-			    (evmeaning (vector-ref value 3) stack))))
+		     (fun   (eval-global-value (vector-ref value 1)))
+		     (a0    (evmeaning (vector-ref value 2) stack))
+		     (a1    (evmeaning (vector-ref value 3) stack)))
+		 (set! *current-bcode* exp)
+		 (funcall-2 name fun a0 a1)))
 	     ;; l'appel de fonction d'arite 3
 	     ((22)
 	      (let* ((value (value-of-byte-code exp))
 		     (name  (vector-ref value 0))
 		     (fun   (__evmeaning_address-ref (eval-global-value
-						      (vector-ref value 1)))))
-		 (funcall-3 name fun
-			    (evmeaning (vector-ref value 2) stack)
-			    (evmeaning (vector-ref value 3) stack)
-			    (evmeaning (vector-ref value 4) stack))))
+						      (vector-ref value 1))))
+		     (a0    (evmeaning (vector-ref value 2) stack))
+		     (a1    (evmeaning (vector-ref value 3) stack))
+		     (a2    (evmeaning (vector-ref value 4) stack)))
+		 (set! *current-bcode* exp)
+		 (funcall-3 name fun a0 a1 a2)))
 	     ((60)
 	      (let* ((value (value-of-byte-code exp))
 		     (name  (vector-ref value 0))
-		     (fun   (eval-global-value (vector-ref value 1))))
-		 (funcall-3 name fun
-			    (evmeaning (vector-ref value 2) stack)
-			    (evmeaning (vector-ref value 3) stack)
-			    (evmeaning (vector-ref value 4) stack))))
+		     (fun   (eval-global-value (vector-ref value 1)))
+		     (a0    (evmeaning (vector-ref value 2) stack))
+		     (a1    (evmeaning (vector-ref value 3) stack))
+		     (a2    (evmeaning (vector-ref value 4) stack)))
+		 (set! *current-bcode* exp)
+		 (funcall-3 name fun a0 a1 a2)))
 	     ;; l'appel de fonction d'arite 4
 	     ((23)
 	      (let* ((value (value-of-byte-code exp))
 		     (name  (vector-ref value 0))
 		     (fun   (__evmeaning_address-ref (eval-global-value
-						      (vector-ref value 1)))))
-		 (funcall-4 name fun
-			    (evmeaning (vector-ref value 2) stack)
-			    (evmeaning (vector-ref value 3) stack)
-			    (evmeaning (vector-ref value 4) stack)
-			    (evmeaning (vector-ref value 5) stack))))
+						      (vector-ref value 1))))
+		     (a0    (evmeaning (vector-ref value 2) stack))
+		     (a1    (evmeaning (vector-ref value 3) stack))
+		     (a2    (evmeaning (vector-ref value 4) stack))
+		     (a3    (evmeaning (vector-ref value 5) stack)))
+		 (set! *current-bcode* exp)
+		 (funcall-4 name fun a0 a1 a2 a3)))
 	     ((61)
 	      (let* ((value (value-of-byte-code exp))
 		     (name  (vector-ref value 0))
-		     (fun   (eval-global-value (vector-ref value 1))))
-		 (funcall-4 name fun
-			    (evmeaning (vector-ref value 2) stack)
-			    (evmeaning (vector-ref value 3) stack)
-			    (evmeaning (vector-ref value 4) stack)
-			    (evmeaning (vector-ref value 5) stack))))
+		     (fun   (eval-global-value (vector-ref value 1)))
+		     (a0    (evmeaning (vector-ref value 2) stack))
+		     (a1    (evmeaning (vector-ref value 3) stack))
+		     (a2    (evmeaning (vector-ref value 4) stack))
+		     (a3    (evmeaning (vector-ref value 5) stack)))
+		 (set! *current-bcode* exp)
+		 (funcall-4 name fun a0 a1 a2 a3)))
 	     ;; l'appel de fonction de plus de quatres arguments
 	     ((24)
 	      (let* ((value (value-of-byte-code exp))
 		     (name  (vector-ref value 0))
 		     (fun   (__evmeaning_address-ref (eval-global-value
 						      (vector-ref value 1))))
-		     (args  (vector-ref value 2)))
-		 (eval-apply name
-			     fun
-			     (let loop ((args args)
-					(res  '()))
-				(if (null? args)
-				    (reverse! res)
-				    (loop (cdr args)
-					  (cons (evmeaning (car args) stack)
-						res)))))))
+		     (args  (vector-ref value 2))
+		     (eargs (let loop ((args args)
+				       (res  '())
+				       (len   0))
+			       (if (null? args)
+				   (cons len (reverse! res))
+				   (loop (cdr args)
+					 (cons (evmeaning (car args) stack)
+					       res)
+					 (+fx 1 len))))))
+		 (set! *current-bcode* exp)
+		 (eval-apply name fun eargs)))
 	     ((62)
 	      (let* ((value (value-of-byte-code exp))
 		     (name  (vector-ref value 0))
 		     (fun   (eval-global-value (vector-ref value 1)))
-		     (args  (vector-ref value 2)))
-		 (eval-apply name
-			     fun
-			     (let loop ((args args)
-					(res  '()))
+		     (args  (vector-ref value 2))
+		     (eargs (let loop ((args args)
+					(res  '())
+					(len   0))
 				(if (null? args)
-				    (reverse! res)
+				    (cons len (reverse! res))
 				    (loop (cdr args)
 					  (cons (evmeaning (car args) stack)
-						res)))))))
+						res)
+					  (+fx len 1))))))
+		 (set! *current-bcode* exp)
+		 (eval-apply name fun eargs)))
 	     ;; l'appel de fonction de compilee anonyme d'arite 0
 	     ((25)
 	      ((value-of-byte-code exp)))
@@ -377,34 +378,43 @@
 	     ((26)
 	      (let* ((value (value-of-byte-code exp))
 		     (fun   (vector-ref value 0)))
+		 (set! *current-bcode* exp)
 		 (fun (evmeaning (vector-ref value 1) stack))))
 	     ;; l'appel de fonction de compilee anonyme d'arite 2
 	     ((27)
 	      (let* ((value (value-of-byte-code exp))
-		     (fun   (vector-ref value 0)))
-		 (fun (evmeaning (vector-ref value 1) stack)
-		      (evmeaning (vector-ref value 2) stack))))
+		     (fun   (vector-ref value 0))
+		     (a0    (evmeaning (vector-ref value 1) stack))
+		     (a1    (evmeaning (vector-ref value 2) stack)))
+		 (set! *current-bcode* exp)
+		 (fun a0 a1)))
 	     ;; l'appel de fonction de compilee anonyme d'arite 3
 	     ((28)
 	      (let* ((value (value-of-byte-code exp))
-		     (fun   (vector-ref value 0)))
-		 (fun (evmeaning (vector-ref value 1) stack)
-		      (evmeaning (vector-ref value 2) stack)
-		      (evmeaning (vector-ref value 3) stack))))
+		     (fun   (vector-ref value 0))
+		     (a0    (evmeaning (vector-ref value 1) stack))
+		     (a1    (evmeaning (vector-ref value 2) stack))
+		     (a2    (evmeaning (vector-ref value 3) stack)))
+		 (set! *current-bcode* exp)
+		 (fun a0 a1 a2)))
 	     ;; l'appel de fonction de compilee anonyme d'arite 4
 	     ((29)
 	      (let* ((value (value-of-byte-code exp))
-		     (fun   (vector-ref value 0)))
-		 (fun (evmeaning (vector-ref value 1) stack)
-		      (evmeaning (vector-ref value 2) stack)
-		      (evmeaning (vector-ref value 3) stack)
-		      (evmeaning (vector-ref value 4) stack))))
+		     (fun   (vector-ref value 0))
+		     (a0    (evmeaning (vector-ref value 1) stack))
+		     (a1    (evmeaning (vector-ref value 2) stack))
+		     (a2    (evmeaning (vector-ref value 3) stack))
+		     (a3    (evmeaning (vector-ref value 4) stack)))
+		 (set! *current-bcode* exp)
+		 (fun a0 a1 a2 a3)))
 	     ;; l'appel de fonction de compilee anonyme d'arite plus que 4
 	     ((30)
-	      (let ((value (value-of-byte-code exp)))
-		 (apply (car value) (map (lambda (x)
-					    (evmeaning x stack))
-					 (cdr value)))))
+	      (let* ((value (value-of-byte-code exp))
+		     (eargs (map (lambda (x)
+				    (evmeaning x stack))
+				 (cdr value))))
+		 (set! *current-bcode* exp)
+		 (apply (car value) eargs)))
 	     ;; l'application classique d'arite 0
 	     ((31)
 	      (let* ((value (value-of-byte-code exp))
@@ -415,45 +425,56 @@
 	     ((32)
 	      (let* ((value (value-of-byte-code exp))
 		     (name  (vector-ref value 0))
-		     (fun   (evmeaning (vector-ref value 1) stack)))
-		 (funcall-1 name fun
-			    (evmeaning (vector-ref value 2) stack))))
+		     (fun   (evmeaning (vector-ref value 1) stack))
+		     (a0    (evmeaning (vector-ref value 2) stack)))
+		 (set! *current-bcode* exp)
+		 (funcall-1 name fun a0)))
 	     ;; l'application classique d'arite 2
 	     ((33)
 	      (let* ((value (value-of-byte-code exp))
 		     (name  (vector-ref value 0))
-		     (fun   (evmeaning (vector-ref value 1) stack)))
-		 (funcall-2 name fun
-			    (evmeaning (vector-ref value 2) stack)
-			    (evmeaning (vector-ref value 3) stack))))
+		     (fun   (evmeaning (vector-ref value 1) stack))
+		     (a0    (evmeaning (vector-ref value 2) stack))
+		     (a1    (evmeaning (vector-ref value 3) stack)))
+		 (set! *current-bcode* exp)
+		 (funcall-2 name fun a0 a1)))
 	     ;; l'application classique d'arite 3
 	     ((34)
 	      (let* ((value (value-of-byte-code exp))
 		     (name  (vector-ref value 0))
-		     (fun   (evmeaning (vector-ref value 1) stack)))
-		 (funcall-3 name fun
-			    (evmeaning (vector-ref value 2) stack)
-			    (evmeaning (vector-ref value 3) stack)
-			    (evmeaning (vector-ref value 4) stack))))
+		     (fun   (evmeaning (vector-ref value 1) stack))
+		     (a0    (evmeaning (vector-ref value 2) stack))
+		     (a1    (evmeaning (vector-ref value 3) stack))
+		     (a2    (evmeaning (vector-ref value 4) stack)))
+		 (set! *current-bcode* exp)
+		 (funcall-3 name fun a0 a1 a2)))
 	     ;; l'application classique d'arite 4
 	     ((35)
 	      (let* ((value (value-of-byte-code exp))
 		     (name  (vector-ref value 0))
-		     (fun   (evmeaning (vector-ref value 1) stack)))
-		 (funcall-4 name fun
-			    (evmeaning (vector-ref value 2) stack)
-			    (evmeaning (vector-ref value 3) stack)
-			    (evmeaning (vector-ref value 4) stack)
-			    (evmeaning (vector-ref value 5) stack))))
+		     (fun   (evmeaning (vector-ref value 1) stack))
+		     (a0    (evmeaning (vector-ref value 2) stack))
+		     (a1    (evmeaning (vector-ref value 3) stack))
+		     (a2    (evmeaning (vector-ref value 4) stack))
+		     (a3    (evmeaning (vector-ref value 5) stack)))
+		 (set! *current-bcode* exp)
+		 (funcall-4 name fun a0 a1 a2 a3)))
 	     ;; l'application classique de plus de 4 arguments
 	     ((36)
 	      (let* ((value (value-of-byte-code exp))
 		     (name  (vector-ref value 0))
-		     (fun   (evmeaning (vector-ref value 1) stack)))
-		 (eval-apply name fun
-			     (map (lambda (v)
-				     (evmeaning v stack))
-				  (vector-ref value 2)))))
+		     (fun   (evmeaning (vector-ref value 1) stack))
+		     (eargs (let loop ((args (vector-ref value 2))
+				       (new '())
+				       (len 0))
+			       (if (null? args)
+				   (cons len (reverse! new))
+				   (loop (cdr args)
+					 (cons (evmeaning (car args) stack)
+					       new)
+					 (+fx 1 len))))))
+		 (set! *current-bcode* exp)
+		 (eval-apply name fun eargs)))
 	     ;; les lambda tracees d'arite 0
 	     ((37)
 	      (let* ((value (value-of-byte-code exp))
@@ -722,7 +743,7 @@
 (define (evmeaning-error bcode proc mes obj)
    (if (byte-code? bcode)
        (match-case (pos-of-byte-code bcode)
-	  ((at ?fname ?loc)
+	  ((at ?fname ?loc ?line)
 	   (error/location proc mes obj fname loc))
 	  (else
 	   (error proc mes obj)))
@@ -734,7 +755,7 @@
 (define (evmeaning-warning bcode . args)
    (if (byte-code? bcode)
        (match-case (pos-of-byte-code bcode)
-	  ((at ?fname ?loc)
+	  ((at ?fname ?loc ?line)
 	   (apply warning/location `(,fname ,loc ,@args)))
 	  (else
 	   (apply warning args)))
@@ -746,13 +767,29 @@
 (define (evmeaning-notify-error proc mes obj)
    (if (byte-code? *current-bcode*)
        (match-case (pos-of-byte-code *current-bcode*)
-	  ((at ?fname ?loc)
+	  ((at ?fname ?loc ?line)
 	   (set! *current-bcode* #f)
-	   ((error/location-file fname loc) proc mes obj))
+	   (let ((err (error/location-file fname loc)))
+	      (if (procedure? err)
+		  (err proc mes obj)
+		  (notify-error proc mes obj))))
 	  (else
 	   (notify-error proc mes obj)))
        (notify-error proc mes obj)))
 
+;*---------------------------------------------------------------------*/
+;*    evmeaning-location ...                                           */
+;*---------------------------------------------------------------------*/
+(define (evmeaning-location)
+   (if (byte-code? *current-bcode*)
+       (let ((p (pos-of-byte-code *current-bcode*)))
+	  (match-case p
+	     ((at ?fname ?loc ?line)
+	      p)
+	     (else
+	      #f))
+	  #f)))
+   
 ;*---------------------------------------------------------------------*/
 ;*    evmeaning-reset-error! ...                                       */
 ;*---------------------------------------------------------------------*/
@@ -773,6 +810,78 @@
 ;*---------------------------------------------------------------------*/
 (define *current-bcode* #f)
 
+;*---------------------------------------------------------------------*/
+;*    funcall-0 ...                                                    */
+;*---------------------------------------------------------------------*/
+(define-inline (funcall-0 name fun)
+   (cond
+      ((not (procedure? fun))
+       (error "eval" "Not a procedure" name))
+      ((not (correct-arity? fun 0))
+       (error "eval" "Wrong number of argument" name))
+      (else
+       (%funcall-0 fun))))
+
+;*---------------------------------------------------------------------*/
+;*    funcall-1 ...                                                    */
+;*---------------------------------------------------------------------*/
+(define-inline (funcall-1 name fun a0)
+   (cond
+      ((not (procedure? fun))
+       (error "eval" "Not a procedure" name))
+      ((not (correct-arity? fun 1))
+       (error "eval" "Wrong number of argument" name))
+      (else
+       (%funcall-1 fun a0))))
+
+;*---------------------------------------------------------------------*/
+;*    funcall-2 ...                                                    */
+;*---------------------------------------------------------------------*/
+(define-inline (funcall-2 name fun a0 a1)
+   (cond
+      ((not (procedure? fun))
+       (error "eval" "Not a procedure" name))
+      ((not (correct-arity? fun 2))
+       (error "eval" "Wrong number of argument" name))
+      (else
+       (%funcall-2 fun a0 a1))))
+
+;*---------------------------------------------------------------------*/
+;*    funcall-3 ...                                                    */
+;*---------------------------------------------------------------------*/
+(define-inline (funcall-3 name fun a0 a1 a2)
+   (cond
+      ((not (procedure? fun))
+       (error "eval" "Not a procedure" name))
+      ((not (correct-arity? fun 3))
+       (error "eval" "Wrong number of argument" name))
+      (else
+       (%funcall-3 fun a0 a1 a2))))
+
+;*---------------------------------------------------------------------*/
+;*    funcall-4 ...                                                    */
+;*---------------------------------------------------------------------*/
+(define-inline (funcall-4 name fun a0 a1 a2 a3)
+   (cond
+      ((not (procedure? fun))
+       (error "eval" "Not a procedure" name))
+      ((not (correct-arity? fun 4))
+       (error "eval" "Wrong number of argument" name))
+      (else
+       (%funcall-4 fun a0 a1 a2 a3))))
+
+;*---------------------------------------------------------------------*/
+;*    eval-apply ...                                                   */
+;*---------------------------------------------------------------------*/
+(define-inline (eval-apply name fun len.lst)
+   (cond
+      ((not (procedure? fun))
+       (error "apply" "Not a procedure" name))
+      ((not (correct-arity? fun (car len.lst)))
+       (error "eval" "Wrong number of argument" name))
+      (else
+       (%eval-apply fun (cdr len.lst)))))
+       
 ;*---------------------------------------------------------------------*/
 ;*    Les environments ...                                             */
 ;*---------------------------------------------------------------------*/
