@@ -1,41 +1,9 @@
-;*---------------------------------------------------------------------*/
-;*   A pratical implementation for the Scheme programming language     */
-;*                                                                     */
-;*                                    ,--^,                            */
-;*                              _ ___/ /|/                             */
-;*                          ,;'( )__, ) '                              */
-;*                         ;;  //   L__.                               */
-;*                         '   \\   /  '                               */
-;*                              ^   ^                                  */
-;*                                                                     */
-;*   Copyright (c) 1992-1999 Manuel Serrano                            */
-;*                                                                     */
-;*     Bug descriptions, use reports, comments or suggestions are      */
-;*     welcome. Send them to                                           */
-;*       bigloo-request@kaolin.unice.fr                                */
-;*       http://kaolin.unice.fr/bigloo                                 */
-;*                                                                     */
-;*   This program is free software; you can redistribute it            */
-;*   and/or modify it under the terms of the GNU General Public        */
-;*   License as published by the Free Software Foundation; either      */
-;*   version 2 of the License, or (at your option) any later version.  */
-;*                                                                     */
-;*   This program is distributed in the hope that it will be useful,   */
-;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
-;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
-;*   GNU General Public License for more details.                      */
-;*                                                                     */
-;*   You should have received a copy of the GNU General Public         */
-;*   License along with this program; if not, write to the Free        */
-;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
-;*   MA 02111-1307, USA.                                               */
-;*---------------------------------------------------------------------*/
 ;*=====================================================================*/
-;*    serrano/prgm/project/bigloo/runtime1.9/Ieee/fixnum.scm           */
+;*    serrano/prgm/project/bigloo/runtime/Ieee/fixnum.scm              */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Fri Jan 20 10:06:37 1995                          */
-;*    Last change :  Fri Sep  6 15:02:05 1996 (serrano)                */
+;*    Last change :  Sun Jun  3 12:06:59 2001 (serrano)                */
 ;*    -------------------------------------------------------------    */
 ;*    6.5. Numbers (page 18, r4) The `fixnum' functions                */
 ;*=====================================================================*/
@@ -45,112 +13,157 @@
 ;*---------------------------------------------------------------------*/
 (module __r4_numbers_6_5_fixnum
    
-   (import  (__error                   "Llib/error.scm"))
+   (import  __error)
    
-   (use     (__type                    "Llib/type.scm")
-	    (__bigloo                  "Llib/bigloo.scm")
-	    (__tvector                 "Llib/tvector.scm")
-	    (__r4_booleans_6_1         "Ieee/boolean.scm")
-	    (__r4_vectors_6_8          "Ieee/vector.scm")
-	    (__r4_strings_6_7          "Ieee/string.scm")
-	    (__r4_characters_6_6       "Ieee/char.scm")
-	    (__r4_symbols_6_4          "Ieee/symbol.scm")
-	    (__r4_pairs_and_lists_6_3  "Ieee/pair-list.scm")
+   (use     __type
+	    __bigloo
+	    __tvector
+	    __r4_booleans_6_1
+	    __r4_vectors_6_8
+	    __r4_strings_6_7
+	    __r4_characters_6_6
+	    __r4_symbols_6_4
+	    __r4_pairs_and_lists_6_3
 	    
-	    (__evenv                   "Eval/evenv.scm"))
+	    __evenv)
 
-   (foreign (macro bool       c-fixnum?     (obj)         "INTEGERP")
-	    (macro bool       c-elong?      (obj)         "ELONGP")
-	    (macro bool       c-llong?      (obj)         "LLONGP")
-	    (infix macro bool c-=fx         (long long)   "==")
-	    (infix macro bool c-<fx         (long long)   "<")
-	    (infix macro bool c-<=fx        (long long)   "<=")
-	    (infix macro bool c->fx         (long long)   ">")
-	    (infix macro bool c->=fx        (long long)   ">=")
-	    (macro bool       c-even?       (long)        "EVENP_FX")
-	    (macro bool       c-odd?        (long)        "ODDP_FX")
-	    (infix macro long c-+fx         (long long)   "+")
-	    (infix macro long c--fx         (long long)   "-")
-	    (infix macro long c-*fx         (long long)   "*")
-	    (infix macro long c-/fx         (long long)   "/")
-	    (macro long       c-negfx       (long)        "NEG")
-	    (infix macro long c-quotient    (long long)   "/")
-	    (infix macro long c-remainder   (long long)   "%")
-	    (string           c-int->string (long long)   "integer_to_string")
-	    (macro long       strtol   (string long long) "strtol"))
-
-   (export  (inline integer?::bool        ::obj)
-	    (inline fixnum?::bool         ::obj)
-	    (inline elong?::bool          ::obj)
-	    (inline llong?::bool          ::obj)
-	    (inline make-elong::belong    ::long)
-	    (inline =fx::bool             ::long ::long)
-	    (inline >fx::bool             ::long ::long)
-	    (inline >=fx::bool            ::long ::long)
-	    (inline <fx::bool             ::long ::long)
-	    (inline <=fx::bool            ::long ::long)
-	    (inline zerofx?::bool         ::long)
-	    (inline positivefx?::bool     ::long)
-	    (inline negativefx?::bool     ::long)
-	    (inline odd?::bool            ::long)
-	    (inline even?::bool           ::long)
-	    (maxfx::long                  ::long . pair)
-	    (minfx::long                  ::long . pair)
-	    (inline +fx::long             ::long ::long)
-	    (inline -fx::long             ::long ::long)
-	    (inline *fx::long             ::long ::long)
-	    (inline /fx::long             ::long ::long)
-	    (inline negfx::long           ::long)
-	    (inline absfx::long           ::long)
-	    (inline quotient::long        ::long ::long)
-	    (inline remainder::long       ::long ::long)
-	    (modulo::long                 ::long ::long)
-	    (gcd::long                    . pair)
-	    (lcm::long                    . pair)
-	    (integer->string::string      ::long . pair)
-	    (string->integer::long        ::string . pair)
-	    (string->elong::belong        ::string . pair)
-	    (string->llong::bllong        ::string . pair))
-
-   (pragma  (fixnum? (predicate-of bint) no-cfa-top)
-	    (c-fixnum? (predicate-of bint) no-cfa-top)
-	    (c-elong? (predicate-of belong) no-cfa-top)
-	    (c-llong? (predicate-of bllong) no-cfa-top)
-	    (integer? side-effect-free no-cfa-top)
-	    (=fx side-effect-free no-cfa-top)
-	    (>fx side-effect-free no-cfa-top)
-	    (>=fx side-effect-free no-cfa-top)
-	    (<fx side-effect-free no-cfa-top)
-	    (<=fx side-effect-free no-cfa-top)
-	    (odd? side-effect-free no-cfa-top)
-	    (even? side-effect-free no-cfa-top)
-	    (+fx side-effect-free no-cfa-top)
-	    (-fx side-effect-free no-cfa-top)
-	    (*fx side-effect-free no-cfa-top)
-	    (/fx side-effect-free no-cfa-top)
-	    (remainder side-effect-free no-cfa-top)
-	    (integer->string side-effect-free no-cfa-top)
-	    (string->integer side-effect-free no-cfa-top)
-	    (modulo side-effect-free no-cfa-top)
-	    (gcd side-effect-free no-cfa-top)
-	    (lcm side-effect-free no-cfa-top)
-	    (quotient side-effect-free no-cfa-top)
-	    (positivefx? side-effect-free no-cfa-top)
-	    (negativefx? side-effect-free no-cfa-top)
-	    (zerofx? side-effect-free no-cfa-top)
-	    (negfx side-effect-free no-cfa-top)
-	    (c-=fx side-effect-free no-cfa-top)
-	    (c->fx side-effect-free no-cfa-top)
-	    (c->=fx side-effect-free no-cfa-top)
-	    (c-<fx side-effect-free no-cfa-top)
-	    (c-<=fx side-effect-free no-cfa-top)
-	    (c-odd? side-effect-free no-cfa-top)
-	    (c-even? side-effect-free no-cfa-top)
-	    (c-+fx side-effect-free no-cfa-top)
-	    (c--fx side-effect-free no-cfa-top)
-	    (c-*fx side-effect-free no-cfa-top)
-	    (c-/fx side-effect-free no-cfa-top)
-	    (c-negfx side-effect-free no-cfa-top)))
+   (extern  (macro c-fixnum?::bool (::obj) "INTEGERP")
+	    (macro c-elong?::bool (::obj) "ELONGP")
+	    (macro c-llong?::bool (::obj) "LLONGP")
+	    (infix macro c-=fx::bool (::long ::long) "==")
+	    (infix macro c-<fx::bool (::long ::long) "<")
+	    (infix macro c-<=fx::bool (::long ::long) "<=")
+	    (infix macro c->fx::bool (::long ::long) ">")
+	    (infix macro c->=fx::bool (::long ::long) ">=")
+	    (macro c-even?::bool (::long) "EVENP_FX")
+	    (macro c-odd?::bool (::long) "ODDP_FX")
+	    (infix macro c-+fx::long (::long ::long) "+")
+	    (infix macro c--fx::long (::long ::long) "-")
+	    (infix macro c-*fx::long (::long ::long) "*")
+	    (infix macro c-/fx::long (::long ::long) "/")
+	    (macro c-negfx::long (::long) "NEG")
+	    (infix macro c-quotient::long (::long ::long) "/")
+	    (infix macro c-remainder::long (::long ::long) "%")
+	    (macro strtol::long (::string ::long ::long) "strtol")
+	    (c-int->string::bstring  (::long ::long) "integer_to_string")
+	    (macro %%rand::int () "rand"))
+   
+   (java    (class foreign
+	       (method static c-fixnum?::bool (::obj)
+		       "INTEGERP")
+	       (method static c-elong?::bool (::obj)
+		       "ELONGP")
+	       (method static c-llong?::bool (::obj)
+		       "LLONGP")
+	       (method static c-=fx::bool (::long ::long)
+		       "EQ_FX")
+	       (method static c-<fx::bool (::long ::long)
+		       "LT_FX")
+	       (method static c-<=fx::bool (::long ::long)
+		       "LE_FX")
+	       (method static c->fx::bool (::long ::long)
+		       "GT_FX")
+	       (method static c->=fx::bool (::long ::long)
+		       "GE_FX")
+	       (method static c-even?::bool (::long)
+		       "EVENP_FX")
+	       (method static c-odd?::bool (::long)
+		       "ODDP_FX")
+	       (method static c-+fx::long (::long ::long)
+		       "PLUS_FX")
+	       (method static c--fx::long (::long ::long)
+		       "MINUS_FX")
+	       (method static c-*fx::long (::long ::long)
+		       "MUL_FX")
+	       (method static c-/fx::long (::long ::long)
+		       "DIV_FX")
+	       (method static c-negfx::long (::long)
+		       "NEG_FX")
+	       (method static c-quotient::long (::long ::long)
+		       "QUOTIENT_FX")
+	       (method static c-remainder::long (::long ::long)
+		       "REMAINDER_FX")
+	       (method static strtol::long (::string ::long ::long)
+		       "strtol")
+	       (method static c-int->string::bstring (::long ::long)
+		       "integer_to_string")
+	       (method static %%rand::int ()
+		       "rand")))
+   
+   (export  (inline integer?::bool ::obj)
+	    (inline fixnum?::bool ::obj)
+	    (inline elong?::bool ::obj)
+	    (inline llong?::bool ::obj)
+	    (inline make-elong::belong ::long)
+	    (inline =fx::bool ::long ::long)
+	    (inline >fx::bool ::long ::long)
+	    (inline >=fx::bool ::long ::long)
+	    (inline <fx::bool ::long ::long)
+	    (inline <=fx::bool ::long ::long)
+	    (inline zerofx?::bool ::long)
+	    (inline positivefx?::bool ::long)
+	    (inline negativefx?::bool ::long)
+	    (inline odd?::bool ::long)
+	    (inline even?::bool ::long)
+	    (maxfx::long ::long . pair)
+	    (minfx::long ::long . pair)
+	    (inline +fx::long ::long ::long)
+	    (inline -fx::long ::long ::long)
+	    (inline *fx::long ::long ::long)
+	    (inline /fx::long ::long ::long)
+	    (inline negfx::long ::long)
+	    (inline absfx::long ::long)
+	    (inline quotient::long ::long ::long)
+	    (inline remainder::long ::long ::long)
+	    (modulo::long ::long ::long)
+	    (gcd::long . pair)
+	    (lcm::long . pair)
+	    (integer->string::bstring ::long . pair)
+	    (string->integer::long ::bstring . pair)
+	    (string->elong::belong ::string . pair)
+	    (string->llong::bllong ::string . pair)
+	    (inline random::int ::int))
+   
+   (pragma  (fixnum? (predicate-of bint) no-cfa-top nesting)
+	    (c-fixnum? (predicate-of bint) no-cfa-top nesting)
+	    (c-elong? (predicate-of belong) no-cfa-top nesting)
+	    (c-llong? (predicate-of bllong) no-cfa-top nesting)
+	    (integer? side-effect-free no-cfa-top nesting)
+	    (=fx side-effect-free no-cfa-top nesting)
+	    (>fx side-effect-free no-cfa-top nesting)
+	    (>=fx side-effect-free no-cfa-top nesting)
+	    (<fx side-effect-free no-cfa-top nesting)
+	    (<=fx side-effect-free no-cfa-top nesting)
+	    (odd? side-effect-free no-cfa-top nesting)
+	    (even? side-effect-free no-cfa-top nesting)
+	    (+fx side-effect-free no-cfa-top nesting)
+	    (-fx side-effect-free no-cfa-top nesting)
+	    (*fx side-effect-free no-cfa-top nesting)
+	    (/fx side-effect-free no-cfa-top nesting)
+	    (remainder side-effect-free no-cfa-top nesting)
+	    (integer->string side-effect-free no-cfa-top nesting)
+	    (string->integer side-effect-free no-cfa-top nesting)
+	    (modulo side-effect-free no-cfa-top nesting)
+	    (gcd side-effect-free no-cfa-top nesting)
+	    (lcm side-effect-free no-cfa-top nesting)
+	    (quotient side-effect-free no-cfa-top nesting)
+	    (positivefx? side-effect-free no-cfa-top nesting)
+	    (negativefx? side-effect-free no-cfa-top nesting)
+	    (zerofx? side-effect-free no-cfa-top nesting)
+	    (negfx side-effect-free no-cfa-top nesting)
+	    (c-=fx side-effect-free no-cfa-top nesting)
+	    (c->fx side-effect-free no-cfa-top nesting)
+	    (c->=fx side-effect-free no-cfa-top nesting)
+	    (c-<fx side-effect-free no-cfa-top nesting)
+	    (c-<=fx side-effect-free no-cfa-top nesting)
+	    (c-odd? side-effect-free no-cfa-top nesting)
+	    (c-even? side-effect-free no-cfa-top nesting)
+	    (c-+fx side-effect-free no-cfa-top nesting)
+	    (c--fx side-effect-free no-cfa-top nesting)
+	    (c-*fx side-effect-free no-cfa-top nesting)
+	    (c-/fx side-effect-free no-cfa-top nesting)
+	    (c-negfx side-effect-free no-cfa-top nesting)
+	    (random side-effect-free no-cfa-top nesting)))
 
 ;*---------------------------------------------------------------------*/
 ;*    integer? ...                                                     */
@@ -401,5 +414,11 @@
 ;*---------------------------------------------------------------------*/
 (define (string->llong string . radix)
    (if (pair? radix)
-       (llong->bllong (string->integer string (car radix)))
-       (llong->bllong (string->integer string))))
+       (long->llong (string->integer string (car radix)))
+       (long->llong (string->integer string))))
+
+;*---------------------------------------------------------------------*/
+;*    random ...                                                       */
+;*---------------------------------------------------------------------*/
+(define-inline (random max::int)
+   (modulo (%%rand) max))
