@@ -1,43 +1,15 @@
-;*---------------------------------------------------------------------*/
-;*   A pratical implementation for the Scheme programming language     */
-;*                                                                     */
-;*                                    ,--^,                            */
-;*                              _ ___/ /|/                             */
-;*                          ,;'( )__, ) '                              */
-;*                         ;;  //   L__.                               */
-;*                         '   \\   /  '                               */
-;*                              ^   ^                                  */
-;*                                                                     */
-;*   Copyright (c) 1992-1999 Manuel Serrano                            */
-;*                                                                     */
-;*     Bug descriptions, use reports, comments or suggestions are      */
-;*     welcome. Send them to                                           */
-;*       bigloo-request@kaolin.unice.fr                                */
-;*       http://kaolin.unice.fr/bigloo                                 */
-;*                                                                     */
-;*   This program is free software; you can redistribute it            */
-;*   and/or modify it under the terms of the GNU General Public        */
-;*   License as published by the Free Software Foundation; either      */
-;*   version 2 of the License, or (at your option) any later version.  */
-;*                                                                     */
-;*   This program is distributed in the hope that it will be useful,   */
-;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
-;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
-;*   GNU General Public License for more details.                      */
-;*                                                                     */
-;*   You should have received a copy of the GNU General Public         */
-;*   License along with this program; if not, write to the Free        */
-;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
-;*   MA 02111-1307, USA.                                               */
-;*---------------------------------------------------------------------*/
 ;*=====================================================================*/
-;*    serrano/prgm/project/bigloo/runtime1.9b/Ieee/number.scm          */
+;*    serrano/prgm/project/bigloo/runtime/Ieee/number.scm              */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Fri Mar 24 09:59:43 1995                          */
-;*    Last change :  Mon Apr 14 11:33:44 1997 (serrano)                */
+;*    Last change :  Tue Jun 26 06:04:38 2001 (serrano)                */
 ;*    -------------------------------------------------------------    */
 ;*    6.5. Numbers (page 18, r4)                                       */
+;*    -------------------------------------------------------------    */
+;*    Source documentation:                                            */
+;*       @path ../../manuals/body.texi@                                */
+;*       @node Numbers@                                                */
 ;*=====================================================================*/
  
 ;*---------------------------------------------------------------------*/
@@ -45,33 +17,41 @@
 ;*---------------------------------------------------------------------*/
 (module __r4_numbers_6_5
    
-   (import  (__error                   "Llib/error.scm"))
+   (import  __error)
 
-   (use     (__type                    "Llib/type.scm")
-	    (__bigloo                  "Llib/bigloo.scm")
-	    (__tvector                 "Llib/tvector.scm")
-	    (__r4_equivalence_6_2      "Ieee/equiv.scm")
-	    (__r4_numbers_6_5_fixnum   "Ieee/fixnum.scm")
-	    (__r4_booleans_6_1         "Ieee/boolean.scm")
-	    (__r4_characters_6_6       "Ieee/char.scm")
-	    (__r4_pairs_and_lists_6_3  "Ieee/pair-list.scm")
-	    (__r4_vectors_6_8          "Ieee/vector.scm")
-	    (__r4_numbers_6_5_flonum   "Ieee/flonum.scm")
-	    (__r4_symbols_6_4          "Ieee/symbol.scm")
-	    (__r4_strings_6_7          "Ieee/string.scm")
+   (use     __type
+	    __bigloo
+	    __tvector
+	    __r4_equivalence_6_2
+	    __r4_numbers_6_5_fixnum
+	    __r4_booleans_6_1
+	    __r4_characters_6_6
+	    __r4_pairs_and_lists_6_3
+	    __r4_vectors_6_8
+	    __r4_numbers_6_5_flonum
+	    __r4_symbols_6_4
+	    __r4_strings_6_7
 
-	    (__evenv                   "Eval/evenv.scm"))
+	    __evenv)
    
-   (foreign (macro double c-fixnum->flonum (long)   "(double)")
-	    (macro long   c-flonum->fixnum (double) "(long)"))
+   (extern  (macro c-fixnum->flonum::double (::long)   "(double)")
+	    (macro c-flonum->fixnum::long   (::double) "(long)")
+	    (export exact->inexact "bgl_exact_to_inexact")
+	    (export inexact->exact "bgl_inexact_to_exact"))
+   
+   (java    (class foreign
+	       (method static c-fixnum->flonum::double (::long)
+		       "FIXNUM_TO_FLONUM")
+	       (method static c-flonum->fixnum::long   (::double)
+		       "FLONUM_TO_FIXNUM")))
    
    (export  (inline number?::bool           obj)
 	    (inline exact?::bool            z)
 	    (inline inexact?::bool          z)
 	    (complex?::bool                 x)
 	    (rational?::bool                x)
-	    (inline flonum->fixnum::long    x)
-	    (inline fixnum->flonum::double  x)
+	    (inline flonum->fixnum::long    ::double)
+	    (inline fixnum->flonum::double  ::long)
 	    (2=::bool                       x y)
 	    (=::bool                        x y . z)
 	    (2<::bool                       x y) 
@@ -157,7 +137,6 @@
 	    (inexact->exact side-effect-free)
 	    (number->string side-effect-free)
 	    (string->number side-effect-free)))
-		  
 
 ;*---------------------------------------------------------------------*/
 ;*    number? ...                                                      */
@@ -836,28 +815,86 @@
        (error "number->string" "Argument not a number" x))))
 
 ;*---------------------------------------------------------------------*/
-;*    string->number ...                                               */
+;*    @deffn string->number@ ...                                       */
 ;*---------------------------------------------------------------------*/
 (define (string->number x . radix)
-   (define (integer-string? x)
+   (define (integer-string? x r)
       (let loop ((i (-fx (string-length x) 1)))
-	 (cond
-	    ((=fx i -1)
-	     #t)
-	    ((char=? (string-ref x i) #\-)
-	     (=fx i 0))
-	    ((and (char>=? (string-ref x i) #\0)
-		  (char<=? (string-ref x i) #\9))
-	     (loop (-fx i 1)))
-	    ((and (char>=? (string-ref x i) #\a)
-		  (char<=? (string-ref x i) #\f))
-	     (loop (-fx i 1)))
-	    ((and (char>=? (string-ref x i) #\A)
-		  (char<=? (string-ref x i) #\F))
-	     (loop (-fx i 1)))
-	    (else
-	     #f))))
-   (if (integer-string? x)
-       (apply string->integer (cons x radix))
-       (apply string->real (cons x radix))))
-
+	 (cond ((=fx -1 i)
+		#t)
+	       ((and (char>=? (string-ref x i) #\0)
+		     (char<=? (string-ref x i) #\1)
+		     (>=fx r 2))
+		(loop (-fx i 1)))
+	       ((and (char>=? (string-ref x i) #\2)
+		     (char<=? (string-ref x i) #\7)
+		     (>=fx r 8))
+		(loop (-fx i 1)))
+	       ((and (char>=? (string-ref x i) #\8)
+		     (char<=? (string-ref x i) #\9)
+		     (>=fx r 10))
+		(loop (-fx i 1)))
+	       ((and (char>=? (string-ref x i) #\a)
+		     (char<=? (string-ref x i) #\f)
+		     (=fx r 16))
+		(loop (-fx i 1)))
+	       ((and (char>=? (string-ref x i) #\A)
+		     (char<=? (string-ref x i) #\F)
+		     (=fx r 16))
+		(loop (-fx i 1)))
+	       ((or (char=? (string-ref x i) #\-)
+		    (char=? (string-ref x i) #\+))
+		(=fx i 0))
+	       (else #f))))
+   (define (real-string? x)
+      (let ((len (string-length x)))
+	 (let loop ((i 0)
+		    (e #f)
+		    (p 0))
+	    (cond ((=fx i len)
+		   #t)
+		  ((and (char>=? (string-ref x i) #\0)
+			(char<=? (string-ref x i) #\9))
+		   (loop (+fx i 1)
+			 e
+			 0))
+		  ((char=? (string-ref x i) #\.)
+		   (loop (+fx i 1)
+			 e
+			 0))
+		  ((or (char=? (string-ref x i) #\e)
+		       (char=? (string-ref x i) #\E))
+		   (if e
+		       #f
+		       (loop (+fx i 1)
+			     #t
+			     (+fx i 1))))
+		  ((or (char=? (string-ref x i) #\-)
+		       (char=? (string-ref x i) #\+))
+		   (and (or (=fx i 0) (=fx i p))
+			(loop (+fx i 1)
+			      e
+			      0)))
+		  (else #f)))))
+   (let ((rx (match-case radix
+		(() 10)
+		((?val)
+		 (case val
+		    ((2 8 10 16) val)
+		    (else (error "string->number"
+				 "Illegal radix"
+				 val))))
+		(else (error "string->number"
+			     "Illegal number of optional arguments"
+			     radix)))))
+      (cond
+	 ((integer-string? x rx)
+	  (string->integer x rx))
+	 ((real-string? x)
+	  (if (=fx rx 10)
+	      (string->real x)
+	      (error "string->number"
+		     "Only radix `10' is legal for floating point number"
+		     rx)))
+	 (else
+	  #f))))
