@@ -1,41 +1,9 @@
-;*---------------------------------------------------------------------*/
-;*   A pratical implementation for the Scheme programming language     */
-;*                                                                     */
-;*                                    ,--^,                            */
-;*                              _ ___/ /|/                             */
-;*                          ,;'( )__, ) '                              */
-;*                         ;;  //   L__.                               */
-;*                         '   \\   /  '                               */
-;*                              ^   ^                                  */
-;*                                                                     */
-;*   Copyright (c) 1992-1999 Manuel Serrano                            */
-;*                                                                     */
-;*     Bug descriptions, use reports, comments or suggestions are      */
-;*     welcome. Send them to                                           */
-;*       bigloo-request@kaolin.unice.fr                                */
-;*       http://kaolin.unice.fr/bigloo                                 */
-;*                                                                     */
-;*   This program is free software; you can redistribute it            */
-;*   and/or modify it under the terms of the GNU General Public        */
-;*   License as published by the Free Software Foundation; either      */
-;*   version 2 of the License, or (at your option) any later version.  */
-;*                                                                     */
-;*   This program is distributed in the hope that it will be useful,   */
-;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
-;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
-;*   GNU General Public License for more details.                      */
-;*                                                                     */
-;*   You should have received a copy of the GNU General Public         */
-;*   License along with this program; if not, write to the Free        */
-;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
-;*   MA 02111-1307, USA.                                               */
-;*---------------------------------------------------------------------*/
 ;*=====================================================================*/
 ;*    serrano/prgm/project/bigloo/runtime/Ieee/output.scm              */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Sun Jul  5 11:13:01 1992                          */
-;*    Last change :  Fri Feb  6 16:55:20 1998 (serrano)                */
+;*    Last change :  Thu Dec  6 05:49:18 2001 (serrano)                */
 ;*    -------------------------------------------------------------    */
 ;*    6.10.3 Output (page 31, r4)                                      */
 ;*    -------------------------------------------------------------    */
@@ -44,6 +12,10 @@
 ;*    means, error during the initialization time will be correctly    */
 ;*    prompted. In particular, we must be very carefull not to launch  */
 ;*    unexpected errors.                                               */
+;*    -------------------------------------------------------------    */
+;*    Source documentation:                                            */
+;*       @path ../../manuals/body.texi@                                */
+;*       @node Input And Output@                                       */
 ;*=====================================================================*/
 
 ;*---------------------------------------------------------------------*/
@@ -51,83 +23,125 @@
 ;*---------------------------------------------------------------------*/
 (module __r4_output_6_10_3
 
-   (import  (__error                   "Llib/error.scm")
-	    (__bexit                   "Llib/bexit.scm")
-	    (__r4_ports_6_10_1         "Ieee/port.scm"))
+   (import  __error
+	    __bexit
+	    __r4_ports_6_10_1
+	    __bigloo)
    
-   (use     (__type                    "Llib/type.scm")
-	    (__bigloo                  "Llib/bigloo.scm")
-	    (__tvector                 "Llib/tvector.scm")
-	    (__structure               "Llib/struct.scm")
-	    (__bexit                   "Llib/bexit.scm")
-	    (__tvector                 "Llib/tvector.scm")
-	    (__object                  "Llib/object.scm")
-	    (__ucs2                    "Llib/ucs2.scm")
-	    (__unicode                 "Llib/unicode.scm")
-	    
-	    (__r4_numbers_6_5_fixnum   "Ieee/fixnum.scm")
-	    (__r4_numbers_6_5_flonum   "Ieee/flonum.scm")
-	    (__r4_characters_6_6       "Ieee/char.scm")
-	    (__r4_strings_6_7          "Ieee/string.scm")
-	    (__r4_vectors_6_8          "Ieee/vector.scm")
-	    (__r4_equivalence_6_2      "Ieee/equiv.scm")
-	    (__r4_booleans_6_1         "Ieee/boolean.scm")
-	    (__r4_symbols_6_4          "Ieee/symbol.scm")
-	    (__r4_pairs_and_lists_6_3  "Ieee/pair-list.scm")
-	    (__r4_control_features_6_9 "Ieee/control.scm")
+   (use     __type
+	    __tvector
+	    __structure
+	    __bexit
+	    __object
+	    __ucs2
+	    __unicode
+	     
+	    __r4_numbers_6_5_fixnum
+	    __r4_numbers_6_5_flonum
+	    __r4_characters_6_6
+	    __r4_strings_6_7
+	    __r4_vectors_6_8
+	    __r4_equivalence_6_2
+	    __r4_booleans_6_1
+	    __r4_symbols_6_4
+	    __r4_pairs_and_lists_6_3
+	    __r4_control_features_6_9
 
-	    (__evenv                   "Eval/evenv.scm"))
+	    __evenv)
      
    (extern  (macro output-string-port::bool (::obj)
-            		                    "OUTPUT_STRING_PORTP")
-	    (macro c-write-char::uchar      (::uchar ::output-port)
-		                            "WRITE_CHAR")
-	    (c-write_char::obj              (::bchar ::output-port)
-					    "write_char")
-	    (display-char::obj              (::bchar ::output-port)
-					    "display_char")
-	    (c-write_ucs2::obj              (::bucs2 ::output-port)
-					    "write_ucs2")
-	    (display-ucs2::obj              (::bucs2 ::output-port)
-					    "display_ucs2")
-	    (write-object::obj              (::obj ::output-port)
-					    "write_object")
-	    (write-string::obj              (::bstring ::output-port)
-					    "write_string")
-	    (display-string::obj            (::bstring ::output-port)
-					    "display_string")
-	    (display-fixnum::obj            (::blong ::output-port)
-					    "display_fixnum")
-	    (display-flonum::obj            (::real ::output-port)
-					    "display_flonum")
-	    (ill-char-rep::obj              (::uchar)
-					    "ill_char_rep")
-	    (write-utf8string::obj          (::bstring ::output-port)
-					    "write_utf8string")
-	    (display-ucs2string::obj        (::ucs2string ::output-port)
-					    "display_ucs2string")
-	    (export c-debugging-print "dprint"))
+		   "OUTPUT_STRING_PORTP")
+	    (macro c-write-char::uchar (::uchar ::output-port)
+		   "WRITE_CHAR")
+	    (c-write_char::obj (::bchar ::output-port)
+			       "write_char")
+	    (display-char::obj (::bchar ::output-port)
+			       "display_char")
+	    (c-write_ucs2::obj (::bucs2 ::output-port)
+			       "write_ucs2")
+	    (display-ucs2::obj (::bucs2 ::output-port)
+			       "display_ucs2")
+	    (write-object::obj (::obj ::output-port)
+			       "write_object")
+	    (write-string::obj (::bstring ::bool ::output-port)
+			       "write_string")
+	    (display-string::obj (::bstring ::output-port)
+				 "display_string")
+	    (write-symbol::obj (::symbol ::output-port)
+			       "write_symbol")
+	    (display-symbol::obj (::symbol ::output-port)
+				 "display_symbol")
+	    (display-fixnum::obj (::blong ::output-port)
+				 "display_fixnum")
+	    (display-flonum::obj (::real ::output-port)
+				 "display_flonum")
+	    (ill-char-rep::obj (::uchar)
+			       "ill_char_rep")
+	    (write-utf8string::obj (::bstring ::output-port)
+				   "write_utf8string")
+	    (display-ucs2string::obj (::ucs2string ::output-port)
+				     "display_ucs2string"))
     
-   (export  (set-write-length!        ::long)
+   (java    (class foreign
+	       (method static output-string-port::bool (::obj)
+		       "OUTPUT_STRING_PORTP")
+	       (method static c-write-char::uchar (::uchar ::output-port)
+		       "WRITE_CHAR")
+	       (method static c-write_char::obj (::bchar ::output-port)
+		       "write_char")
+	       (method static display-char::obj (::bchar ::output-port)
+		       "display_char")
+	       (method static c-write_ucs2::obj (::bucs2 ::output-port)
+		       "write_ucs2")
+	       (method static display-ucs2::obj (::bucs2 ::output-port)
+		       "display_ucs2")
+	       (method static write-object::obj (::obj ::output-port)
+		       "write_object")
+	       (method static write-string::obj (::bstring ::bool ::output-port)
+		       "write_string")
+	       (method static display-string::obj (::bstring ::output-port)
+		       "display_string")
+	       (method static write-symbol::obj (::symbol ::output-port)
+		       "write_symbol")
+	       (method static display-symbol::obj (::symbol ::output-port)
+		       "display_symbol")
+	       (method static display-fixnum::obj (::bint ::output-port)
+		       "display_fixnum")
+	       (method static display-flonum::obj (::real ::output-port)
+		       "display_flonum")
+	       (method static ill-char-rep::obj (::uchar)
+		       "ill_char_rep")
+	       (method static write-utf8string::obj (::bstring ::output-port)
+		       "write_utf8string")
+	       (method static display-ucs2string::obj (::ucs2string ::output-port)
+		       "display_ucs2string")))
+    
+   (export  (set-write-length! ::long)
 	    (get-write-length::long)
-	    (newline                  . port)
-	    (write                    obj . port)
-	    (display                  obj . port)  
-	    (write-char               ::uchar . port)
-	    (illegal-char-rep         ::uchar) 
-	    (display*                 . obj)
-	    (write*                   . obj)
-	    (print                    . obj)
-	    (fprint                   ::output-port . obj)
-	    (set-printer!             ::procedure)
-	    (native-printer::procedure)
-	    (c-debugging-print        ::obj)))
+	    (newline . port)
+	    (inline newline-1 ::output-port)
+	    (write obj . port)
+	    (write-2 obj ::output-port)
+	    (display obj . port)  
+	    (display-2 obj ::output-port)  
+	    (write-char ::uchar . port)
+	    (inline write-char-2 ::uchar ::output-port)
+	    (illegal-char-rep ::uchar) 
+	    (display* . obj)
+	    (write* . obj)
+	    (print . obj)
+	    (fprint ::output-port . obj)
+	    (set-printer! ::procedure)
+	    (native-printer::procedure))
+
+   (pragma  (output-string-port nesting)
+	    (c-write-char nesting)))
    
 ;*---------------------------------------------------------------------*/
 ;*    *max-length-write* ...                                           */
 ;*    -------------------------------------------------------------    */
-;*    Cette variable contient le nombre maximum d'appels recursifs     */
-;*    toleres lors d'un write.                                         */
+;*    This variable contains the number of recursive calls that are    */
+;*    accepted during a write.                                         */
 ;*---------------------------------------------------------------------*/
 (define *max-length-write* -1)
 
@@ -154,34 +168,51 @@
 ;*    newline ...                                                      */
 ;*---------------------------------------------------------------------*/
 (define (newline . port)
-   (c-write-char #\Newline
-		 (if (or (not (pair? port))
-			 (not (output-port? (car port))))
-		     (current-output-port)
-		     (car port))))
+   (let ((port (match-case port
+		  (()
+		   (current-output-port))
+		  ((?-)
+		   (car port))
+		  (else
+		   (error "newline"
+			  "wrong number of optional arguments"
+			  port)))))
+      (newline-1 port)))
+
+;*---------------------------------------------------------------------*/
+;*    newline-1 ...                                                    */
+;*---------------------------------------------------------------------*/
+(define-inline (newline-1 port::output-port)
+   (c-write-char #\Newline port))
 
 ;*---------------------------------------------------------------------*/
 ;*    display ...                                                      */
 ;*---------------------------------------------------------------------*/
 (define (display obj . port)
+   (let ((port (match-case port
+		  (()
+		   (current-output-port))
+		  ((?-)
+		   (car port))
+		  (else
+		   (error "display"
+			  "wrong number of optional arguments"
+			  port)))))
+      (display-2 obj port)))
+
+;*---------------------------------------------------------------------*/
+;*    display-2 ...                                                    */
+;*---------------------------------------------------------------------*/
+(define (display-2 obj port::output-port)
    (set! *displayed* 0)
    (if (or (eq? *current-printer* native-display)
 	   (not (procedure? *current-printer*)))
-       (write/display obj
-		      (if (or (not (pair? port))
-			      (not (output-port? (car port))))
-			  (current-output-port)
-			  (car port))
-		      #t)
-       (try (*current-printer* obj
-			       (if (or (not (pair? port))
-				       (not (output-port? (car port))))
-				   (current-output-port)
-				   (car port)))
+       (write/display obj port #t)
+       (try (*current-printer* obj port)
 	    (lambda (esc proc mes obj)
 	       (set-printer! (native-printer))
 	       (error proc mes obj)))))
-
+   
 ;*---------------------------------------------------------------------*/
 ;*    native-display ...                                               */
 ;*---------------------------------------------------------------------*/
@@ -197,29 +228,65 @@
 ;*    write ...                                                        */
 ;*---------------------------------------------------------------------*/
 (define (write obj . port)
+   (let ((port (match-case port
+		  (()
+		   (current-output-port))
+		  ((?-)
+		   (car port))
+		  (else
+		   (error "write"
+			  "wrong number of optional arguments"
+			  port)))))
+      (write-2 obj port)))
+
+;*---------------------------------------------------------------------*/
+;*    write-2 ...                                                      */
+;*---------------------------------------------------------------------*/
+(define (write-2 obj port::output-port)
    (set! *displayed* 0)
-   (write/display obj
-		  (if (or (not (pair? port))
-			  (not (output-port? (car port))))
-		      (current-output-port)
-		      (car port))
-		  #f))
+   (write/display obj port #f))
 
 ;*---------------------------------------------------------------------*/
 ;*    write-char ...                                                   */
 ;*---------------------------------------------------------------------*/
 (define (write-char char . port)
-   (c-write-char char
-		 (if (or (not (pair? port))
-			 (not (output-port? (car port))))
-		     (current-output-port)
-		     (car port))))
+   (let ((port (match-case port
+		  (()
+		   (current-output-port))
+		  ((?-)
+		   (car port))
+		  (else
+		   (error "write"
+			  "wrong number of optional arguments"
+			  port)))))
+      (write-char-2 char port)))
+
+;*---------------------------------------------------------------------*/
+;*    write-char-2 ...                                                 */
+;*---------------------------------------------------------------------*/
+(define-inline (write-char-2 char::uchar port::output-port)
+   (c-write-char char port))
 
 ;*---------------------------------------------------------------------*/
 ;*    illegal-char-rep ...                                             */
 ;*---------------------------------------------------------------------*/
 (define (illegal-char-rep char)
-   (ill-char-rep char))
+   (if (or (char-alphabetic? char) (char-numeric? char))
+       char
+       (case char
+	  ((#\Newline)
+	   "#\Newline")
+	  ((#\Return)
+	   "#\Return")
+	  ((#\Space)
+	   "#\Space")
+	  ((#\Tab)
+	   "#\Tab")
+	  (else
+	   (let ((i (char->integer char)))
+	      (if (<fx i (char->integer #\!))
+		  (ill-char-rep char)
+		  char))))))
 
 ;*---------------------------------------------------------------------*/
 ;*    write/display ...                                                */
@@ -227,7 +294,7 @@
 ;*    As we can't be sure that this module is initialized before       */
 ;*    used (for example when an error occurs during module             */
 ;*    intialization process), we can't use symbol to set               */
-;*    flag. So is flag is equal to #t it means that                    */
+;*    flag. So if flag is equal to #t it means that                    */
 ;*    `write-display' is used to display and if flags is equal         */
 ;*    to #f it writes.                                                 */
 ;*---------------------------------------------------------------------*/
@@ -240,6 +307,22 @@
        (display-string "..." port))
       ((fixnum? obj)
        (display-fixnum obj port))
+      ((symbol? obj) 
+       (if flag
+	   (display-symbol obj port)
+	   (write-symbol obj port)))
+      ((string? obj) 
+       (if flag
+	   (display-string obj port)
+	   (write-string (string-for-read obj)
+			 *bigloo-strict-r5rs-strings*
+			 port)))
+      ((pair? obj)
+       (write/display-pair obj port flag))
+      ((class? obj)
+       (write/display-class obj port flag))
+      ((vector? obj)
+       (write/display-vector obj port flag))
       ((char? obj)
        (if flag 
 	   (display-char obj port)
@@ -248,14 +331,6 @@
        (if flag 
 	   (display-ucs2 obj port)
 	   (c-write_ucs2 obj port)))
-      ((string? obj) 
-       (if flag
-	   (display-string obj port)
-	   (write-string (string-for-read obj) port)))
-      ((vector? obj)
-       (write/display-vector obj port flag))
-      ((pair? obj)
-       (write/display-pair obj port flag))
       ((flonum? obj)
        (display-flonum obj port))
       ((cell? obj)
@@ -271,10 +346,20 @@
 	   (display-ucs2string obj port) 
 	   (write-utf8string (string-for-read (ucs2-string->utf8-string obj))
 			     port)))
+      ((opaque? obj)
+       (display-string "#<opaque>" port))
       (else
        (write-object obj port)))
    #unspecified)
 
+;*---------------------------------------------------------------------*/
+;*    write/display-class ...                                          */
+;*---------------------------------------------------------------------*/
+(define (write/display-class obj port flag)
+   (display-string "#<class:" port)
+   (display-symbol (class-name obj) port)
+   (display-string ">" port))
+   
 ;*---------------------------------------------------------------------*/
 ;*    write/display-cell ...                                           */
 ;*---------------------------------------------------------------------*/
@@ -412,9 +497,6 @@
 	  (c-write-char #\space port)
 	  (loop (cdr l))))))
  
-;*---------------------------------------------------------------------*/
-;*    Les fonctions qui suivent ne sont pas r4                         */
-;*---------------------------------------------------------------------*/
 ;*---------------------------------------------------------------------*/
 ;*    print ...                                                        */
 ;*---------------------------------------------------------------------*/
@@ -494,14 +576,4 @@
 (define (native-printer)
    native-display)
 
-;*---------------------------------------------------------------------*/
-;*    c-debugging-print ...                                            */
-;*---------------------------------------------------------------------*/
-(define (c-debugging-print obj)
-   (set! *displayed* 0)
-   (let ((port (current-output-port)))
-      (write/display obj port #t)
-      (c-write-char #\Newline port)
-      obj))
-
 
