@@ -1,139 +1,205 @@
-;*---------------------------------------------------------------------*/
-;*   A pratical implementation for the Scheme programming language     */
-;*                                                                     */
-;*                                    ,--^,                            */
-;*                              _ ___/ /|/                             */
-;*                          ,;'( )__, ) '                              */
-;*                         ;;  //   L__.                               */
-;*                         '   \\   /  '                               */
-;*                              ^   ^                                  */
-;*                                                                     */
-;*   Copyright (c) 1992-1999 Manuel Serrano                            */
-;*                                                                     */
-;*     Bug descriptions, use reports, comments or suggestions are      */
-;*     welcome. Send them to                                           */
-;*       bigloo-request@kaolin.unice.fr                                */
-;*       http://kaolin.unice.fr/bigloo                                 */
-;*                                                                     */
-;*   This program is free software; you can redistribute it            */
-;*   and/or modify it under the terms of the GNU General Public        */
-;*   License as published by the Free Software Foundation; either      */
-;*   version 2 of the License, or (at your option) any later version.  */
-;*                                                                     */
-;*   This program is distributed in the hope that it will be useful,   */
-;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
-;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
-;*   GNU General Public License for more details.                      */
-;*                                                                     */
-;*   You should have received a copy of the GNU General Public         */
-;*   License along with this program; if not, write to the Free        */
-;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
-;*   MA 02111-1307, USA.                                               */
-;*---------------------------------------------------------------------*/
 ;*=====================================================================*/
 ;*    serrano/prgm/project/bigloo/runtime/Ieee/port.scm                */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Mon Feb 20 16:53:27 1995                          */
-;*    Last change :  Thu Oct 29 08:14:05 1998 (serrano)                */
+;*    Last change :  Thu Nov  8 11:58:50 2001 (serrano)                */
 ;*    -------------------------------------------------------------    */
 ;*    6.10.1 Ports (page 29, r4)                                       */
+;*    -------------------------------------------------------------    */
+;*    Source documentation:                                            */
+;*       @path ../../manuals/body.texi@                                */
+;*       @node Input And Output@                                       */
 ;*=====================================================================*/
 
 ;*---------------------------------------------------------------------*/
 ;*    Le module                                                        */
 ;*---------------------------------------------------------------------*/
 (module __r4_ports_6_10_1
-
-   (import  (__error                   "Llib/error.scm"))
    
-   (use     (__type                    "Llib/type.scm")
-	    (__bigloo                  "Llib/bigloo.scm")
-	    (__tvector                 "Llib/tvector.scm")
-	    (__bexit                   "Llib/bexit.scm")
-	    (__r4_vectors_6_8          "Ieee/vector.scm")
-	    (__r4_strings_6_7          "Ieee/string.scm")
-	    (__r4_characters_6_6       "Ieee/char.scm")
-	    (__r4_control_features_6_9 "Ieee/control.scm")
-	    (__r4_numbers_6_5_fixnum   "Ieee/fixnum.scm")
-	    (__r4_equivalence_6_2      "Ieee/equiv.scm")
-	    (__r4_booleans_6_1         "Ieee/boolean.scm")
-	    (__r4_symbols_6_4          "Ieee/symbol.scm")
-	    (__r4_pairs_and_lists_6_3  "Ieee/pair-list.scm")
-
-	    (__evenv                   "Eval/evenv.scm"))
-
-   (foreign (macro bool  c-input-port?  (obj)             "INPUT_PORTP")
-	    (macro bool  c-output-port? (obj)             "OUTPUT_PORTP")
-	    (output-port c-current-output-port           "current_output_port")
-	    (output-port c-current-error-port             "current_error_port")
-	    (input-port  c-current-input-port             "current_input_port")
-	    (obj         c-open-input-file (bstring obj)  "open_input_file")
-	    (obj         c-open-input-c-string (string)   "open_input_c_string")
-	    (obj         c-open-input-string (bstring)    "open_input_string")
-	    (obj         c-open-output-file  (bstring)    "open_output_file")
-	    (obj         c-append-output-file (bstring)   "append_output_file")
-	    (obj         c-open-output-string ()          "open_output_string")
-	    (obj         c-close-input-port  (obj)        "close_input_port")
-	    (obj         c-close-output-port (obj)        "close_output_port")
-	    (long        c-default-io-bufsiz              "default_io_bufsiz")
-	    (bool        c-reset-eof         (obj)        "reset_eof")
-	    (macro obj  c-flush-output-port (output-port) "FLUSH_OUTPUT_PORT")
-	    (bool       c-fexists?          (string)      "fexists")
-	    (macro bool c-delete-file       (string)        "unlink")
-	    (macro bool c-delete-directory  (string)        "rmdir")
-	    (macro int  c-rename-file       (string string) "rename")
-	    
-	    (macro long   c-input-port-position (input-port)
-		   "INPUT_PORT_FILEPOS")
-	    (macro string c-input-port-name     (input-port)
-		   "INPUT_PORT_NAME")
-	    
-	    (bool       __directory?        (string)      "directoryp")
-	    (obj        __directory->list   (string)      "directory_to_list"))
+   (import  __error)
    
-   (export  (call-with-input-file  ::bstring ::procedure)
+   (use     __type
+	    __bigloo
+	    __tvector
+	    __bexit
+	    __socket
+	    __r4_vectors_6_8
+	    __r4_strings_6_7
+	    __r4_characters_6_6
+	    __r4_control_features_6_9
+	    __r4_numbers_6_5_fixnum
+	    __r4_equivalence_6_2
+	    __r4_booleans_6_1
+	    __r4_symbols_6_4
+	    __r4_pairs_and_lists_6_3
+	    __r4_output_6_10_3
+	    
+	    __evenv)
+   
+   (extern  (macro c-input-port?::bool  (::obj) "INPUT_PORTP")
+	    (macro c-output-port?::bool (::obj) "OUTPUT_PORTP")
+	    
+	    (c-current-output-port::output-port "current_output_port")
+	    (c-current-error-port::output-port "current_error_port")
+	    (c-current-input-port::input-port "current_input_port")
+	    
+	    (c-open-input-file::obj (::bstring ::obj) "open_input_file")
+	    (c-open-input-pipe::obj (::bstring ::obj) "open_input_pipe")
+ 	    (c-open-input-c-string::obj (::string) "open_input_c_string")
+	    (c-reopen-input-c-string::obj (::input-port ::string) "reopen_input_c_string")
+	    (c-open-input-string::obj (::bstring) "open_input_string")
+	    (c-open-output-file::obj (::bstring) "open_output_file")
+	    (c-append-output-file::obj (::bstring) "append_output_file")
+	    (c-open-output-string::obj () "open_output_string")
+	    (c-close-input-port::obj (::obj) "close_input_port")
+	    (c-input-port-reopen!::obj (::input-port) "bgl_input_port_reopen")
+	    (c-input-port-seek!::obj (::input-port ::long) "bgl_input_port_seek")
+	    
+	    (macro c-closed-input-port?::bool (::obj) "!RGC_BUFFER")
+	    (c-close-output-port::obj (::obj) "close_output_port")
+	    (c-get-output-string::bstring (::output-port)"get_output_string")
+	    (c-default-io-bufsiz::long "default_io_bufsiz")
+	    (c-reset-eof::bool (::obj) "reset_eof")
+	    (macro c-flush-output-port::obj (::output-port) "FLUSH_OUTPUT_PORT")
+	    
+	    (c-fexists?::bool (::string) "fexists")
+	    (macro c-delete-file::bool (::string) "unlink")
+	    (macro c-delete-directory::bool  (::string) "rmdir")
+	    (macro c-rename-file::int (::string ::string) "rename")
+	    
+	    (macro c-input-port-position::long (::input-port) "INPUT_PORT_FILEPOS")
+ 	    (macro c-input-port-name::string (::input-port) "INPUT_PORT_NAME")
+	    
+	    (__directory?::bool (::string) "directoryp")
+	    (__directory->list::obj (::string) "directory_to_list")
+	    (c-modification-time::long (::string) "bgl_last_modification_time")
+	    (c-file-size::long (::string) "bgl_file_size"))
+   
+   (java    (class foreign
+	       (method static c-input-port?::bool  (::obj)
+		       "INPUT_PORTP")
+	       (method static c-output-port?::bool (::obj)
+		       "OUTPUT_PORTP")
+	       
+	       (field static c-current-output-port::output-port
+		      "current_output_port")
+	       (field static c-current-error-port::output-port
+		      "current_error_port")
+	       (field static c-current-input-port::input-port
+		      "current_input_port")
+	       (field static c-default-io-bufsiz::long
+		      "default_io_bufsiz")
+	       
+	       (method static c-open-input-file::obj (::bstring ::bint)
+		       "open_input_file")
+	       (method static c-open-input-pipe::obj (::bstring ::bint)
+		       "open_input_pipe")
+	       (method static c-open-input-c-string::obj (::string)
+		       "open_input_c_string")
+	       (method static c-reopen-input-c-string::obj (::input-port ::string)
+		       "reopen_input_c_string")
+	       (method static c-open-input-string::obj (::bstring)
+		       "open_input_string")
+	       (method static c-open-output-file::obj (::bstring)
+		       "open_output_file")
+	       (method static c-append-output-file::obj (::bstring)
+		       "append_output_file")
+	       (method static c-open-output-string::obj ()
+		       "open_output_string")
+	       (method static c-close-input-port::obj (::input-port)
+		       "close_input_port")
+	       (method static c-input-port-reopen!::obj (::input-port)
+		       "bgl_input_port_reopen")
+	       (method static c-input-port-seek!::obj (::input-port ::long)
+		       "bgl_input_port_seek")
+	       
+	       (method static c-closed-input-port?::bool (::input-port)
+		       "CLOSED_RGC_BUFFER")
+	       (method static c-close-output-port::obj (::output-port)
+		       "close_output_port")
+	       (method static c-get-output-string::bstring (::output-port)
+		       "get_output_string")
+	       (method static c-reset-eof::bool (::obj)
+		       "reset_eof")
+	       (method static c-flush-output-port::obj (::output-port)
+		       "FLUSH_OUTPUT_PORT")
+	       
+	       (method static c-fexists?::bool (::string)
+		       "fexists")
+	       (method static c-delete-file::bool (::string)
+		       "unlink")
+	       (method static c-delete-directory::bool  (::string)
+		       "rmdir")
+	       (method static c-rename-file::int (::string ::string)
+		       "rename")
+	       
+	       (method static c-input-port-position::long (::input-port)
+		       "INPUT_PORT_FILEPOS")
+	       (method static c-input-port-name::string (::input-port)
+		       "INPUT_PORT_NAME")
+	       
+	       (method static __directory?::bool (::string)
+		       "directoryp")
+	       (method static __directory->list::obj (::string)
+		       "directory_to_list")
+	       (method static c-modification-time::long (::string)
+		       "bgl_last_modification_time")
+	       (method static c-file-size::long (::string)
+		       "bgl_file_size")))
+   
+   (export  (call-with-input-file ::bstring ::procedure)
 	    (call-with-output-file ::bstring ::procedure)
-
-	    (inline input-port?    ::obj)
-	    (inline output-port?   ::obj)
-	    (inline port?::bool    ::obj)
+	    
+	    (inline input-port? ::obj)
+	    (inline output-port? ::obj)
+	    (inline port?::bool ::obj)
 	    
 	    (inline current-input-port::input-port) 
 	    (inline current-error-port::output-port)
 	    (inline current-output-port::output-port)
 	    
 	    (with-input-from-file ::bstring ::procedure)
+	    (with-input-from-string ::bstring ::procedure)
 	    (with-input-from-port ::input-port ::procedure)
-	    (with-output-to-file  ::bstring ::procedure)
-	    (with-output-to-port  ::output-port ::procedure)
+	    (with-output-to-file ::bstring ::procedure)
+	    (with-output-to-string ::procedure)
+	    (with-error-to-string ::procedure)
+	    (with-output-to-port ::output-port ::procedure)
 	    
-	    (with-error-to-file   ::bstring ::procedure)
-	    (with-error-to-port   ::output-port ::procedure)
+	    (with-error-to-file ::bstring ::procedure)
+	    (with-error-to-port ::output-port ::procedure)
 	    
-	    (open-input-file                       ::bstring . bufsiz)
-	    (inline open-input-string              ::bstring)
-	    (inline open-input-c-string            ::string)
-	    (inline open-output-file               ::bstring)
-	    (inline append-output-file             ::bstring)
+	    (open-input-file ::bstring . bufsiz)
+	    (inline open-input-string ::bstring)
+	    (inline open-input-c-string ::string)
+	    (inline reopen-input-c-string ::input-port ::string)
+	    (inline open-output-file ::bstring)
+	    (inline append-output-file ::bstring)
 	    (inline open-output-string)
-	    (inline close-input-port               ::input-port)
-	    (inline close-output-port              ::output-port)
-	    (inline flush-output-port              ::output-port)
-	    (inline reset-eof::bool                ::input-port)
-	    (inline input-port-position::int       ::input-port)
-	    (inline input-port-name::string        ::input-port)
+	    (inline closed-input-port?::bool ::input-port)
+	    (inline close-input-port ::input-port)
+	    (inline get-output-string::bstring ::output-port)
+	    (inline close-output-port ::output-port)
+	    (inline flush-output-port ::output-port)
+	    (inline reset-eof::bool ::input-port)
+	    (inline input-port-seek! ::input-port ::long)
+	    (inline input-port-reopen! ::input-port)
+	    (inline input-port-position::int ::input-port)
+	    (inline input-port-name::string ::input-port)
 	    
-	    (inline file-exists?     ::string)
-	    (inline delete-file      ::string)
+	    (inline file-exists?::bool ::string)
+	    (inline delete-file ::string)
 	    (inline delete-directory ::string)
-	    (inline rename-file      ::string ::string)
+	    (inline rename-file ::string ::string)
 	    (inline directory?::bool ::string)
-	    (inline directory->list  ::string))
+	    (inline directory->list ::string)
+	    (file-modification-time::long ::string)
+	    (file-size::long ::string))
    
-   (pragma  (c-input-port? (predicate-of input-port))
-	    (c-output-port? (predicate-of output-port))
-	    (file-exists? side-effect-free)))
+   (pragma  (c-input-port? (predicate-of input-port) nesting)
+	    (c-output-port? (predicate-of output-port) nesting)
+	    (file-exists? side-effect-free nesting)))
    
 
 ;*---------------------------------------------------------------------*/
@@ -189,7 +255,21 @@
    c-current-error-port)
 
 ;*---------------------------------------------------------------------*/
-;*    with-input-from-file ...                                         */
+;*    @deffn input-port-seek!@ ...                                     */
+;*---------------------------------------------------------------------*/
+(define-inline (input-port-seek! port::input-port pos::long)
+   (if (not (c-input-port-seek! port pos))
+       (error "input-port-seek!" "Can't seek port" port)))
+   
+;*---------------------------------------------------------------------*/
+;*    @deffn input-port-reopen!@ ...                                   */
+;*---------------------------------------------------------------------*/
+(define-inline (input-port-reopen! port::input-port)
+   (if (not (c-input-port-reopen! port))
+       (error "input-port-reopen!" "Can't reopen port" port)))
+
+;*---------------------------------------------------------------------*/
+;*    @deffn with-input-from-file@ ...                                 */
 ;*---------------------------------------------------------------------*/
 (define (with-input-from-file string thunk)
    (let ((port (open-input-file string)))
@@ -204,6 +284,22 @@
 		   (close-input-port port))))
 	  (error "with-input-from-file" "can't open file" string))))
 
+;*---------------------------------------------------------------------*/
+;*    @deffn with-input-from-string@ ...                               */
+;*---------------------------------------------------------------------*/
+(define (with-input-from-string string thunk)
+   (let ((port (open-input-string string)))
+      (if (input-port? port)
+	  (let ((old-input-port (current-input-port)))
+	     (unwind-protect
+		(begin
+		   (set! c-current-input-port port)
+		   (thunk))
+		(begin
+		   (set! c-current-input-port old-input-port)
+		   (close-input-port port))))
+	  (error "with-input-from-string" "can't open string" string))))
+
 ;*---------------------------------------------------------------------*/
 ;*    with-input-from-port ...                                         */
 ;*---------------------------------------------------------------------*/
@@ -232,7 +328,7 @@
 	  (error "with-output-to-file" "can't open file" string))))
 
 ;*---------------------------------------------------------------------*/
-;*    with-output-to-port ...                                          */
+;*    @deffn with-output-to-port@ ...                                  */
 ;*---------------------------------------------------------------------*/
 (define (with-output-to-port port thunk)
    (let ((old-output-port (current-output-port)))
@@ -243,7 +339,43 @@
 	 (set! c-current-output-port old-output-port))))
 
 ;*---------------------------------------------------------------------*/
-;*    with-error-to-file ...                                           */
+;*    @deffn with-output-to-string@ ...                                */
+;*---------------------------------------------------------------------*/
+(define (with-output-to-string thunk)
+   (let ((port (open-output-string)))
+      (if (output-port? port)
+	  (let ((old-output-port (current-output-port))
+		(res #unspecified))
+	     (unwind-protect
+		(begin
+		   (set! c-current-output-port port)
+		   (thunk))
+		(begin
+		   (set! c-current-output-port old-output-port)
+		   (set! res (close-output-port port))))
+	     res)
+	  (error "with-output-to-string" "can't open string" #unspecified))))
+
+;*---------------------------------------------------------------------*/
+;*    @deffn with-error-to-string@ ...                                 */
+;*---------------------------------------------------------------------*/
+(define (with-error-to-string thunk)
+   (let ((port (open-output-string)))
+      (if (output-port? port)
+	  (let ((old-error-port (current-error-port))
+		(res #unspecified))
+	     (unwind-protect
+		(begin
+		   (set! c-current-error-port port)
+		   (thunk))
+		(begin
+		   (set! c-current-error-port old-error-port)
+		   (set! res (close-output-port port))))
+	     res)
+	  (error "with-error-to-string" "can't open string" #unspecified))))
+
+;*---------------------------------------------------------------------*/
+;*    @deffn with-error-to-file@ ...                                   */
 ;*---------------------------------------------------------------------*/
 (define (with-error-to-file string thunk)
    (let ((port (open-output-file string)))
@@ -271,6 +403,23 @@
 
 ;*---------------------------------------------------------------------*/
 ;*    open-input-file ...                                              */
+;*    -------------------------------------------------------------    */
+;*    This new version of open-input-file accept extended syntax.      */
+;*    It may open plain file, strings, pipes, http/ftp files. The      */
+;*    syntax is inspired by Web browsers, e.g.:                        */
+;*        "/etc/passwd"                                                */
+;*        "file:/etc/passwd"                                           */
+;*        "string:bozo"                                                */
+;*        "pipe:ls -lR /"                                              */
+;*        "http://kaolin.unice.fr/Bigloo"                              */
+;*        "http://localhost:10000/"                                    */
+;*                                                                     */
+;*    it also accepts the former form for pipes:                       */
+;*        "| ls -lR /"                                                 */
+;*    -------------------------------------------------------------    */
+;*    Implementation note: remember that STRING-LENGTH is a O-macro    */
+;*    thus, (STRING-LENGTH <a-constant-string>) is replaced with the   */
+;*    actual length of the constant.                                   */
 ;*---------------------------------------------------------------------*/
 (define (open-input-file string . bufsiz)
    (let ((size (if (null? bufsiz)
@@ -278,7 +427,83 @@
 		   (car bufsiz))))
       (if (not (integer? size))
 	  (error "open-input-file" "not a number" size)
-	  (c-open-input-file string size))))
+	  (cond
+	     ((substring=? string "file:" (string-length "file:"))
+	      ;; an plain file
+	      (c-open-input-file (substring string
+					    (string-length "file:")
+					    (string-length string))
+				 size))
+	     ((substring=? string "| " 2)
+	      ;; a former pipe
+	      (c-open-input-pipe (substring string
+					    2
+					    (string-length string))
+				 size))
+	     ((substring=? string "pipe:" (string-length "pipe:"))
+	      ;; a new pipe notation
+	      (c-open-input-pipe (substring string
+					    (string-length "pipe:")
+					    (string-length string))
+				 size))
+	     ((substring=? string "http:" (string-length "http:"))
+	      (open-input-socket (substring string
+					    (string-length "http:")
+					    (string-length string))))
+	     ;; an http file
+	     ((substring=? string "ftp:" (string-length "ftp:"))
+	      ;; an ftp file
+	      (open-input-socket (substring string
+					    (string-length "ftp:")
+					    (string-length string))))
+	     ((substring=? string "string:" (string-length "string:"))
+	      ;; a string port
+	      (c-open-input-string (substring string
+					      (string-length "string:")
+					      (string-length string))))
+	     (else
+	      (c-open-input-file string size))))))
+
+;*---------------------------------------------------------------------*/
+;*    open-input-socket ...                                            */
+;*---------------------------------------------------------------------*/
+(define (open-input-socket string)
+   (define (socket->port server port path)
+      ;; given a server name, a port number and a file name, opens
+      ;; a socket connection to that server and returns the INPUT-PORT
+      ;; associated to that socket
+      (let ((s (make-client-socket server 80)))
+	 (fprint (socket-output s) "GET " path)
+	 (flush-output-port (socket-output s))
+	 (socket-input s)))
+   ;; find the port name (and the optional port number),
+   (if (not (substring=? string "//" 2))
+       #f
+       (let ((len (string-length string)))
+	  (let loop ((r 2)
+		     (pstart #f))
+	     (cond
+		((>=fx r len)
+		 #f)
+		((char=? (string-ref string r) #\/)
+		 ;; we have the name of the server 
+		 (if (fixnum? pstart)
+		     (let ((server (substring string 2 pstart))
+			   (port (string->integer (substring string
+							     pstart
+							     r)))
+			   (file (substring string (+fx r 1) len)))
+			(socket->port server port file))
+		     (let ((server (substring string 2 r))
+			   (port 80)
+			   (file (substring string (+fx r 1) len)))
+			(socket->port server port file))))
+		((char=? (string-ref string r) #\:)
+		 ;; we have the name of the server, we have to decode
+		 ;; the port number
+		 (loop (+fx r 1) (+fx r 1)))
+		(else
+		 (loop (+fx r 1) pstart)))))))
 
 ;*---------------------------------------------------------------------*/
 ;*    open-input-string ...                                            */
@@ -292,6 +517,12 @@
 (define-inline (open-input-c-string string)
    (c-open-input-c-string string))
 
+;*---------------------------------------------------------------------*/
+;*    reopen-input-c-string ...                                        */
+;*---------------------------------------------------------------------*/
+(define-inline (reopen-input-c-string port::input-port string)
+   (c-reopen-input-c-string port string))
+
 ;*---------------------------------------------------------------------*/
 ;*    open-output-file ...                                             */
 ;*---------------------------------------------------------------------*/
@@ -304,12 +535,24 @@
 (define-inline (open-output-string)
    (c-open-output-string))
 
+;*---------------------------------------------------------------------*/
+;*    closed-input-port? ...                                           */
+;*---------------------------------------------------------------------*/
+(define-inline (closed-input-port? port)
+   (c-closed-input-port? port))
+
 ;*---------------------------------------------------------------------*/
 ;*    close-input-port ...                                             */
 ;*---------------------------------------------------------------------*/
 (define-inline (close-input-port port)
    (c-close-input-port port))
 
+;*---------------------------------------------------------------------*/
+;*    get-output-string ...                                            */
+;*---------------------------------------------------------------------*/
+(define-inline (get-output-string port)
+   (c-get-output-string port))
+
 ;*---------------------------------------------------------------------*/
 ;*    close-output-port ...                                            */
 ;*---------------------------------------------------------------------*/
@@ -317,7 +560,7 @@
    (c-close-output-port port))
 
 ;*---------------------------------------------------------------------*/
-;*    Les fonctions qui suivent ne sont pas r4                         */
+;*    Non R5Rs functions.                                              */
 ;*---------------------------------------------------------------------*/
 ;*---------------------------------------------------------------------*/
 ;*    flush-output-port ...                                            */
@@ -392,3 +635,21 @@
 ;*---------------------------------------------------------------------*/
 (define-inline (directory->list string)
    (__directory->list string))
+
+;*---------------------------------------------------------------------*/
+;*    @deffn file-modification-time@ ...                               */
+;*---------------------------------------------------------------------*/
+(define (file-modification-time file)
+   (let ((mtime (c-modification-time file)))
+      (if (<fx mtime 0)
+          (error "file-modification-time" "Can't find file info" file)
+          mtime)))
+
+;*---------------------------------------------------------------------*/
+;*    @deffn file-size@ ...                                            */
+;*---------------------------------------------------------------------*/
+(define (file-size file)
+   (let ((size (c-file-size file)))
+      (if (<fx size 0)
+          (error "file-size" "Can't find file info" file)
+          size)))
