@@ -1,41 +1,9 @@
-;*---------------------------------------------------------------------*/
-;*   A pratical implementation for the Scheme programming language     */
-;*                                                                     */
-;*                                    ,--^,                            */
-;*                              _ ___/ /|/                             */
-;*                          ,;'( )__, ) '                              */
-;*                         ;;  //   L__.                               */
-;*                         '   \\   /  '                               */
-;*                              ^   ^                                  */
-;*                                                                     */
-;*   Copyright (c) 1992-1999 Manuel Serrano                            */
-;*                                                                     */
-;*     Bug descriptions, use reports, comments or suggestions are      */
-;*     welcome. Send them to                                           */
-;*       bigloo-request@kaolin.unice.fr                                */
-;*       http://kaolin.unice.fr/bigloo                                 */
-;*                                                                     */
-;*   This program is free software; you can redistribute it            */
-;*   and/or modify it under the terms of the GNU General Public        */
-;*   License as published by the Free Software Foundation; either      */
-;*   version 2 of the License, or (at your option) any later version.  */
-;*                                                                     */
-;*   This program is distributed in the hope that it will be useful,   */
-;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
-;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
-;*   GNU General Public License for more details.                      */
-;*                                                                     */
-;*   You should have received a copy of the GNU General Public         */
-;*   License along with this program; if not, write to the Free        */
-;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
-;*   MA 02111-1307, USA.                                               */
-;*---------------------------------------------------------------------*/
 ;*=====================================================================*/
 ;*    serrano/prgm/project/bigloo/runtime/Ieee/string.scm              */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Mon Mar 20 19:17:18 1995                          */
-;*    Last change :  Sun Aug 16 09:39:00 1998 (serrano)                */
+;*    Last change :  Wed Jul  4 15:19:41 2001 (serrano)                */
 ;*    -------------------------------------------------------------    */
 ;*    6.7. Strings (page 25, r4)                                       */
 ;*    -------------------------------------------------------------    */
@@ -48,118 +16,179 @@
 ;*    The module                                                       */
 ;*---------------------------------------------------------------------*/
 (module __r4_strings_6_7
-
-   (import  (__error                   "Llib/error.scm"))
    
-   (use     (__type                    "Llib/type.scm")
-	    (__bigloo                  "Llib/bigloo.scm")
-	    (__tvector                 "Llib/tvector.scm")
-	    (__r4_numbers_6_5_fixnum   "Ieee/fixnum.scm")
-	    (__r4_equivalence_6_2      "Ieee/equiv.scm")
-	    (__r4_vectors_6_8          "Ieee/vector.scm")
-	    (__r4_booleans_6_1         "Ieee/boolean.scm")
-	    (__r4_characters_6_6       "Ieee/char.scm")
-	    (__r4_symbols_6_4          "Ieee/symbol.scm")
-	    (__r4_pairs_and_lists_6_3  "Ieee/pair-list.scm")
-
-	    (__evenv                   "Eval/evenv.scm"))
-    
-   (foreign (macro bool c-string?       (obj)              "STRINGP")
-	    (bstring    c-make-string   (long uchar)       "make_string")
-	    
-	    (macro long c-string-length (bstring)          "STRING_LENGTH")
-	    
-	    (macro uchar c-string-ref  (bstring long)      "STRING_REF")
-	    (macro obj  c-string-set! (bstring long uchar) "STRING_SET")
-	    
-	    (bool    c-string=?    (bstring bstring)      "bigloo_strcmp")
-	    (bool    c-substring=? (bstring bstring long) "bigloo_strncmp")
-	    (bool    c-substring-ci=? (bstring bstring long) "bigloo_strncmp_ci")
-	    (bool    strcicmp      (bstring bstring)      "strcicmp")
-	    (bool    string_lt     (bstring bstring)      "string_lt")
-	    (bool    string_le     (bstring bstring)      "string_le")
-	    (bool    string_gt     (bstring bstring)      "string_gt")
-	    (bool    string_ge     (bstring bstring)      "string_ge")
-	    (bool    string_cilt   (bstring bstring)      "string_cilt")
-	    (bool    string_cile   (bstring bstring)      "string_cile")
-	    (bool    string_cigt   (bstring bstring)      "string_cigt")
-	    (bool    string_cige   (bstring bstring)      "string_cige")
-	    
-	    (bstring c-string-copy (string)            "string_to_bstring")
-	    (bstring c-substring (bstring long long)   "c_substring")
-	    (bstring c-string-append (bstring bstring) "string_append")
-	    
-	    (bstring escape-C-string (string)          "escape_C_string")
-	    (bstring escape-scheme-string (string)     "escape_scheme_string")
-	    (bstring c-constant-string-to-string (string)
-		     "c_constant_string_to_string")
-	    
-	    (macro bool string-bound-check? (long long) "BOUND_CHECK")
-	    (bstring c-string-for-read (bstring)        "string_for_read")
-	    (obj   c-blit-string (bstring bint bstring bint bint)
-		   "blit_string"))
+   (import  __error)
    
-   (export  (inline string?::bool               ::obj)
-	    (inline make-string::bstring        ::int . char)
-	    (inline string::bstring             . chars)
-	    (inline string-length::long         ::bstring)
-	    (inline string-ref::uchar           ::bstring ::long)
-	    (inline string-set!::obj            ::bstring ::long ::uchar)
-	    (inline string-ref-ur::uchar        ::bstring ::long)
-	    (inline string-set-ur!::obj         ::bstring ::long ::uchar)
-	    (inline string=?::bool              ::bstring ::bstring)
-	    (inline substring=?::bool           ::bstring ::bstring ::long)
-	    (inline string-ci=?::bool           ::bstring ::bstring)
-	    (inline substring-ci=?::bool        ::bstring ::bstring ::long)
-	    (inline string<?::bool              ::bstring ::bstring)
-	    (inline string>?::bool              ::bstring ::bstring)
-	    (inline string<=?::bool             ::bstring ::bstring)
-	    (inline string>=?::bool             ::bstring ::bstring)
-	    (inline string-ci<?::bool           ::bstring ::bstring)
-	    (inline string-ci>?::bool           ::bstring ::bstring)
-	    (inline string-ci<=?::bool          ::bstring ::bstring)
-	    (inline string-ci>=?::bool          ::bstring ::bstring)
-	    (inline substring::bstring          ::bstring ::long ::long)
-	    (inline substring-ur::bstring       ::bstring ::long ::long)
-	    (string-append::bstring             . strings)
-	    (string->list                       ::bstring)
-	    (list->string::bstring              ::obj)
-	    (inline string-copy::bstring        ::bstring)
-	    (string-fill!::bstring              ::bstring ::uchar)
-	    (string-upcase::bstring             ::bstring)
-	    (string-downcase::bstring           ::bstring)
-	    (string-upcase!::bstring            ::bstring)
-	    (string-downcase!::bstring          ::bstring)
-	    (string-capitalize!::bstring        ::bstring)
-	    (string-capitalize::bstring         ::bstring)
-	    (inline string-for-read::bstring    ::bstring)
-	    (inline blit-string-ur!             ::bstring ::bint
-		    ::bstring ::bint
-		    ::bint)
-	    (blit-string!                       ::bstring ::bint
-						  ::bstring ::bint
-						  ::bint))
-    
-   (pragma  (c-string? (predicate-of bstring) no-cfa-top)
-	    (string? side-effect-free no-cfa-top)
-	    (c-string-ref side-effect-free no-cfa-top)
-	    (string-ref-ur side-effect-free no-cfa-top)
-	    (string-ref side-effect-free no-cfa-top)
-	    (c-string-length side-effect-free no-cfa-top)
-	    (string-length side-effect-free no-cfa-top)
-	    (string-bound-check? side-effect-free no-cfa-top)
-	    (string=? side-effect-free)
-	    (substring=? side-effect-free)
-	    (substring-ci=? side-effect-free)
-	    (string-ci=? side-effect-free)
-	    (string<? side-effect-free)
-	    (string>? side-effect-free)
-	    (string<=? side-effect-free)
-	    (string>=? side-effect-free)
-	    (string-ci<? side-effect-free)
-	    (string-ci>? side-effect-free)
-	    (string-ci<=? side-effect-free)
-	    (string-ci>=? side-effect-free)))
+   (use     __type
+	    __bigloo
+	    __tvector
+	    __r4_numbers_6_5_fixnum
+	    __r4_equivalence_6_2
+	    __r4_vectors_6_8
+	    __r4_booleans_6_1
+	    __r4_characters_6_6
+	    __r4_symbols_6_4
+	    __r4_pairs_and_lists_6_3
+	    
+	    __evenv)
+   
+   (extern  (macro c-string?::bool (::obj) "STRINGP")
+	    (c-make-string::bstring (::int ::uchar) "make_string")
+	    (c-make-string/wo-fill::bstring (::int) "make_string_sans_fill")
+	    
+	    (macro c-string-length::int (::bstring) "STRING_LENGTH")
+	    (macro c-string-ref::uchar (::bstring ::int) "STRING_REF")
+	    (macro c-string-set!::obj (::bstring ::int ::uchar) "STRING_SET")
+	    
+	    (c-string=?::bool (::bstring ::bstring) "bigloo_strcmp")
+	    (c-substring=?::bool (::bstring ::bstring ::int) "bigloo_strncmp")
+	    (c-substring-ci=?::bool (::bstring ::bstring ::int) "bigloo_strncmp_ci")
+	    (strcicmp::bool (::bstring ::bstring) "strcicmp")
+	    (string_lt::bool (::bstring ::bstring) "string_lt")
+	    (string_le::bool (::bstring ::bstring) "string_le")
+	    (string_gt::bool (::bstring ::bstring) "string_gt")
+	    (string_ge::bool (::bstring ::bstring) "string_ge")
+	    (string_cilt::bool (::bstring ::bstring) "string_cilt")
+	    (string_cile::bool (::bstring ::bstring) "string_cile")
+	    (string_cigt::bool (::bstring ::bstring) "string_cigt")
+	    (string_cige::bool (::bstring ::bstring) "string_cige")
+	    
+	    (c-substring::bstring (::bstring ::int ::int) "c_substring")
+	    (c-string-append::bstring (::bstring ::bstring) "string_append")
+	    (c-string-append-3::bstring (::bstring ::bstring ::bstring)
+					"string_append_3")
+	    
+	    (escape-C-string::bstring (::string) "escape_C_string")
+	    (escape-scheme-string::bstring (::string) "escape_scheme_string")
+	    (c-constant-string-to-string::bstring (::string)
+						  "c_constant_string_to_string")
+	    
+	    (macro  string-bound-check?::bool (::int ::int) "BOUND_CHECK")
+	    (c-string-for-read::bstring (::bstring) "string_for_read")
+	    (c-blit-string::obj (::bstring ::int ::bstring ::int ::int)
+				"blit_string"))
+   
+   (java    (class foreign
+	       (method static c-string?::bool (::obj)
+		       "STRINGP")
+	       (method static c-make-string::bstring (::int ::uchar)
+		       "make_string")
+	       
+	       (method static c-string-length::int (::bstring)
+		       "STRING_LENGTH")
+	       
+	       (method static c-string-ref::uchar (::bstring ::int)
+		       "STRING_REF")
+	       (method static c-string-set!::obj (::bstring ::int ::uchar)
+		       "STRING_SET")
+	       
+	       (method static c-string=?::bool (::bstring ::bstring)
+		       "bigloo_strcmp")
+	       (method static c-substring=?::bool (::bstring ::bstring ::int)
+		       "bigloo_strncmp")
+	       (method static c-substring-ci=?::bool (::bstring ::bstring ::int)
+		       "bigloo_strncmp_ci")
+	       (method static strcicmp::bool (::bstring ::bstring)
+		       "strcicmp")
+	       (method static string_lt::bool (::bstring ::bstring)
+		       "string_lt")
+	       (method static string_le::bool (::bstring ::bstring)
+		       "string_le")
+	       (method static string_gt::bool (::bstring ::bstring)
+		       "string_gt")
+	       (method static string_ge::bool (::bstring ::bstring)
+		       "string_ge")
+	       (method static string_cilt::bool (::bstring ::bstring)
+		       "string_cilt")
+	       (method static string_cile::bool (::bstring ::bstring)
+		       "string_cile")
+	       (method static string_cigt::bool (::bstring ::bstring)
+		       "string_cigt")
+	       (method static string_cige::bool (::bstring ::bstring)
+		       "string_cige")
+	       
+	       (method static c-substring::bstring (::bstring ::int ::int)
+		       "c_substring")
+	       (method static c-string-append::bstring (::bstring ::bstring)
+		       "string_append")
+	       (method static c-string-append-3::bstring (::bstring ::bstring ::bstring)
+		       "string_append_3")
+	       
+	       (method static escape-C-string::bstring (::string)
+		       "escape_C_string")
+	       (method static escape-scheme-string::bstring (::string)
+		       "escape_scheme_string")
+	       (method static c-constant-string-to-string::bstring (::string)
+		       "c_constant_string_to_string")
+	       
+	       (method static  string-bound-check?::bool (::int ::int)
+		       "BOUND_CHECK")
+	       (method static c-string-for-read::bstring (::bstring)
+		       "string_for_read")
+	       (method static c-blit-string::obj (::bstring ::int ::bstring ::int ::int)
+		       "blit_string")
+	       (method static c-make-string/wo-fill::bstring (::int)
+		       "make_string_sans_fill")))
+   
+   (export  (inline string?::bool ::obj)
+	    (inline make-string::bstring ::int . char)
+	    (inline string::bstring . chars)
+	    (inline string-length::int ::bstring)
+	    (inline string-ref::uchar ::bstring ::int)
+	    (inline string-set!::obj ::bstring ::int ::uchar)
+	    (inline string-ref-ur::uchar ::bstring ::int)
+	    (inline string-set-ur!::obj ::bstring ::int ::uchar)
+	    (inline string=?::bool ::bstring ::bstring)
+	    (inline substring=?::bool ::bstring ::bstring ::int)
+	    (inline string-ci=?::bool ::bstring ::bstring)
+	    (inline substring-ci=?::bool ::bstring ::bstring ::int)
+	    (inline empty-string?::bool ::bstring)
+	    (inline string<?::bool ::bstring ::bstring)
+	    (inline string>?::bool ::bstring ::bstring)
+	    (inline string<=?::bool ::bstring ::bstring)
+	    (inline string>=?::bool ::bstring ::bstring)
+	    (inline string-ci<?::bool ::bstring ::bstring)
+	    (inline string-ci>?::bool ::bstring ::bstring)
+	    (inline string-ci<=?::bool ::bstring ::bstring)
+	    (inline string-ci>=?::bool ::bstring ::bstring)
+	    (inline substring::bstring ::bstring ::int ::int)
+	    (inline substring-ur::bstring ::bstring ::int ::int)
+	    (string-append::bstring . strings)
+	    (string->list ::bstring)
+	    (list->string::bstring ::obj)
+	    (string-copy::bstring ::bstring)
+	    (string-fill!::obj ::bstring ::uchar)
+	    (string-upcase::bstring ::bstring)
+	    (string-downcase::bstring ::bstring)
+	    (string-upcase!::bstring ::bstring)
+	    (string-downcase!::bstring ::bstring)
+	    (string-capitalize!::bstring ::bstring)
+	    (string-capitalize::bstring ::bstring)
+	    (inline string-for-read::bstring ::bstring)
+	    (inline blit-string-ur! ::bstring ::int ::bstring ::int ::int)
+	    (blit-string! ::bstring ::int ::bstring ::int ::int))
+   
+   (pragma  (c-string? (predicate-of bstring) no-cfa-top nesting)
+	    (string? side-effect-free no-cfa-top nesting)
+	    (c-string-ref side-effect-free no-cfa-top nesting)
+	    (string-ref-ur side-effect-free no-cfa-top nesting)
+	    (string-ref side-effect-free no-cfa-top nesting)
+	    (c-string-length side-effect-free no-cfa-top nesting)
+	    (string-length side-effect-free no-cfa-top nesting)
+	    (string-bound-check? side-effect-free no-cfa-top nesting)
+	    (string=? side-effect-free nesting)
+	    (substring=? side-effect-free nesting)
+	    (substring-ci=? side-effect-free nesting)
+	    (string-ci=? side-effect-free nesting)
+	    (string<? side-effect-free nesting)
+	    (string>? side-effect-free nesting)
+	    (string<=? side-effect-free nesting)
+	    (string>=? side-effect-free nesting)
+	    (string-ci<? side-effect-free nesting)
+	    (string-ci>? side-effect-free nesting)
+	    (string-ci<=? side-effect-free nesting)
+	    (string-ci>=? side-effect-free nesting)
+	    (empty-string? side-effect-free nesting)))
  
 ;*---------------------------------------------------------------------*/
 ;*    @deffn string?@ ...                                              */
@@ -233,6 +262,12 @@
 (define-inline (substring-ci=? string1 string2 len)
    (c-substring-ci=? string1 string2 len))
 
+;*---------------------------------------------------------------------*/
+;*    @deffn empty-string?@ ...                                        */
+;*---------------------------------------------------------------------*/
+(define-inline (empty-string? string)
+   (=fx (string-length string) 0))
+
 ;*---------------------------------------------------------------------*/
 ;*    @deffn string-ci=?@ ...                                          */
 ;*---------------------------------------------------------------------*/
@@ -308,15 +343,31 @@
 
 ;*---------------------------------------------------------------------*/
 ;*    @deffn string-append@ ...                                        */
+;*    -------------------------------------------------------------    */
+;*    To avoid allocating N strings where N is the size of the list    */
+;*    of string, we precompute the global string size and we fill      */
+;*    it.                                                              */
 ;*---------------------------------------------------------------------*/
 (define (string-append . list)
    (if (null? list)
        ""
-       (let loop ((list list))
-	  (if (null? (cdr list))
-	      (car list)
-	      (c-string-append (car list) (loop (cdr list)))))))
-
+       (let* ((len (let string-append ((list list)
+				       (res  0))
+		      (if (null? list)
+			  res
+			  (string-append (cdr list)
+					 (+fx res
+					      (string-length (car list)))))))
+	      (res (c-make-string/wo-fill len)))
+	  (let string-append ((list list)
+			      (w    0))
+	     (if (null? list)
+		 res
+		 (let* ((s (car list))
+			(l (string-length s)))
+		    (blit-string-ur! s 0 res w l)
+		    (string-append (cdr list) (+fx w l))))))))
+ 
 ;*---------------------------------------------------------------------*/
 ;*    @deffn list->string@ ...                                         */
 ;*---------------------------------------------------------------------*/
@@ -346,8 +397,15 @@
 ;*---------------------------------------------------------------------*/
 ;*    @deffn string-copy@ ...                                          */
 ;*---------------------------------------------------------------------*/
-(define-inline (string-copy string)
-   (c-string-copy string))
+(define (string-copy string)
+   (let* ((len (string-length string))
+	  (new (make-string len)))
+      (let loop ((i (-fx len 1)))
+	 (if (=fx i -1)
+	     new
+	     (begin
+		(string-set! new i (string-ref string i))
+		(loop (-fx i 1)))))))
 
 ;*---------------------------------------------------------------------*/
 ;*    @deffn string-fill!@ ...                                         */
@@ -356,7 +414,7 @@
    (let ((len (string-length string)))
       (let loop ((i 0))
 	 (if (=fx i len)
-	     string
+	     #unspecified
 	     (begin
 		(string-set! string i char)
 		(loop (+fx i 1)))))))
@@ -391,26 +449,24 @@
 ;*    @deffn string-upcase!@ ...                                       */
 ;*---------------------------------------------------------------------*/
 (define (string-upcase! string)
-   (let* ((len (string-length string))
-	  (res string))
+   (let ((len (string-length string)))
       (let loop ((i 0))
 	 (if (=fx i len)
-	     res
+	     string
 	     (begin
-		(string-set! res i (char-upcase (string-ref string i)))
+		(string-set! string i (char-upcase (string-ref string i)))
 		(loop (+fx i 1)))))))
 
 ;*---------------------------------------------------------------------*/
 ;*    @deffn string-downcase!@ ...                                     */
 ;*---------------------------------------------------------------------*/
 (define (string-downcase! string)
-   (let* ((len (string-length string))
-	  (res string))
+   (let ((len (string-length string)))
       (let loop ((i 0))
 	 (if (=fx i len)
-	     res
+	     string
 	     (begin
-		(string-set! res i (char-downcase (string-ref string i)))
+		(string-set! string i (char-downcase (string-ref string i)))
 		(loop (+fx i 1)))))))
 
 ;*---------------------------------------------------------------------*/
@@ -456,9 +512,10 @@
 ;*    @deffn blit-string!@ ...                                         */
 ;*---------------------------------------------------------------------*/
 (define (blit-string! s1 o1 s2 o2 l)
-  (if (and (string-bound-check? (+fx l o1) (+fx (string-length s1) 1))
-	   (string-bound-check? (+fx l o2) (+fx (string-length s2) 1)))
-      (c-blit-string s1 o1 s2 o2 l)
-      (error "blit-string!" "Index and length out of range" 
-	     (list (string-length s1) o1 (string-length s2) o2 l))))
+   (if (and (string-bound-check? (+fx l o1) (+fx (string-length s1) 1))
+	    (string-bound-check? (+fx l o2) (+fx (string-length s2) 1)))
+       (c-blit-string s1 o1 s2 o2 l)
+       (error "blit-string!:Index and length out of range"
+	      (string-append "[src:" s1 "] [dest:" s2 "]")
+	      (list (string-length s1) o1 (string-length s2) o2 l))))
 
