@@ -1,35 +1,3 @@
-;*---------------------------------------------------------------------*/
-;*   A pratical implementation for the Scheme programming language     */
-;*                                                                     */
-;*                                    ,--^,                            */
-;*                              _ ___/ /|/                             */
-;*                          ,;'( )__, ) '                              */
-;*                         ;;  //   L__.                               */
-;*                         '   \\   /  '                               */
-;*                              ^   ^                                  */
-;*                                                                     */
-;*   Copyright (c) 1992-1999 Manuel Serrano                            */
-;*                                                                     */
-;*     Bug descriptions, use reports, comments or suggestions are      */
-;*     welcome. Send them to                                           */
-;*       bigloo-request@kaolin.unice.fr                                */
-;*       http://kaolin.unice.fr/bigloo                                 */
-;*                                                                     */
-;*   This program is free software; you can redistribute it            */
-;*   and/or modify it under the terms of the GNU General Public        */
-;*   License as published by the Free Software Foundation; either      */
-;*   version 2 of the License, or (at your option) any later version.  */
-;*                                                                     */
-;*   This program is distributed in the hope that it will be useful,   */
-;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
-;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
-;*   GNU General Public License for more details.                      */
-;*                                                                     */
-;*   You should have received a copy of the GNU General Public         */
-;*   License along with this program; if not, write to the Free        */
-;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
-;*   MA 02111-1307, USA.                                               */
-;*---------------------------------------------------------------------*/
 ;; ---------------------------------------------------------------------- ;;
 ;; FICHIER               : lalr.scm                                       ;;
 ;; DATE DE CREATION      : Thu Jun 29 15:07:55 1995                       ;;
@@ -44,35 +12,36 @@
    
    (include "Lalr/lalr.sch")
 
-   (import (__error                   "Llib/error.scm")
-	   (__lalr_util               "Lalr/util.scm")
-	   (__lalr_gen                "Lalr/gen.scm")
-	   (__lalr_global             "Lalr/global.scm")
-	   (__lalr_rewrite            "Lalr/rewrite.scm"))
+   (import __error
+	   __lalr_util
+	   __lalr_gen
+	   __lalr_global
+	   __lalr_rewrite
 		
-   (import (__type                    "Llib/type.scm")
+	   __type
 		
-	   (__bigloo                  "Llib/bigloo.scm")
-	   (__tvector                 "Llib/tvector.scm")
-	   (__structure               "Llib/struct.scm")
-	   (__tvector                 "Llib/tvector.scm")
+	   __bigloo
+	   __tvector
+	   __structure
+	   __tvector
+	   __bexit
 
-	   (__r4_numbers_6_5          "Ieee/number.scm")
-	   (__r4_numbers_6_5_fixnum   "Ieee/fixnum.scm")
-	   (__r4_numbers_6_5_flonum   "Ieee/flonum.scm")
-	   (__r4_characters_6_6       "Ieee/char.scm")
-	   (__r4_equivalence_6_2      "Ieee/equiv.scm")
-	   (__r4_booleans_6_1         "Ieee/boolean.scm")
-	   (__r4_symbols_6_4          "Ieee/symbol.scm")
-	   (__r4_strings_6_7          "Ieee/string.scm")
-	   (__r4_pairs_and_lists_6_3  "Ieee/pair-list.scm")
-	   (__r4_input_6_10_2         "Ieee/input.scm")
-	   (__r4_control_features_6_9 "Ieee/control.scm")
-	   (__r4_vectors_6_8          "Ieee/vector.scm")
-	   (__r4_ports_6_10_1         "Ieee/port.scm")
-	   (__r4_output_6_10_3        "Ieee/output.scm")
-	   (__bit                     "Llib/bit.scm")
-	   (__evenv                   "Eval/evenv.scm"))
+	   __r4_numbers_6_5
+	   __r4_numbers_6_5_fixnum
+	   __r4_numbers_6_5_flonum
+	   __r4_characters_6_6
+	   __r4_equivalence_6_2
+	   __r4_booleans_6_1
+	   __r4_symbols_6_4
+	   __r4_strings_6_7
+	   __r4_pairs_and_lists_6_3
+	   __r4_input_6_10_2
+	   __r4_control_features_6_9
+	   __r4_vectors_6_8
+	   __r4_ports_6_10_1
+	   __r4_output_6_10_3
+	   __bit
+	   __evenv)
 
    (export (expand-lalr-grammar x e)))
 
@@ -81,23 +50,29 @@
 ;*---------------------------------------------------------------------*/
 (define *default* (gensym 'default))
 
+
 ;; ---------------------------------------------------------------------- ;;
 ;; C'est le point d'entre ...                                            ;;
 ;; ---------------------------------------------------------------------- ;;
 (define (expand-lalr-grammar x e)
    (match-case x
-      ((?- . (and (? pair?) ?rules))
-       (initialize-all)
-       (rewrite-grammar! rules)
-       (pack-grammar)
-       (set-derives)
-       (set-nullable)
-       (generate-states)
-       (lalr)
-       (build-tables)
-       (compact-action-table)
-       (let ((code (gen-lalr-code)))
-	  (clean-plist)
+      ((?- . ?rules)
+       (match-case rules
+	  (((and (? list?) ?tokens) . (and (? list?) ?rules))
+	   (check-lalr-rules x tokens rules)))
+       (let ((code (unwind-protect
+		      (begin
+			 (initialize-all)
+			 (rewrite-grammar! rules)
+			 (pack-grammar)
+			 (set-derives)
+			 (set-nullable)
+			 (generate-states)
+			 (lalr)
+			 (build-tables)
+			 (compact-action-table)
+			 (gen-lalr-code))
+		      (clean-plist))))
 	  (e code e)))
       (else
        (error "lalr-grammar" "Illegal form" x))))
@@ -131,6 +106,34 @@
 			  (vector-set! ritem it-no3 sym-no)
 			  (loop3 (cdr rhs) (+fx it-no3 1))))))))))))
 
+;; ---------------------------------------------------------------------- ;;
+;; On verifie la syntaxe des regles
+;; ---------------------------------------------------------------------- ;;
+(define (check-lalr-rules x tokens rules)
+   (define (check-lalr-rule rule)
+      (match-case rule
+	 (((? symbol?) . (and (? list?) ?subrules))
+	  (for-each (lambda (sr)
+		       (match-case sr
+			  (((and ?sr (? list?)) . ?-)
+			   (for-each (lambda (sym)
+					(if (not (symbol? sym))
+					    (error "lalr-grammar"
+						   "Illegal form "
+						   rule)))
+				     sr))
+			  (else
+			   (error "lalr-grammar"
+				  "Illegal form (Illegal left hand side)"
+				  rule))))
+		    subrules))
+	 (else
+	  (error "lalr-grammar" "Illegal form" x))))
+   (for-each (lambda (t)
+		(if (not (symbol? t))
+		    (error "lalr-grammar" "Illegal token" tokens)))
+	     tokens)
+   (for-each check-lalr-rule rules))
 
 ;; ---------------------------------------------------------------------- ;;
 ;; Les autres fonctions sont tires du code de Bison ...                  ;;
@@ -851,7 +854,7 @@
 
 (define (build-tables)
   (define (add-action St Sym Act)
-     (let* ((x (vector-ref ACTION-TABLE St))
+     (let* ((x (vector-ref action-table St))
 	    (y (assv Sym x)))
 	(if y
 	    (if (not (= Act (cdr y)))
@@ -872,10 +875,10 @@
 				   "** Shift/Reduce conflict: "
 				   #"\n - shift to state "  Act
 				   #"\n - reduce rule " (build-rule (negfx (cdr y)))
-				   #"\non token `"  (vector-ref *symv* (+fx nvars sym))
+				   #"\non token `"  (vector-ref *symv* (+fx nvars Sym))
 				   "'")
 			  (set-cdr! y Act)))))
-	    (vector-set! ACTION-TABLE St
+	    (vector-set! action-table St
 			 (cons (cons Sym Act) x)))))
 
   (set! action-table (make-vector nstates '()))
