@@ -1,41 +1,9 @@
-;*---------------------------------------------------------------------*/
-;*   A pratical implementation for the Scheme programming language     */
-;*                                                                     */
-;*                                    ,--^,                            */
-;*                              _ ___/ /|/                             */
-;*                          ,;'( )__, ) '                              */
-;*                         ;;  //   L__.                               */
-;*                         '   \\   /  '                               */
-;*                              ^   ^                                  */
-;*                                                                     */
-;*   Copyright (c) 1992-1999 Manuel Serrano                            */
-;*                                                                     */
-;*     Bug descriptions, use reports, comments or suggestions are      */
-;*     welcome. Send them to                                           */
-;*       bigloo-request@kaolin.unice.fr                                */
-;*       http://kaolin.unice.fr/bigloo                                 */
-;*                                                                     */
-;*   This program is free software; you can redistribute it            */
-;*   and/or modify it under the terms of the GNU General Public        */
-;*   License as published by the Free Software Foundation; either      */
-;*   version 2 of the License, or (at your option) any later version.  */
-;*                                                                     */
-;*   This program is distributed in the hope that it will be useful,   */
-;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
-;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
-;*   GNU General Public License for more details.                      */
-;*                                                                     */
-;*   You should have received a copy of the GNU General Public         */
-;*   License along with this program; if not, write to the Free        */
-;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
-;*   MA 02111-1307, USA.                                               */
-;*---------------------------------------------------------------------*/
 ;*=====================================================================*/
 ;*    serrano/prgm/project/bigloo/runtime/Llib/error.scm               */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Fri Jan 20 08:19:23 1995                          */
-;*    Last change :  Wed Feb 10 19:55:52 1999 (serrano)                */
+;*    Last change :  Wed Jul 25 14:56:05 2001 (serrano)                */
 ;*    -------------------------------------------------------------    */
 ;*    The error machinery                                              */
 ;*    -------------------------------------------------------------    */
@@ -49,74 +17,103 @@
 ;*---------------------------------------------------------------------*/
 (module __error
 
-   (use     (__type                    "Llib/type.scm")
-	    (__bigloo                  "Llib/bigloo.scm")
-	    (__os                      "Llib/os.scm")
-	    (__foreign                 "Llib/foreign.scm")
-	    (__binary                  "Llib/binary.scm")
-	    (__structure               "Llib/struct.scm")
-	    (__dsssl                   "Llib/dsssl.scm")
-	    (__object                  "Llib/object.scm")
-	    (__tvector                 "Llib/tvector.scm")
+   (use     __type
+	    __bigloo
+	    __os
+	    __foreign
+	    __binary
+ 	    __structure
+	    __dsssl
+	    __object
+	    __tvector
+	    __socket
+	    __process
+	    __custom
+	    __unicode
+	    __ucs2
 	    
-	    (__reader                  "Read/reader.scm")
-
-	    (__rgc                     "Rgc/rgc.scm")
+	    __pp_circle
 	    
-	    (__r4_vectors_6_8          "Ieee/vector.scm")
-	    (__r4_numbers_6_5_fixnum   "Ieee/fixnum.scm")
-	    (__r4_numbers_6_5_flonum   "Ieee/flonum.scm")
-	    (__r4_equivalence_6_2      "Ieee/equiv.scm")
-	    (__r4_booleans_6_1         "Ieee/boolean.scm")
-	    (__r4_pairs_and_lists_6_3  "Ieee/pair-list.scm")
-	    (__r4_characters_6_6       "Ieee/char.scm")
-	    (__r4_symbols_6_4          "Ieee/symbol.scm")
-	    (__r4_strings_6_7          "Ieee/string.scm")
-	    (__r4_ports_6_10_1         "Ieee/port.scm")
-	    (__r4_control_features_6_9 "Ieee/control.scm")
-	    (__r4_output_6_10_3        "Ieee/output.scm")
-	    (__evenv                   "Eval/evenv.scm"))
+	    __reader
 
-   (import  (__r4_input_6_10_2         "Ieee/input.scm"))
+	    __rgc
+	    
+	    __r4_vectors_6_8
+	    __r4_numbers_6_5_fixnum
+	    __r4_numbers_6_5_flonum
+	    __r4_equivalence_6_2
+	    __r4_booleans_6_1
+	    __r4_pairs_and_lists_6_3
+	    __r4_characters_6_6
+	    __r4_symbols_6_4
+	    __r4_strings_6_7
+	    __r4_ports_6_10_1
+	    __r4_control_features_6_9
+	    __r4_output_6_10_3
+	    
+	    __evenv)
 
-   (foreign (include "signal.h")
+   (import  __r4_input_6_10_2)
+
+   (extern  (include "signal.h")
 	    
 	    (export the_failure "the_failure")
 	    (export the_c_failure "the_c_failure")
 	    
-	    (obj   c-dump-trace-stack (output-port long) "dump_trace_stack")
-	    (macro obj c-push-trace   (obj)              "PUSH_TRACE")
-	    (macro obj c-pop-trace    ()                 "POP_TRACE")
-	    (macro obj c-restore-trace ()                "RESTORE_TRACE")
-	    (macro void c-exit        (long)             "exit")
-	    (macro int sigfpe                            "SIGFPE")
-	    (macro int sigill                            "SIGILL")
-	    (macro int sigbus                            "SIGBUS")
-	    (macro int sigsegv                           "SIGSEGV"))
+	    (c-dump-trace-stack::obj (::output-port ::int) "dump_trace_stack")
+	    (macro c-push-trace::obj (::obj) "PUSH_TRACE")
+	    (macro c-pop-trace::obj () "POP_TRACE")
+	    (macro c-restore-trace::obj () "RESTORE_TRACE")
+	    (macro sigfpe::int "SIGFPE")
+	    (macro sigill::int "SIGILL")
+	    (macro sigbus::int "SIGBUS")
+	    (macro sigsegv::int "SIGSEGV")
+	    (macro c-find-runtime-type::string (::obj) "FOREIGN_TYPE_NAME"))
 
-   (export  (inline exit::magic          ::long)
-	    (inline error::obj           ::obj ::obj ::obj)
-	    (warning                     . args)
-	    (warning/location::obj       ::obj ::obj . obj)
-	    (error/location::obj         ::obj ::obj ::obj ::obj ::obj)
-	    (debug-error/location::obj   ::obj ::obj ::obj ::obj ::obj)
-	    (error/location-file::obj    ::obj ::obj)
-	    (notify-error                ::obj ::obj ::obj)
-	    (add-error-handler!          ::obj ::obj)
+   (java    (class foreign
+	       (method static c-dump-trace-stack::obj (::output-port ::int)
+		       "dump_trace_stack")
+	       (method static c-push-trace::obj (::obj)
+		       "PUSH_TRACE")
+	       (method static c-pop-trace::obj ()
+		       "POP_TRACE")
+	       (method static c-restore-trace::obj ()
+		       "RESTORE_TRACE")
+	       (method static c-find-runtime-type::string (::obj)
+		       "FOREIGN_TYPE_NAME")
+	       (field static sigfpe::int "SIGFPE")
+	       (field static sigill::int "SIGILL")
+	       (field static sigbus::int "SIGBUS")
+	       (field static sigsegv::int "SIGSEGV")))
+
+   (export  (exit::magic . obj)
+	    (inline error::obj ::obj ::obj ::obj)
+	    (warning . args)
+	    (warning/location::obj ::obj ::obj . obj)
+	    (warning/c-location::obj ::string ::long . obj)
+	    (error/location::obj ::obj ::obj ::obj ::obj ::obj)
+	    (error/c-location::obj ::obj ::obj ::obj ::string ::long)
+	    (debug-error/location::obj ::obj ::obj ::obj ::obj ::obj)
+	    (error/location-file::obj ::obj ::obj)
+	    (notify-error ::obj ::obj ::obj)
+	    (notify-interrupt ::int)
+	    (add-error-handler! ::obj ::obj)
 	    (remove-error-handler!)
-	    (the_failure::magic          ::obj ::obj ::obj)
-	    (the_c_failure::magic        ::string ::string ::obj)
+	    (the_failure::magic ::obj ::obj ::obj)
+	    (the_c_failure::magic ::string ::string ::obj)
 	    (find-runtime-type::bstring  ::obj)
 	    *error-notifier*
+	    *interrupt-notifier*
 	    *trace-stack-depth*
 	    *debug*
 	    *warning*
 	    (module-init-error ::string ::string)
 	    (bigloo-type-error-msg::bstring  ::bstring ::bstring ::bstring)
-	    (bigloo-type-error::obj          ::obj ::obj ::obj)
+	    (bigloo-type-error::obj ::obj ::obj ::obj)
 	    (bigloo-type-error/location::obj ::obj ::obj ::obj ::obj ::obj))
 
    (option  (set! *compiler-debug* 0)
+	    (set! *optim-O-macro?* #f)
 	    (set! *unsafe-type*    #t)
 	    (set! *unsafe-arity*   #t)
 	    (set! *unsafe-range*   #t)))
@@ -143,9 +140,16 @@
 ;*---------------------------------------------------------------------*/
 ;*    exit ...                                                         */
 ;*---------------------------------------------------------------------*/
-(define-inline (exit n)
-   (c-exit n)
-   n)
+(define (exit . n)
+   (let ((val (cond
+		 ((null? n)
+		  0)
+		 ((not (fixnum? (car n)))
+		  0)
+		 (else
+		  (car n)))))
+      (bigloo-exit val)
+      val))
 
 ;*---------------------------------------------------------------------*/
 ;*    La valeur par defaut du *error-handler*                          */
@@ -165,25 +169,19 @@
 ;*    add-error-handler! ...                                           */
 ;*---------------------------------------------------------------------*/
 (define (add-error-handler! handler escape)
-   (set! *error-handler* (cons (cons handler escape) *error-handler*))
-;*    (print ">> add-error-handler![" (length *error-handler*) "]: " *error-handler*) */
-   )
+   (set! *error-handler* (cons (cons handler escape) *error-handler*)))
 
 ;*---------------------------------------------------------------------*/
 ;*    remove-error-handler! ...                                        */
 ;*---------------------------------------------------------------------*/
 (define (remove-error-handler!)
-;*    (print "=== avant remove-error-handler![" (length *error-handler*) "]: " */
-;* 	  *error-handler*)                                             */
    (if (pair? *error-handler*)
        (set! *error-handler* (cdr *error-handler*))
        (begin
 	  (notify-error "remove-error-handler! (this is an internal error)"
 			"Can't remove handler"
 			"error handler stack empty")
-	  (bigloo-exit 1)))
-;*    (print "<<< remove-error-handler![" (length *error-handler*) "]: " */
-;* 	  *error-handler*)                                             */
+	  (%exit 1)))
    *error-handler*)
 
 ;*---------------------------------------------------------------------*/
@@ -203,9 +201,10 @@
 	  (display "*** WARNING:bigloo:" (current-error-port))
 	  (if (not (null? args))
 	      (begin
-		 (fprint (current-error-port) (car args))
+		 (display-circle (car args) (current-error-port))
+		 (newline (current-error-port))
 		 (for-each (lambda (a)
-			      (display a (current-error-port)))
+			      (display-circle a (current-error-port)))
 			   (cdr args))))
 	  (newline (current-error-port))
 	  (flush-output-port (current-error-port)))))
@@ -223,6 +222,16 @@
 	  (else
 	   (warning/location-file fname location args)))))
 
+;*---------------------------------------------------------------------*/
+;*    warning/c-location ...                                           */
+;*    -------------------------------------------------------------    */
+;*    This function implements a cast for its first two arguments.     */
+;*    It impliclty casts the fname and location from C values to       */
+;*    Bigloo values.                                                   */
+;*---------------------------------------------------------------------*/
+(define (warning/c-location fname location . args)
+   (apply warning/location (cons fname (cons location args))))
+			    
 ;*---------------------------------------------------------------------*/
 ;*    the_failure ...                                                  */
 ;*---------------------------------------------------------------------*/
@@ -242,24 +251,55 @@
 (define (the_c_failure proc message object)
    (the_failure proc message object))
 
+;*---------------------------------------------------------------------*/
+;*    notify-dump-trace-stack ...                                      */
+;*---------------------------------------------------------------------*/
+(define (notify-dump-trace-stack)
+   (if (or (and (integer? *debug*) (>fx *debug* 0))
+	   (string? (getenv "BIGLOOSTACKDEPTH")))
+       (dump-trace-stack)))
+   
+;*---------------------------------------------------------------------*/
+;*    notify-interrupt ...                                             */
+;*---------------------------------------------------------------------*/
+(define (notify-interrupt sig)
+   ((if (procedure? *interrupt-notifier*)
+ 	*interrupt-notifier*
+ 	default-interrupt-notifier)
+    sig))
+
+;*---------------------------------------------------------------------*/
+;*    *interrupt-notifier* ...                                         */
+;*---------------------------------------------------------------------*/
+(define *interrupt-notifier* #unspecified)
+
+;*---------------------------------------------------------------------*/
+;*    default-interrupt-notifier ...                                   */
+;*---------------------------------------------------------------------*/
+(define (default-interrupt-notifier sig)
+   (let ((port (current-error-port)))
+      (newline port)
+      (fprint port "*** INTERRUPT:bigloo:")
+      (flush-output-port port)))
+
 ;*---------------------------------------------------------------------*/
 ;*    notify-error ...                                                 */
 ;*---------------------------------------------------------------------*/
 (define (notify-error proc mes obj)
    (if (procedure? *error-notifier*)
        (*error-notifier* proc mes obj)
-       (begin
+       (let ((port (current-error-port)))
 	  (flush-output-port (current-output-port))
-	  (newline (current-error-port))
-	  (let ((old-length (get-write-length)))
-	     (set-write-length! 80)
-	     (display "*** ERROR:bigloo:" (current-error-port))
-	     (fprint (current-error-port) proc ":" #\Newline mes " -- " obj)
-	     (if (and (integer? *debug*)
-		      (>fx *debug* 0))
-		 (dump-trace-stack))
-	     (set-write-length! old-length)
-	     (flush-output-port (current-error-port))))))
+	  (newline port)
+	  (display "*** ERROR:bigloo:" port)
+	  (display-circle proc port)
+	  (display #":\n" port)
+	  (display-circle mes port)
+	  (display " -- " port)
+	  (display-circle obj port)
+	  (newline port)
+	  (notify-dump-trace-stack)
+	  (flush-output-port port))))
 
 ;*---------------------------------------------------------------------*/
 ;*    bigloo-type-error-msg ...                                        */
@@ -340,6 +380,16 @@
        (set! *error-notifier* (error/location-file fname location))
        (error proc message object))))
 
+;*---------------------------------------------------------------------*/
+;*    error/c-location ...                                             */
+;*    -------------------------------------------------------------    */
+;*    This function implements a cast for its last two arguments.      */
+;*    It impliclty casts the fname and location from C values to       */
+;*    Bigloo values.                                                   */
+;*---------------------------------------------------------------------*/
+(define (error/c-location proc message object fname location)
+   (error/location proc message object fname location))
+
 ;*---------------------------------------------------------------------*/
 ;*    debug-error/location ...                                         */
 ;*    -------------------------------------------------------------    */
@@ -365,7 +415,8 @@
 	  #f
 	  ;; we readlines until we reach location
 	  (let loop ((line-string (read-line port))
-		     (line-num    1))
+		     (line-num    1)
+		     (old-pos     0))
 	     (if (eof-object? line-string)
 		 (begin
 		    ;; an error we don't know how to print
@@ -373,23 +424,18 @@
 		    (lambda (proc msg obj)
 		       (set! *error-notifier* #f)
 		       (notify-error proc msg obj)))
-		 (if (>=fx (+fx (string-length line-string)
-				(input-port-position port))
-			   location)
+		 (if (>=fx (input-port-position port) location)
 		     (begin
 			(close-input-port port)
 			(make-location-notifier file-name
 						line-num
 						location
 						line-string
-						(+fx
-						 (-fx location
-						      (input-port-position
-						       port))
-						 1)))
+						(-fx location old-pos)))
 		     (let ((old-pos (input-port-position port)))
 			(loop (read-line port) 
-			      (+fx line-num 1)))))))))
+			      (+fx line-num 1)
+			      old-pos))))))))
 
 ;*---------------------------------------------------------------------*/
 ;*    make-location-notifier ...                                       */
@@ -398,28 +444,32 @@
    (lambda (proc msg obj)
       ;; we first re-install default notifier
       (set! *error-notifier* #f)
-      ;; we flush error-port
-      (flush-output-port (current-output-port))
-      (newline (current-error-port))
-      (let ((old-length   (get-write-length))
-	    (space-string (if (>fx marker 0)
-			      (make-string (-fx marker 1) #\space)
-			      "")))
-	 ;; we ajust tabulation in space string.
-	 (fix-tabulation! marker string space-string)
-	 ;; we now print the error message
-	 (print-cursor fname line char string space-string)
-	 ;; we set the write-length before printing the error message.
-	 (set-write-length! 80)
-	 ;; we display the error message
-	 (fprint (current-error-port) "# *** ERROR:bigloo:" proc)
-	 (fprint (current-error-port) "# " msg " -- " obj)
-	 (if (and (integer? *debug*)
-		  (>fx *debug* 0))
-	     (dump-trace-stack))
-	 ;; we restore the current-printer
-	 (set-write-length! old-length)
-	 (flush-output-port (current-error-port)))))
+      (let ((port (current-error-port)))
+	 ;; we flush error-port
+	 (flush-output-port port)
+	 (newline port)
+	 (let ((space-string (if (>fx marker 0)
+				 (make-string (-fx marker 1) #\space)
+				 ""))
+	       (n-marker (if (>=fx marker (string-length string))
+			     (-fx marker 1)
+			     marker)))
+	    ;; we ajust tabulation in space string.
+	    (fix-tabulation! n-marker string space-string)
+	    ;; we now print the error message
+	    (print-cursor fname line char string space-string)
+	    ;; we display the error message
+	    (display "# *** ERROR:bigloo:" port)
+	    (display-circle proc port)
+	    (newline port)
+	    (display "# " port)
+	    (display-circle msg port)
+	    (display " -- " port)
+	    (display-circle obj port)
+	    (newline port)
+	    (notify-dump-trace-stack)
+	    ;; we are now done, we flush
+	    (flush-output-port (current-error-port))))))
 
 ;*---------------------------------------------------------------------*/
 ;*    warning/location-file ...                                        */
@@ -433,30 +483,26 @@
 	  (apply warning args)
 	  ;; we readlines until we reach location
 	  (let loop ((line-string (read-line port))
-		     (line-num    1))
+		     (line-num    1)
+		     (old-pos     0))
 	     (if (eof-object? line-string)
 		 (begin
 		    ;; an error we don't know how to print
 		    (close-input-port port)
 		    (apply warning args))
-		 (if (>=fx (+fx (string-length line-string)
-				(input-port-position port))
-			   location)
+		 (if (>=fx (input-port-position port) location)
 		     (begin
 			(close-input-port port)
 			(do-warn/location file-name
 					  line-num
 					  location
 					  line-string
-					  (+fx
-					   (-fx location
-						(input-port-position
-						 port))
-					   1)
+					  (-fx location old-pos)
 					  args))
 		     (let ((old-pos (input-port-position port)))
 			(loop (read-line port)
-			      (+fx line-num 1)))))))))
+			      (+fx line-num 1)
+			      old-pos))))))))
 
 ;*---------------------------------------------------------------------*/
 ;*    do-warn/location ...                                             */
@@ -467,18 +513,22 @@
    (let ((old-length   (get-write-length))
 	 (space-string (if (>fx marker 0)
 			   (make-string (-fx marker 1) #\space)
-			   "")))
+			   ""))
+	 (n-marker (if (>=fx marker (string-length string))
+		       (-fx marker 1)
+		       marker)))
       ;; we ajust tabulation in space string.
-      (fix-tabulation! marker string space-string)
+      (fix-tabulation! n-marker string space-string)
       ;; we now print the warning message
       (print-cursor fname line char string space-string)
       ;; we display the warning message
       (display "# *** WARNING:bigloo:" (current-error-port))
       (if (not (null? args))
-	  (begin
-	     (fprint (current-error-port) (car args))
+	  (let ((port (current-error-port)))
+	     (display-circle (car args) port)
+	     (newline port)
 	     (for-each (lambda (a)
-			  (display a (current-error-port)))
+			  (display-circle a (current-error-port)))
 		       (cdr args))))
       (newline (current-error-port))
       (flush-output-port (current-error-port))))
@@ -596,43 +646,55 @@
 (define (find-runtime-type obj)
    (cond
       ((fixnum? obj)
-       "BINT")
+       "bint")
       ((flonum? obj)
-       "REAL")
+       "real")
       ((string? obj)
-       "BSTRING")
+       "bstring")
       ((symbol? obj)
-       "SYMBOL")
+       "symbol")
       ((keyword? obj)
-       "KEYWORD")
+       "keyword")
       ((char? obj)
-       "BCHAR")
+       "bchar")
       ((boolean? obj)
-       "BBOOL")
+       "bbool")
       ((null? obj)
-       "BNIL")
+       "bnil")
+      ((epair? obj)
+       "epair")
       ((pair? obj)
-       "PAIR")
+       "pair")
+      ((class? obj)
+       "class")
       ((vector? obj)
-       "VECTOR")
+       "vector")
       ((tvector? obj)
-       "TVECTOR")
+       "tvector")
       ((struct? obj)
-       "STRUCT")
+       "struct")
       ((procedure? obj)
-       "PROCEDURE")
+       "procedure")
       ((input-port? obj)
-       "INPUT-port")
+       "input-port")
       ((output-port? obj)
-       "OUTPUT-port")
+       "output-port")
       ((binary-port? obj)
-       "BINARY-port")
+       "binary-port")
       ((cell? obj)
-       "CELL")
+       "cell")
       ((foreign? obj)
-       "FOREIGN")
+       (string-append "foreign:" (symbol->string (foreign-id obj))))
       ((cnst? obj)
-       "BCNST")
+       "bcnst")
+      ((socket? obj)
+       "socket")
+      ((process? obj)
+       "process")
+      ((custom? obj)
+       "custom")
+      ((opaque? obj)
+       "opaque")
       ((object? obj)
        (let ((class  (object-class obj)))
 	  ;; here we are very chicken here because want to ensure that
@@ -644,16 +706,22 @@
 		     (symbol->string sym)
 		     "_"))
 	      "_")))
+      ((ucs2-string? obj)
+       "ucs2string")
+      ((ucs2? obj)
+       "ucs2")
       (else
-       "_")))
+       (c-find-runtime-type obj))))
 
 ;*---------------------------------------------------------------------*/
 ;*    module-init-error ...                                            */
 ;*---------------------------------------------------------------------*/
 (define (module-init-error current::string from::string)
-   (warning current
-	    #"Inconsistent module initialization.\nAt least, one of the two modules has to be recompiled (see also -unsafev option)"
-	    from))
+   (fprint (current-error-port)
+	   #"*** ERROR:Inconsistent module initialization:\n"
+	   #"At least, one of the two modules has to be recompiled (see also -unsafev option):\n"
+	   current " -- " from)
+   (%exit -1))
 
 ;*---------------------------------------------------------------------*/
 ;*    bigloo-type-error ...                                            */
