@@ -1,44 +1,16 @@
-;*---------------------------------------------------------------------*/
-;*   A pratical implementation for the Scheme programming language     */
-;*                                                                     */
-;*                                    ,--^,                            */
-;*                              _ ___/ /|/                             */
-;*                          ,;'( )__, ) '                              */
-;*                         ;;  //   L__.                               */
-;*                         '   \\   /  '                               */
-;*                              ^   ^                                  */
-;*                                                                     */
-;*   Copyright (c) 1992-1999 Manuel Serrano                            */
-;*                                                                     */
-;*     Bug descriptions, use reports, comments or suggestions are      */
-;*     welcome. Send them to                                           */
-;*       bigloo-request@kaolin.unice.fr                                */
-;*       http://kaolin.unice.fr/bigloo                                 */
-;*                                                                     */
-;*   This program is free software; you can redistribute it            */
-;*   and/or modify it under the terms of the GNU General Public        */
-;*   License as published by the Free Software Foundation; either      */
-;*   version 2 of the License, or (at your option) any later version.  */
-;*                                                                     */
-;*   This program is distributed in the hope that it will be useful,   */
-;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
-;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
-;*   GNU General Public License for more details.                      */
-;*                                                                     */
-;*   You should have received a copy of the GNU General Public         */
-;*   License along with this program; if not, write to the Free        */
-;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
-;*   MA 02111-1307, USA.                                               */
-;*---------------------------------------------------------------------*/
 ;*=====================================================================*/
 ;*    serrano/prgm/project/bigloo/runtime/Llib/hash.scm                */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Thu Sep  1 08:51:06 1994                          */
-;*    Last change :  Thu Oct 29 09:37:03 1998 (serrano)                */
+;*    Last change :  Thu Nov  8 09:23:27 2001 (serrano)                */
 ;*    -------------------------------------------------------------    */
 ;*    The hash tables.                                                 */
 ;*    -------------------------------------------------------------    */
+;*    Source documentation:                                            */
+;*       @path ../../manuals/body.texi@                                */
+;*       @node Hash Tables@                                            */
+;*    -------------------------------------------------------------    */
 ;*    This implementation is cumbersome because the keys are inside    */
 ;*    the objects. Unfortunately I can't change this because these     */
 ;*    hash table are extensively used by the compiler. This approach   */
@@ -50,39 +22,91 @@
 ;*---------------------------------------------------------------------*/
 (module __hash
 
-   (import  (__error                   "Llib/error.scm"))
-   
-   (use     (__type                    "Llib/type.scm")
-	    (__bigloo                  "Llib/bigloo.scm")
-	    (__structure               "Llib/struct.scm")
-	    (__bit                     "Llib/bit.scm")
-	    (__tvector                 "Llib/tvector.scm")
-	    (__r4_numbers_6_5_fixnum   "Ieee/fixnum.scm")
-	    (__r4_equivalence_6_2      "Ieee/equiv.scm")
-	    (__r4_control_features_6_9 "Ieee/control.scm")
-	    (__r4_characters_6_6       "Ieee/char.scm")
-	    (__r4_booleans_6_1         "Ieee/boolean.scm")
-	    (__r4_vectors_6_8          "Ieee/vector.scm")
-	    (__r4_pairs_and_lists_6_3  "Ieee/pair-list.scm")
-	    (__r4_symbols_6_4          "Ieee/symbol.scm")
-	    (__r4_strings_6_7          "Ieee/string.scm")
-	    (__evenv                   "Eval/evenv.scm"))
+   (import  __error
+	    __r4_symbols_6_4)
 
-   (foreign (long get_hash_number                          (string)
-		  "get_hash_number")
-	    (long get_hash_power_number                    (string long)
-		  "get_hash_power_number")
-	    
-	    (macro long get_hash_number_from_int           (long)
-		  "get_hash_number_from_int")
-	    (macro long get_hash_power_number_from_int     (long long)
-		  "get_hash_power_number_from_int")
-	    
-	    (macro long get_hash_number_from_pointer       (obj)
-		   "get_hash_number_from_int")
-	    (macro long get_hash_power_number_from_pointer (obj long)
-		   "get_hash_power_number_from_int"))
+   (use     __type
+	    __bigloo
+	    __structure
+	    __bit
+	    __tvector
+	    __object
+	    __r4_numbers_6_5_fixnum
+	    __r4_equivalence_6_2
+	    __r4_control_features_6_9
+	    __r4_characters_6_6
+	    __r4_booleans_6_1
+	    __r4_vectors_6_8
+	    __r4_pairs_and_lists_6_3
+	    __r4_strings_6_7
+	    __evenv)
+
+   ;; the new interface
+   (extern  (string-hash-number::int (::string) "bgl_string_hash_number")
+	    (symbol-hash-number::int (::symbol) "bgl_symbol_hash_number")
+	    (keyword-hash-number::int (::keyword) "bgl_keyword_hash_number")
+	    (obj-hash-number::int (::obj) "bgl_obj_hash_number"))
    
+   ;; the old interface
+   (extern  (get_hash_number::long (::string)
+				   "get_hash_number")
+	    (get_hash_power_number::long (::string ::long)
+					 "get_hash_power_number")
+	    
+	    (macro get_hash_number_from_int::long (::long)
+		   "get_hash_number_from_int")
+	    (macro get_hash_power_number_from_int::long (::long ::long)
+		   "get_hash_power_number_from_int")
+	    
+	    (macro get_hash_number_from_pointer::long (::obj)
+		   "get_hash_number_from_int")
+	    (macro get_hash_power_number_from_pointer::long (::obj ::long)
+		   "get_hash_power_number_from_int"))
+
+   ;; the new interface
+   (java    (class foreign
+	       (method static string-hash-number::int (::string)
+		       "bgl_string_hash_number")
+	       (method static symbol-hash-number::int (::symbol)
+		       "bgl_symbol_hash_number")
+	       (method static keyword-hash-number::int (::keyword)
+		       "bgl_keyword_hash_number")
+	       (method static obj-hash-number::int (::obj)
+		       "bgl_obj_hash_number")))
+
+   ;; the old interface
+   (java    (class foreign
+	       (method static get_hash_number::long (::string)
+		       "get_hash_number")
+	       (method static get_hash_power_number::long (::string ::long)
+		       "get_hash_power_number")
+	       
+	       (method static get_hash_number_from_int::long (::long)
+		       "get_hash_number_from_int")
+	       (method static get_hash_power_number_from_int::long (::long ::long)
+		       "get_hash_power_number_from_int")
+	       
+	       (method static get_hash_number_from_pointer::long (::obj)
+		       "get_hash_number_from_int")
+	       (method static get_hash_power_number_from_pointer::long (::obj ::long)
+		       "get_hash_power_number_from_int")))
+
+
+   ;; the new interface
+   (export  (make-hashtable . args)
+	    (get-hashnumber::int ::obj)
+	    (hashtable?::bool ::obj)
+	    (hashtable-size::int ::struct)
+	    (hashtable-get::obj ::struct ::obj)
+	    (hashtable-put! ::struct ::obj ::obj)
+	    (hashtable-update! ::struct ::obj ::procedure ::obj)
+	    (hashtable-remove!::bool ::struct ::obj)
+	    (hashtable->vector::vector ::struct)
+	    (hashtable->list::pair-nil ::struct)
+	    (hashtable-key-list::pair-nil ::struct)
+	    (hashtable-for-each ::struct ::procedure))
+
+   ;; the old interface
    (export  (make-hash-table  ::long ::procedure ::procedure
 				::procedure . <size>)
 	    (hash-table?::bool         <obj>)
@@ -103,8 +127,274 @@
 	    (obj->0..255::long             ::obj)
 	    (obj->0..2^x-1::long           ::obj ::long))
 
-   (pragma  (get-hash side-effect-free)))
-	     
+   (pragma  (hashtable-get side-effect-free))
+
+   (pragma  (get-hash side-effect-free)
+	    (get_hash_number_from_int nesting)
+	    (get_hash_power_number_from_int nesting)
+	    (get_hash_number_from_pointer nesting)
+	    (get_hash_power_number_from_pointer nesting)))
+
+;*---------------------------------------------------------------------*/
+;*    Default hashtable configuration                                  */
+;*---------------------------------------------------------------------*/
+(define default-hashtable-bucket-length 128)
+(define default-max-bucket-length 10)
+
+;*---------------------------------------------------------------------*/
+;*    %hashtable                                                       */
+;*---------------------------------------------------------------------*/
+(define-struct %hashtable size max-bucket-len buckets)
+
+;*---------------------------------------------------------------------*/
+;*    make-hashtable ...                                               */
+;*---------------------------------------------------------------------*/
+(define (make-hashtable . args)
+   (let ((size (if (pair? args)
+		   (if (fixnum? (car args))
+		       (car args)
+		       (error "make-hashtable"
+			      "Illegal default bucket length"
+			      args))
+		   default-hashtable-bucket-length))
+	 (mblen (if (and (pair? args) (pair? (cdr args)))
+		    (if (fixnum? (cadr args))
+			(cadr args)
+			(error "make-hashtable"
+			       "Illegal max bucket length"
+			       args))
+		    default-max-bucket-length)))
+      (%hashtable 0 mblen (make-vector size '()))))
+
+;*---------------------------------------------------------------------*/
+;*    hashtable? ...                                                   */
+;*---------------------------------------------------------------------*/
+(define (hashtable?::bool obj::obj)
+   (%hashtable? obj))
+
+;*---------------------------------------------------------------------*/
+;*    hashtable-size ...                                               */
+;*---------------------------------------------------------------------*/
+(define (hashtable-size::int table::struct)
+   (%hashtable-size table))
+
+;*---------------------------------------------------------------------*/
+;*    hashtable->vector ...                                            */
+;*---------------------------------------------------------------------*/
+(define (hashtable->vector table::struct)
+   (let* ((vec (make-vector (hashtable-size table)))
+	  (buckets (%hashtable-buckets table))
+	  (buckets-len (vector-length buckets)))
+      (let loop ((i 0)
+		 (w 0))
+	 (if (=fx i buckets-len)
+	     vec
+	     (let liip ((bucket (vector-ref buckets i))
+			(w w))
+		(if (null? bucket)
+		    (loop (+fx i 1) w)
+		    (begin
+		       (vector-set! vec w (cdar bucket))
+		       (liip (cdr bucket) (+fx w 1)))))))))
+
+;*---------------------------------------------------------------------*/
+;*    hashtable->list ...                                              */
+;*---------------------------------------------------------------------*/
+(define (hashtable->list table::struct)
+   (let* ((vec (make-vector (hashtable-size table)))
+	  (buckets (%hashtable-buckets table))
+	  (buckets-len (vector-length buckets)))
+      (let loop ((i 0)
+		 (res '()))
+	 (if (=fx i buckets-len)
+	     res
+	     (let liip ((bucket (vector-ref buckets i))
+			(res res))
+		(if (null? bucket)
+		    (loop (+fx i 1) res)
+		    (liip (cdr bucket) (cons (cdar bucket) res))))))))
+
+;*---------------------------------------------------------------------*/
+;*    hashtable-key-list ...                                           */
+;*---------------------------------------------------------------------*/
+(define (hashtable-key-list table::struct)
+   (let* ((vec (make-vector (hashtable-size table)))
+	  (buckets (%hashtable-buckets table))
+	  (buckets-len (vector-length buckets)))
+      (let loop ((i 0)
+		 (res '()))
+	 (if (=fx i buckets-len)
+	     res
+	     (let liip ((bucket (vector-ref buckets i))
+			(res res))
+		(if (null? bucket)
+		    (loop (+fx i 1) res)
+		    (liip (cdr bucket) (cons (caar bucket) res))))))))
+
+;*---------------------------------------------------------------------*/
+;*    hashtable-for-each ...                                           */
+;*---------------------------------------------------------------------*/
+(define (hashtable-for-each table::struct fun::procedure)
+   (let* ((buckets (%hashtable-buckets table))
+	  (buckets-len (vector-length buckets)))
+      (let loop ((i 0))
+	 (if (<fx i buckets-len)
+	     (begin
+		(for-each (lambda (cell)
+			     (fun (car cell) (cdr cell)))
+			  (vector-ref buckets i))
+		(loop (+fx i 1)))))))
+
+;*---------------------------------------------------------------------*/
+;*    hashtable-get ...                                                */
+;*---------------------------------------------------------------------*/
+(define (hashtable-get table::struct key::obj)
+   (let* ((buckets (%hashtable-buckets table))
+	  (bucket-len (vector-length buckets))
+	  (bucket-num (remainder (get-hashnumber key) bucket-len))
+	  (bucket (vector-ref buckets bucket-num)))
+      (let loop ((bucket bucket))
+	 (cond
+	    ((null? bucket)
+	     #f)
+	    ((hashtable-equal? (caar bucket) key)
+	     (cdar bucket))
+	    (else
+	     (loop (cdr bucket)))))))
+
+;*---------------------------------------------------------------------*/
+;*    hashtable-put! ...                                               */
+;*---------------------------------------------------------------------*/
+(define (hashtable-put! table::struct key::obj obj::obj)
+   (let* ((buckets (%hashtable-buckets table))
+	  (bucket-len (vector-length buckets))
+	  (bucket-num (remainder (get-hashnumber key) bucket-len))
+	  (bucket (vector-ref buckets bucket-num))
+	  (max-bucket-len (%hashtable-max-bucket-len table)))
+      (if (null? bucket)
+	  (begin
+	     (%hashtable-size-set! table (+fx (%hashtable-size table) 1))
+	     (vector-set! buckets bucket-num (list (cons key obj)))
+	     obj)
+	  (let loop ((buck bucket)
+		     (count 0))
+	     (cond
+		((null? buck)
+		 (%hashtable-size-set! table (+fx (%hashtable-size table) 1))
+		 (vector-set! buckets bucket-num (cons (cons key obj) bucket))
+		 (if (>fx count max-bucket-len)
+		     (hashtable-expand! table))
+		 obj)
+		((hashtable-equal? (caar buck) key)
+		 (set-cdr! (car buck) obj))
+		(else
+		 (loop (cdr buck) (+fx count 1))))))))
+
+;*---------------------------------------------------------------------*/
+;*    hashtable-update! ...                                            */
+;*---------------------------------------------------------------------*/
+(define (hashtable-update! table::struct key::obj proc::procedure obj)
+   (let* ((buckets (%hashtable-buckets table))
+	  (bucket-len (vector-length buckets))
+	  (bucket-num (remainder (get-hashnumber key) bucket-len))
+	  (bucket (vector-ref buckets bucket-num))
+	  (max-bucket-len (%hashtable-max-bucket-len table)))
+      (if (null? bucket)
+	  (begin
+	     (%hashtable-size-set! table (+fx (%hashtable-size table) 1))
+	     (vector-set! buckets bucket-num (list (cons key obj)))
+	     obj)
+	  (let loop ((buck bucket)
+		     (count 0))
+	     (cond
+		((null? buck)
+		 (%hashtable-size-set! table (+fx (%hashtable-size table) 1))
+		 (vector-set! buckets bucket-num (cons (cons key obj) bucket))
+		 (if (>fx count max-bucket-len)
+		     (hashtable-expand! table))
+		 obj)
+		((hashtable-equal? (caar buck) key)
+		 (set-cdr! (car buck) (proc (cdar buck))))
+		(else
+		 (loop (cdr buck) (+fx count 1))))))))
+   
+;*---------------------------------------------------------------------*/
+;*    hashtable-remove! ...                                            */
+;*---------------------------------------------------------------------*/
+(define (hashtable-remove! table::struct key::obj)
+   (let* ((buckets (%hashtable-buckets table))
+	  (bucket-len (vector-length buckets))
+	  (bucket-num (remainder (get-hashnumber key) bucket-len))
+	  (bucket (vector-ref buckets bucket-num)))
+      (cond
+	 ((null? bucket)
+	  #f)
+	 ((hashtable-equal? (caar bucket) key)
+	  (vector-set! buckets bucket-num (cdr bucket))
+	  (%hashtable-size-set! table (-fx (%hashtable-size table) 1))
+	  #t)
+	 (else
+	  (let loop ((bucket (cdr bucket))
+		     (prev bucket))
+	     (if (pair? bucket)
+		 (if (hashtable-equal? (caar bucket) key)
+		     (begin
+			(set-cdr! prev (cdr bucket))
+			(%hashtable-size-set! table
+					      (-fx (%hashtable-size table) 1))
+			#t)
+		     (loop (cdr bucket)
+			   bucket))
+		 #f))))))
+   
+;*---------------------------------------------------------------------*/
+;*    hashtable-expand! ...                                            */
+;*---------------------------------------------------------------------*/
+(define (hashtable-expand! table)
+   (let* ((old-bucks (%hashtable-buckets table))
+	  (old-bucks-len (vector-length old-bucks))
+	  (new-bucks-len (*fx 2 old-bucks-len))
+	  (new-bucks (make-vector new-bucks-len '())))
+      (%hashtable-buckets-set! table new-bucks)
+      (let loop ((i 0))
+	 (if (<fx i old-bucks-len)
+	     (begin
+		(for-each (lambda (cell)
+			     (let* ((key (car cell))
+				    (h   (remainder (get-hashnumber key)
+						    new-bucks-len)))
+				(vector-set! new-bucks
+					     h
+					     (cons cell
+						   (vector-ref new-bucks h)))))
+			  (vector-ref old-bucks i))
+		(loop (+fx i 1)))))))
+
+;*---------------------------------------------------------------------*/
+;*    hashtable-equal? ...                                             */
+;*---------------------------------------------------------------------*/
+(define-inline (hashtable-equal? obj1 obj2)
+   (if (string? obj1)
+       (if (string? obj2)
+	   (string=? obj1 obj2)
+	   #f)
+       (equal? obj1 obj2)))
+
+;*---------------------------------------------------------------------*/
+;*    get-hashnumber ...                                               */
+;*---------------------------------------------------------------------*/
+(define (get-hashnumber::int key)
+   (cond
+      ((string? key)
+       (absfx (string-hash-number key)))
+      ((symbol? key)
+       (absfx (symbol-hash-number key)))
+      ((fixnum? key)
+       (absfx key))
+      ((object? key)
+       (absfx (object-hashnumber key)))
+      (else
+       (absfx (obj-hash-number key)))))
 
 ;*---------------------------------------------------------------------*/
 ;*    hash-table                                                       */
@@ -132,18 +422,8 @@
 		       max-size
 		       (car init-size)))
 	 (2^power? (lambda (size)
-		      ;; je suis sur qu'on peut ecrire cette fonction
-		      ;; vraiment mieux mais comme ses perfs n'ont aucune
-		      ;; importance, je laisse ca comme ca pour le moment.
-		      (let loop ((n 1))
-			 (let ((num (bit-lsh 1 n)))
-			    (cond
-			       ((=fx num size)
-				#t)
-			       ((<fx num size)
-				(loop (+fx n 1)))
-			       (else
-				#f)))))))
+		      (and (>fx size 1)
+			   (=fx 0 (bit-and size (-fx size 1)))))))
       (cond
 	 ((not (2^power? size))
 	  (error "make-hash-table"
