@@ -1,41 +1,9 @@
-;*---------------------------------------------------------------------*/
-;*   A pratical implementation for the Scheme programming language     */
-;*                                                                     */
-;*                                    ,--^,                            */
-;*                              _ ___/ /|/                             */
-;*                          ,;'( )__, ) '                              */
-;*                         ;;  //   L__.                               */
-;*                         '   \\   /  '                               */
-;*                              ^   ^                                  */
-;*                                                                     */
-;*   Copyright (c) 1992-1999 Manuel Serrano                            */
-;*                                                                     */
-;*     Bug descriptions, use reports, comments or suggestions are      */
-;*     welcome. Send them to                                           */
-;*       bigloo-request@kaolin.unice.fr                                */
-;*       http://kaolin.unice.fr/bigloo                                 */
-;*                                                                     */
-;*   This program is free software; you can redistribute it            */
-;*   and/or modify it under the terms of the GNU General Public        */
-;*   License as published by the Free Software Foundation; either      */
-;*   version 2 of the License, or (at your option) any later version.  */
-;*                                                                     */
-;*   This program is distributed in the hope that it will be useful,   */
-;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
-;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
-;*   GNU General Public License for more details.                      */
-;*                                                                     */
-;*   You should have received a copy of the GNU General Public         */
-;*   License along with this program; if not, write to the Free        */
-;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
-;*   MA 02111-1307, USA.                                               */
-;*---------------------------------------------------------------------*/
 ;*=====================================================================*/
 ;*    serrano/prgm/project/bigloo/runtime/Llib/unicode.scm             */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Mon Mar 20 19:17:18 1995                          */
-;*    Last change :  Fri Feb  6 16:58:46 1998 (serrano)                */
+;*    Last change :  Wed Jun  6 18:59:31 2001 (serrano)                */
 ;*    -------------------------------------------------------------    */
 ;*    Unicode (UCS-2) strings handling.                                */
 ;*=====================================================================*/
@@ -44,48 +12,51 @@
 ;*    The module                                                       */
 ;*---------------------------------------------------------------------*/
 (module __unicode
-
-   (import  (__error                   "Llib/error.scm"))
    
-   (use     (__type                    "Llib/type.scm")
-	    (__bigloo                  "Llib/bigloo.scm")
-	    (__tvector                 "Llib/tvector.scm")
-	    (__ucs2                    "Llib/ucs2.scm")
+   (import  __error)
+   
+   (use     __type
+	    __bigloo
+	    __tvector
+	    __ucs2
 	    
-	    (__r4_numbers_6_5_fixnum   "Ieee/fixnum.scm")
-	    (__r4_equivalence_6_2      "Ieee/equiv.scm")
-	    (__r4_vectors_6_8          "Ieee/vector.scm")
-	    (__r4_booleans_6_1         "Ieee/boolean.scm")
-	    (__r4_characters_6_6       "Ieee/char.scm")
-	    (__r4_symbols_6_4          "Ieee/symbol.scm")
-	    (__r4_pairs_and_lists_6_3  "Ieee/pair-list.scm")
-	    (__r4_strings_6_7          "Ieee/string.scm")
-
-	    (__evenv                   "Eval/evenv.scm"))
+	    __r4_ports_6_10_1
+	    __r4_output_6_10_3
+	    
+	    __r4_numbers_6_5_fixnum
+	    __r4_equivalence_6_2
+	    __r4_vectors_6_8
+	    __r4_booleans_6_1
+	    __r4_characters_6_6
+	    __r4_symbols_6_4
+	    __r4_pairs_and_lists_6_3
+	    __r4_strings_6_7
+	    
+	    __evenv)
    
-   (extern (macro c-ucs2-string?::bool       (::obj)
+   (extern (macro c-ucs2-string?::bool (::obj)
 		  "UCS2_STRINGP")
-	   (c-make-ucs2-string::ucs2string   (::long ::ucs2)
-		  "make_ucs2_string")
-	   (macro c-ucs2-string-length::long (::ucs2string)
+	   (c-make-ucs2-string::ucs2string (::int ::ucs2)
+					   "make_ucs2_string")
+	   (macro c-ucs2-string-length::int (::ucs2string)
 		  "UCS2_STRING_LENGTH")
-	   (macro c-ucs2-string-ref::ucs2    (::ucs2string ::long)
+	   (macro c-ucs2-string-ref::ucs2 (::ucs2string ::int)
 		  "UCS2_STRING_REF")
-	   (macro c-ucs2-string-set!::obj    (::ucs2string ::long ::ucs2)
+	   (macro c-ucs2-string-set!::obj (::ucs2string ::int ::ucs2)
 		  "UCS2_STRING_SET")
-	    
-	   (c-ucs2-string=?::bool  (::ucs2string ::ucs2string)
-				   "ucs2_strcmp")
-	   (ucs2-strcicmp::bool    (::ucs2string ::ucs2string)
-				   "ucs2_strcicmp")
-	   (ucs2-string_lt::bool   (::ucs2string ::ucs2string)
-				   "ucs2_string_lt")
-	   (ucs2-string_le::bool   (::ucs2string ::ucs2string)
-				   "ucs2_string_le")
-	   (ucs2-string_gt::bool   (::ucs2string ::ucs2string)
-				   "ucs2_string_gt")
-	   (ucs2-string_ge::bool   (::ucs2string ::ucs2string)
-				   "ucs2_string_ge")
+	   
+	   (c-ucs2-string=?::bool (::ucs2string ::ucs2string)
+				  "ucs2_strcmp")
+	   (ucs2-strcicmp::bool (::ucs2string ::ucs2string)
+				"ucs2_strcicmp")
+	   (ucs2-string_lt::bool (::ucs2string ::ucs2string)
+				 "ucs2_string_lt")
+	   (ucs2-string_le::bool (::ucs2string ::ucs2string)
+				 "ucs2_string_le")
+	   (ucs2-string_gt::bool (::ucs2string ::ucs2string)
+				 "ucs2_string_gt")
+	   (ucs2-string_ge::bool (::ucs2string ::ucs2string)
+				 "ucs2_string_ge")
 	   (ucs2-string_cilt::bool (::ucs2string ::ucs2string)
 				   "ucs2_string_cilt")
 	   (ucs2-string_cile::bool (::ucs2string ::ucs2string)
@@ -94,72 +65,113 @@
 				   "ucs2_string_cigt")
 	   (ucs2-string_cige::bool (::ucs2string ::ucs2string)
 				   "ucs2_string_cige")
-
-	   (escape-C-string::ucs2string      (::string)
-					     "escape_ucs2_string")
-	   (c-ucs2-string-copy::ucs2string   (::ucs2string)
-					     "c_ucs2_string_copy")
-	   (c-subucs2-string::ucs2string     (::ucs2string ::long ::long)
-					     "c_subucs2_string")
+	   
+	   (c-ucs2-string-copy::ucs2string (::ucs2string)
+					   "c_ucs2_string_copy")
+	   (c-subucs2-string::ucs2string (::ucs2string ::int ::int)
+					 "c_subucs2_string")
 	   (c-ucs2-string-append::ucs2string (::ucs2string ::ucs2string)
 					     "ucs2_string_append")
-	    
-	   (ucs2-string-bound-check?::bool      (::long ::long)
-						"BOUND_CHECK")
+	   
 	   (c-ucs2-string->utf8-string::bstring (::ucs2string)
 						"ucs2_string_to_utf8_string")
 	   (c-utf8-string->ucs2-string::ucs2string (::bstring)
-						"utf8_string_to_ucs2_string"))
+						   "utf8_string_to_ucs2_string"))
    
-   (export  (inline ucs2-string?::bool             ::obj)
-	    (inline make-ucs2-string::ucs2string   ::int . ucs2)
-	    (inline ucs2-string::ucs2string        . ucs2s)
-	    (inline ucs2-string-length::long       ::ucs2string)
-	    (inline ucs2-string-ref::ucs2          ::ucs2string ::long)
-	    (inline ucs2-string-set!::obj          ::ucs2string ::long ::ucs2)
-	    (inline ucs2-string-ref-ur::ucs2       ::ucs2string ::long)
-	    (inline ucs2-string-set-ur!::obj       ::ucs2string ::long ::ucs2)
-	    (inline ucs2-string=?::bool            ::ucs2string ::ucs2string)
-	    (inline ucs2-string-ci=?::bool         ::ucs2string ::ucs2string)
-	    (inline ucs2-string<?::bool            ::ucs2string ::ucs2string)
-	    (inline ucs2-string>?::bool            ::ucs2string ::ucs2string)
-	    (inline ucs2-string<=?::bool           ::ucs2string ::ucs2string)
-	    (inline ucs2-string>=?::bool           ::ucs2string ::ucs2string)
-	    (inline ucs2-string-ci<?::bool         ::ucs2string ::ucs2string)
-	    (inline ucs2-string-ci>?::bool         ::ucs2string ::ucs2string)
-	    (inline ucs2-string-ci<=?::bool        ::ucs2string ::ucs2string)
-	    (inline ucs2-string-ci>=?::bool        ::ucs2string ::ucs2string)
-	    (inline subucs2-string::ucs2string     ::ucs2string ::long ::long)
-	    (inline subucs2-string-ur::ucs2string  ::ucs2string ::long ::long)
-	    (ucs2-string-append::ucs2string        . ucs2-strings)
-	    (ucs2-string->list                     ::ucs2string)
-	    (list->ucs2-string::ucs2string         ::obj)
-	    (inline ucs2-string-copy::ucs2string   ::ucs2string)
-	    (ucs2-string-fill!::ucs2string         ::ucs2string ::ucs2)
-	    (ucs2-string-upcase::ucs2string        ::ucs2string)
-	    (ucs2-string-downcase::ucs2string      ::ucs2string)
-	    (ucs2-string-upcase!::ucs2string       ::ucs2string)
-	    (ucs2-string-downcase!::ucs2string     ::ucs2string)
-	    (inline ucs2-string->utf8-string::bstring    ::ucs2string)
+   (java   (class foreign
+	      (method static c-ucs2-string?::bool (::obj)
+		      "UCS2_STRINGP")
+	      (method static c-make-ucs2-string::ucs2string (::int ::ucs2)
+		      "make_ucs2_string")
+	      (method static c-ucs2-string-length::int (::ucs2string)
+		      "UCS2_STRING_LENGTH")
+	      (method static c-ucs2-string-ref::ucs2 (::ucs2string ::int)
+		      "UCS2_STRING_REF")
+	      (method static c-ucs2-string-set!::obj (::ucs2string ::int ::ucs2)
+		      "UCS2_STRING_SET")
+	      
+	      (method static c-ucs2-string=?::bool (::ucs2string ::ucs2string)
+		      "ucs2_strcmp")
+	      (method static ucs2-strcicmp::bool (::ucs2string ::ucs2string)
+		      "ucs2_strcicmp")
+	      (method static ucs2-string_lt::bool (::ucs2string ::ucs2string)
+		      "ucs2_string_lt")
+	      (method static ucs2-string_le::bool (::ucs2string ::ucs2string)
+		      "ucs2_string_le")
+	      (method static ucs2-string_gt::bool (::ucs2string ::ucs2string)
+		      "ucs2_string_gt")
+	      (method static ucs2-string_ge::bool (::ucs2string ::ucs2string)
+		      "ucs2_string_ge")
+	      (method static ucs2-string_cilt::bool (::ucs2string ::ucs2string)
+		      "ucs2_string_cilt")
+	      (method static ucs2-string_cile::bool (::ucs2string ::ucs2string)
+		      "ucs2_string_cile")
+	      (method static ucs2-string_cigt::bool (::ucs2string ::ucs2string)
+		      "ucs2_string_cigt")
+	      (method static ucs2-string_cige::bool (::ucs2string ::ucs2string)
+		      "ucs2_string_cige")
+	      
+	      (method static c-ucs2-string-copy::ucs2string (::ucs2string)
+		      "c_ucs2_string_copy")
+	      (method static c-subucs2-string::ucs2string (::ucs2string ::int ::int)
+		      "c_subucs2_string")
+	      (method static c-ucs2-string-append::ucs2string (::ucs2string ::ucs2string)
+		      "c_ucs2_string_append")
+	      
+	      (method static c-ucs2-string->utf8-string::bstring (::ucs2string)
+		      "ucs2_string_to_utf8_string")
+	      (method static c-utf8-string->ucs2-string::ucs2string (::bstring)
+		      "utf8_string_to_ucs2_string")))
+   
+   (export  (inline ucs2-string?::bool ::obj)
+	    (inline make-ucs2-string::ucs2string ::int . ucs2)
+	    (inline ucs2-string::ucs2string . ucs2s)
+	    (inline ucs2-string-length::int ::ucs2string)
+	    (inline ucs2-string-ref::ucs2 ::ucs2string ::int)
+	    (inline ucs2-string-set!::obj ::ucs2string ::int ::ucs2)
+	    (inline ucs2-string-ref-ur::ucs2 ::ucs2string ::int)
+	    (inline ucs2-string-set-ur!::obj ::ucs2string ::int ::ucs2)
+	    (inline ucs2-string=?::bool ::ucs2string ::ucs2string)
+	    (inline ucs2-string-ci=?::bool ::ucs2string ::ucs2string)
+	    (inline ucs2-string<?::bool ::ucs2string ::ucs2string)
+	    (inline ucs2-string>?::bool ::ucs2string ::ucs2string)
+	    (inline ucs2-string<=?::bool ::ucs2string ::ucs2string)
+	    (inline ucs2-string>=?::bool ::ucs2string ::ucs2string)
+	    (inline ucs2-string-ci<?::bool ::ucs2string ::ucs2string)
+	    (inline ucs2-string-ci>?::bool ::ucs2string ::ucs2string)
+	    (inline ucs2-string-ci<=?::bool ::ucs2string ::ucs2string)
+	    (inline ucs2-string-ci>=?::bool ::ucs2string ::ucs2string)
+	    (inline subucs2-string::ucs2string ::ucs2string ::int ::int)
+	    (inline subucs2-string-ur::ucs2string ::ucs2string ::int ::int)
+	    (ucs2-string-append::ucs2string . ucs2-strings)
+	    (ucs2-string->list ::ucs2string)
+	    (list->ucs2-string::ucs2string ::obj)
+	    (inline ucs2-string-copy::ucs2string ::ucs2string)
+	    (ucs2-string-fill!::ucs2string ::ucs2string ::ucs2)
+	    (ucs2-string-upcase::ucs2string ::ucs2string)
+	    (ucs2-string-downcase::ucs2string ::ucs2string)
+	    (ucs2-string-upcase!::ucs2string ::ucs2string)
+	    (ucs2-string-downcase!::ucs2string ::ucs2string)
+	    (inline ucs2-string->utf8-string::bstring ::ucs2string)
 	    (inline utf8-string->ucs2-string::ucs2string ::bstring))
-    
-   (pragma  (c-ucs2-string? (predicate-of ucs2string) no-cfa-top)
-	    (ucs2-string? side-effect-free no-cfa-top)
-	    (c-ucs2-string-ref side-effect-free no-cfa-top)
-	    (ucs2-string-ref-ur side-effect-free no-cfa-top)
-	    (ucs2-string-ref side-effect-free no-cfa-top)
-	    (c-ucs2-string-length side-effect-free no-cfa-top)
-	    (ucs2-string-length side-effect-free no-cfa-top)
-	    (ucs2-string=? side-effect-free)
-	    (ucs2-string-ci=? side-effect-free)
-	    (ucs2-string<? side-effect-free)
-	    (ucs2-string>? side-effect-free)
-	    (ucs2-string<=? side-effect-free)
-	    (ucs2-string>=? side-effect-free)
-	    (ucs2-string-ci<? side-effect-free)
-	    (ucs2-string-ci>? side-effect-free)
-	    (ucs2-string-ci<=? side-effect-free)
-	    (ucs2-string-ci>=? side-effect-free)))
+   
+   (pragma  (c-ucs2-string? (predicate-of ucs2string) no-cfa-top nesting)
+	    (ucs2-string? side-effect-free no-cfa-top nesting)
+	    (c-ucs2-string-ref side-effect-free no-cfa-top nesting)
+	    (ucs2-string-ref-ur side-effect-free no-cfa-top nesting)
+	    (ucs2-string-ref side-effect-free no-cfa-top nesting)
+	    (c-ucs2-string-length side-effect-free no-cfa-top nesting)
+	    (ucs2-string-length side-effect-free no-cfa-top nesting)
+	    (ucs2-string=? side-effect-free nesting)
+	    (ucs2-string-ci=? side-effect-free nesting)
+	    (ucs2-string<? side-effect-free nesting)
+	    (ucs2-string>? side-effect-free nesting)
+	    (ucs2-string<=? side-effect-free nesting)
+	    (ucs2-string>=? side-effect-free nesting)
+	    (ucs2-string-ci<? side-effect-free nesting)
+	    (ucs2-string-ci>? side-effect-free nesting)
+	    (ucs2-string-ci<=? side-effect-free nesting)
+	    (ucs2-string-ci>=? side-effect-free nesting)))
   
 ;*---------------------------------------------------------------------*/
 ;*    ucs2-string? ...                                                 */
