@@ -1,41 +1,9 @@
 ;*---------------------------------------------------------------------*/
-;*   A pratical implementation for the Scheme programming language     */
-;*                                                                     */
-;*                                    ,--^,                            */
-;*                              _ ___/ /|/                             */
-;*                          ,;'( )__, ) '                              */
-;*                         ;;  //   L__.                               */
-;*                         '   \\   /  '                               */
-;*                              ^   ^                                  */
-;*                                                                     */
-;*   Copyright (c) 1992-1999 Manuel Serrano                            */
-;*                                                                     */
-;*     Bug descriptions, use reports, comments or suggestions are      */
-;*     welcome. Send them to                                           */
-;*       bigloo-request@kaolin.unice.fr                                */
-;*       http://kaolin.unice.fr/bigloo                                 */
-;*                                                                     */
-;*   This program is free software; you can redistribute it            */
-;*   and/or modify it under the terms of the GNU General Public        */
-;*   License as published by the Free Software Foundation; either      */
-;*   version 2 of the License, or (at your option) any later version.  */
-;*                                                                     */
-;*   This program is distributed in the hope that it will be useful,   */
-;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
-;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
-;*   GNU General Public License for more details.                      */
-;*                                                                     */
-;*   You should have received a copy of the GNU General Public         */
-;*   License along with this program; if not, write to the Free        */
-;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
-;*   MA 02111-1307, USA.                                               */
-;*---------------------------------------------------------------------*/
-;*---------------------------------------------------------------------*/
 ;*    serrano/prgm/project/bigloo/runtime/Match/compiler.scm           */
 ;*                                                                     */
 ;*    Author      :  Jean-Marie Geffroy                                */
 ;*    Creation    :  Thu Jan 14 10:29:50 1993                          */
-;*    Last change :  Fri Sep 18 18:00:58 1998 (serrano)                */
+;*    Last change :  Fri Jul  6 07:21:29 2001 (serrano)                */
 ;*                                                                     */
 ;*---------------------------------------------------------------------*/
 ;*    A hand-written pattern "compiler"                                */
@@ -59,31 +27,31 @@
    
    (export  (pcompile f))
 
-   (import  (__error                   "Llib/error.scm")
-	    (__match_s2cfun            "Match/s2cfun.scm")
-	    (__match_descriptions      "Match/descr.scm"))
+   (import  __error
+	    __match_s2cfun
+	    __match_descriptions)
 
-   (use     (__type                    "Llib/type.scm")
-	    (__bigloo                  "Llib/bigloo.scm")
-	    (__tvector                 "Llib/tvector.scm")
-	    (__structure               "Llib/struct.scm")
-	    (__tvector                 "Llib/tvector.scm")
+   (use     __type
+	    __bigloo
+	    __tvector
+	    __structure
+	    __tvector
 
-	    (__r4_numbers_6_5          "Ieee/number.scm")
-	    (__r4_numbers_6_5_fixnum   "Ieee/fixnum.scm")
-	    (__r4_numbers_6_5_flonum   "Ieee/flonum.scm")
-	    (__r4_characters_6_6       "Ieee/char.scm")
-	    (__r4_equivalence_6_2      "Ieee/equiv.scm")
-	    (__r4_booleans_6_1         "Ieee/boolean.scm")
-	    (__r4_symbols_6_4          "Ieee/symbol.scm")
-	    (__r4_strings_6_7          "Ieee/string.scm")
-	    (__r4_pairs_and_lists_6_3  "Ieee/pair-list.scm")
-	    (__r4_input_6_10_2         "Ieee/input.scm")
-	    (__r4_control_features_6_9 "Ieee/control.scm")
-	    (__r4_vectors_6_8          "Ieee/vector.scm")
-	    (__r4_ports_6_10_1         "Ieee/port.scm")
-	    (__r4_output_6_10_3        "Ieee/output.scm")
-	    (__evenv                   "Eval/evenv.scm")))
+	    __r4_numbers_6_5
+	    __r4_numbers_6_5_fixnum
+	    __r4_numbers_6_5_flonum
+	    __r4_characters_6_6
+	    __r4_equivalence_6_2
+	    __r4_booleans_6_1
+	    __r4_symbols_6_4
+	    __r4_strings_6_7
+	    __r4_pairs_and_lists_6_3
+	    __r4_input_6_10_2
+	    __r4_control_features_6_9
+	    __r4_vectors_6_8
+	    __r4_ports_6_10_1
+	    __r4_output_6_10_3
+	    __evenv))
 	    
 (define (pcompile f)
    (let ((s (jim-gensym "E-")))
@@ -137,10 +105,26 @@
        ,(k r z d)  
        ,(z d)))
 
+(define (build-atom-equality-test e const)
+   (cond
+      ((or (fixnum? const)
+	   (char? const)
+	   (boolean? const)
+	   (symbol? const))
+       `(eq? ,e (quote ,const)))
+      ((flonum? const)
+       `(and (flonum? ,e) (=fl ,e ,const)))
+      ((number? const)
+       `(and (number? ,e) (= ,e ,const)))
+      ((string? const)
+       `(and (string? ,e) (string=? ,e ,const)))
+      (else
+       `(equal? ,e (quote ,const)))))
+
 (define (compile-quote f e r m k z d)
-  (build-if `(eq? ,e (quote ,(cadr f)))
-	    (k r z `(quote ,(cadr f)))
-	    (z (pattern-minus d f))))
+   (build-if (build-atom-equality-test e (cadr f))
+	     (k r z `(quote ,(cadr f)))
+	     (z (pattern-minus d f))))
 
 (define (compile-not f e r m k z d)
    (compile f e r m (lambda (r2 z2 d2) (z d2)) 
@@ -300,7 +284,7 @@
 
 (define (compile-times n f1 f2 e r m k0 z0 d0)
 
-   (let ((F-Env '()) (D-env '()))
+   (let ((F-env '()) (D-env '()))
       (letrec
 	    ((instanciate-try
 	      (lambda (r m k z d)
