@@ -1,35 +1,3 @@
-;*---------------------------------------------------------------------*/
-;*   A pratical implementation for the Scheme programming language     */
-;*                                                                     */
-;*                                    ,--^,                            */
-;*                              _ ___/ /|/                             */
-;*                          ,;'( )__, ) '                              */
-;*                         ;;  //   L__.                               */
-;*                         '   \\   /  '                               */
-;*                              ^   ^                                  */
-;*                                                                     */
-;*   Copyright (c) 1992-1999 Manuel Serrano                            */
-;*                                                                     */
-;*     Bug descriptions, use reports, comments or suggestions are      */
-;*     welcome. Send them to                                           */
-;*       bigloo-request@kaolin.unice.fr                                */
-;*       http://kaolin.unice.fr/bigloo                                 */
-;*                                                                     */
-;*   This program is free software; you can redistribute it            */
-;*   and/or modify it under the terms of the GNU General Public        */
-;*   License as published by the Free Software Foundation; either      */
-;*   version 2 of the License, or (at your option) any later version.  */
-;*                                                                     */
-;*   This program is distributed in the hope that it will be useful,   */
-;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
-;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
-;*   GNU General Public License for more details.                      */
-;*                                                                     */
-;*   You should have received a copy of the GNU General Public         */
-;*   License along with this program; if not, write to the Free        */
-;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
-;*   MA 02111-1307, USA.                                               */
-;*---------------------------------------------------------------------*/
 ;;;--------------------------------------------------------------------*/
 ;;;   geffroy/Match3.0/descriptions.scm ...                            */
 ;;;   Author      :  Jean-Marie Geffroy                                */
@@ -76,30 +44,30 @@
 	    (inline isVector-end? c)
 	    (inline isTree? c) )
 
-   (import  (__error                   "Llib/error.scm")
-	    (__match_s2cfun            "Match/s2cfun.scm"))
+   (import  __error
+	    __match_s2cfun)
    
-   (use     (__type                    "Llib/type.scm")
-	    (__bigloo                  "Llib/bigloo.scm")
-	    (__tvector                 "Llib/tvector.scm")
-	    (__structure               "Llib/struct.scm")
-	    (__tvector                 "Llib/tvector.scm")
+   (use     __type
+	    __bigloo
+	    __tvector
+	    __structure
+	    __tvector
 
-	    (__r4_numbers_6_5          "Ieee/number.scm")
-	    (__r4_numbers_6_5_fixnum   "Ieee/fixnum.scm")
-	    (__r4_numbers_6_5_flonum   "Ieee/flonum.scm")
-	    (__r4_characters_6_6       "Ieee/char.scm")
-	    (__r4_equivalence_6_2      "Ieee/equiv.scm")
-	    (__r4_booleans_6_1         "Ieee/boolean.scm")
-	    (__r4_symbols_6_4          "Ieee/symbol.scm")
-	    (__r4_strings_6_7          "Ieee/string.scm")
-	    (__r4_pairs_and_lists_6_3  "Ieee/pair-list.scm")
-	    (__r4_input_6_10_2         "Ieee/input.scm")
-	    (__r4_control_features_6_9 "Ieee/control.scm")
-	    (__r4_vectors_6_8          "Ieee/vector.scm")
-	    (__r4_ports_6_10_1         "Ieee/port.scm")
-	    (__r4_output_6_10_3        "Ieee/output.scm")
-	    (__evenv                   "Eval/evenv.scm")))
+	    __r4_numbers_6_5
+	    __r4_numbers_6_5_fixnum
+	    __r4_numbers_6_5_flonum
+	    __r4_characters_6_6
+	    __r4_equivalence_6_2
+	    __r4_booleans_6_1
+	    __r4_symbols_6_4
+	    __r4_strings_6_7
+	    __r4_pairs_and_lists_6_3
+	    __r4_input_6_10_2
+	    __r4_control_features_6_9
+	    __r4_vectors_6_8
+	    __r4_ports_6_10_1
+	    __r4_output_6_10_3
+	    __evenv))
 	    
 ;;;--------------------------------------------------------------------*/
 ;;;   Renvoie la liste des variables apparaissant dans un filtre       */
@@ -360,7 +328,7 @@
    (cond
       ((or (isAny? descr)
 	   (isAny? pat)
-	   (isOr? pat) (isT-Or? pat) (isTagged-or? pat)
+	   (isOr? pat) (isT-Or? pat) (isTagged-Or? pat)
 	   (isSuccess? pat)
 	   (isCheck? pat)
 	   (isTimes? pat)
@@ -569,7 +537,7 @@
 ;;;   to extend it.                                                    */
 ;;;--------------------------------------------------------------------*/
 
-(define (Vector-Plus v i d)
+(define (vector-plus v i d)
    (if (>=fx i (vector-length (caddr v)))
        (set-car! (cddr v)
 		 (extend-vector (caddr v) i '(any)))
@@ -580,7 +548,7 @@
 		   (pattern-plus (vector-ref (caddr v) i) d))
       res))
 
-(define (Vector-Minus v i d)
+(define (vector-minus v i d)
    (if (>=fx i (vector-length (caddr v)))
        (set-car! (cddr v)
 		 (extend-vector (caddr v) i '(any)))
