@@ -1,41 +1,9 @@
-;*---------------------------------------------------------------------*/
-;*   A pratical implementation for the Scheme programming language     */
-;*                                                                     */
-;*                                    ,--^,                            */
-;*                              _ ___/ /|/                             */
-;*                          ,;'( )__, ) '                              */
-;*                         ;;  //   L__.                               */
-;*                         '   \\   /  '                               */
-;*                              ^   ^                                  */
-;*                                                                     */
-;*   Copyright (c) 1992-1999 Manuel Serrano                            */
-;*                                                                     */
-;*     Bug descriptions, use reports, comments or suggestions are      */
-;*     welcome. Send them to                                           */
-;*       bigloo-request@kaolin.unice.fr                                */
-;*       http://kaolin.unice.fr/bigloo                                 */
-;*                                                                     */
-;*   This program is free software; you can redistribute it            */
-;*   and/or modify it under the terms of the GNU General Public        */
-;*   License as published by the Free Software Foundation; either      */
-;*   version 2 of the License, or (at your option) any later version.  */
-;*                                                                     */
-;*   This program is distributed in the hope that it will be useful,   */
-;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
-;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
-;*   GNU General Public License for more details.                      */
-;*                                                                     */
-;*   You should have received a copy of the GNU General Public         */
-;*   License along with this program; if not, write to the Free        */
-;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
-;*   MA 02111-1307, USA.                                               */
-;*---------------------------------------------------------------------*/
 ;*=====================================================================*/
 ;*    serrano/prgm/project/bigloo/runtime/Pp/pp.scm                    */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Sat Nov 27 14:10:31 1993                          */
-;*    Last change :  Sat Sep 26 18:10:21 1998 (serrano)                */
+;*    Last change :  Mon Nov 19 15:10:39 2001 (serrano)                */
 ;*    -------------------------------------------------------------    */
 ;*    Le pretty-printer de Marc Feeley.                                */
 ;*=====================================================================*/
@@ -45,29 +13,29 @@
 ;*---------------------------------------------------------------------*/
 (module __pp
 
-   (import  (__error                   "Llib/error.scm")
-	    (__r4_output_6_10_3        "Ieee/output.scm"))
+   (import  __error
+	    __r4_output_6_10_3)
    
-   (use     (__type                    "Llib/type.scm")
-	    (__bigloo                  "Llib/bigloo.scm")
-	    (__tvector                 "Llib/tvector.scm")
-	    (__structure               "Llib/struct.scm")
-	    (__tvector                 "Llib/tvector.scm")
+   (use     __type
+	    __bigloo
+	    __tvector
+	    __structure
+	    __tvector
 
-	    (__r4_numbers_6_5          "Ieee/number.scm")
-	    (__r4_numbers_6_5_fixnum   "Ieee/fixnum.scm")
-	    (__r4_numbers_6_5_flonum   "Ieee/flonum.scm")
-	    (__r4_characters_6_6       "Ieee/char.scm")
-	    (__r4_equivalence_6_2      "Ieee/equiv.scm")
-	    (__r4_booleans_6_1         "Ieee/boolean.scm")
-	    (__r4_symbols_6_4          "Ieee/symbol.scm")
-	    (__r4_strings_6_7          "Ieee/string.scm")
-	    (__r4_pairs_and_lists_6_3  "Ieee/pair-list.scm")
-	    (__r4_input_6_10_2         "Ieee/input.scm")
-	    (__r4_control_features_6_9 "Ieee/control.scm")
-	    (__r4_vectors_6_8          "Ieee/vector.scm")
-	    (__r4_ports_6_10_1         "Ieee/port.scm")
-	    (__evenv                   "Eval/evenv.scm"))
+	    __r4_numbers_6_5
+	    __r4_numbers_6_5_fixnum
+	    __r4_numbers_6_5_flonum
+	    __r4_characters_6_6
+	    __r4_equivalence_6_2
+	    __r4_booleans_6_1
+	    __r4_symbols_6_4
+	    __r4_strings_6_7
+	    __r4_pairs_and_lists_6_3
+	    __r4_input_6_10_2
+	    __r4_control_features_6_9
+	    __r4_vectors_6_8
+	    __r4_ports_6_10_1
+	    __evenv)
 
    (export (pp::unspecified exp . port)
 	   *pp-width*
@@ -91,7 +59,7 @@
 		   (current-output-port)
 		   (let ((port (car port)))
 		      (if (not (output-port? port))
-			  (error "pp" "not an input-port" port)
+			  (error "pp" "not an output-port" port)
 			  port)))))
       (generic-write exp #f *pp-width* (lambda (s) (display s port) #t))
       #unspecified))
@@ -157,71 +125,89 @@
     (and col (output str) (+fx col (string-length str))))
 
   (define (wr obj col)
-
-    (define (wr-expr expr col)
-      (if (read-macro? expr)
-        (wr (read-macro-body expr) (out (read-macro-prefix expr) col))
-        (wr-lst expr col)))
-
-    (define (wr-lst l col)
-      (if (pair? l)
-        (let loop ((l (cdr l)) (col (wr (car l) (out "(" col))))
-          (and col
-               (cond ((pair? l) (loop (cdr l) (wr (car l) (out " " col))))
-                     ((null? l) (out ")" col))
-                     (else      (out ")" (wr l (out " . " col)))))))
-        (out "()" col)))
-
-    (cond ((match-case obj
-	      ((comment (? integer?) (? string?))
-	       #t)
-	      (else
-	       #f))
-	   (let ((add (- *pp-width* (string-length (caddr obj)) 3)))
-	      (if (<=fx add 0)
-		  (out (caddr obj) col)
-		  (out (string-append (caddr obj) (make-string add #\space))
-		       col))))
-	  ((pair? obj)        (wr-expr obj col))
-          ((null? obj)        (wr-lst obj col))
-          ((vector? obj)      (wr-lst (vector->list obj)
-				      (out (vector-prefix obj) col)))
-          ((boolean? obj)     (out (if obj "#t" "#f") col))
-          ((number? obj)      (out (number->string obj) col))
-          ((symbol? obj)      (case *pp-case*
-				 ((respect)
-				  (out (symbol->string obj) col))
-				 ((upper)
-				  (out (string-upcase (symbol->string obj))
-				       col))
-				 (else
-				  (out (string-downcase (symbol->string obj))
-				       col))))
-          ((procedure? obj)   (out "#<procedure>" col))
-          ((string? obj)      (let ((obj (string-for-read obj)))
-				 (if display?
-				     (out obj col)
-				     (let loop ((i 0)
-						(j 0) 
-						(col (out "#\"" col)))
-					(if (and col (<fx j (string-length obj)))
-					    (let ((c (string-ref obj j)))
-					       (loop i (+fx j 1) col))
-					    (out "\""
-						 (out (substring obj i j)
-						      col)))))))
-          ((char? obj)        (if display?
-                                (out (make-string 1 obj) col)
-				(let ((p (open-output-string)))
-				   (write obj p)
-				   (out (close-output-port p) col))))
-          ((input-port? obj)  (out "#[input-port]" col))
-          ((output-port? obj) (out "#[output-port]" col))
-          ((eof-object? obj)  (out "#[eof-object]" col))
-          (else               (out (let ((p (open-output-string)))
-				      (write obj p)
-				      (close-output-port p))
-				   col))))
+     
+     (define (wr-expr expr col)
+	(if (read-macro? expr)
+	    (wr (read-macro-body expr) (out (read-macro-prefix expr) col))
+	    (wr-lst expr col)))
+     
+     (define (wr-lst l col)
+	(if (pair? l)
+	    (let loop ((l (cdr l)) (col (wr (car l) (out "(" col))))
+	       (and col
+		    (cond ((pair? l) (loop (cdr l) (wr (car l) (out " " col))))
+			  ((null? l) (out ")" col))
+			  (else      (out ")" (wr l (out " . " col)))))))
+	    (out "()" col)))
+     
+     (cond ((match-case obj
+	       ((comment (? integer?) (? string?))
+		#t)
+	       (else
+		#f))
+	    (let ((add (- *pp-width* (string-length (caddr obj)) 3)))
+	       (if (<=fx add 0)
+		   (out (caddr obj) col)
+		   (out (string-append (caddr obj) (make-string add #\space))
+			col))))
+	   ((pair? obj)
+	    (wr-expr obj col))
+	   ((null? obj)
+	    (wr-lst obj col))
+	   ((vector? obj)
+	    (wr-lst (vector->list obj)
+		    (out (vector-prefix obj) col)))
+	   ((boolean? obj)
+	    (out (if obj "#t" "#f") col))
+	   ((number? obj)
+	    (out (number->string obj) col))
+	   ((symbol? obj)
+	    (let ((p (open-output-string)))
+	       (if display?
+		   (display obj p)
+		   (write obj p))
+	       (case *pp-case*
+		  ((respect)
+		   (out (close-output-port p) col))
+		  ((upper)
+		   (out (string-upcase (close-output-port p))
+			col))
+		  (else
+		   (out (string-downcase (close-output-port p))
+			col)))))
+	   ((procedure? obj)
+	    (out "#<procedure>" col))
+	   ((string? obj)
+	    (let ((obj (string-for-read obj)))
+	       (if display?
+		   (out obj col)
+		   (let loop ((i 0)
+			      (j 0) 
+			      (col (out (if *bigloo-strict-r5rs-strings* "#\"" "\"")
+					col)))
+		      (if (and col (<fx j (string-length obj)))
+			  (let ((c (string-ref obj j)))
+			     (loop i (+fx j 1) col))
+			  (out "\""
+			       (out (substring obj i j)
+				    col)))))))
+	   ((char? obj)
+	    (if display?
+		(out (make-string 1 obj) col)
+		(let ((p (open-output-string)))
+		   (write obj p)
+		   (out (close-output-port p) col))))
+	   ((input-port? obj)
+	    (out "#[input-port]" col))
+	   ((output-port? obj)
+	    (out "#[output-port]" col))
+	   ((eof-object? obj)
+	    (out "#[eof-object]" col))
+	   (else
+	    (out (let ((p (open-output-string)))
+		    (write obj p)
+		    (close-output-port p))
+		 col))))
 
   (define (pp obj col)
 
