@@ -1,41 +1,9 @@
-;*---------------------------------------------------------------------*/
-;*   A pratical implementation for the Scheme programming language     */
-;*                                                                     */
-;*                                    ,--^,                            */
-;*                              _ ___/ /|/                             */
-;*                          ,;'( )__, ) '                              */
-;*                         ;;  //   L__.                               */
-;*                         '   \\   /  '                               */
-;*                              ^   ^                                  */
-;*                                                                     */
-;*   Copyright (c) 1992-1999 Manuel Serrano                            */
-;*                                                                     */
-;*     Bug descriptions, use reports, comments or suggestions are      */
-;*     welcome. Send them to                                           */
-;*       bigloo-request@kaolin.unice.fr                                */
-;*       http://kaolin.unice.fr/bigloo                                 */
-;*                                                                     */
-;*   This program is free software; you can redistribute it            */
-;*   and/or modify it under the terms of the GNU General Public        */
-;*   License as published by the Free Software Foundation; either      */
-;*   version 2 of the License, or (at your option) any later version.  */
-;*                                                                     */
-;*   This program is distributed in the hope that it will be useful,   */
-;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
-;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
-;*   GNU General Public License for more details.                      */
-;*                                                                     */
-;*   You should have received a copy of the GNU General Public         */
-;*   License along with this program; if not, write to the Free        */
-;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
-;*   MA 02111-1307, USA.                                               */
-;*---------------------------------------------------------------------*/
 ;*=====================================================================*/
 ;*    serrano/prgm/project/bigloo/runtime/R5rs/expand5.scm             */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Willian Clinger                                   */
 ;*    Creation    :  Sat Mar 21 17:18:51 1998                          */
-;*    Last change :  Fri Sep 18 17:25:43 1998 (serrano)                */
+;*    Last change :  Wed Dec  5 13:24:54 2001 (serrano)                */
 ;*    -------------------------------------------------------------    */
 ;*  Copyright 1992 William Clinger                                     */
 ;*                                                                     */
@@ -51,6 +19,8 @@
 ;*  The external entry points and kernel of the macro expander.        */
 ;*                                                                     */
 ;*  Part of this code is snarfed from the Twobit macro expander.       */
+;*    -------------------------------------------------------------    */
+;*    Many thanks to Leonardo de Moura who fixes this code.            */
 ;*=====================================================================*/
 
 ;*---------------------------------------------------------------------*/
@@ -58,38 +28,38 @@
 ;*---------------------------------------------------------------------*/
 (module __r5_syntax_expand
    
+   (use    __type
+           __error
+           __bigloo
+           __tvector
+           __structure
+           __tvector
+           __bexit
+	   
+           __r4_numbers_6_5
+           __r4_numbers_6_5_fixnum
+           __r4_numbers_6_5_flonum
+           __r4_characters_6_6
+           __r4_equivalence_6_2
+           __r4_booleans_6_1
+           __r4_symbols_6_4
+           __r4_strings_6_7
+           __r4_pairs_and_lists_6_3
+           __r4_input_6_10_2
+           __r4_control_features_6_9
+           __r4_vectors_6_8
+           __r4_ports_6_10_1
+           __r4_output_6_10_3
+	   
+	   __r5_syntax_misc
+	   __r5_syntax_syntaxenv
+	   __r5_syntax_syntaxrules
+	   __r5_syntax_prefs)
+   
    (export (internal-expand-syntax ::obj)
 	   (desugar-definitions ::obj ::obj)
 	   define-syntax-scope
-	   m-quit)
-
-   (use    (__type                       "Llib/type.scm")
-           (__error                      "Llib/error.scm")
-           (__bigloo                     "Llib/bigloo.scm")
-           (__tvector                    "Llib/tvector.scm")
-           (__structure                  "Llib/struct.scm")
-           (__tvector                    "Llib/tvector.scm")
-           (__bexit                      "Llib/bexit.scm")
-            
-           (__r4_numbers_6_5             "Ieee/number.scm")
-           (__r4_numbers_6_5_fixnum      "Ieee/fixnum.scm")
-           (__r4_numbers_6_5_flonum      "Ieee/flonum.scm")
-           (__r4_characters_6_6          "Ieee/char.scm")
-           (__r4_equivalence_6_2         "Ieee/equiv.scm")
-           (__r4_booleans_6_1            "Ieee/boolean.scm")
-           (__r4_symbols_6_4             "Ieee/symbol.scm")
-           (__r4_strings_6_7             "Ieee/string.scm")
-           (__r4_pairs_and_lists_6_3     "Ieee/pair-list.scm")
-           (__r4_input_6_10_2            "Ieee/input.scm")
-           (__r4_control_features_6_9    "Ieee/control.scm")
-           (__r4_vectors_6_8             "Ieee/vector.scm")
-           (__r4_ports_6_10_1            "Ieee/port.scm")
-           (__r4_output_6_10_3           "Ieee/output.scm")
-
-	   (__r5_syntax_misc             "R5rs/misc5.scm")
-	   (__r5_syntax_syntaxenv        "R5rs/syntaxenv5.scm")
-	   (__r5_syntax_syntaxrules      "R5rs/syntaxrules5.scm")
-	   (__r5_syntax_prefs            "R5rs/prefs5.scm")))
+	   m-quit))
 
 (define define-syntax-scope
   (let ((flag 'letrec))
@@ -221,8 +191,8 @@
             ((eq? keyword denotation-of-begin)         (m-begin exp env))
             ((eq? keyword denotation-of-let-syntax)    (m-let-syntax exp env))
             ((eq? keyword denotation-of-letrec-syntax) (m-letrec-syntax exp env))
-            ((or (eq? keyword denotation-of-define)
-                 (eq? keyword denotation-of-define-syntax))
+	    ((eq? keyword denotation-of-define)        (m-define exp env))
+            ((eq? keyword denotation-of-define-syntax)
              (m-error "Definition out of context" exp))
             (else (m-bug "Bug detected in m-expand" exp env))))
           ((macro) (m-macro exp env))
@@ -247,15 +217,34 @@
       (list quote1 (m-strip (cadr exp)))
       (m-error "Malformed quoted constant" exp)))
 
+(define (m-define exp env)
+   (if (> (safe-length exp) 2)
+       (let ((definition (cadr exp)))
+	  (if (list? definition)
+	      ;; (define (fun ...) ...)
+	      (let* ((name (car definition))
+		     (formals (cdr definition))
+		     (alist (rename-vars (make-null-terminated formals)))
+		     (env (syntactic-rename env alist))
+		     (body (cddr exp)))
+		 (list define1
+		       (cons name (rename-formals formals alist))
+		       (m-body body env)))
+	      ;; (define constant ...)
+	      (let ((name definition)
+		    (body (cddr exp)))
+		 (list define1 name (m-body body env)))))
+       (m-error "Malformed define expression" exp)))
+
 (define (m-lambda exp env)
   (if (> (safe-length exp) 2)
       (let* ((formals (cadr exp))
              (alist (rename-vars (make-null-terminated formals)))
              (env (syntactic-rename env alist))
              (body (cddr exp)))
-        (list lambda1
-              (rename-formals formals alist)
-              (m-body body env)))
+	 (list lambda1
+	       (rename-formals formals alist)
+	       (m-body body env)))
       (m-error "Malformed lambda expression" exp)))
 
 (define (m-body body env)
