@@ -1,41 +1,9 @@
-;*---------------------------------------------------------------------*/
-;*   A pratical implementation for the Scheme programming language     */
-;*                                                                     */
-;*                                    ,--^,                            */
-;*                              _ ___/ /|/                             */
-;*                          ,;'( )__, ) '                              */
-;*                         ;;  //   L__.                               */
-;*                         '   \\   /  '                               */
-;*                              ^   ^                                  */
-;*                                                                     */
-;*   Copyright (c) 1992-1999 Manuel Serrano                            */
-;*                                                                     */
-;*     Bug descriptions, use reports, comments or suggestions are      */
-;*     welcome. Send them to                                           */
-;*       bigloo-request@kaolin.unice.fr                                */
-;*       http://kaolin.unice.fr/bigloo                                 */
-;*                                                                     */
-;*   This program is free software; you can redistribute it            */
-;*   and/or modify it under the terms of the GNU General Public        */
-;*   License as published by the Free Software Foundation; either      */
-;*   version 2 of the License, or (at your option) any later version.  */
-;*                                                                     */
-;*   This program is distributed in the hope that it will be useful,   */
-;*   but WITHOUT ANY WARRANTY; without even the implied warranty of    */
-;*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the     */
-;*   GNU General Public License for more details.                      */
-;*                                                                     */
-;*   You should have received a copy of the GNU General Public         */
-;*   License along with this program; if not, write to the Free        */
-;*   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,   */
-;*   MA 02111-1307, USA.                                               */
-;*---------------------------------------------------------------------*/
 ;*=====================================================================*/
 ;*    serrano/prgm/project/bigloo/runtime/R5rs/init5.scm               */
 ;*    -------------------------------------------------------------    */
 ;*    Author      :  Manuel Serrano                                    */
 ;*    Creation    :  Sat Mar 21 17:13:11 1998                          */
-;*    Last change :  Fri Sep 18 17:25:26 1998 (serrano)                */
+;*    Last change :  Mon May  7 18:37:42 2001 (serrano)                */
 ;*    -------------------------------------------------------------    */
 ;*    The entry point of the Will Clinger expand-syntax code.          */
 ;*    -------------------------------------------------------------    */
@@ -359,39 +327,39 @@
 ;*---------------------------------------------------------------------*/
 (module __r5_macro_4_3_init
    
-   (use    (__type                       "Llib/type.scm")
-           (__error                      "Llib/error.scm")
-           (__bigloo                     "Llib/bigloo.scm")
-           (__tvector                    "Llib/tvector.scm")
-           (__structure                  "Llib/struct.scm")
-           (__tvector                    "Llib/tvector.scm")
-           (__bexit                      "Llib/bexit.scm")
-           (__os                         "Llib/os.scm")
+   (use    __type
+           __error
+           __bigloo
+           __tvector
+           __structure
+           __tvector
+           __bexit
+           __os
             
-           (__r4_numbers_6_5             "Ieee/number.scm")
-           (__r4_numbers_6_5_fixnum      "Ieee/fixnum.scm")
-           (__r4_numbers_6_5_flonum      "Ieee/flonum.scm")
-           (__r4_characters_6_6          "Ieee/char.scm")
-           (__r4_equivalence_6_2         "Ieee/equiv.scm")
-           (__r4_booleans_6_1            "Ieee/boolean.scm")
-           (__r4_symbols_6_4             "Ieee/symbol.scm")
-           (__r4_strings_6_7             "Ieee/string.scm")
-           (__r4_pairs_and_lists_6_3     "Ieee/pair-list.scm")
-           (__r4_input_6_10_2            "Ieee/input.scm")
-           (__r4_control_features_6_9    "Ieee/control.scm")
-           (__r4_vectors_6_8             "Ieee/vector.scm")
-           (__r4_ports_6_10_1            "Ieee/port.scm")
-           (__r4_output_6_10_3           "Ieee/output.scm")
+           __r4_numbers_6_5
+           __r4_numbers_6_5_fixnum
+           __r4_numbers_6_5_flonum
+           __r4_characters_6_6
+           __r4_equivalence_6_2
+           __r4_booleans_6_1
+           __r4_symbols_6_4
+           __r4_strings_6_7
+           __r4_pairs_and_lists_6_3
+           __r4_input_6_10_2
+           __r4_control_features_6_9
+           __r4_vectors_6_8
+           __r4_ports_6_10_1
+           __r4_output_6_10_3
             
-           (define-primop!     __evenv   "Eval/evenv.scm")
-           (define-primop-ref! __evenv   "Eval/evenv.scm"))
+           (define-primop!     __evenv)
+           (define-primop-ref! __evenv))
    
-   (import (__r5_syntax_expand      "R5rs/expand5.scm")
-	   (__r5_syntax_misc        "R5rs/misc5.scm")
-	   (__r5_syntax_prefs       "R5rs/prefs5.scm")
-	   (__r5_syntax_syntaxenv   "R5rs/syntaxenv5.scm")
-	   (__r5_syntax_syntaxrules "R5rs/syntaxrules5.scm")
-	   (__r5_syntax_usual       "R5rs/usual5.scm"))
+   (import __r5_syntax_expand
+	   __r5_syntax_misc
+	   __r5_syntax_prefs
+	   __r5_syntax_syntaxenv
+	   __r5_syntax_syntaxrules
+	   __r5_syntax_usual)
 
    (eval   (export expand-syntax)
            (export *hygien?*))
